@@ -33,11 +33,7 @@
 #define IPEBASE_H
 
 #include <cstdio>
-#include <cstring>
 #include <vector>
-#include <map>
-#include <unordered_map>
-#include <list>
 #include <algorithm>
 #include <memory>
 
@@ -47,37 +43,38 @@
 
 // --------------------------------------------------------------------
 
-#define IPE_PI 3.14159265358979323846
-
-using uchar = unsigned char;
-using uint = unsigned int;
-
-#ifdef WIN32
-#define IPESEP ('\\')
-#else
-#define IPESEP ('/')
-#endif
-
 #undef assert
 #define assert(e) ((e) ? (void)0 : ipeAssertionFailed(__FILE__, __LINE__, #e))
 extern void ipeAssertionFailed(const char *, int, const char *);
 
-// --------------------------------------------------------------------
-
 extern void ipeDebug(const char *msg, ...) noexcept;
 
+// --------------------------------------------------------------------
+
 namespace ipe {
 
+  template <typename T>
+  inline int size(const std::vector<T> &v) {
+    return static_cast<int>(v.size()); }
+
+  constexpr double IPE_PI = 3.14159265358979323846;
+
+#ifdef WIN32
+  constexpr char IPESEP = '\\';
+#else
+  constexpr char IPESEP = '/';
+#endif
+
   //! Ipelib version.
   /*! \ingroup base */
-  const int IPELIB_VERSION = 70209;
+  const int IPELIB_VERSION = 70213;
 
   //! Oldest readable file format version.
   /*! \ingroup base */
   const int OLDEST_FILE_FORMAT = 70000;
   //! Current file format version.
   /*! \ingroup base */
-  const int FILE_FORMAT = 70206;
+  const int FILE_FORMAT = 70212;
 
   enum class LatexType { Default, Pdftex, Xetex, Luatex };
 
@@ -94,7 +91,8 @@ namespace ipe {
     String(const String &rhs, int index, int len) noexcept;
     String &operator=(const String &rhs) noexcept;
 #ifdef WIN32
-    String(const wchar_t *data) noexcept;
+    String(const wchar_t *data);
+    std::wstring w() const noexcept;
 #endif
     ~String() noexcept;
     static String withData(char *data, int len = 0) noexcept;
@@ -227,34 +225,22 @@ namespace ipe {
 
   class Buffer {
   public:
-    Buffer() noexcept = default;
-    ~Buffer() noexcept;
-    Buffer(const Buffer &rhs) noexcept;
-    Buffer(Buffer &&rhs) noexcept;
-    Buffer &operator=(const Buffer &rhs) noexcept;
-    Buffer &operator=(Buffer &&rhs) noexcept;
-
-    explicit Buffer(int size) noexcept;
-    explicit Buffer(const char *data, int size) noexcept;
+    explicit Buffer() = default;
+    explicit Buffer(int size);
+    explicit Buffer(const char *data, int size);
     //! Character access.
-    inline char &operator[](int index) noexcept { return iImp->iData[index]; }
+    inline char &operator[](int index) noexcept { return (*iData)[index]; }
     //! Character access (const version).
-    inline const char &operator[](int index) const noexcept {
-      return iImp->iData[index]; }
+    inline const char &operator[](int index) const noexcept { return (*iData)[index]; }
     //! Return size of buffer;
-    inline int size() const noexcept { return iImp ? iImp->iSize : 0; }
+    inline int size() const noexcept { return iData ? static_cast<int>((*iData).size()) : 0; }
     //! Return pointer to buffer data.
-    inline char *data() noexcept { return iImp->iData; }
+    inline char *data() noexcept { return iData ? iData->data() : nullptr; }
     //! Return pointer to buffer data (const version).
-    inline const char *data() const noexcept { return iImp->iData; }
+    inline const char *data() const noexcept { return iData ? iData->data() : nullptr; }
 
   private:
-    struct Imp {
-      int iRefCount;
-      char *iData;
-      int iSize;
-    };
-    Imp *iImp = { nullptr };
+    std::shared_ptr<std::vector<char>> iData;
   };
 
   // --------------------------------------------------------------------
@@ -353,15 +339,13 @@ namespace ipe {
     using DebugHandler = void (*)(const char *);
 
 #ifdef IPEBUNDLE
-    static String ipeDir(const char *suffix, const char *fname = 0);
+    static String ipeDir(const char *suffix, const char *fname = nullptr);
 #endif
 #ifdef WIN32
-    static String winConv(const String &utf8, const String &encoding);
-    static void utf8ToWide(const char *utf8, std::vector<wchar_t> &wbuf);
     static FILE *fopen(const char *fname, const char *mode);
     static int mkdir(const char *dname);
 #else
-    static FILE *fopen(const char *fname, const char *mode) {
+    inline static FILE *fopen(const char *fname, const char *mode) {
       return ::fopen(fname, mode);
     }
 #endif
@@ -374,9 +358,11 @@ namespace ipe {
     static String latexPath();
     static bool fileExists(String fname);
     static bool listDirectory(String path, std::vector<String> &files);
+    static String realPath(String fname);
     static String readFile(String fname);
-    static int runLatex(String dir, LatexType engine) noexcept;
+    static int runLatex(String dir, LatexType engine, String docname) noexcept;
     static double toDouble(String s);
+    static int toNumber(String s, int &iValue, double &dValue);
   };
 
   // --------------------------------------------------------------------
