@@ -44,6 +44,27 @@ namespace ipe {
   class BitmapFinder;
   class PdfResources;
 
+  //! Flags for saving Ipe documents (to PDF)
+  class SaveFlag {
+  public:
+    enum {
+      SaveNormal = 0,   //!< Nothing special
+      Export = 1,       //!< Don't include Ipe markup
+      NoZip = 2,        //!< Do not compress streams
+      MarkedView = 4,   //!< Create marked views only
+      KeepNotes = 8,    //!< Keep page notes as PDF annotations even when exporting
+    };
+  };
+
+  //! There are several Ipe document formats.
+  enum class FileFormat {
+    Xml,  //!< Save as XML
+    Pdf,  //!< Save as PDF
+    Eps,  //!< Encapsulated Postscript (loading only)
+    Ipe5,  //!< Ancient Ipe format
+    Unknown //!< Unknown file format
+  };
+
   class Document {
   public:
     //! Properties of a document
@@ -63,23 +84,6 @@ namespace ipe {
       String iCreator;
     };
 
-    //! There are several Ipe document formats.
-    enum TFormat {
-      EXml,  //!< Save as XML
-      EPdf,  //!< Save as PDF
-      EEps,  //!< Encapsulated Postscript (loading only)
-      EIpe5,  //!< Ancient Ipe format
-      EUnknown //!< Unknown file format
-    };
-
-    //! Options for saving Ipe documents (to PDF and Postscript)
-    enum {
-      ESaveNormal = 0, //!< Nothing special
-      EExport = 1,     //!< Don't include Ipe markup
-      ENoZip = 2,      //!< Do not compress streams
-      EMarkedView = 4, //!< Create marked views only
-    };
-
     //! Errors that can happen while loading documents
     enum LoadErrors {
       EVersionTooOld = -1,    //!< The version of the file is too old.
@@ -93,26 +97,26 @@ namespace ipe {
     Document &operator=(const Document &rhs) = delete;
     ~Document();
 
-    static TFormat fileFormat(DataSource &source);
-    static TFormat formatFromFilename(String fn);
+    static FileFormat fileFormat(DataSource &source);
+    static FileFormat formatFromFilename(String fn);
 
-    static Document *load(DataSource &source, TFormat format, int &reason);
+    static Document *load(DataSource &source, FileFormat format, int &reason);
 
     static Document *load(const char *fname, int &reason);
     static Document *loadWithErrorReport(const char *fname);
 
-    bool save(TellStream &stream, TFormat format, uint flags) const;
-    bool save(const char *fname, TFormat format, uint flags) const;
-    bool exportPages(const char *fname, uint flags,
+    bool save(TellStream &stream, FileFormat format, uint32_t flags) const;
+    bool save(const char *fname, FileFormat format, uint32_t flags) const;
+    bool exportPages(const char *fname, uint32_t flags,
 		     int fromPage, int toPage) const;
-    bool exportView(const char *fname, TFormat format,
-		    uint flags, int pno, int vno) const;
+    bool exportView(const char *fname, FileFormat format,
+		    uint32_t flags, int pno, int vno) const;
 
     void saveAsXml(Stream &stream, bool usePdfBitmaps = false) const;
 
     //! Return number of pages of document.
-    size_t countPages() const { return int(iPages.size()); }
-    size_t countTotalViews() const;
+    int countPages() const { return int(iPages.size()); }
+    int countTotalViews() const;
 
     //! Return page (const version).
     /*! The first page is no 0. */
@@ -148,8 +152,8 @@ namespace ipe {
     //! Error codes returned by RunLatex.
     enum { ErrNone, ErrNoText, ErrNoDir, ErrWritingSource,
 	   ErrOldPdfLatex, ErrRunLatex, ErrLatex, ErrLatexOutput };
-    int runLatex(String &logFile);
-    int runLatex();
+    int runLatex(String docname, String &logFile);
+    int runLatex(String docname);
 
 
   private:
