@@ -45,32 +45,40 @@ namespace ipe {
     explicit Image(const XmlAttributes &attr, String data);
     explicit Image(const XmlAttributes &attr, Bitmap bitmap);
 
-    virtual Object *clone() const;
+    virtual Object *clone() const override;
 
-    virtual Image *asImage();
+    virtual Image *asImage() override;
 
-    virtual Type type() const;
+    virtual Type type() const override;
 
-    virtual void saveAsXml(Stream &stream, String layer) const;
-    virtual void draw(Painter &painter) const;
-    virtual void drawSimple(Painter &painter) const;
+    virtual void saveAsXml(Stream &stream, String layer) const override;
+    virtual void draw(Painter &painter) const override;
+    virtual void drawSimple(Painter &painter) const override;
 
-    virtual void accept(Visitor &visitor) const;
+    virtual void accept(Visitor &visitor) const override;
 
     virtual double distance(const Vector &v, const Matrix &m,
-			    double bound) const;
-    virtual void addToBBox(Rect &box, const Matrix &m, bool) const;
+			    double bound) const override;
+    virtual void addToBBox(Rect &box, const Matrix &m, bool) const override;
     virtual void snapCtl(const Vector &mouse, const Matrix &m,
-			 Vector &pos, double &bound) const;
+			 Vector &pos, double &bound) const override;
 
     inline Rect rect() const;
     inline Bitmap bitmap() const;
 
+    virtual bool setAttribute(Property prop, Attribute value) override;
+    virtual Attribute getAttribute(Property prop) const noexcept override;
+
+    void setOpacity(Attribute opaq);
+    //! Return opacity of the opject.
+    inline Attribute opacity() const { return iOpacity; }
+
   private:
     void init(const XmlAttributes &attr);
   private:
     Rect iRect;
     Bitmap iBitmap;
+    Attribute iOpacity;
   };
 
   // --------------------------------------------------------------------
