@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2016  Otfried Cheong
+    Copyright (c) 1993-2019 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -61,11 +61,14 @@ namespace ipe {
     void setLineJoin(TLineJoin s);
     void setFillRule(TFillRule s);
     void setOpacity(Attribute opaq);
+    void setStrokeOpacity(Attribute opaq);
     void setTiling(Attribute a);
     void setGradient(Attribute a);
 
     //! Return opacity of the opject.
     inline Attribute opacity() const { return iOpacity; }
+    //! Return stroke opacity of the opject.
+    inline Attribute strokeOpacity() const { return iStrokeOpacity; }
     //! Return tiling pattern.
     inline Attribute tiling() const { return iTiling; }
     //! Return gradient fill.
@@ -140,6 +143,7 @@ namespace ipe {
     Attribute iDashStyle;
     Attribute iPen;
     Attribute iOpacity;
+    Attribute iStrokeOpacity;
     Attribute iTiling;
     Attribute iGradient;
 
