@@ -93,8 +93,8 @@ namespace ipe {
     virtual bool setAttribute(Property prop, Attribute value);
     virtual Attribute getAttribute(Property prop) const noexcept;
 
-    inline uint flags() const { return iFlags; }
-    static uint flagsFromName(String name);
+    inline uint32_t flags() const { return iFlags; }
+    static uint32_t flagsFromName(String name);
 
   private:
     Attribute iName;
@@ -103,7 +103,8 @@ namespace ipe {
     Attribute iStroke;
     Attribute iFill;
     Attribute iPen;
-    uint iFlags;
+    uint32_t iFlags;
+    mutable std::vector<Vector> iSnap;   // caching info from the symbol itself
   };
 
 } // namespace
