@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2016  Otfried Cheong
+    Copyright (c) 1993-2019 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -41,7 +41,34 @@
 
 namespace ipe {
 
-  class PdfResources {
+  class PdfResourceBase {
+  public:
+    PdfResourceBase();
+    virtual ~PdfResourceBase();
+
+    virtual const PdfObj *object(int num) const noexcept = 0;
+
+    const PdfObj *getDeep(const PdfDict *d, String key) const noexcept;
+    const PdfDict *getDict(const PdfDict *d, String key) const noexcept;
+    const PdfDict *resourcesOfKind(String kind) const noexcept;
+    const PdfDict *findResource(String kind, String name) const noexcept;
+    const PdfDict *findResource(const PdfDict *xf, String kind,
+				String name) const noexcept;
+  protected:
+    std::unique_ptr<PdfDict> iPageResources;
+  };
+
+  class PdfFileResources : public PdfResourceBase {
+  public:
+    PdfFileResources(const PdfFile *file);
+    ~PdfFileResources() = default;
+
+    virtual const PdfObj *object(int num) const noexcept;
+  private:
+    const PdfFile *iPdf;
+  };
+
+  class PdfResources : public PdfResourceBase {
   public:
     struct SPageNumber {
       int page;
@@ -50,14 +77,10 @@ namespace ipe {
     };
   public:
     PdfResources();
+    virtual ~PdfResources() = default;
     bool collect(const PdfDict *resources, PdfFile *file);
-    const PdfObj *object(int num) const noexcept;
-    const PdfDict *resourcesOfKind(String kind) const noexcept;
-    const PdfDict *findResource(String kind, String name) const noexcept;
-    const PdfDict *findResource(const PdfDict *xf, String kind,
-				String name) const noexcept;
-    const PdfObj *getDeep(const PdfDict *d, String key) const noexcept;
-    const PdfDict *getDict(const PdfDict *d, String key) const noexcept;
+    virtual const PdfObj *object(int num) const noexcept;
+    virtual const PdfDict *baseResources() const noexcept;
     void addPageNumber(SPageNumber &pn) noexcept;
     const Text *pageNumber(int page, int view) const noexcept;
     inline const std::vector<int> &embedSequence() const noexcept {
@@ -69,7 +92,6 @@ namespace ipe {
     bool addToResource(PdfDict *d, String key,
 		       const PdfObj *el, PdfFile *file);
   private:
-    std::unique_ptr<PdfDict> iPageResources;
     std::unordered_map<int, std::unique_ptr<const PdfObj>> iObjects;
     std::vector<int> iEmbedSequence;
     //! Page number objects.
