@@ -71,7 +71,7 @@ namespace ipe {
 		 Vector &pos, double &bound) const;
   private:
     CurveSegment(Type type, int num, const Vector *cp,
-		 const Matrix *m = 0);
+		 const Matrix *m = nullptr);
   private:
     Type iType;
     const Vector *iCP;
