@@ -66,10 +66,10 @@ namespace ipe {
 				 const Page *page, int view,
 				 double snapDist) const noexcept;
     TSnapModes simpleSnap(Vector &pos, const Page *page, int view,
-			  double snapDist, Tool *tool = 0) const noexcept;
+			  double snapDist, Tool *tool = nullptr) const noexcept;
     TSnapModes snap(Vector &pos, const Page *page, int view,
-		    double snapDist, Tool *tool = 0,
-		    Vector *autoOrg = 0) const noexcept;
+		    double snapDist, Tool *tool = nullptr,
+		    Vector *autoOrg = nullptr) const noexcept;
     Line getLine(const Vector &mouse, const Vector &base) const noexcept;
     bool setEdge(const Vector &pos, const Page *page, int view) noexcept;
   };
