@@ -83,7 +83,7 @@ namespace ipe {
     virtual void close();
   private:
     Stream &iStream;
-    uchar iCh[4];
+    uint8_t iCh[4];
     int iN;
     int iCol;
   };
@@ -95,7 +95,7 @@ namespace ipe {
     virtual void close();
   private:
     Stream &iStream;
-    uchar iCh[3];
+    uint8_t iCh[3];
     int iN;
     int iCol;
   };
@@ -130,7 +130,7 @@ namespace ipe {
     bool iEof;
     int iN;
     int iIndex;
-    uchar iBuf[4];
+    uint8_t iBuf[4];
   };
 
   class Base64Source : public DataSource {
@@ -143,7 +143,7 @@ namespace ipe {
     bool  iEof;
     int   iIndex;
     int   iBufLen;
-    uchar iBuf[3];
+    uint8_t iBuf[3];
   };
 
   class InflateSource : public DataSource {
