@@ -57,32 +57,32 @@ AppUiBase::AppUiBase(lua_State *L0, int model)
   lua_getglobal(L, "prefs");
   lua_getfield(L, -1, "coordinates_format");
   if (lua_isstring(L, -1)) {
-    iCoordinatesFormat = lua_tostring(L, -1);
+    iCoordinatesFormat = lua_tolstring(L, -1, nullptr);
   }
   lua_pop(L, 1); // coordinates_format
 
   lua_getfield(L, -1, "width_notes_bookmarks");
   if (lua_isnumber(L, -1)) {
-    iWidthNotesBookmarks = (int) lua_tointeger(L, -1);
+    iWidthNotesBookmarks = lua_tointegerx(L, -1, nullptr);
   }
   lua_pop(L, 1); // width_notes_bookmarks
 
   iUiScale = 100;
   lua_getfield(L, -1, "ui_scale");
   if (lua_isnumber(L, -1))
-    iUiScale = (int) lua_tointeger(L, -1);
+    iUiScale = lua_tointegerx(L, -1, nullptr);
   lua_pop(L, 1); // ui_scale
 
   iToolbarScale = 100;
   lua_getfield(L, -1, "toolbar_scale");
   if (lua_isnumber(L, -1))
-    iToolbarScale = (int) lua_tointeger(L, -1);
+    iToolbarScale = lua_tointegerx(L, -1, nullptr);
   lua_pop(L, 1); // toolbar_scale
 
   iUiGap = 0;
   lua_getfield(L, -1, "ui_gap");
   if (lua_isnumber(L, -1))
-    iUiGap = (int) lua_tointeger(L, -1);
+    iUiGap = lua_tointegerx(L, -1, nullptr);
   lua_pop(L, 1); // ui_gap
 
   lua_getfield(L, -1, "mini_edit");
@@ -100,7 +100,7 @@ AppUiBase::AppUiBase(lua_State *L0, int model)
     for (int i = 1; i <= n; ++i) {
       lua_rawgeti(L, -1, i);
       if (lua_isnumber(L, -1))
-	iScalings.push_back(lua_tointeger(L, -1));
+	iScalings.push_back(lua_tointegerx(L, -1, nullptr));
       lua_pop(L, 1); // number
     }
   }
@@ -139,6 +139,7 @@ void AppUiBase::buildMenus()
   addItem(EFileMenu);
   addItem(EFileMenu, "Export as PNG", "export_png");
   addItem(EFileMenu, "Export as EPS", "export_eps");
+  addItem(EFileMenu, "Export as SVG", "export_svg");
   addItem(EFileMenu);
   addItem(EFileMenu, "Insert image", "insert_image");
   addItem(EFileMenu);
@@ -277,8 +278,9 @@ void AppUiBase::buildMenus()
   startSubMenu(ESnapMenu, "Radial angle", ESubmenuAngleSize);
   iAngleSizeMenu = endSubMenu();
   addItem(ESnapMenu);
+  addItem(ESnapMenu, "Set origin", "@set_origin");
   addItem(ESnapMenu, "Set origin && snap", "@set_origin_snap");
-  addItem(ESnapMenu, "Hide axes", "@hide_axes");
+  addItem(ESnapMenu, "Show axes", "@*show_axes");
   addItem(ESnapMenu, "Set direction", "@set_direction");
   addItem(ESnapMenu, "Set tangent direction", "@set_tangent_direction");
   addItem(ESnapMenu, "Reset direction", "@reset_direction");
@@ -297,10 +299,9 @@ void AppUiBase::buildMenus()
   endSubMenu();
 
   startSubMenu(EZoomMenu, "Coordinate scale");
-  for (uint i = 0; i < iScalings.size(); ++i) {
+  for (int s : iScalings) {
     char display[32];
     char action[32];
-    int s = iScalings[i];
     if (s < 0)
       sprintf(display, "%d:1", -s);
     else
@@ -343,7 +344,7 @@ void AppUiBase::buildMenus()
   addItem(EViewMenu, "Delete view", "delete_view");
   addItem(EViewMenu);
   addItem(EViewMenu, "Jump to view", "jump_view");
-  addItem(EViewMenu, "Edit effects", "edit_effects");
+  addItem(EViewMenu, "Edit view", "edit_view");
 
   addItem(EPageMenu, "Next page", "next_page");
   addItem(EPageMenu, "Previous page", "previous_page");
@@ -368,10 +369,14 @@ void AppUiBase::buildMenus()
 #endif
 
   addItem(EHelpMenu, "Ipe &manual", "manual");
+  addItem(EHelpMenu, "Preferences", "preferences");
   addItem(EHelpMenu, "Onscreen keyboard", "@keyboard");
   addItem(EHelpMenu, "Show &configuration", "show_configuration");
   addItem(EHelpMenu, "Show &libraries", "show_libraries");
   addItem(EHelpMenu, "&Ipelet information", "about_ipelets");
+#ifdef IPECURL
+  addItem(EHelpMenu, "Enable online Latex-compilation", "cloud_latex");
+#endif
 
   lua_getglobal(L, "prefs");
   lua_getfield(L, -1, "developer");
@@ -397,10 +402,10 @@ void AppUiBase::buildMenus()
       lua_pop(L, 2); // label, ipelet
       continue;
     }
-    String label(lua_tostring(L, -1));
+    String label(lua_tolstring(L, -1, nullptr));
     lua_pop(L, 1);
     lua_getfield(L, -1, "name");
-    String name(lua_tostring(L, -1));
+    String name(lua_tolstring(L, -1, nullptr));
     lua_pop(L, 1);
     lua_getfield(L, -1, "methods");
     char buf[20];
@@ -417,7 +422,7 @@ void AppUiBase::buildMenus()
 	sprintf(buf, "ipelet_%d_", j);
 	String action(buf);
 	action += name;
-	addSubItem(lua_tostring(L, -1), action.z());
+	addSubItem(lua_tolstring(L, -1, nullptr), action.z());
 	lua_pop(L, 2); // sublabel, method
       }
       endSubMenu();
@@ -439,7 +444,7 @@ void AppUiBase::canvasObserverWheelMoved(double xDegrees, double yDegrees,
     lua_pushnumber(L, xDegrees);
     lua_pushnumber(L, yDegrees);
     lua_pushinteger(L, kind);
-    lua_call(L, 4, 0);
+    luacall(L, 4, 0);
   } else
     // result of a zoom gesture (Windows only, currently)
     setZoom(iCanvas->zoom());
@@ -494,7 +499,7 @@ void AppUiBase::canvasObserverMouseAction(int button)
   lua_getfield(L, -1, "mouseButtonAction");
   lua_insert(L, -2); // model
   push_button(L, button);
-  lua_call(L, 3, 0);
+  luacall(L, 3, 0);
 }
 
 void AppUiBase::canvasObserverSizeChanged()
@@ -502,7 +507,7 @@ void AppUiBase::canvasObserverSizeChanged()
   lua_rawgeti(L, LUA_REGISTRYINDEX, iModel);
   lua_getfield(L, -1, "sizeChanged");
   lua_insert(L, -2); // model
-  lua_call(L, 1, 0);
+  luacall(L, 1, 0);
 }
 
 // --------------------------------------------------------------------
@@ -531,7 +536,7 @@ void AppUiBase::luaSelector(String name, String value)
     lua_pushboolean(L, false);
   else
     push_string(L, value);
-  lua_call(L, 3, 0);
+  luacall(L, 3, 0);
 }
 
 void AppUiBase::luaAbsoluteButton(const char *s)
@@ -541,7 +546,7 @@ void AppUiBase::luaAbsoluteButton(const char *s)
   lua_getfield(L, -1, "absoluteButton");
   lua_insert(L, -2); // method, model
   lua_pushstring(L, s);
-  lua_call(L, 2, 0);
+  luacall(L, 2, 0);
 }
 
 // --------------------------------------------------------------------
@@ -576,7 +581,7 @@ void AppUiBase::luaAction(String name)
     lua_getfield(L, -1, "action");
     lua_insert(L, -2); // before model
     push_string(L, name);
-    lua_call(L, 2, 0);
+    luacall(L, 2, 0);
   }
 }
 
@@ -586,7 +591,7 @@ void AppUiBase::luaShowPathStylePopup(Vector v)
   lua_getfield(L, -1, "showPathStylePopup");
   lua_insert(L, -2); // before model
   push_vector(L, v);
-  lua_call(L, 2, 0);
+  luacall(L, 2, 0);
 }
 
 void AppUiBase::luaShowLayerBoxPopup(Vector v, String layer)
@@ -596,7 +601,7 @@ void AppUiBase::luaShowLayerBoxPopup(Vector v, String layer)
   lua_insert(L, -2); // before model
   push_vector(L, v);
   push_string(L, layer);
-  lua_call(L, 3, 0);
+  luacall(L, 3, 0);
 }
 
 void AppUiBase::luaLayerAction(String name, String layer)
@@ -606,7 +611,7 @@ void AppUiBase::luaLayerAction(String name, String layer)
   lua_insert(L, -2); // before model
   push_string(L, name);
   push_string(L, layer);
-  lua_call(L, 3, 0);
+  luacall(L, 3, 0);
 }
 
 void AppUiBase::luaBookmarkSelected(int index)
@@ -615,7 +620,7 @@ void AppUiBase::luaBookmarkSelected(int index)
   lua_getfield(L, -1, "bookmark");
   lua_insert(L, -2); // method, model
   lua_pushnumber(L, index + 1);
-  lua_call(L, 2, 0);
+  luacall(L, 2, 0);
 }
 
 // --------------------------------------------------------------------
@@ -635,12 +640,12 @@ void AppUiBase::showInCombo(const Cascade *sheet, Kind kind,
 {
   AttributeSeq seq;
   sheet->allNames(kind, seq);
-  if (!seq.size() && deflt != 0) {
+  if (!seq.size() && deflt != nullptr) {
     addCombo(sel, deflt);
     iComboContents[sel].push_back(deflt);
   }
-  for (uint i = 0; i < seq.size(); ++i) {
-    String s = seq[i].string();
+  for (const auto & att : seq) {
+    String s = att.string();
     addCombo(sel, s);
     iComboContents[sel].push_back(s);
   }
@@ -650,8 +655,8 @@ void AppUiBase::showMarksInCombo(const Cascade *sheet)
 {
   AttributeSeq seq;
   sheet->allNames(ESymbol, seq);
-  for (uint i = 0; i < seq.size(); ++i) {
-    String s = stripMark(seq[i]);
+  for (const auto & att : seq) {
+    String s = stripMark(att);
     if (!s.empty()) {
       addCombo(EUiMarkShape, s);
       iComboContents[EUiMarkShape].push_back(s);
@@ -666,8 +671,8 @@ void AppUiBase::setupSymbolicNames(const Cascade *sheet)
     iComboContents[i].clear();
   AttributeSeq seq, absColor;
   sheet->allNames(EColor, seq);
-  for (uint i = 0; i < seq.size(); ++i)
-    absColor.push_back(sheet->find(EColor, seq[i]));
+  for (const auto & att : seq)
+    absColor.push_back(sheet->find(EColor, att));
   addComboColors(seq, absColor);
   showInCombo(sheet, EPen, EUiPen);
   showInCombo(sheet, ETextSize, EUiTextSize);
@@ -683,7 +688,7 @@ void AppUiBase::setGridAngleSize(Attribute abs_grid, Attribute abs_angle)
   iCascade->allNames(EGridSize, seq);
   if (!seq.size())
     setComboCurrent(EUiGridSize, 0);
-  for (uint i = 0; i < seq.size(); ++i) {
+  for (int i = 0; i < size(seq); ++i) {
     if (iCascade->find(EGridSize, seq[i]) == abs_grid) {
       setComboCurrent(EUiGridSize, i);
       break;
@@ -693,7 +698,7 @@ void AppUiBase::setGridAngleSize(Attribute abs_grid, Attribute abs_angle)
   iCascade->allNames(EAngleSize, seq);
   if (!seq.size())
     setComboCurrent(EUiAngleSize, 0);
-  for (uint i = 0; i < seq.size(); ++i) {
+  for (int i = 0; i < size(seq); ++i) {
     if (iCascade->find(EAngleSize, seq[i]) == abs_angle) {
       setComboCurrent(EUiAngleSize, i);
       break;
