@@ -130,8 +130,7 @@ public:  // What platforms must implement:
 protected:   // What platforms must implement:
   virtual void addRootMenu(int id, const char *name) = 0;
   // if title == 0, add separator
-  virtual void addItem(int id, const char *title = 0,
-		       const char *name = 0) = 0;
+  virtual void addItem(int id, const char *title = nullptr, const char *name = nullptr) = 0;
   virtual void startSubMenu(int id, const char *name, int tag = 0) = 0;
   virtual void addSubItem(const char *title, const char *name) = 0;
   virtual MENUHANDLE endSubMenu() = 0;
@@ -163,7 +162,7 @@ protected:
 
   void buildMenus();
   void showInCombo(const Cascade *sheet, Kind kind,
-		   int sel, const char *deflt = 0);
+		   int sel, const char *deflt = nullptr);
   void showMarksInCombo(const Cascade *sheet);
   void setAttribute(int sel, Attribute a);
 
