@@ -3,7 +3,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2016  Otfried Cheong
+    Copyright (c) 1993-2019 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -51,6 +51,23 @@ static const char * const snapbutton_action[] = {
   "snapgrid", "snapangle", "snapauto",
   "grid_visible" };
 
+static const char * const touchbar_action[] = {
+  "escape", "set_origin", "set_direction", "set_line",
+  "show_axes", "reset_direction", "set_tangent_direction",
+  "snapvtx", "snapctl", "snapbd", "snapint",
+  "snapgrid", "snapangle", "snapauto" };
+
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= 101202
+static const char * const touchbar_titles[] = {
+  "esc", "org", "dir", "line", "axes", "reset", "tangent",
+  nullptr, nullptr, nullptr, nullptr, nullptr, nullptr, nullptr };
+
+static const char * const touchbar_customization_label[] = {
+  "Escape", "Set origin", "Set direction", "Set to edge", "Show axes", "Reset direction", "Set to tangent direction",
+  "Snap to vertex", "Snap to control point", "Snap to boundary", "Snap to intersection", "Snap to grid",
+  "Angular snap", "Automatic snap" };
+#endif
+
 @interface IpeAction : NSObject
 @property NSString *name;
 @property NSString *title;
@@ -145,22 +162,14 @@ static NSImage *loadIcon(String action)
 
 // --------------------------------------------------------------------
 
-@interface IpeWindowDelegate :
-  NSObject <NSWindowDelegate, NSToolbarDelegate, IpeControlsDelegate>
+#if MAC_OS_X_VERSION_MAX_ALLOWED < 101202
+@protocol NSTouchBarDelegate
+@end
+#endif
 
-- (instancetype) initFor:(AppUi *) ui;
-- (void) ipeMenuAction:(id) sender;
-- (void) ipeToolbarAction:(id) sender;
-- (void) ipeAbsoluteButton:(id) sender;
-- (void) ipeSnapButton:(id) sender;
-- (void) ipeSelectorChanged:(id) sender;
-- (void) toggleSnapbarShown:(id) sender;
-- (void) ipeSubmenu:(id) sender;
-- (BOOL) ipeIsModified:(id) sender;
-#pragma clang diagnostic push
-#pragma clang diagnostic ignored "-Wdeprecated-declarations"
-- (void) ipeMagnificationGesture:(NSGestureRecognizer *) gestureRecognizer;
-#pragma clang diagnostic pop
+// Window delegate is also an NSWindowController object to handle touch bar
+@interface IpeWindowDelegate : NSWindowController
+  <NSWindowDelegate, NSToolbarDelegate, IpeControlsDelegate, NSTouchBarDelegate>
 @end
 
 @implementation IpeWindowDelegate {
@@ -169,7 +178,7 @@ static NSImage *loadIcon(String action)
 
 - (instancetype) initFor:(AppUi *) ui
 {
-  self = [super init];
+  self = [super initWithWindow:nil];
   if (self) {
     appui = ui;
   }
@@ -264,17 +273,6 @@ static NSImage *loadIcon(String action)
   appui->snapButton([sender tag]);
 }
 
-#pragma clang diagnostic push
-#pragma clang diagnostic ignored "-Wdeprecated-declarations"
-- (void) ipeMagnificationGesture:(NSGestureRecognizer *) gestureRecognizer
-{
-  NSMagnificationGestureRecognizer *p = (NSMagnificationGestureRecognizer *)
-    gestureRecognizer;
-  NSPoint q = [p locationInView:p.view];
-  NSLog(@"magnify! %g %g %ld %g" , q.x, q.y, p.state, 1.0 + p.magnification);
-}
-#pragma clang diagnostic pop
-
 - (void) ipeSelectorChanged:(id) sender
 {
   appui->selectorChanged([sender tag]);
@@ -386,6 +384,58 @@ static NSImage *loadIcon(String action)
 	     @"mode_circle1", @"mode_circle2", @"mode_circle3",
 	     @"mode_ink" ];
 }
+
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= 101202
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wdeprecated-declarations"
+- (NSTouchBar *) makeTouchBar
+{
+  NSTouchBar *bar = [[NSTouchBar alloc] init];
+  bar.delegate = self;
+  bar.defaultItemIdentifiers = @[ @"show_axes", @"set_origin", @"set_direction", @"snapvtx", @"snapctl", @"snapbd", @"snapint", @"snapgrid" ];
+  bar.customizationIdentifier = @"org.otfried.ipe.mainbar7_2_11";
+  bar.customizationAllowedItemIdentifiers =
+    @[ @"show_axes", @"set_origin", @"set_direction", @"reset_direction",
+	@"set_line", @"set_tangent_direction",
+	@"snapvtx", @"snapctl", @"snapbd", @"snapint", @"snapgrid", @"snapangle", @"snapauto" ];
+  bar.escapeKeyReplacementItemIdentifier = @"escape";
+  return bar;
+}
+
+- (NSTouchBarItem *) touchBar: (NSTouchBar *) bar
+	makeItemForIdentifier: (NSTouchBarItemIdentifier) identifier
+{
+  String idf = N2I(identifier);
+  for (size_t tag = 0; tag < (sizeof(touchbar_action)/sizeof(const char *)); ++tag) {
+    if (idf == touchbar_action[tag]) {
+      NSCustomTouchBarItem *item = [[NSCustomTouchBarItem alloc] initWithIdentifier:identifier];
+      auto title = touchbar_titles[tag];
+      NSButton *b;
+      if (title == nullptr) {
+	auto image = loadIcon(touchbar_action[tag]);
+	b = [NSButton buttonWithTitle:identifier image:image target:self action:@selector(ipeTouchBar:)];
+	b.buttonType = NSPushOnPushOffButton;
+	b.imagePosition = NSImageOnly;
+      } else {
+	b = [NSButton buttonWithTitle:C2N(title) target:self action:@selector(ipeTouchBar:)];
+	if (idf == "show_axes")
+	  b.buttonType = NSPushOnPushOffButton;
+      }
+      b.tag = tag;
+      item.view = b;
+      item.customizationLabel = C2N(touchbar_customization_label[tag]);
+      return item;
+    }
+  }
+  return nil;
+}
+#pragma clang diagnostic pop
+#endif
+
+- (void) ipeTouchBar:(id) sender
+{
+  appui->action(touchbar_action[[sender tag]]);
+}
 @end
 
 // --------------------------------------------------------------------
@@ -399,7 +449,7 @@ void AppUi::addRootMenu(int id, const char *name)
 
 void AppUi::addItem(NSMenu *menu, const char *title, const char *name)
 {
-  if (title == 0) {
+  if (title == nullptr) {
     if (build_menus)
       [menu addItem:[NSMenuItem separatorItem]];
   } else {
@@ -507,7 +557,7 @@ void AppUi::addItem(int id, const char *title, const char *name)
   addItem(menu, title, name);
 }
 
-static NSMenu * __unsafe_unretained current_submenu = 0;
+static NSMenu * __unsafe_unretained current_submenu = nullptr;
 
 void AppUi::startSubMenu(int id, const char *name, int tag)
 {
@@ -518,7 +568,7 @@ void AppUi::startSubMenu(int id, const char *name, int tag)
 
   NSString *title = ipeui_set_mnemonic(C2N(name), nil);
   NSMenuItem *item = [[NSMenuItem alloc] initWithTitle:title
-						action:NULL
+						action:nullptr
 					 keyEquivalent:@""];
   NSMenu *submenu = [[NSMenu alloc] initWithTitle:title];
   item.submenu = submenu;
@@ -570,18 +620,6 @@ AppUi::AppUi(lua_State *L0, int model)
   [iWindow setDelegate:iDelegate];
   [iWindow setAcceptsMouseMovedEvents:YES];
 
-#if 0
-  if ([iView respondsToSelector:@selector(addGestureRecognizer:)]) {
-#pragma clang diagnostic push
-#pragma clang diagnostic ignored "-Wdeprecated-declarations"
-    NSGestureRecognizer *p = [[NSMagnificationGestureRecognizer alloc] init];
-    p.target = iDelegate;
-    p.action = @selector(ipeMagnificationGesture:);
-    [iView addGestureRecognizer:p];
-#pragma clang diagnostic pop
-  }
-#endif
-
   iActions = [NSMutableDictionary dictionaryWithCapacity:100];
 
   buildMenus();
@@ -595,6 +633,8 @@ AppUi::AppUi(lua_State *L0, int model)
   [tb setSizeMode:NSToolbarSizeModeSmall];
   [iWindow setToolbar:tb];
 
+  [iWindow setWindowController:iDelegate];
+
   iContent = [[NSView alloc] initWithFrame:contentRect];
 
   makePropertiesTool();
@@ -1071,6 +1111,7 @@ void AppUi::setNotes(String notes)
     NSAttributedString *n =
       [[NSAttributedString alloc] initWithString:I2N(notes)];
     [s setAttributedString:n];
+    [iNotesField setTextColor: [NSColor textColor]];
   }
 }
 
@@ -1169,7 +1210,7 @@ IpeAction *AppUi::findAction(NSString *name) const
 }
 
 // Determine if an action is checked.
-// Used for viewmarked, pagemarked, snapXXX, grid_visible,
+// Used for viewmarked, pagemarked, snapXXX, grid_visible, show_axes,
 // pretty_display, toggle_notes, toggle_bookmarks.
 bool AppUi::actionState(const char *name)
 {
@@ -1197,11 +1238,26 @@ void AppUi::setActionState(const char *name, bool value)
       iModeIndicator.image = loadIcon(name);
       iWindow.toolbar.selectedItemIdentifier = a;
     }
-  } else if ([a hasPrefix:@"snap"] || [a isEqualToString:@"grid_visible"]) {
+  } else if ([a hasPrefix:@"snap"] || [a isEqualToString:@"grid_visible"] ||
+	     [a isEqualToString:@"show_axes"]) {
     for (int i = 0; i < 8; ++i) {
       if ([a isEqualToString:C2N(snapbutton_action[i])])
 	iSnapButton[i].state = value ? NSOnState : NSOffState;
     }
+    // update touch bar if it exists
+#if MAC_OS_X_VERSION_MAX_ALLOWED >= 101202
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wdeprecated-declarations"
+    if (iDelegate.touchBar != nil) {
+      auto bar = (NSTouchBar *) iDelegate.touchBar;
+      auto item = [bar itemForIdentifier:a];
+      if (item != nil) {
+	auto b = (NSButton *) item.view;
+	b.state =  value ? NSOnState : NSOffState;
+      }
+    }
+#pragma clang diagnostic push
+#endif
   } else if ([a isEqualToString:@"shift_key"]) {
     int mod = s.state ? CanvasBase::EShift : 0;
     if (iCanvas)
@@ -1212,6 +1268,11 @@ void AppUi::setActionState(const char *name, bool value)
 void AppUi::action(String name)
 {
   ipeDebug("action %s", name.z());
+  if (name == "escape") {
+    if (iCanvas && iCanvas->tool())
+      iCanvas->tool()->key("\033", 0);
+    return;
+  }
   // Implement radio buttons
   int i = name.find("|");
   if (i >= 0)
