@@ -115,7 +115,7 @@ QAction *AppUi::findAction(const char *name) const
   if (it != iActions.end())
     return it->second;
   else
-    return 0;
+    return nullptr;
 }
 
 void AppUi::addItem(QMenu *m, const QString &title, const char *name)
@@ -177,13 +177,13 @@ void AppUi::addRootMenu(int id, const char *name)
 // if title and name == 0, add separator (default)
 void AppUi::addItem(int id, const char *title, const char *name)
 {
-  if (title == 0)
+  if (title == nullptr)
     iMenu[id]->addSeparator();
   else
     addItem(iMenu[id], QString::fromUtf8(title), name);
 }
 
-static QMenu *submenu = 0;
+static QMenu *submenu = nullptr;
 static int submenuId = 0;
 
 void AppUi::startSubMenu(int id, const char *name, int tag)
@@ -211,7 +211,7 @@ Qt::DockWidgetArea getDockSide(lua_State *L, const char *name,
   lua_getfield(L, -1, "tools_placement");
   if (lua_istable(L, -1)) {
     lua_getfield(L, -1, name);
-    const char *s = lua_tolstring(L, -1, 0);
+    const char *s = lua_tolstring(L, -1, nullptr);
     if (!strcmp(s, "left"))
       r = Qt::LeftDockWidgetArea;
     else if (!strcmp(s, "right"))
@@ -241,7 +241,7 @@ AppUi::AppUi(lua_State *L0, int model, Qt::WFlags f)
 #else
 AppUi::AppUi(lua_State *L0, int model, Qt::WindowFlags f)
 #endif
-  : QMainWindow(0, f), AppUiBase(L0, model)
+    : QMainWindow(nullptr, f), AppUiBase(L0, model)
 {
   setWindowIcon(prefsIcon("ipe"));
   QMainWindow::setAttribute(Qt::WA_DeleteOnClose);
@@ -389,7 +389,7 @@ AppUi::AppUi(lua_State *L0, int model, Qt::WindowFlags f)
   QButtonGroup *bg = new QButtonGroup(wg);
   bg->setExclusive(false);
   connect(bg, SIGNAL(buttonClicked(int)), this, SLOT(absoluteButton(int)));
-  iButton[EUiMarkShape] = 0; // no such button
+  iButton[EUiMarkShape] = nullptr;   // no such button
   for (int i = 0; i <= EUiSymbolSize; ++i) {
     if (i != EUiMarkShape) {
       iButton[i] = new QToolButton();
@@ -528,13 +528,22 @@ AppUi::~AppUi()
 
 // --------------------------------------------------------------------
 
+static void populateLayerMenu(LayerBox *layerList, QMenu *menu)
+{
+  menu->clear();
+  for (int i = 0; i < layerList->count(); ++i) {
+    LayerItem *item = dynamic_cast<LayerItem *>(layerList->item(i));
+    if (item) {
+      QAction *a = new QAction(QIpe(item->ipeLayerName), menu);
+      menu->addAction(a);
+    }
+  }
+
+}
+
 void AppUi::aboutToShowSelectLayerMenu()
 {
-  iSelectLayerMenu->clear();
-  for (int i = 0; i < iLayerList->count(); ++i) {
-    QAction *a = new QAction(iLayerList->item(i)->text(), iSelectLayerMenu);
-    iSelectLayerMenu->addAction(a);
-  }
+  populateLayerMenu(iLayerList, iSelectLayerMenu);
 }
 
 void AppUi::selectLayerAction(QAction *a)
@@ -544,12 +553,7 @@ void AppUi::selectLayerAction(QAction *a)
 
 void AppUi::aboutToShowMoveToLayerMenu()
 {
-  iMoveToLayerMenu->clear();
-  for (int i = 0; i < iLayerList->count(); ++i) {
-    ipeDebug("action: %s", IpeQ(iLayerList->item(i)->text()).z());
-    QAction *a = new QAction(iLayerList->item(i)->text(), iMoveToLayerMenu);
-    iMoveToLayerMenu->addAction(a);
-  }
+  populateLayerMenu(iLayerList, iMoveToLayerMenu);
 }
 
 void AppUi::moveToLayerAction(QAction *a)
@@ -747,7 +751,7 @@ void AppUi::bookmarkSelected(QListWidgetItem *item)
 
 void AppUi::setToolVisible(int m, bool vis)
 {
-  QWidget *tool = 0;
+  QWidget *tool = nullptr;
   switch (m) {
   case 0: tool = iPropertiesTools; break;
   case 1: tool = iBookmarkTools; break;
@@ -868,8 +872,11 @@ static const char * const aboutText =
   " for further information.</p>"
   "<p>You can \"like\" Ipe and follow Ipe announcements on "
   "<a href=\"http://www.facebook.com/drawing.editor.Ipe7\">Facebook</a>.</p>"
+  "<p>If you are an Ipe fan and want to show others, have a look at the "
+  "<a href=\"https://www.shirtee.com/en/store/ipe\">Ipe T-shirts</a>.</p>"
   "<h3>Platinum sponsors</h3>"
-  "<ul><li>Martin Ziegler</li></ul>"
+  "<ul><li>Hee-Kap Ahn</li>"
+  "<li>Martin Ziegler</li></ul>"
   "<p>If you enjoy Ipe, you can become a member of the exclusive community of "
   "<a href=\"http://patreon.com/otfried\">Ipe patrons</a>. "
   "For the price of a cup of coffee per month you can make a meaningful contribution "
@@ -1017,26 +1024,16 @@ static int getImage(lua_State *L, QImage &img)
   QImage im1 = img.convertToFormat(QImage::Format_ARGB32);
   int w = im1.width();
   int h = im1.height();
-  bool isGray = im1.allGray();
-
-  Bitmap::TColorSpace cs = isGray ? Bitmap::EDeviceGray : Bitmap::EDeviceRGB;
-  int datalen = w * h * (isGray ? 1 : 3);
-  Buffer data(datalen);
-  char *d = data.data();
+  Buffer data(w * h * sizeof(uint32_t));
+  uint32_t *d = (uint32_t *) data.data();
   for (int y = 0; y < h; ++y) {
-    uint *p = (uint *) im1.scanLine(y);
+    uint32_t *p = (uint32_t *) im1.scanLine(y);
     for (int x = 0; x < w; ++x) {
-      if (isGray) {
-	*d++ = qRed(*p++);
-      } else {
-	*d++ = qRed(*p);
-	*d++ = qGreen(*p);
-	*d++ = qBlue(*p++);
-      }
+      *d++ = *p++;
     }
   }
 
-  Bitmap bitmap(w, h, cs, 8, data, Bitmap::EDirect, true);
+  Bitmap bitmap(w, h, Bitmap::ENative, data);
   Rect r(Vector::ZERO, Vector(w, h));
   Image *im = new Image(r, bitmap);
   push_object(L, im);
