@@ -64,7 +64,7 @@ public:
 #if QT_VERSION < 0x050000
   AppUi(lua_State *L0, int model, Qt::WFlags f=0);
 #else
-  AppUi(lua_State *L0, int model, Qt::WindowFlags f=0);
+  AppUi(lua_State *L0, int model, Qt::WindowFlags f=nullptr);
 #endif
   ~AppUi();
 
