@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2016  Otfried Cheong
+    Copyright (c) 1993-2019 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -35,10 +35,6 @@
 
 #include "ipelua.h"
 
-// for version info only
-#include "ipefonts.h"
-#include <zlib.h>
-
 #include <cstdio>
 #include <cstdlib>
 
@@ -53,7 +49,6 @@
 #define IDC_LAYERS 7300
 #define IDBASE 8000
 #define TEXT_STYLE_BASE 8300
-#define ID_LAYERLIST 8400
 #define ID_SELECTOR_BASE 8500
 #define ID_ABSOLUTE_BASE 8600
 #define ID_PATHVIEW 8700
@@ -69,7 +64,7 @@ using namespace ipelua;
 
 const int TBICONSIZE = 24;
 
-static HINSTANCE win_hInstance = 0;
+static HINSTANCE win_hInstance = nullptr;
 static int win_nCmdShow = 0;
 
 const wchar_t AppUi::className[] = L"ipeWindowClass";
@@ -92,13 +87,12 @@ static HBITMAP loadIcon(String action, int w, int h, int r0, int g0, int b0)
     fname = base + ".png";
   // ipeDebug("loadIcon %s at %d x %d from %s", action.z(), w, h, fname.z());
   if (!Platform::fileExists(fname))
-    return NULL;
+    return nullptr;
 
-  WString wname(fname);
-  Gdiplus::Bitmap *bitmap = Gdiplus::Bitmap::FromFile(wname, FALSE);
+  Gdiplus::Bitmap *bitmap = Gdiplus::Bitmap::FromFile(WString(fname).data(), FALSE);
   if (bitmap->GetLastStatus() != Gdiplus::Ok) {
     delete bitmap;
-    return NULL;
+    return nullptr;
   }
 
   int wd = bitmap->GetWidth();
@@ -120,8 +114,8 @@ static HBITMAP loadIcon(String action, int w, int h, int r0, int g0, int b0)
   bmi.bmiHeader.biPlanes = 1;
   bmi.bmiHeader.biBitCount = (r0 >= 0) ? 24 : 32;
   bmi.bmiHeader.biCompression = BI_RGB;
-  void *pbits = 0;
-  HBITMAP bm = CreateDIBSection(NULL, &bmi, DIB_RGB_COLORS, &pbits, NULL, 0);
+  void *pbits = nullptr;
+  HBITMAP bm = CreateDIBSection(nullptr, &bmi, DIB_RGB_COLORS, &pbits, nullptr, 0);
 
   unsigned char *p = (unsigned char *) bitmapData->Scan0;
   unsigned char *q = (unsigned char *) pbits;
@@ -171,9 +165,9 @@ static int loadIcon(String action, HIMAGELIST il, int scale)
 {
   int size = scale * TBICONSIZE / 100;
   HBITMAP bm = loadIcon(action, size, size, -1, -1, -1);
-  if (bm == NULL)
+  if (bm == nullptr)
     return -1;
-  int r = ImageList_Add(il, bm, NULL);
+  int r = ImageList_Add(il, bm, nullptr);
   DeleteObject(bm);
   return r;
 }
@@ -196,7 +190,7 @@ static HBITMAP colorIcon(Color color, int scale)
   COLORREF rgb = RGB(r, g, b);
   int cx = scale * 22 / 100;
   int cy = scale * 22 / 100;
-  HDC hdc = GetDC(NULL);
+  HDC hdc = GetDC(nullptr);
   HDC memDC = CreateCompatibleDC(hdc);
   HBITMAP bm = CreateCompatibleBitmap(hdc, cx, cy);
   SelectObject(memDC, bm);
@@ -206,7 +200,7 @@ static HBITMAP colorIcon(Color color, int scale)
     }
   }
   DeleteDC(memDC);
-  ReleaseDC(NULL, hdc);
+  ReleaseDC(nullptr, hdc);
   return bm;
 }
 
@@ -216,7 +210,7 @@ void AppUi::addRootMenu(int id, const char *name)
 {
   hRootMenu[id] = CreatePopupMenu();
   WString wname(name);
-  AppendMenuW(hMenuBar, MF_STRING | MF_POPUP, UINT(hRootMenu[id]),
+  AppendMenuW(hMenuBar, MF_STRING | MF_POPUP, (UINT_PTR) hRootMenu[id],
 	      wname.data());
 }
 
@@ -232,8 +226,8 @@ void AppUi::createAction(String name, String tooltip, bool canWhileDrawing)
 
 void AppUi::addItem(HMENU menu, const char *title, const char *name)
 {
-  if (title == 0) {
-    AppendMenu(menu, MF_SEPARATOR, 0, 0);
+  if (title == nullptr) {
+    AppendMenu(menu, MF_SEPARATOR, 0, nullptr);
   } else {
     bool canUseWhileDrawing = false;
     if (name[0] == '@') {
@@ -248,7 +242,7 @@ void AppUi::addItem(HMENU menu, const char *title, const char *name)
     lua_getfield(L, -1, name);
     String sc;
     if (lua_isstring(L, -1))
-      sc = lua_tostring(L, -1);
+      sc = lua_tolstring(L, -1, nullptr);
     lua_pop(L, 2); // shortcuts, shortcuts[name]
 
     String tooltip = title;
@@ -257,14 +251,12 @@ void AppUi::addItem(HMENU menu, const char *title, const char *name)
     createAction(name, tooltip, canUseWhileDrawing);
 
     if (sc.empty()) {
-      WString wtitle(title);
-      AppendMenuW(menu, MF_STRING, iActions.size() - 1 + IDBASE, wtitle);
+      AppendMenuW(menu, MF_STRING, iActions.size() - 1 + IDBASE, WString(title).data());
     } else {
       String t = title;
       t += "\t";
       t += sc;
-      WString wt(t);
-      AppendMenu(menu, MF_STRING, iActions.size() - 1 + IDBASE, wt);
+      AppendMenu(menu, MF_STRING, iActions.size() - 1 + IDBASE, WString(t).data());
     }
   }
 }
@@ -274,13 +266,12 @@ void AppUi::addItem(int id, const char *title, const char *name)
   addItem(hRootMenu[id], title, name);
 }
 
-static HMENU submenu = 0;
+static HMENU submenu = nullptr;
 
 void AppUi::startSubMenu(int id, const char *name, int tag)
 {
   submenu = CreatePopupMenu();
-  WString wname(name);
-  AppendMenuW(hRootMenu[id], MF_STRING | MF_POPUP, UINT(submenu), wname);
+  AppendMenuW(hRootMenu[id], MF_STRING | MF_POPUP, (UINT_PTR) submenu, WString(name).data());
 }
 
 void AppUi::addSubItem(const char *title, const char *name)
@@ -295,11 +286,11 @@ MENUHANDLE AppUi::endSubMenu()
 
 HWND AppUi::createToolBar(HINSTANCE hInst)
 {
-  HWND tb = CreateWindowExW(0, TOOLBARCLASSNAMEW, NULL,
+  HWND tb = CreateWindowExW(0, TOOLBARCLASSNAMEW, nullptr,
 			    WS_CHILD | WS_VISIBLE |
 			    TBSTYLE_TOOLTIPS |
 			    CCS_NOPARENTALIGN | CCS_NORESIZE,
-			    0, 0, 0, 0, hwnd, NULL, hInst, NULL);
+			    0, 0, 0, 0, hwnd, nullptr, hInst, nullptr);
 
   // needed to set version of control library to be used
   SendMessage(tb, TB_BUTTONSTRUCTSIZE, (WPARAM) sizeof(TBBUTTON), 0);
@@ -312,7 +303,7 @@ void AppUi::addTButton(HWND tb, const char *name, int flags)
 {
   TBBUTTON tbb;
   ZeroMemory(&tbb, sizeof(tbb));
-  if (name == 0) {
+  if (name == nullptr) {
     tbb.iBitmap = 1;
     tbb.fsStyle = BTNS_SEP | flags;
     SendMessageW(tb, TB_ADDBUTTONS, 1, (LPARAM) &tbb);
@@ -344,8 +335,8 @@ void AppUi::setTooltip(HWND h, String tip, bool isComboBoxEx)
 
 HWND AppUi::createButton(HINSTANCE hInst, int id, int flags)
 {
-  return CreateWindowExW(0, L"button", NULL, WS_CHILD | WS_VISIBLE | flags,
-			 0, 0, 0, 0, hwnd, (HMENU) id, hInst, NULL);
+  return CreateWindowExW(0, L"button", nullptr, WS_CHILD | WS_VISIBLE | flags,
+			 0, 0, 0, 0, hwnd, (HMENU) (INT_PTR) id, hInst, nullptr);
 }
 
 void AppUi::toggleFullscreen()
@@ -378,7 +369,7 @@ void AppUi::toggleFullscreen()
     SetWindowLong(hwnd, GWL_STYLE, iWindowStyle);
     SetWindowLong(hwnd, GWL_EXSTYLE, iWindowExStyle);
 
-    SetWindowPos(hwnd, NULL, iWindowRect.left, iWindowRect.top,
+    SetWindowPos(hwnd, nullptr, iWindowRect.left, iWindowRect.top,
 		 iWindowRect.right - iWindowRect.left,
 		 iWindowRect.bottom - iWindowRect.top,
 		 SWP_NOZORDER | SWP_NOACTIVATE | SWP_FRAMECHANGED);
@@ -394,15 +385,15 @@ AppUi::AppUi(lua_State *L0, int model)
   : AppUiBase(L0, model)
 {
   // windows procedure may receive WM_SIZE message before canvas is created
-  iCanvas = 0;
+  iCanvas = nullptr;
   HWND hwnd = CreateWindowExW(WS_EX_CLIENTEDGE, className, L"Ipe",
 			      WS_OVERLAPPEDWINDOW,
 			      CW_USEDEFAULT, CW_USEDEFAULT,
 			      CW_USEDEFAULT, CW_USEDEFAULT,
-			      NULL, NULL, win_hInstance, this);
+			      nullptr, nullptr, win_hInstance, this);
 
-  if (hwnd == NULL) {
-    MessageBoxW(NULL, L"AppUi window creation failed!", L"Error!",
+  if (hwnd == nullptr) {
+    MessageBoxW(nullptr, L"AppUi window creation failed!", L"Error!",
 		MB_ICONEXCLAMATION | MB_OK);
     exit(9);
   }
@@ -421,25 +412,32 @@ static void insert_tb(HWND hRebar, REBARBANDINFO &rbBand, HWND tb, int size)
 // called during Window creation
 void AppUi::initUi()
 {
+  // update scale if dpi information is available
+  int dpi = Canvas::getDpiForWindow(hwnd);
+  if (iUiScale == 100)
+    iUiScale = dpi;
+  if (iToolbarScale == 100)
+    iToolbarScale = dpi;
+
   int tbsize = iToolbarScale * TBICONSIZE / 100;
   hIcons = ImageList_Create(tbsize, tbsize, ILC_COLOR32, 20, 4);
   hColorIcons = ImageList_Create(uiscale(22), uiscale(22), ILC_COLOR32, 20, 4);
-  HINSTANCE hInst = (HINSTANCE) GetWindowLong(hwnd, GWL_HINSTANCE);
+  HINSTANCE hInst = (HINSTANCE) GetWindowLongPtr(hwnd, GWLP_HINSTANCE);
 
   hMenuBar = CreateMenu();
   buildMenus();
   SetMenu(hwnd, hMenuBar);
 
-  hStatusBar = CreateWindowExW(0, STATUSCLASSNAMEW, NULL,
+  hStatusBar = CreateWindowExW(0, STATUSCLASSNAMEW, nullptr,
 			       WS_CHILD | WS_VISIBLE | SBARS_SIZEGRIP,
 			       0, 0, 0, 0,
-			       hwnd, (HMENU) IDC_STATUSBAR, hInst, NULL);
+			       hwnd, (HMENU) IDC_STATUSBAR, hInst, nullptr);
 
-  hTip = CreateWindowEx(0, TOOLTIPS_CLASS, NULL,
+  hTip = CreateWindowEx(0, TOOLTIPS_CLASS, nullptr,
 			WS_POPUP | TTS_NOPREFIX | TTS_ALWAYSTIP,
 			CW_USEDEFAULT, CW_USEDEFAULT,
 			CW_USEDEFAULT, CW_USEDEFAULT,
-			hwnd, NULL, hInst, NULL);
+			hwnd, nullptr, hInst, nullptr);
   SetWindowPos(hTip, HWND_TOPMOST, 0, 0, 0, 0,
 	       SWP_NOMOVE | SWP_NOSIZE | SWP_NOACTIVATE);
 
@@ -459,26 +457,39 @@ void AppUi::initUi()
 #define SNAP_BUTTONS 7
 
   hEditTools = createToolBar(hInst);
+  int EDIT_BUTTONS = 0;
 
-  addTButton(hEditTools, "copy");
-  addTButton(hEditTools, "cut");
-  addTButton(hEditTools, "paste");
-  addTButton(hEditTools, "delete");
+  if (iLeftDockFloats) {
+    createAction("dock_visible", "Show properties and layers", true);
+    addTButton(hEditTools, "dock_visible", BTNS_CHECK);
+    EDIT_BUTTONS += 1;
+  }
+
+  if (!isMiniEdit) {
+    addTButton(hEditTools, "copy");
+    addTButton(hEditTools, "cut");
+    addTButton(hEditTools, "paste");
+    addTButton(hEditTools, "delete");
+    EDIT_BUTTONS += 4;
+  }
   addTButton(hEditTools, "undo");
   addTButton(hEditTools, "redo");
-  addTButton(hEditTools, "zoom_in");
-  addTButton(hEditTools, "zoom_out");
-  addTButton(hEditTools, "fit_objects");
-  addTButton(hEditTools, "fit_page");
-  addTButton(hEditTools, "fit_width");
-  addTButton(hEditTools, "keyboard");
+  EDIT_BUTTONS += 2;
+  if (!isMiniEdit) {
+    addTButton(hEditTools, "zoom_in");
+    addTButton(hEditTools, "zoom_out");
+    addTButton(hEditTools, "fit_objects");
+    addTButton(hEditTools, "fit_page");
+    addTButton(hEditTools, "fit_width");
+    addTButton(hEditTools, "keyboard");
+    EDIT_BUTTONS += 6;
+  }
   createAction("shift_key", "Press the Shift key", true);
   addTButton(hEditTools, "shift_key", BTNS_CHECK);
   addTButton(hEditTools, "grid_visible", BTNS_CHECK);
   createAction("stop", "Abort object being drawn [Esc]", true);
   addTButton(hEditTools, "stop");
-
-#define EDIT_BUTTONS 15
+  EDIT_BUTTONS += 3;
 
   hObjectTools = createToolBar(hInst);
 
@@ -490,7 +501,7 @@ void AppUi::initUi()
   addTButton(hObjectTools, "mode_graph", BTNS_CHECKGROUP);
   addTButton(hObjectTools, "mode_pan", BTNS_CHECKGROUP);
   addTButton(hObjectTools, "mode_shredder", BTNS_CHECKGROUP);
-  addTButton(hObjectTools, 0, BTNS_CHECKGROUP);
+  addTButton(hObjectTools, nullptr, BTNS_CHECKGROUP);
   addTButton(hObjectTools, "mode_label", BTNS_CHECKGROUP);
   addTButton(hObjectTools, "mode_math", BTNS_CHECKGROUP);
   addTButton(hObjectTools, "mode_paragraph", BTNS_CHECKGROUP);
@@ -516,18 +527,18 @@ void AppUi::initUi()
   // size is used for dropdown list!  Don't set to zero.
   for (int i = EUiGridSize; i <= EUiAngleSize; ++i)
     hSelector[i] =
-      CreateWindowExW(0, WC_COMBOBOXEXW, NULL,
+      CreateWindowExW(0, WC_COMBOBOXEXW, nullptr,
 		      WS_CHILD | WS_VISIBLE | CBS_DROPDOWNLIST,
 		      0, 0, uiscale(100), uiscale(300), hwnd,
-		      (HMENU) (ID_SELECTOR_BASE + i), hInst, NULL);
+		      (HMENU) (INT_PTR) (ID_SELECTOR_BASE + i), hInst, nullptr);
 
-  hRebar = CreateWindowEx(WS_EX_TOOLWINDOW, REBARCLASSNAME, NULL,
+  hRebar = CreateWindowEx(WS_EX_TOOLWINDOW, REBARCLASSNAME, nullptr,
 			  WS_CHILD | WS_VISIBLE | WS_CLIPSIBLINGS |
 			  WS_CLIPCHILDREN | RBS_VARHEIGHT | RBS_AUTOSIZE |
 			  CCS_NODIVIDER | RBS_BANDBORDERS,
 			  0, 0, 0, 0,
-			  hwnd, NULL,
-			  hInst, NULL);
+			  hwnd, nullptr,
+			  hInst, nullptr);
 
   // Initialize band info for all bands
   REBARBANDINFO rbBand = { sizeof(REBARBANDINFO) };
@@ -552,7 +563,7 @@ void AppUi::initUi()
     for (int i = 1; i <= n; ++i) {
       lua_rawgeti(L, -1, i);
       if (lua_isstring(L, -1)) {
-	String s = lua_tostring(L, -1);
+	String s = lua_tolstring(L, -1, nullptr);
 	if (s == "edit")
 	  insert_tb(hRebar, rbBand, hEditTools, EDIT_BUTTONS * bw);
 	else if (s == "grid")
@@ -573,7 +584,7 @@ void AppUi::initUi()
 
   hProperties = CreateWindowExW(0, L"button", L"Properties",
 				WS_CHILD | WS_VISIBLE | BS_GROUPBOX,
-				0, 100, 200, 280, hwnd, NULL, hInst, NULL);
+				0, 100, 200, 280, hwnd, nullptr, hInst, nullptr);
 
   for (int i = 0; i <= EUiSymbolSize; ++i) {
     if (i != EUiMarkShape)
@@ -581,10 +592,10 @@ void AppUi::initUi()
   }
   // size is used for dropdown list!  Don't set to zero.
   for (int i = 0; i < EUiGridSize; ++i)
-    hSelector[i] = CreateWindowExW(0, WC_COMBOBOXEXW, NULL,
+    hSelector[i] = CreateWindowExW(0, WC_COMBOBOXEXW, nullptr,
 				   WS_CHILD | WS_VISIBLE | CBS_DROPDOWNLIST,
 				   0, 0, uiscale(100), uiscale(300), hwnd,
-				   (HMENU) (ID_SELECTOR_BASE + i), hInst, NULL);
+				   (HMENU) (INT_PTR) (ID_SELECTOR_BASE + i), hInst, nullptr);
   SendMessage(hButton[EUiPen], BM_SETIMAGE, IMAGE_BITMAP,
 	      (LPARAM) loadButtonIcon("pen", iUiScale));
   SendMessage(hButton[EUiTextSize], BM_SETIMAGE, IMAGE_BITMAP,
@@ -597,9 +608,9 @@ void AppUi::initUi()
 	      (LPARAM) colorIcon(Color(1000, 1000, 0), iUiScale));
 
   hButton[EUiMarkShape] =
-    CreateWindowExW(0, L"static", NULL,
+    CreateWindowExW(0, L"static", nullptr,
 		    WS_CHILD | WS_VISIBLE | SS_BITMAP | SS_CENTERIMAGE,
-		    0, 0, 0, 0, hwnd, NULL, hInst, NULL);
+		    0, 0, 0, 0, hwnd, nullptr, hInst, nullptr);
   SendMessage(hButton[EUiMarkShape], STM_SETIMAGE, IMAGE_BITMAP,
 	      (LPARAM) loadButtonIcon("mode_marks", iUiScale));
 
@@ -616,36 +627,36 @@ void AppUi::initUi()
 
   hLayerGroup = CreateWindowExW(0, L"button", L"Layers",
 				WS_CHILD | WS_VISIBLE | BS_GROUPBOX,
-				0, 0, 0, 0, hwnd, NULL, hInst, NULL);
+				0, 0, 0, 0, hwnd, nullptr, hInst, nullptr);
 
-  hLayers = CreateWindowExW(0, WC_LISTVIEWW, NULL,
+  hLayers = CreateWindowExW(0, WC_LISTVIEWW, nullptr,
 			    WS_CHILD | WS_VISIBLE |
 			    LVS_LIST | LVS_SINGLESEL,
 			    0, 0, 0, 0, hwnd,
 			    (HMENU) IDC_LAYERS,
-			    hInst, NULL);
-  ListView_SetExtendedListViewStyle(hLayers, LVS_EX_CHECKBOXES);
+			    hInst, nullptr);
+  (void) ListView_SetExtendedListViewStyle(hLayers, LVS_EX_CHECKBOXES);
 
   hNotesGroup = CreateWindowExW(0, L"button", L"Notes",
 				WS_CHILD | WS_VISIBLE | BS_GROUPBOX,
-				0, 0, 0, 0, hwnd, NULL, hInst, NULL);
+				0, 0, 0, 0, hwnd, nullptr, hInst, nullptr);
 
-  hNotes = CreateWindowExW(0, L"edit", NULL,
+  hNotes = CreateWindowExW(0, L"edit", nullptr,
 			   WS_CHILD | WS_VISIBLE | WS_BORDER | WS_VSCROLL |
 			   ES_READONLY | ES_LEFT | ES_MULTILINE |
 			   ES_AUTOVSCROLL,
 			   0, 0, 0, 0, hwnd,
-			   (HMENU) IDC_NOTES, hInst, NULL);
+			   (HMENU) IDC_NOTES, hInst, nullptr);
 
   hBookmarksGroup = CreateWindowExW(0, L"button", L"Bookmarks",
 				    WS_CHILD | WS_VISIBLE | BS_GROUPBOX,
-				    0, 0, 0, 0, hwnd, NULL, hInst, NULL);
+				    0, 0, 0, 0, hwnd, nullptr, hInst, nullptr);
 
-  hBookmarks = CreateWindowExW(0, L"listbox", NULL,
+  hBookmarks = CreateWindowExW(0, L"listbox", nullptr,
 			       WS_CHILD | WS_VISIBLE | WS_VSCROLL | WS_BORDER |
 			       LBS_HASSTRINGS | LBS_NOTIFY | LBS_NOSEL,
 			       0, 0, 0, 0,
-			       hwnd, (HMENU) IDC_BOOKMARK, hInst, NULL);
+			       hwnd, (HMENU) IDC_BOOKMARK, hInst, nullptr);
 
   Canvas *canvas = new Canvas(hwnd);
   hwndCanvas = canvas->windowId();
@@ -687,7 +698,7 @@ void AppUi::initUi()
 		      DEFAULT_QUALITY,
 		      DEFAULT_PITCH | FF_SWISS,
 		      L"MS Shell Dlg");
-  if (hFont != NULL) {
+  if (hFont != nullptr) {
     SendMessage(hNotes, WM_SETFONT, WPARAM (hFont), TRUE);
     SendMessage(hNotesGroup, WM_SETFONT, WPARAM (hFont), TRUE);
     SendMessage(hBookmarksGroup, WM_SETFONT, WPARAM (hFont), TRUE);
@@ -700,6 +711,11 @@ void AppUi::initUi()
   // default is on, Lua code will turn off if necessary
   CheckMenuItem(hMenuBar, actionId("toggle_notes"), MF_CHECKED);
   CheckMenuItem(hMenuBar, actionId("toggle_bookmarks"), MF_CHECKED);
+
+  if (iLeftDockFloats)
+    // SendMessage(hEditTools, TB_SETSTATE, actionId("dock_visible"),
+    // (LPARAM) (TBSTATE_ENABLED | TBSTATE_CHECKED));
+    setLeftDockVisibility(false);
 }
 
 AppUi::~AppUi()
@@ -723,44 +739,52 @@ void AppUi::layoutChildren(bool resizeRebar)
   int sbh = rc1.bottom - rc1.top;
   int tbh = rc2.bottom - rc2.top;
   int cvh = rc.bottom - sbh - tbh;
+
   RECT rect;
-  rect.left = 0;
-  rect.top = tbh;
-  rect.right = uiscale(200);
-  rect.bottom = rc.bottom - sbh;
-  InvalidateRect(hwnd, &rect, TRUE);
-  int pbh = uiscale(28 * 7 + 70);
-  MoveWindow(hProperties, uiscale(8), tbh + uiscale(2), uiscale(180), pbh, TRUE);
+  bool visLeftDock = IsWindowVisible(hProperties);
+  if (visLeftDock) {
+    rect.left = 0;
+    rect.top = tbh;
+    rect.right = uiscale(200);
+    rect.bottom = rc.bottom - sbh;
+    InvalidateRect(hwnd, &rect, TRUE);
 
-  for (int i = 0; i <= EUiSymbolSize; ++i) {
-    int y = uiscale(24 + 28 * i + (i > 2 ? 42 : 0));
-    MoveWindow(hButton[i], uiscale(16), tbh + y,
-	       uiscale(26), uiscale(26), TRUE);
-    MoveWindow(hSelector[i], uiscale(50), tbh + y,
-	       uiscale(132), uiscale(26), TRUE);
+    int dy = 28 + iUiGap;
+    int pbh = uiscale(dy * 6 + 28 + 70);
+    MoveWindow(hProperties, uiscale(8), tbh + uiscale(2),
+	       uiscale(180), pbh, TRUE);
+    for (int i = 0; i <= EUiSymbolSize; ++i) {
+      int y = uiscale(24 + dy * i + (i > 2 ? 42 : 0));
+      MoveWindow(hButton[i], uiscale(16), tbh + y,
+		 uiscale(26), uiscale(26), TRUE);
+      MoveWindow(hSelector[i], uiscale(50), tbh + y,
+		 uiscale(132), uiscale(26), TRUE);
+    }
+
+    int y = tbh + uiscale(24 + dy * 6 + 42);
+    MoveWindow(hViewNumber, uiscale(16), y, uiscale(68), uiscale(26), TRUE);
+    MoveWindow(hPageNumber, uiscale(116), y, uiscale(68), uiscale(26), TRUE);
+    MoveWindow(hViewMarked, uiscale(85), y + uiscale(6),
+	       uiscale(13), uiscale(13), TRUE);
+    MoveWindow(hPageMarked, uiscale(100), y + uiscale(6),
+	       uiscale(13), uiscale(13), TRUE);
+
+    MoveWindow(iPathView->windowId(), uiscale(16),
+	       tbh + uiscale(24 + dy * 2 + 28 + iUiGap / 2),
+	       uiscale(168), uiscale(40), TRUE);
+
+    MoveWindow(hLayerGroup, uiscale(8), tbh + uiscale(6) + pbh,
+	       uiscale(180), cvh - pbh - uiscale(8), TRUE);
+    MoveWindow(hLayers, uiscale(16), tbh + uiscale(24) + pbh,
+	       uiscale(164), cvh - pbh - uiscale(32), TRUE);
   }
 
-  int y = tbh + uiscale(24 + 28 * 6 + 42);
-  MoveWindow(hViewNumber, uiscale(16), y, uiscale(68), uiscale(26), TRUE);
-  MoveWindow(hPageNumber, uiscale(116), y, uiscale(68), uiscale(26), TRUE);
-  MoveWindow(hViewMarked, uiscale(85), y + uiscale(5),
-	     uiscale(12), uiscale(12), TRUE);
-  MoveWindow(hPageMarked, uiscale(100), y + uiscale(5),
-	     uiscale(12), uiscale(12), TRUE);
-
-  MoveWindow(iPathView->windowId(), uiscale(16), tbh + uiscale(24 + 28 * 3),
-	     uiscale(168), uiscale(40), TRUE);
-
-  MoveWindow(hLayerGroup, uiscale(8), tbh + uiscale(6) + pbh,
-	     uiscale(180), cvh - pbh - uiscale(8), TRUE);
-  MoveWindow(hLayers, uiscale(16), tbh + uiscale(24) + pbh,
-	     uiscale(164), cvh - pbh - uiscale(32), TRUE);
-
   bool visNotes = IsWindowVisible(hNotes);
   bool visBookmarks = IsWindowVisible(hBookmarks);
-  int cvw = (visNotes || visBookmarks) ?
-    rc.right - uiscale(200 + iWidthNotesBookmarks) : rc.right - uiscale(200);
-  MoveWindow(hwndCanvas, uiscale(200), tbh, cvw, cvh, TRUE);
+  int cvl = visLeftDock ? uiscale(200) : 0;
+  int cvw = rc.right - cvl -
+    ((visNotes || visBookmarks) ? uiscale(iWidthNotesBookmarks) : 0);
+  MoveWindow(hwndCanvas, cvl, tbh, cvw, cvh, TRUE);
   if (visNotes || visBookmarks) {
     rect.left = rc.right - uiscale(iWidthNotesBookmarks);
     rect.top = tbh;
@@ -790,6 +814,7 @@ void AppUi::layoutChildren(bool resizeRebar)
 		      rc.right - uiscale(80), rc.right };
   SendMessage(hStatusBar, SB_SETPARTS, 4, (LPARAM) statwidths);
   iCanvas->update();
+  BringWindowToTop(hProperties);
 }
 
 // --------------------------------------------------------------------
@@ -818,10 +843,10 @@ void AppUi::addComboColors(AttributeSeq &sym, AttributeSeq &abs)
   COMBOBOXEXITEM item;
   item.mask = CBEIF_TEXT | CBEIF_IMAGE;
   item.iItem = (INT_PTR) -1;
-  for (uint i = 0; i < sym.size(); ++i) {
+  for (int i = 0; i < size(sym); ++i) {
     Color color = abs[i].color();
     HBITMAP bm = colorIcon(color, iUiScale);
-    ImageList_Add(hColorIcons, bm, NULL);
+    ImageList_Add(hColorIcons, bm, nullptr);
     DeleteObject(bm);
     String s = sym[i].string();
     WString ws(s);
@@ -903,17 +928,19 @@ void AppUi::setLayers(const Page *page, int view)
   iLayerNames.clear();
   for (int i = 0; i < page->countLayers(); ++i)
     iLayerNames.push_back(page->layer(i));
+  std::vector<int> objCounts;
+  page->objectsPerLayer(objCounts);
 
   iSettingLayers = true;
   // TODO send WM_SETREDRAW
-  ListView_DeleteAllItems(hLayers);
+  (void) ListView_DeleteAllItems(hLayers);
 
   LVITEM lvI;
   lvI.mask      = LVIF_TEXT | LVIF_PARAM;
   lvI.iSubItem  = 0;
 
   for (int i = 0; i < page->countLayers(); ++i) {
-    lvI.iItem   = i;
+    lvI.iItem = i;
     lvI.lParam = 0;
     if (page->layer(i) == page->active(view))
       lvI.lParam |=  1;
@@ -921,9 +948,12 @@ void AppUi::setLayers(const Page *page, int view)
       lvI.lParam |=  2;
     if (!page->hasSnapping(i))
       lvI.lParam |=  4;
-    WString w(page->layer(i));
-    lvI.pszText = w.data();
-    ListView_InsertItem(hLayers, &lvI);
+    WString text(page->layer(i));
+    char buf[16];
+    sprintf(buf, " (%d)", objCounts[i]);
+    text += WString(buf);
+    lvI.pszText = text.data();
+    (void) ListView_InsertItem(hLayers, &lvI);
     ListView_SetCheckState(hLayers, i, page->visible(view, i));
   }
   iSettingLayers = false;
@@ -977,7 +1007,7 @@ void AppUi::closeRequested()
   lua_getfield(L, -1, "closeEvent");
   lua_pushvalue(L, -2); // model
   lua_remove(L, -3);
-  lua_call(L, 1, 1);
+  luacall(L, 1, 1);
   bool result = lua_toboolean(L, -1);
   if (result)
     DestroyWindow(hwnd);
@@ -989,12 +1019,16 @@ void AppUi::closeWindow()
 }
 
 // Determine if an action is checked.
-// Used for viewmarked, pagemarked, snapXXX, grid_visible,
-// pretty_display, toggle_notes, toggle_bookmarks.
+// Used for viewmarked, pagemarked, snapXXX, grid_visible, show_axes,
+// pretty_display, toggle_notes, toggle_bookmarks, auto_latex.
 bool AppUi::actionState(const char *name)
 {
-  // ipeDebug("actionState %s", name);
+  if (!strcmp(name, "viewmarked"))
+    return (Button_GetCheck(hViewMarked) == BST_CHECKED);
+  if (!strcmp(name, "pagemarked"))
+    return (Button_GetCheck(hPageMarked) == BST_CHECKED);
   int idx = actionId(name);
+  // ipeDebug("actionState %s (%d)", name, idx);
   return GetMenuState(hMenuBar, idx, MF_CHECKED);
 }
 
@@ -1049,10 +1083,9 @@ void AppUi::populateTextStyleMenu()
   iCascade->allNames(ETextStyle, seq);
   clearMenu(iTextStyleMenu);
   int check = 0;
-  for (uint i = 0; i < seq.size(); ++i) {
+  for (int i = 0; i < size(seq); ++i) {
     String s = seq[i].string();
-    WString ws(s);
-    AppendMenuW(iTextStyleMenu, MF_STRING, TEXT_STYLE_BASE + i, ws);
+    AppendMenuW(iTextStyleMenu, MF_STRING, TEXT_STYLE_BASE + i, WString(s).data());
     if (s == iAll.iTextStyle.string())
       check = i;
   }
@@ -1066,8 +1099,7 @@ void AppUi::populateSizeMenu(HMENU h, int sel, int base)
   clearMenu(h);
   auto i = 0;
   for (auto &name : iComboContents[sel]) {
-    WString ws(name);
-    AppendMenuW(h, MF_STRING, base + i, ws);
+    AppendMenuW(h, MF_STRING, base + i, WString(name).data());
     ++i;
   }
   auto cur = SendMessage(hSelector[sel], CB_GETCURSEL, 0, 0);
@@ -1088,8 +1120,8 @@ void AppUi::populateLayerMenus()
   clearMenu(iMoveToLayerMenu);
   for (int i = 0; i < int(iLayerNames.size()); ++i) {
     WString ws(iLayerNames[i]);
-    AppendMenuW(iSelectLayerMenu, MF_STRING, ID_SELECTINLAYER_BASE + i, ws);
-    AppendMenuW(iMoveToLayerMenu, MF_STRING, ID_MOVETOLAYER_BASE + i, ws);
+    AppendMenuW(iSelectLayerMenu, MF_STRING, ID_SELECTINLAYER_BASE + i, ws.data());
+    AppendMenuW(iMoveToLayerMenu, MF_STRING, ID_MOVETOLAYER_BASE + i, ws.data());
   }
 }
 
@@ -1118,7 +1150,7 @@ int AppUi::iconId(const char *name) const
 
 int AppUi::actionInfo(lua_State *L) const
 {
-  const char *action = luaL_checkstring(L, 2);
+  const char *action = luaL_checklstring(L, 2, nullptr);
   int i = findAction(action);
   lua_pushinteger(L, (i >= 0) ? i + IDBASE : 0);
   lua_pushboolean(L, (i >= 0) ? iActions[i].alwaysOn : 0);
@@ -1162,36 +1194,59 @@ void AppUi::cmd(int id, int notification)
 
 static const char * const aboutText =
   "Ipe %d.%d.%d\n\n"
-  "Copyright (c) 1993-2016 Otfried Cheong\n\n"
+  "Copyright (c) 1993-%d Otfried Cheong\n\n"
   "The extensible drawing editor Ipe creates figures "
   "in Postscript and PDF format, "
   "using LaTeX to format the text in the figures.\n"
-  "Ipe relies on the following fine pieces of software:\n\n"
-  " * Pdftex, Xetex, or Luatex\n"
-  " * %s (%d kB used)\n" // Lua
-  " * The font rendering library %s\n"
-  " * The rendering library Cairo %s\n"
-  " * The compression library zlib %s\n\n"
   "Ipe is released under the GNU Public License.\n"
-  "See http://ipe.otfried.org for details.";
+  "See http://ipe.otfried.org for details.\n\n"
+  "You can \"like\" Ipe and follow Ipe announcements on Facebook "
+  "(http://www.facebook.com/drawing.editor.Ipe7).\n\n"
+  "If you are an Ipe fan and want to show others, have a look at the "
+  "Ipe T-shirts (www.shirtee.com/en/store/ipe).\n\n"
+  "Platinum sponsors\n\n"
+  " * Hee-Kap Ahn\n"
+  " * Martin Ziegler\n\n"
+  "If you enjoy Ipe, you can become a member of the exclusive community of "
+  "Ipe patrons (http://patreon.com/otfried). "
+  "For the price of a cup of coffee per month you can make a meaningful contribution "
+  "to the continuing development of Ipe.";
 
 void AppUi::aboutIpe()
 {
-  int luaSize = lua_gc(L, LUA_GCCOUNT, 0);
-  std::vector<char> buf(strlen(aboutText) + 200);
+  std::vector<char> buf(strlen(aboutText) + 100);
   sprintf(&buf[0], aboutText,
 	  IPELIB_VERSION / 10000,
 	  (IPELIB_VERSION / 100) % 100,
 	  IPELIB_VERSION % 100,
-	  LUA_RELEASE, luaSize,
-	  Fonts::freetypeVersion().z(),
-	  cairo_version_string(),
-	  ZLIB_VERSION);
+	  COPYRIGHT_YEAR);
 
   MessageBoxA(hwnd, &buf[0], "About Ipe",
 	      MB_OK | MB_ICONINFORMATION | MB_APPLMODAL);
 }
 
+void AppUi::setLeftDockVisibility(bool vis)
+{
+  int cmd = vis ?SW_SHOW : SW_HIDE;
+  ShowWindow(hProperties, cmd);
+  ShowWindow(hLayerGroup, cmd);
+  ShowWindow(hLayers, cmd);
+  for (int i = 0; i < EUiGridSize; ++i)
+    ShowWindow(hButton[i], cmd);
+  for (int i = 0; i < EUiView; ++i)
+    if (i != EUiGridSize && i != EUiAngleSize)
+      ShowWindow(hSelector[i], cmd);
+  ShowWindow(hViewNumber, cmd);
+  ShowWindow(hPageNumber, cmd);
+  ShowWindow(hViewMarked, cmd);
+  ShowWindow(hPageMarked, cmd);
+  ShowWindow(iPathView->windowId(), cmd);
+  layoutChildren(false);
+  auto pan = iCanvas->pan();
+  double delta = (vis ? +1 : -1) * uiscale(100) / iCanvas->zoom();
+  iCanvas->setPan(pan + Vector(delta, 0.0));
+}
+
 void AppUi::action(String name)
 {
   ipeDebug("action %s", name.z());
@@ -1208,6 +1263,10 @@ void AppUi::action(String name)
 	mod |= CanvasBase::EShift;
       iCanvas->setAdditionalModifiers(mod);
     }
+  } else if (name == "dock_visible") {
+    bool vis = (SendMessage(hEditTools, TB_GETSTATE, (WPARAM) id, 0)
+		& TBSTATE_CHECKED);
+    setLeftDockVisibility(vis);
   } else {
     // Implement radio buttons
     int i = name.find("|");
@@ -1215,8 +1274,8 @@ void AppUi::action(String name)
       setCheckMark(name.left(i+1), name.substr(i+1));
     if (name.hasPrefix("mode_"))
       setCheckMark("mode_", name.substr(5));
-    if (name.hasPrefix("snap") || name == "grid_visible" ||
-	name == "pretty_display" || name.hasPrefix("toggle_"))
+    if (name.hasPrefix("snap") || name == "grid_visible" || name == "auto_latex" ||
+	name == "pretty_display" || name == "show_axes" || name.hasPrefix("toggle_"))
       setActionState(name.z(), !GetMenuState(hMenuBar, id, MF_CHECKED));
     luaAction(name);
   }
@@ -1234,7 +1293,7 @@ void AppUi::setWindowCaption(bool mod, const char *s)
 
 void AppUi::showWindow(int width, int height)
 {
-  SetWindowPos(hwnd, NULL, 0, 0, width, height, SWP_NOMOVE);
+  SetWindowPos(hwnd, nullptr, 0, 0, width, height, SWP_NOMOVE);
   ShowWindow(hwnd, win_nCmdShow);
   UpdateWindow(hwnd);
 }
@@ -1243,7 +1302,7 @@ void AppUi::showWindow(int width, int height)
 void AppUi::explain(const char *s, int t)
 {
   if (t)
-    SetTimer(hwnd, ID_STATUS_TIMER, t, NULL);
+    SetTimer(hwnd, ID_STATUS_TIMER, t, nullptr);
   else
     KillTimer(hwnd, ID_STATUS_TIMER);
   sendMessage(hStatusBar, SB_SETTEXT, s);
@@ -1287,10 +1346,10 @@ static int clipboardBitmap(lua_State *L)
   s->biSize = sizeof(BITMAPINFOHEADER);
   s->biBitCount = 0;
   s->biClrUsed = 0;
-  HDC hdc = GetDC(NULL); // CreateCompatibleDC(NULL);
-  if (GetDIBits(hdc, bm, 0, 0, NULL, &bmi, DIB_RGB_COLORS) == 0) {
+  HDC hdc = GetDC(nullptr); // CreateCompatibleDC(nullptr);
+  if (GetDIBits(hdc, bm, 0, 0, nullptr, &bmi, DIB_RGB_COLORS) == 0) {
     ipeDebug("AppUi::clipboard() GetDIBits failed");
-    ReleaseDC(NULL, hdc);
+    ReleaseDC(nullptr, hdc);
     CloseClipboard();
     return 0;
   }
@@ -1305,34 +1364,32 @@ static int clipboardBitmap(lua_State *L)
   s->biCompression = BI_RGB;
   s->biSizeImage = 0;
 
-  Buffer pixels(w * h * 3);
   DWORD *bits = (DWORD *)
-    VirtualAlloc(NULL, 4 * w, MEM_COMMIT | MEM_RESERVE, PAGE_READWRITE);
+    VirtualAlloc(nullptr, 4 * w, MEM_COMMIT | MEM_RESERVE, PAGE_READWRITE);
+
+  Buffer pixels(w * h * 4);
+  uint32_t *p = (uint32_t *) pixels.data();
 
-  char *p = pixels.data();
   for (int y = h-1; y >= 0; --y) {
     if (GetDIBits(hdc, bm, y, 1, bits, &bmi, DIB_RGB_COLORS) == 0) {
       ipeDebug("AppUi::clipboard() GetDIBits failed to retrieve bits");
       VirtualFree(bits, 0, MEM_RELEASE);
-      ReleaseDC(NULL, hdc);
+      ReleaseDC(nullptr, hdc);
       CloseClipboard();
       return 0;
     }
     DWORD *r = bits;
-    for (int x = 0; x < w; ++x) {
-      DWORD rr = *r++;
-      *p++ = (rr >> 16) & 0xff;
-      *p++ = (rr >> 8) & 0xff;
-      *p++ = rr & 0xff;
-    }
+    for (int x = 0; x < w; ++x)
+      *p++ = *r++;
   }
 
-  Bitmap ibm(w, h, Bitmap::EDeviceRGB, 8, pixels, Bitmap::EDirect, true);
+  Bitmap ibm(w, h, Bitmap::ENative, pixels);
+
   Rect r(Vector::ZERO, Vector(w, h));
   Image *img = new Image(r, ibm);
   push_object(L, img);
   VirtualFree(bits, 0, MEM_RELEASE);
-  ReleaseDC(NULL, hdc);
+  ReleaseDC(nullptr, hdc);
   CloseClipboard();
   return 1;
 }
@@ -1351,7 +1408,7 @@ int AppUi::clipboard(lua_State *L)
     return clipboardBitmap(L);
   if (format == CF_UNICODETEXT){
     HGLOBAL hGlobal = GetClipboardData(CF_UNICODETEXT);
-    if (hGlobal == NULL) {
+    if (hGlobal == nullptr) {
       CloseClipboard();
       return 0;
     }
@@ -1367,7 +1424,7 @@ int AppUi::clipboard(lua_State *L)
 
 int AppUi::setClipboard(lua_State *L)
 {
-  const char *s = luaL_checkstring(L, 2);
+  const char *s = luaL_checklstring(L, 2, nullptr);
   WString ws(s);
   size_t size = (ws.size() + 1) * sizeof(wchar_t);
   HGLOBAL hGlobal = GlobalAlloc(GMEM_MOVEABLE, size);
@@ -1376,7 +1433,7 @@ int AppUi::setClipboard(lua_State *L)
   GlobalUnlock(hGlobal);
   OpenClipboard(hwnd);
   EmptyClipboard();
-  if (SetClipboardData(CF_UNICODETEXT, hGlobal) == NULL)
+  if (SetClipboardData(CF_UNICODETEXT, hGlobal) == nullptr)
     GlobalFree(hGlobal);  // clipboard didn't take the data
   CloseClipboard();
   return 0;
@@ -1456,7 +1513,7 @@ LRESULT CALLBACK AppUi::wndProc(HWND hwnd, UINT message, WPARAM wParam,
 	return CDRF_NOTIFYITEMDRAW;
       case CDDS_ITEMPREPAINT: {
 	LPNMLVCUSTOMDRAW q = (LPNMLVCUSTOMDRAW) lParam;
-	uint flags = q->nmcd.lItemlParam;
+	UINT flags = q->nmcd.lItemlParam;
 	if (flags & 0x01)   // active
 	  q->clrTextBk = 0x0000ffff;
 	if (flags & 0x02)   // locked
@@ -1532,7 +1589,7 @@ LRESULT CALLBACK AppUi::wndProc(HWND hwnd, UINT message, WPARAM wParam,
 bool AppUi::isDrawing(HWND target)
 {
   // check that it really is an AppUi window
-  WNDPROC w = (WNDPROC) GetWindowLongPtr(target, GWL_WNDPROC);
+  WNDPROC w = (WNDPROC) GetWindowLongPtr(target, GWLP_WNDPROC);
   if (w != AppUi::wndProc)
     return false;
   AppUi *ui = (AppUi*) GetWindowLongPtr(target, GWLP_USERDATA);
@@ -1548,17 +1605,17 @@ void AppUi::init(HINSTANCE hInstance, int nCmdShow)
   wc.cbClsExtra	 = 0;
   wc.cbWndExtra	 = 0;
   wc.hInstance	 = hInstance;
-  wc.hCursor = LoadCursor(NULL, IDC_ARROW);
+  wc.hCursor = LoadCursor(nullptr, IDC_ARROW);
   wc.hbrBackground = (HBRUSH)(COLOR_BTNFACE+1);
-  wc.lpszMenuName = NULL;
+  wc.lpszMenuName = nullptr;
   wc.lpszClassName = className;
-  wc.hIcon = LoadIcon(GetModuleHandle(NULL), MAKEINTRESOURCE(IDI_MYICON));
+  wc.hIcon = LoadIcon(GetModuleHandle(nullptr), MAKEINTRESOURCE(IDI_MYICON));
   wc.hIconSm =
-    HICON(LoadImage(GetModuleHandle(NULL),
+    HICON(LoadImage(GetModuleHandle(nullptr),
 		    MAKEINTRESOURCE(IDI_MYICON), IMAGE_ICON, 16, 16, 0));
 
   if (!RegisterClassEx(&wc)) {
-    MessageBoxA(NULL, "AppUi registration failed!", "Error!",
+    MessageBoxA(nullptr, "AppUi registration failed!", "Error!",
 		MB_ICONEXCLAMATION | MB_OK);
     exit(9);
   }
