@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2016  Otfried Cheong
+    Copyright (c) 1993-2019 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -109,7 +109,7 @@ private:
   void closeRequested();
   int iconId(const char *name) const;
   HWND createToolBar(HINSTANCE hInst);
-  void addTButton(HWND tb, const char *name = 0, int flags = 0);
+  void addTButton(HWND tb, const char *name = nullptr, int flags = 0);
   void setTooltip(HWND h, String tip, bool isComboBoxEx = false);
   void toggleVisibility(String action, HWND h);
   void setLeftDockVisibility(bool vis);
