@@ -46,7 +46,7 @@
 {
   self = [super initWithFrame:rect];
   if (self) {
-    iCascade = 0;
+    iCascade = nullptr;
   }
   return self;
 }
@@ -83,7 +83,7 @@
     Vector u3 = (1.0/zoom) * Vector(0.88 * w, 0.2 * h);
     Vector u4 = (1.0/zoom) * Vector(0.96 * w, 0.5 * h);
 
-    CairoPainter painter(iCascade, 0, cc, 3.0, false);
+    CairoPainter painter(iCascade, nullptr, cc, 3.0, false);
     painter.setPen(iAll.iPen);
     painter.setDashStyle(iAll.iDashStyle);
     painter.setStroke(iAll.iStroke);
@@ -314,7 +314,7 @@ objectValueForTableColumn:(NSTableColumn *) col
   if ([[col identifier] isEqualToString:@"checks"])
     return [NSNumber numberWithBool:iLayers[row].checked];
   else
-    return iLayers[row].name;
+    return iLayers[row].text;
 }
 
 - (NSView *) tableView:(NSTableView *) tv
