@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2016  Otfried Cheong
+    Copyright (c) 1993-2019 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -57,19 +57,22 @@ void LayerBox::layerChanged(QListWidgetItem *item)
 {
   if (iInSet)
     return;
+  LayerItem *litem = (LayerItem *) item;
   if (item->checkState() == Qt::Checked)
-    emit activated("selecton", IpeQ(item->text()));
+    emit activated("selecton", litem->ipeLayerName);
   else
-    emit activated("selectoff", IpeQ(item->text()));
+    emit activated("selectoff", litem->ipeLayerName);
 }
 
 void LayerBox::set(const Page *page, int view)
 {
+  std::vector<int> objCounts;
+  page->objectsPerLayer(objCounts);
   iInSet = true;
   clear();
   for (int i = 0; i < page->countLayers(); ++i) {
-    // int i = idx[j].second;
-    QListWidgetItem *item = new QListWidgetItem(QIpe(page->layer(i)), this);
+    QString text = QString("%1 (%2)").arg(QIpe(page->layer(i))).arg(objCounts[i]);
+    QListWidgetItem *item = new LayerItem(text, page->layer(i), this);
     item->setFlags(Qt::ItemIsUserCheckable|Qt::ItemIsEnabled);
     item->setCheckState(page->visible(view, i) ? Qt::Checked : Qt::Unchecked);
     if (page->layer(i) == page->active(view))
@@ -84,20 +87,20 @@ void LayerBox::set(const Page *page, int view)
 
 void LayerBox::mousePressEvent(QMouseEvent *ev)
 {
-  QListWidgetItem *item = itemAt(ev->pos());
+  LayerItem *item = (LayerItem *) itemAt(ev->pos());
   if (item && ev->button()== Qt::LeftButton && ev->x() > 30) {
-    emit activated("active", IpeQ(item->text()));
+    emit activated("active", item->ipeLayerName);
   }
   QListWidget::mousePressEvent(ev);
 }
 
 void LayerBox::mouseReleaseEvent(QMouseEvent *ev)
 {
-  QListWidgetItem *item = itemAt(ev->pos());
+  LayerItem *item = (LayerItem *) itemAt(ev->pos());
   if (item && ev->button() == Qt::RightButton) {
     // make popup menu
     emit showLayerBoxPopup(Vector(ev->globalPos().x(), ev->globalPos().y()),
-			   IpeQ(item->text()));
+			   item->ipeLayerName);
   } else
     QListWidget::mouseReleaseEvent(ev);
 }
@@ -110,7 +113,7 @@ PathView::PathView(QWidget* parent, Qt::WFlags f) : QWidget(parent, f)
 PathView::PathView(QWidget* parent, Qt::WindowFlags f) : QWidget(parent, f)
 #endif
 {
-  iCascade = 0;
+  iCascade = nullptr;
 }
 
 void PathView::set(const AllAttributes &all, Cascade *sheet)
@@ -143,7 +146,7 @@ void PathView::paintEvent(QPaintEvent *ev)
     Vector u3 = (1.0/zoom) * Vector(0.88 * w, 0.2 * h);
     Vector u4 = (1.0/zoom) * Vector(0.96 * w, 0.5 * h);
 
-    CairoPainter painter(iCascade, 0, cc, 3.0, false);
+    CairoPainter painter(iCascade, nullptr, cc, 3.0, false);
     painter.setPen(iAll.iPen);
     painter.setDashStyle(iAll.iDashStyle);
     painter.setStroke(iAll.iStroke);
