@@ -40,10 +40,19 @@ using namespace ipe;
 
 // --------------------------------------------------------------------
 
+class LayerItem: public QListWidgetItem {
+public:
+  LayerItem(const QString &text, String name, QListWidget *parent) :
+    QListWidgetItem(text, parent), ipeLayerName(name) {
+    // nothing
+  }
+  String ipeLayerName;
+};
+
 class LayerBox : public QListWidget {
   Q_OBJECT
 public:
-  LayerBox(QWidget *parent = 0);
+  LayerBox(QWidget *parent = nullptr);
 
   void set(const Page *page, int view);
 
@@ -72,9 +81,9 @@ class PathView : public QWidget {
 
 public:
 #if QT_VERSION < 0x050000
-  PathView(QWidget* parent = 0, Qt::WFlags f = 0);
+  PathView(QWidget* parent = nullptr, Qt::WFlags f = nullptr);
 #else
-  PathView(QWidget* parent = 0, Qt::WindowFlags f = 0);
+  PathView(QWidget* parent = nullptr, Qt::WindowFlags f = nullptr);
 #endif
 
   void set(const AllAttributes &all, Cascade *sheet);
@@ -102,7 +111,7 @@ class PageSorter : public QListWidget {
   Q_OBJECT
 
 public:
-  PageSorter(Document *doc, int width, QWidget *parent = 0);
+  PageSorter(Document *doc, int width, QWidget *parent = nullptr);
 
   int pageAt(int r) const;
 
