@@ -4,7 +4,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2016  Otfried Cheong
+    Copyright (c) 1993-2019 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -1003,8 +1003,6 @@ end
 
 ----------------------------------------------------------------------
 
-local join_threshold = 1e-9
-
 local function flip(sub)
   sub[1], sub[2] = sub[2], sub[1]
   sub.flip = not sub.flip
@@ -1014,9 +1012,9 @@ local function findPartner(l, src, beg)
   local v = l[src][2]
   while beg <= #l do
     local w1 = l[beg][1]
-    if (v - w1):sqLen() < join_threshold then return beg end
+    if (v - w1):sqLen() < prefs.join_threshold then return beg end
     local w2 = l[beg][2]
-    if (v - w2):sqLen() < join_threshold then flip(l[beg]) return beg end
+    if (v - w2):sqLen() < prefs.join_threshold then flip(l[beg]) return beg end
     beg = beg + 1
   end
   return nil
@@ -1115,7 +1113,7 @@ function MODEL:saction_join()
   end
 
   -- check if it is closed
-  if (nsp[1][1] - nsp[#nsp][#nsp[#nsp]]):sqLen() < join_threshold then
+  if (nsp[1][1] - nsp[#nsp][#nsp[#nsp]]):sqLen() < prefs.join_threshold then
     nsp.closed = true
     if nsp[#nsp].type == "segment" then
       table.remove(nsp)
