@@ -206,6 +206,7 @@ else
 end
 filter_png = { "Images (*.png)", "*.png" }
 filter_eps = { "Postscript files (*.eps)", "*.eps" }
+filter_svg = { "SVG files (*.svg)", "*.svg" }
 
 ----------------------------------------------------------------------
 -- This function is called to launch a file on MacOS X
@@ -450,7 +451,7 @@ while i <= #argv do
     i = i + 2
   else
     if i ~= #argv then usage() return end
-    first_file = argv[i]
+    first_file = ipe.realPath(argv[i])
     i = i + 1
   end
 end
