@@ -43,34 +43,7 @@ end
 
 function MODEL:init(fname)
 
-  self.attributes = {
-    pathmode = "stroked",
-    stroke = "black",
-    fill = "white",
-    pen = "normal",
-    dashstyle = "normal",
-    farrowshape = "arrow/normal(spx)",
-    rarrowshape = "arrow/normal(spx)",
-    farrowsize = "normal",
-    rarrowsize = "normal",
-    farrow = false,
-    rarrow = false,
-    symbolsize = "normal",
-    textsize = "normal",
-    textstyle = "normal",
-    transformabletext = prefs.transformable_text,
-    horizontalalignment = "left",
-    verticalalignment = "baseline",
-    pinned = "none",
-    transformations = "affine",
-    linejoin = "normal",
-    linecap = "normal",
-    fillrule = "normal",
-    markshape = "mark/disk(sx)",
-    tiling = "normal",
-    gradient = "normal",
-    opacity = "opaque",
-  }
+  self.attributes = prefs.initial_attributes
 
   self.snap = {
     snapvtx = prefs.snap.vertex,
@@ -80,10 +53,10 @@ function MODEL:init(fname)
     snapgrid = prefs.snap.grid,
     snapangle = prefs.snap.angle,
     snapauto = prefs.snap.autoangle,
-    grid_visible = prefs.grid_visible,
+    grid_visible = prefs.initial.grid_visible,
     pretty_display = false,
-    gridsize = prefs.grid_size,
-    anglesize = prefs.angle_size,
+    gridsize = prefs.initial.grid_size,
+    anglesize = prefs.initial.angle_size,
     snap_distance = prefs.snap_distance,
     with_axes = false,
     origin = ipe.Vector(0,0),
@@ -101,6 +74,7 @@ function MODEL:init(fname)
   self.mode = "select"
   self.ui:setActionState("mode_select", true)
   self.ui:setActionState("grid_visible", self.snap.grid_visible)
+  self.ui:setActionState("show_axes", self.snap.with_axes)
 
   for i,e in ipairs(MODEL.snapmodes) do
     self.ui:setActionState(e, self.snap[e])
@@ -147,8 +121,8 @@ end
 ----------------------------------------------------------------------
 
 function MODEL:resetGridSize()
-  self.snap.gridsize = prefs.grid_size
-  self.snap.anglesize = prefs.angle_size
+  self.snap.gridsize = prefs.initial.grid_size
+  self.snap.anglesize = prefs.initial.angle_size
   local gridsizes = allValues(self.doc:sheets(), "gridsize")
   if #gridsizes == 0 then gridsizes = { 16 } end
   if not indexOf(self.snap.gridsize, gridsizes) then
@@ -185,7 +159,6 @@ end
 
 function MODEL:getDouble(caption, label, value, minv, maxv)
   local s = self:getString(label, caption, tostring(value))
-  -- print("getDouble", s)
   if s then
     local n = tonumber(s)
     if n and minv <= n and n <= maxv then
@@ -458,7 +431,7 @@ function MODEL:latexErrorBox(log)
 end
 
 function MODEL:runLatex()
-  local success, errmsg, result, log = self.doc:runLatex()
+  local success, errmsg, result, log = self.doc:runLatex(self.file_name)
   if success then
     self.ui:setResources(self.doc)
     self.ui:update()
@@ -466,6 +439,22 @@ function MODEL:runLatex()
   elseif result == "latex" then
     self:latexErrorBox(log)
     return false
+  elseif result == "runlatex" then
+    local urlFile = config.latexdir .. prefs.fsep .. "url.txt"
+    if ipe.fileExists(urlFile) then
+      self:warning("Ipe did not succeed to run Latex to process your text objects.",
+		   "Ipe tried to use Latex in the cloud.\n\n"
+		     .. "Perhaps you have no internet connectivity right now?\n\n"
+		     .. "If you have a local Latex installation, you can "
+		     .. "disable Latex-compilation in the cloud "
+		     .. "from the Help menu.")
+    else
+      self:warning("Ipe did not succeed to run Latex to process your text objects.",
+		   "You either have no Latex installation on your computer, "
+		     .. "or the Latex executable is not on your command path.\n\n"
+		     .. "If you prefer, you can enable Latex-compilation in the cloud "
+		     .. "from the Help menu.")
+    end
   else
     self:warning("An error occurred during the Pdflatex run", errmsg)
   end
@@ -548,6 +537,8 @@ function MODEL:tryLoadDocument(fname)
     self.ui:setAttributes(self.doc:sheets(), self.attributes)
     self:resetGridSize()
 
+    local syms = self.doc:checkStyle()
+
     self:action_normal_size()
     self:setSnap()
 
@@ -557,7 +548,6 @@ function MODEL:tryLoadDocument(fname)
       self:runLatex()
     end
 
-    local syms = self.doc:checkStyle()
     if #syms > 0 then
       self:warning("The document contains symbolic attributes " ..
 		   "that are not defined in the style sheet:",
