@@ -168,6 +168,10 @@ function MODEL:insertOpacity(m, obj)
   local opacityL = self.doc:sheets():allNames("opacity")
   local opacity = obj:get("opacity")
   m:add("opacity", "Opacity: " .. opacity, opacityL, nil, opacity)
+  if obj:type() == "path" then
+    local strokeOpacity = obj:get("strokeopacity")
+    m:add("strokeopacity", "Stroke opacity: " .. strokeOpacity, opacityL, nil, strokeOpacity)
+  end
 end
 
 ----------------------------------------------------------------------
@@ -202,15 +206,15 @@ function MODEL:properties_group(obj, m)
   m:add("comment", "Group object")
   local s = string.format("%d elements", obj:count())
   local clip = obj:clip()
-  local url = obj:text()
+  local link_action = obj:text()
   if clip then
     s = s .. ", with clipping"
   end
   m:add("comment", s)
-  if url == "" then
-    m:add("action_set_link_url", "Set link URL")
+  if link_action == "" then
+    m:add("action_set_link_action", "Set link action")
   else
-    m:add("action_set_link_url", "URL: " .. url)
+    m:add("action_set_link_action", "Link action: " .. link_action)
   end
   m:add("decoration", "Decoration", decoration_names(self),
 	decorationToName, obj:get("decoration"))
@@ -362,10 +366,9 @@ function MODEL:properties_image(obj, m)
   local info = obj:info()
   m:add("comment", string.format("%d x %d pixels",
 				  info.width, info.height))
-  m:add("comment", string.format("%d components with %d bits",
-				  info.components, info.bitsPerComponent))
-  m:add("comment", "Filter " .. info.filter)
+  m:add("comment", "Format " .. info.format)
   self:insertBasic(m, obj)
+  self:insertOpacity(m, obj)
 end
 
 function MODEL:properties_reference(obj, m)
@@ -462,6 +465,7 @@ function MODEL:multiAttributes(m, tm)
 
   if tm.reference then m:add("symbolsize", "Size", sizes) end
   if tm.text or tm.path then m:add("opacity", "Opacity", opacity) end
+  if tm.path then m:add("strokeopacity", "Stroke opacity", opacity) end
   m:add("action_apply_properties", "Apply properties")
 end
 
@@ -573,6 +577,7 @@ function MODEL:pick_properties_path(obj)
   a.tiling = obj:get("tiling")
   a.gradient = obj:get("gradient")
   a.opacity = obj:get("opacity")
+  a.strokeopacity = obj:get("strokeopacity")
 end
 
 function MODEL:pick_properties_image(obj)
@@ -658,6 +663,7 @@ function apply_properties_path(obj, a)
   obj:set("tiling", a.tiling)
   obj:set("gradient", a.gradient)
   obj:set("opacity", a.opacity)
+  obj:set("strokeopacity", a.strokeopacity)
 end
 
 function apply_properties_reference(obj, a)
