@@ -1,7 +1,7 @@
 --[[
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2016  Otfried Cheong
+    Copyright (c) 1993-2019 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -281,15 +281,18 @@ function LINESTOOL:key(text, modifiers)
     self:compute()
     self.model.ui:update(false)
     self:explain()
+    return true
   elseif self:has_segs(3) and text == shortcuts_linestool.arc then
     self.t[#self.t - 1] = ARC
     self:compute()
     self.model.ui:update(false)
     self:explain()
+    return true
   elseif #self.v > 2 and self.t[#self.t - 1] == VERTEX and
     text == shortcuts_linestool.set_axis then
     -- set axis
     self.model.snap.with_axes = true
+    self.model.ui:setActionState("show_axes", self.model.snap.with_axes)
     self.model.snap.snapangle = true
     self.model.snap.origin = self.v[#self.v - 1]
     self.model.snap.orientation = self:compute_orientation()
@@ -297,6 +300,7 @@ function LINESTOOL:key(text, modifiers)
     self.model.ui:setActionState("snapangle", true)
     self.model.ui:update(true)   -- redraw coordinate system
     self:explain()
+    return true
   else
     return false
   end
@@ -1323,6 +1327,11 @@ end
 function apply_text_edit(d, data, run_latex)
   -- refuse to do anything with empty text
   if string.match(d:get("text"), "^%s*$") then return end
+  if data.obj:text() == d:get("text") and
+     (not data.size or data.obj:get("textsize") == data.sizes[d:get("size")]) and
+     (not data.style or data.obj:get(data.stylekind) == data.styles[d:get("style")]) then
+     return -- hasn't changed since last time
+  end
   local model = data.model
   local final = data.obj:clone()
   final:setText(d:get("text"))
