@@ -58,8 +58,8 @@ static void setup_globals(lua_State *L)
   lua_pushliteral(L, "cocoa");
   lua_setfield(L, -2, "toolkit");
 
-  setup_config(L, "system_styles", 0, "styles");
-  setup_config(L, "system_ipelets", 0, "ipelets");
+  setup_config(L, "system_styles", nullptr, "styles");
+  setup_config(L, "system_ipelets", nullptr, "ipelets");
   setup_config(L, "docdir", "IPEDOCDIR", "doc");
 
   setup_common_config(L);
@@ -103,10 +103,8 @@ static int mainloop(lua_State *L)
 
 // --------------------------------------------------------------------
 
-// Changed because of bug #147
-// @interface AppDelegate : NSObject <NSApplicationDelegate>
-
-@interface AppDelegate : NSObject
+// Is bug #147 still relevant?
+@interface AppDelegate : NSObject <NSApplicationDelegate>
 @end
 
 // --------------------------------------------------------------------
@@ -120,7 +118,10 @@ static const char * const about_text =
   "See http://ipe.otfried.org for details.\n"
   "You can \"like\" Ipe and follow Ipe announcements on Facebook "
   "(http://www.facebook.com/drawing.editor.Ipe7).\n\n"
+  "If you are an Ipe fan and want to show others, have a look at the "
+  "Ipe T-shirts (www.shirtee.com/en/store/ipe).\n\n"
   "Platinum sponsors\n\n"
+  " * Hee-Kap Ahn\n"
   " * Martin Ziegler\n\n"
   "If you enjoy Ipe, you can become a member of the exclusive community of "
   "Ipe patrons (http://patreon.com/otfried). "
@@ -238,6 +239,7 @@ static const char * const about_text =
   lua_call(L, 1, 0);
   return YES;
 }
+
 @end
 
 // --------------------------------------------------------------------
