@@ -68,12 +68,12 @@ static void setup_globals(lua_State *L, int width, int height)
   lua_setfield(L, -2, "toolkit");
 
 #ifdef IPEBUNDLE
-  setup_config(L, "system_styles", 0, "styles");
-  setup_config(L, "system_ipelets", 0, "ipelets");
+  setup_config(L, "system_styles", nullptr, "styles");
+  setup_config(L, "system_ipelets", nullptr, "ipelets");
   setup_config(L, "docdir", "IPEDOCDIR", "doc");
 #else
-  setup_config(L, "system_styles", 0, IPESTYLEDIR);
-  setup_config(L, "system_ipelets", 0, IPELETDIR);
+  setup_config(L, "system_styles", nullptr, IPESTYLEDIR);
+  setup_config(L, "system_ipelets", nullptr, IPELETDIR);
   setup_config(L, "docdir", "IPEDOCDIR", IPEDOCDIR);
 #endif
   lua_pushfstring(L, "%s / %s", QT_VERSION_STR, qVersion());
