@@ -195,7 +195,7 @@ extern NSSize thumbnail_size; // in ipeselector_cocoa.cpp
   NSArray *move = [els objectsAtIndexes:idx];
   [els removeObjectsAtIndexes:idx];
   NSMutableIndexSet *nIdx = [NSMutableIndexSet new];
-  for (int i = 0; i < [move count]; ++i)
+  for (size_t i = 0; i < [move count]; ++i)
     [nIdx addIndex:dest++];
   [els insertObjects:move atIndexes:nIdx];
   return YES;
@@ -266,7 +266,7 @@ int AppUi::pageSorter(lua_State *L, Document *doc,
   [cv        bind:NSContentBinding
 	 toObject:delegate
       withKeyPath:@"pages"
-	  options:NULL];
+	  options:nullptr];
   cv.delegate = delegate;
   [cv registerForDraggedTypes:@[@"ipePageSelectorDragId"]];
 
