@@ -40,6 +40,8 @@
 
 using namespace ipe;
 
+extern HBITMAP createBitmap(uint32_t *p, int w, int h);
+
 // --------------------------------------------------------------------
 
 #define IDBASE 9000
@@ -110,7 +112,7 @@ static void insertItem(HWND h, SData *d, int index, int page)
   else
     sprintf(buf, "Page %d", page+1);
   WString ws(String(buf) + t);
-  lvI.pszText = ws;
+  lvI.pszText = ws.data();
   (void) ListView_InsertItem(h, &lvI);
 }
 
@@ -168,7 +170,7 @@ static void showPopup(HWND parent, POINT pt, int index, SData *d, HWND lv)
   }
   int result = TrackPopupMenu(hMenu,
 			      TPM_NONOTIFY | TPM_RETURNCMD | TPM_RIGHTBUTTON,
-			      pt.x, pt.y, 0, parent, NULL);
+			      pt.x, pt.y, 0, parent, nullptr);
   DestroyMenu(hMenu);
   switch (result) {
   case 1: // Delete
@@ -252,9 +254,8 @@ int AppUi::pageSorter(lua_State *L, Document *doc,
     sData.pages.push_back(i);
     Page *p = doc->page(i);
     Buffer bx = r.render(p, p->countViews() - 1);
-    HBITMAP b = Canvas::createBitmap((unsigned char *) bx.data(),
-				     thumbWidth, r.height());
-    ImageList_Add(sData.hImageList, b, NULL);
+    HBITMAP b = createBitmap((uint32_t *) bx.data(), thumbWidth, r.height());
+    ImageList_Add(sData.hImageList, b, nullptr);
   }
 
   // Convert to dialog units:
@@ -301,8 +302,8 @@ int AppUi::pageSorter(lua_State *L, Document *doc,
 
 
   int res =
-    DialogBoxIndirectParamW(NULL, (LPCDLGTEMPLATE) &t[0],
-			    NULL, dialogProc, (LPARAM) &sData);
+    DialogBoxIndirectParamW(nullptr, (LPCDLGTEMPLATE) &t[0],
+			    nullptr, dialogProc, (LPARAM) &sData);
 
   if (res == 1) {
     int n = sData.pages.size();
