@@ -43,17 +43,17 @@ const wchar_t PathView::className[] = L"ipePathViewClass";
 
 PathView::PathView(HWND parent, int id)
 {
-  iCascade = 0;
+  iCascade = nullptr;
   idBase = id;
 
   HWND hwnd = CreateWindowExW(WS_EX_CLIENTEDGE, className, L"",
 			      WS_CHILD | WS_VISIBLE,
 			      0, 0, 0, 0,
-			      parent, NULL,
+			      parent, nullptr,
 			      (HINSTANCE) GetWindowLongPtr(parent, GWLP_HINSTANCE),
 			      this);
-  if (hwnd == NULL) {
-    MessageBoxA(NULL, "PathView creation failed!", "Error!",
+  if (hwnd == nullptr) {
+    MessageBoxA(nullptr, "PathView creation failed!", "Error!",
 		MB_ICONEXCLAMATION | MB_OK);
     exit(9);
   }
@@ -64,7 +64,7 @@ void PathView::set(const AllAttributes &all, Cascade *sheet)
 {
   iCascade = sheet;
   iAll = all;
-  InvalidateRect(hwnd, NULL, FALSE);
+  InvalidateRect(hwnd, nullptr, FALSE);
 }
 
 LRESULT CALLBACK PathView::wndProc(HWND hwnd, UINT message, WPARAM wParam,
@@ -74,7 +74,7 @@ LRESULT CALLBACK PathView::wndProc(HWND hwnd, UINT message, WPARAM wParam,
   switch (message) {
   case WM_CREATE:
     {
-      assert(pv == 0);
+      assert(pv == nullptr);
       LPCREATESTRUCT p = (LPCREATESTRUCT) lParam;
       pv = (PathView *) p->lpCreateParams;
       pv->hwnd = hwnd;
@@ -108,7 +108,7 @@ LRESULT CALLBACK PathView::wndProc(HWND hwnd, UINT message, WPARAM wParam,
 
 void PathView::wndPaint()
 {
-  InvalidateRect(hwnd, NULL, FALSE);
+  InvalidateRect(hwnd, nullptr, FALSE);
 
   RECT rc;
   GetClientRect(hwnd, &rc);
@@ -135,7 +135,7 @@ void PathView::wndPaint()
     Vector u3 = (1.0/zoom) * Vector(0.88 * w, 0.2 * h);
     Vector u4 = (1.0/zoom) * Vector(0.96 * w, 0.5 * h);
 
-    CairoPainter painter(iCascade, 0, cc, 3.0, false);
+    CairoPainter painter(iCascade, nullptr, cc, 3.0, false);
     painter.setPen(iAll.iPen);
     painter.setDashStyle(iAll.iDashStyle);
     painter.setStroke(iAll.iStroke);
@@ -196,7 +196,7 @@ void PathView::button(int x, int y)
     SendMessage(GetParent(hwnd), WM_COMMAND, idBase+1, (LPARAM) hwnd);
   }
   // show new status
-  InvalidateRect(hwnd, NULL, FALSE);
+  InvalidateRect(hwnd, nullptr, FALSE);
 }
 
 void PathView::init(HINSTANCE hInstance)
@@ -208,15 +208,15 @@ void PathView::init(HINSTANCE hInstance)
   wc.cbClsExtra	 = 0;
   wc.cbWndExtra	 = 0;
   wc.hInstance	 = hInstance;
-  wc.hCursor = LoadCursor(NULL, IDC_ARROW);
+  wc.hCursor = LoadCursor(nullptr, IDC_ARROW);
   wc.hbrBackground = (HBRUSH) GetStockObject(NULL_BRUSH);
-  wc.lpszMenuName = NULL;  // actually child id
+  wc.lpszMenuName = nullptr;  // actually child id
   wc.lpszClassName = className;
-  wc.hIcon = NULL;
-  wc.hIconSm = NULL;
+  wc.hIcon = nullptr;
+  wc.hIconSm = nullptr;
 
   if (!RegisterClassEx(&wc)) {
-    MessageBoxA(NULL, "PathView control registration failed!", "Error!",
+    MessageBoxA(nullptr, "PathView control registration failed!", "Error!",
 		MB_ICONEXCLAMATION | MB_OK);
     exit(9);
   }
