@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2016  Otfried Cheong
+    Copyright (c) 1993-2019 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -66,7 +66,7 @@ void IpeTransformTool::report()
   // call back to Lua to report final transformation
   lua_rawgeti(L, LUA_REGISTRYINDEX, iMethod);
   push_matrix(L, iTransform);
-  lua_call(L, 1, 0);
+  lua_callk(L, 1, 0, 0, nullptr);
 }
 
 // --------------------------------------------------------------------
@@ -115,7 +115,7 @@ void LuaTool::mouseButton(int button, bool press)
   lua_remove(L, -3);
   push_button(L, button);
   lua_pushboolean(L, press);
-  lua_call(L, 4, 0);
+  lua_callk(L, 4, 0, 0, nullptr);
 }
 
 void LuaTool::mouseMove()
@@ -124,7 +124,7 @@ void LuaTool::mouseMove()
   lua_getfield(L, -1, "mouseMove");
   lua_pushvalue(L, -2); // model
   lua_remove(L, -3);
-  lua_call(L, 1, 0);
+  lua_callk(L, 1, 0, 0, nullptr);
 }
 
 bool LuaTool::key(String text, int modifiers)
@@ -136,7 +136,7 @@ bool LuaTool::key(String text, int modifiers)
   push_string(L, text);
   push_modifiers(L, modifiers);
   lua_State *L0 = L; // need to save L since
-  lua_call(L, 3, 1); // this may delete tool
+  lua_callk(L, 3, 1, 0, nullptr); // this may delete tool
   bool used = lua_toboolean(L0, -1);
   return used;
 }
@@ -166,7 +166,7 @@ void ShapeTool::draw(Painter &painter) const
   painter.newPath();
   iAuxShape.draw(painter);
   painter.drawPath(EStrokedOnly);
-  for (uint i = 0; i < iMarks.size(); ++i) {
+  for (int i = 0; i < size(iMarks); ++i) {
     switch (iMarks[i].t) {
     case EVertex:
       painter.setFill(Attribute(Color(1000, 0, 1000)));
