@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2016  Otfried Cheong
+    Copyright (c) 1993-2019 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -127,7 +127,7 @@ static int appui_setZoom(lua_State *L)
 static int appui_setSnapIndicator(lua_State *L)
 {
   AppUiBase **ui = check_appui(L, 1);
-  (*ui)->setSnapIndicator(luaL_checkstring(L, 2));
+  (*ui)->setSnapIndicator(luaL_checklstring(L, 2, nullptr));
   return 0;
 }
 
@@ -182,54 +182,11 @@ static int appui_setSelectionVisible(lua_State *L)
   return 0;
 }
 
-static void snapFlag(lua_State *L, int &flags, const char *mode, uint bits)
-{
-  lua_getfield(L, -1, mode);
-  if (!lua_isnil(L, -1))
-    flags = lua_toboolean(L, -1) ? (flags | bits) : (flags & ~bits);
-  lua_pop(L, 1);
-}
-
 static int appui_setSnap(lua_State *L)
 {
   CanvasBase *canvas = check_canvas(L, 1);
-  luaL_checktype(L, 2, LUA_TTABLE);
   Snap snap = canvas->snap();
-  snapFlag(L, snap.iSnap, "snapvtx", Snap::ESnapVtx);
-  snapFlag(L, snap.iSnap, "snapctl", Snap::ESnapCtl);
-  snapFlag(L, snap.iSnap, "snapbd", Snap::ESnapBd);
-  snapFlag(L, snap.iSnap, "snapint", Snap::ESnapInt);
-  snapFlag(L, snap.iSnap, "snapgrid", Snap::ESnapGrid);
-  snapFlag(L, snap.iSnap, "snapangle", Snap::ESnapAngle);
-  snapFlag(L, snap.iSnap, "snapauto", Snap::ESnapAuto);
-  lua_getfield(L, -1, "grid_visible");
-  if (!lua_isnil(L, -1))
-    snap.iGridVisible = lua_toboolean(L, -1);
-  lua_pop(L, 1);
-  lua_getfield(L, -1, "gridsize");
-  if (!lua_isnil(L, -1))
-    snap.iGridSize = (int)luaL_checkinteger(L, -1);
-  lua_pop(L, 1);
-  lua_getfield(L, -1, "anglesize");
-  if (!lua_isnil(L, -1))
-    snap.iAngleSize = IpePi * luaL_checknumber(L, -1) / 180.0;
-  lua_pop(L, 1);
-  lua_getfield(L, -1, "snap_distance");
-  if (!lua_isnil(L, -1))
-    snap.iSnapDistance = (int)luaL_checkinteger(L, -1);
-  lua_pop(L, 1);
-  lua_getfield(L, -1, "with_axes");
-  if (!lua_isnil(L, -1))
-    snap.iWithAxes = lua_toboolean(L, -1);
-  lua_pop(L, 1);
-  lua_getfield(L, -1, "origin");
-  if (is_type(L, -1, "Ipe.vector"))
-    snap.iOrigin = *check_vector(L, -1);
-  lua_pop(L, 1);
-  lua_getfield(L, -1, "orientation");
-  if (!lua_isnil(L, -1))
-    snap.iDir = luaL_checknumber(L, -1);
-  lua_pop(L, 1);
+  get_snap(L, 2, snap);
   canvas->setSnap(snap);
   return 0;
 }
@@ -291,16 +248,16 @@ static int appui_setCursor(lua_State *L)
 {
   CanvasBase *canvas = check_canvas(L, 1);
   if (lua_isnumber(L, 2)) {
-    double s = lua_tonumber(L, 2);
+    double s = lua_tonumberx(L, 2, nullptr);
     int r, g, b;
     check_rgb(L, 3, r, g, b);
     Color color(r, g, b);
     canvas->setCursor(CanvasBase::EDotCursor, s, &color);
   } else if (lua_isstring(L, 2)) {
     static const char * const cursor_names[] =
-      { "standard", "hand", "cross", 0 };
+      { "standard", "hand", "cross", nullptr };
     CanvasBase::TCursor t =
-      CanvasBase::TCursor(luaL_checkoption(L, 2, 0, cursor_names));
+      CanvasBase::TCursor(luaL_checkoption(L, 2, nullptr, cursor_names));
     canvas->setCursor(t);
   } else
     canvas->setCursor(CanvasBase::EStandardCursor);
@@ -356,11 +313,11 @@ static int appui_selecttool(lua_State *L)
 static int appui_transformtool(lua_State *L)
 {
   static const char * const option_names[] = {
-    "translate", "scale", "stretch", "rotate", "shear", 0 };
+    "translate", "scale", "stretch", "rotate", "shear", nullptr };
   CanvasBase *canvas = check_canvas(L, 1);
   Page *page = check_page(L, 2)->page;
   int view = check_viewno(L, 3, page);
-  int type = luaL_checkoption(L, 4, 0, option_names);
+  int type = luaL_checkoption(L, 4, nullptr, option_names);
   bool withShift = lua_toboolean(L, 5);
   lua_pushvalue(L, 6);
   int method = luaL_ref(L, LUA_REGISTRYINDEX);
@@ -422,7 +379,7 @@ static int shapetool_setmarks(lua_State *L)
     Vector *v = check_vector(L, -1);
     lua_rawgeti(L, 1, i+1);
     luaL_argcheck(L, lua_isnumber(L, -1), 1, "element is not a number");
-    int t = lua_tointeger(L, -1);
+    int t = lua_tointegerx(L, -1, nullptr);
     luaL_argcheck(L, ShapeTool::EVertex <= t && t < ShapeTool::ENumMarkTypes,
 		  1, "number is not a mark type");
     lua_pop(L, 2); // v, t
@@ -520,7 +477,7 @@ static int appui_actionInfo(lua_State *L)
 static int appui_actionState(lua_State *L)
 {
   AppUiBase **ui = check_appui(L, 1);
-  const char *name = luaL_checkstring(L, 2);
+  const char *name = luaL_checklstring(L, 2, nullptr);
   lua_pushboolean(L, (*ui)->actionState(name));
   return 1;
 }
@@ -528,7 +485,7 @@ static int appui_actionState(lua_State *L)
 static int appui_setActionState(lua_State *L)
 {
   AppUiBase **ui = check_appui(L, 1);
-  const char *name = luaL_checkstring(L, 2);
+  const char *name = luaL_checklstring(L, 2, nullptr);
   bool val = lua_toboolean(L, 3);
   (*ui)->setActionState(name, val);
   return 0;
@@ -576,10 +533,10 @@ static int appui_setNumbers(lua_State *L)
   String vno;
   String pno;
   if (!lua_isnil(L, 2))
-    vno = luaL_checkstring(L, 2);
+    vno = luaL_checklstring(L, 2, nullptr);
   bool vm = lua_toboolean(L, 3);
   if (!lua_isnil(L, 4))
-    pno = luaL_checkstring(L, 4);
+    pno = luaL_checklstring(L, 4, nullptr);
   bool pm = lua_toboolean(L, 5);
   (*ui)->setNumbers(vno, vm, pno, pm);
   return 0;
@@ -594,7 +551,7 @@ static int appui_setBookmarks(lua_State *L)
   for (int i = 1; i <= no; ++i) {
     lua_rawgeti(L, 2, i);
     luaL_argcheck(L, lua_isstring(L, -1), 2, "item is not a string");
-    bm.push_back(String(lua_tostring(L, -1)));
+    bm.push_back(String(lua_tolstring(L, -1, nullptr)));
     lua_pop(L, 1);
   }
   (*ui)->setBookmarks(no, &bm[0]);
@@ -605,7 +562,7 @@ static int appui_setWindowTitle(lua_State *L)
 {
   AppUiBase **ui = check_appui(L, 1);
   bool mod = lua_toboolean(L, 2);
-  const char *s = luaL_checkstring(L, 3);
+  const char *s = luaL_checklstring(L, 3, nullptr);
   (*ui)->setWindowCaption(mod, s);
   return 0;
 }
@@ -613,7 +570,7 @@ static int appui_setWindowTitle(lua_State *L)
 static int appui_setNotes(lua_State *L)
 {
   AppUiBase **ui = check_appui(L, 1);
-  const char *s = luaL_checkstring(L, 2);
+  const char *s = luaL_checklstring(L, 2, nullptr);
   (*ui)->setNotes(s);
   return 0;
 }
@@ -621,10 +578,10 @@ static int appui_setNotes(lua_State *L)
 static int appui_showTool(lua_State *L)
 {
   static const char * const option_names[] = {
-    "properties", "bookmarks", "notes", "layers", 0 };
+    "properties", "bookmarks", "notes", "layers", nullptr };
 
   AppUiBase **ui = check_appui(L, 1);
-  int m = luaL_checkoption(L, 2, 0, option_names);
+  int m = luaL_checkoption(L, 2, nullptr, option_names);
   bool s = lua_toboolean(L, 3);
   (*ui)->setToolVisible(m, s);
   return 0;
@@ -633,10 +590,10 @@ static int appui_showTool(lua_State *L)
 static int appui_explain(lua_State *L)
 {
   AppUiBase **ui = check_appui(L, 1);
-  const char *s = luaL_checkstring(L, 2);
+  const char *s = luaL_checklstring(L, 2, nullptr);
   int t = 4000;
   if (lua_isnumber(L, 3))
-    t = lua_tointeger(L, 3);
+    t = lua_tointegerx(L, 3, nullptr);
   (*ui)->explain(s, t);
   return 0;
 }
@@ -655,17 +612,17 @@ void get_page_sorter_size(lua_State *L, int &width, int &height,
   if (lua_istable(L, -1)) {
     lua_rawgeti(L, -1, 1);
     if (lua_isnumber(L, -1))
-      width = lua_tointeger(L, -1);
+      width = lua_tointegerx(L, -1, nullptr);
     lua_rawgeti(L, -2, 2);
     if (lua_isnumber(L, -1))
-      height = lua_tointeger(L, -1);
+      height = lua_tointegerx(L, -1, nullptr);
     lua_pop(L, 2);
   }
   lua_pop(L, 1); // page_sorter_size
 
   lua_getfield(L, -1, "thumbnail_width");
   if (lua_isnumber(L, -1))
-    thumbWidth = lua_tointeger(L, -1);
+    thumbWidth = lua_tointegerx(L, -1, nullptr);
   lua_pop(L, 1); // thumbnail_width, prefs
 }
 
@@ -675,14 +632,14 @@ static int appui_selectPage(lua_State *L)
   Document **doc = check_document(L, 2);
   int page = -1;
   if (lua_isnumber(L, 3)) {
-    page = lua_tointeger(L, 3);
+    page = lua_tointegerx(L, 3, nullptr);
     luaL_argcheck(L, 1 <= page && page <= (*doc)->countPages(),
 		  3, "invalid page number");
   }
 
   int startIndex = 1;
   if (lua_isnumber(L, 4)) {
-    startIndex = lua_tointeger(L, 4);
+    startIndex = lua_tointegerx(L, 4, nullptr);
     int maxIndex = (page < 0) ? (*doc)->countPages() :
       (*doc)->page(page-1)->countViews();
     luaL_argcheck(L, 1 <= startIndex && startIndex <= maxIndex,
@@ -712,7 +669,7 @@ static int appui_pageSorter(lua_State *L)
   return (*ui)->pageSorter(L, *doc, width, height, thumbWidth);
 }
 
-static const char *const render_formats[] = {"svg", "png", "eps", "pdf", NULL };
+static const char *const render_formats[] = {"svg", "png", "eps", "pdf", nullptr };
 
 static int appui_renderPage(lua_State *L)
 {
@@ -721,8 +678,8 @@ static int appui_renderPage(lua_State *L)
   int pageno = luaL_checkinteger(L, 3);
   int viewno = luaL_checkinteger(L, 4);
   Thumbnail::TargetFormat fm =
-    Thumbnail::TargetFormat(luaL_checkoption(L, 5, NULL, render_formats));
-  const char *dst = luaL_checkstring(L, 6);
+    Thumbnail::TargetFormat(luaL_checkoption(L, 5, nullptr, render_formats));
+  const char *dst = luaL_checklstring(L, 6, nullptr);
   double zoom = luaL_checknumber(L, 7);
   bool transparent = lua_toboolean(L, 8);
   bool nocrop = lua_toboolean(L, 9);
@@ -787,7 +744,7 @@ static const struct luaL_Reg appui_methods[] = {
   { "selectPage", appui_selectPage },
   { "pageSorter", appui_pageSorter },
   { "renderPage", appui_renderPage },
-  { 0, 0},
+  { nullptr, nullptr }
 };
 
 // --------------------------------------------------------------------
@@ -797,7 +754,7 @@ static int appui_constructor(lua_State *L)
   luaL_checktype(L, 1, LUA_TTABLE); // this is the model
 
   AppUiBase **ui = (AppUiBase **) lua_newuserdata(L, sizeof(AppUiBase *));
-  *ui = 0;
+  *ui = nullptr;
   luaL_getmetatable(L, "Ipe.appui");
   lua_setmetatable(L, -2);
 
@@ -843,22 +800,22 @@ static int appui_constructor(lua_State *L)
 
     lua_getfield(L, -1, "thin_grid_line");
     if (lua_isnumber(L, -1))
-      style.thinLine = lua_tonumber(L, -1);
+      style.thinLine = lua_tonumberx(L, -1, nullptr);
     lua_pop(L, 1); // thin_grid_line
 
     lua_getfield(L, -1, "thick_grid_line");
     if (lua_isnumber(L, -1))
-      style.thickLine = lua_tonumber(L, -1);
+      style.thickLine = lua_tonumberx(L, -1, nullptr);
     lua_pop(L, 1); // thick_grid_line
 
     lua_getfield(L, -1, "thin_step");
     if (lua_isnumber(L, -1))
-      style.thinStep = lua_tointeger(L, -1);
+      style.thinStep = lua_tointegerx(L, -1, nullptr);
     lua_pop(L, 1); // thin_step
 
     lua_getfield(L, -1, "thick_step");
     if (lua_isnumber(L, -1))
-      style.thickStep = lua_tointeger(L, -1);
+      style.thickStep = lua_tointegerx(L, -1, nullptr);
     lua_pop(L, 1); // thick_step
   }
   lua_pop(L, 1); // canvas_style
@@ -868,10 +825,10 @@ static int appui_constructor(lua_State *L)
   if (lua_istable(L, -1)) {
     lua_rawgeti(L, -1, 1);
     if (lua_isnumber(L, -1))
-      width = lua_tointeger(L, -1);
+      width = lua_tointegerx(L, -1, nullptr);
     lua_rawgeti(L, -2, 2);
     if (lua_isnumber(L, -1))
-      height = lua_tointeger(L, -1);
+      height = lua_tointegerx(L, -1, nullptr);
     lua_pop(L, 2);
   }
   lua_pop(L, 1); // window_size
