@@ -555,7 +555,7 @@ bool Parser::parseStyle()
       writeTag("layout", att);
     } else {
       // only standard tags remain
-      uint i = 0;
+      size_t i = 0;
       while (i < (sizeof(styledefs)/sizeof(const char *)) &&
 	     tag != styledefs[i])
 	++i;
