@@ -33,10 +33,13 @@
 #include "ipecairopainter.h"
 #include "ipefonts.h"
 
+// for std::memset
+#include <cstring>
+
 using namespace ipe;
 
 // in ipebitmap:
-extern bool dctDecode(Buffer dctData, Buffer pixelData, int components);
+extern bool dctDecode(Buffer dctData, Buffer pixelData);
 
 // --------------------------------------------------------------------
 
@@ -78,8 +81,7 @@ static bool getFunctionType2(const PdfObj *obj, std::vector<double> &fun)
     return false;
   for (int i = 0; i < 2; ++i) {
     std::vector<double> c;
-    if (!obj->dict()->getNumberArray(i ? "C1" : "C0", nullptr, c)
-	|| c.size() != 3)
+    if (!obj->dict()->getNumberArray(i ? "C1" : "C0", nullptr, c) || c.size() != 3)
       return false;
     for (int j = 0; j < 3; ++j)
       fun.push_back(c[j]);
@@ -87,7 +89,7 @@ static bool getFunctionType2(const PdfObj *obj, std::vector<double> &fun)
   return true;
 }
 
-static void drawShading(cairo_t *cr, const PdfDict *d, const PdfResources *r)
+static void drawShading(cairo_t *cr, const PdfDict *d, const PdfResourceBase *r)
 {
   // ipeDebug("drawShading: %s", d->dictRepr().z());
   double type;
@@ -130,7 +132,7 @@ static void drawShading(cairo_t *cr, const PdfDict *d, const PdfResources *r)
     if (!f->getNumberArray("Bounds", nullptr, bounds))
       return;
     const PdfObj *a = r->getDeep(f, "Functions");
-    if (!a || !a->array() || a->array()->count() != int(bounds.size()) + 1)
+    if (!a || !a->array() || a->array()->count() != size(bounds) + 1)
       return;
     for (int i = 0; i < a->array()->count(); ++i) {
       const PdfObj *af = a->array()->obj(i, nullptr);
@@ -149,20 +151,20 @@ static void drawShading(cairo_t *cr, const PdfDict *d, const PdfResources *r)
 
   if (extend[0] && extend[1])
     // Cairo cannot control this individually, would have
-    // to simulate using transparency
+    // to simulate using transparency or something
     cairo_pattern_set_extend(p, CAIRO_EXTEND_PAD);
   else
     cairo_pattern_set_extend(p, CAIRO_EXTEND_NONE);
 
-  cairo_pattern_add_color_stop_rgb(p, 0.0, funs[0], funs[1], funs[2]);
-  cairo_pattern_add_color_stop_rgb(p, 1.0, funs[3], funs[4], funs[5]);
-  if (ftype == 3.0) {
-    for (int i = 0; i < int(bounds.size()); ++i) {
-      double x = (bounds[i] - domain[0]) / (domain[1] - domain[0]);
-      cairo_pattern_add_color_stop_rgb(p, x, funs[6 * i + 6],
-				       funs[6 * i + 7], funs[6 * i + 8]);
-    }
+  int fi = 0;
+  cairo_pattern_add_color_stop_rgb(p, 0.0, funs[fi+0], funs[fi+1], funs[fi+2]);
+  for (int i = 0; i < size(bounds); ++i) {
+    fi += 6;
+    double x = (bounds[i] - domain[0]) / (domain[1] - domain[0]);
+    cairo_pattern_add_color_stop_rgb(p, x, funs[fi+0], funs[fi+1], funs[fi+2]);
   }
+  cairo_pattern_add_color_stop_rgb(p, 1.0, funs[fi+3], funs[fi+4], funs[fi+5]);
+
 
   cairo_set_source(cr, p);
   cairo_paint(cr);
@@ -170,7 +172,7 @@ static void drawShading(cairo_t *cr, const PdfDict *d, const PdfResources *r)
 }
 
 
-static void drawImage(cairo_t *cr, const PdfDict *d, const PdfResources *r)
+static void drawImage(cairo_t *cr, const PdfDict *d, const PdfResourceBase *r, double opacity)
 {
   // ipeDebug("Image: %s", d->dictRepr().z());
   double width, height, bpc;
@@ -200,38 +202,35 @@ static void drawImage(cairo_t *cr, const PdfDict *d, const PdfResources *r)
     }
   }
   Buffer stream = d->inflate();
-  Buffer pixels;
+  Buffer pixels(4 * width * height);
   if (jpg) {
-    pixels = Buffer(width * height * components);
-    if (!dctDecode(stream, pixels, components))
+    if (!dctDecode(stream, pixels))
       return;
-  } else
-    pixels = stream;
-  Buffer data(width * height * sizeof(uint32_t));
-  // convert pixels to data
-  const char *p = pixels.data();
-  uint32_t *q = (uint *) data.data();
-  if (components == 3) {
-    for (int y = 0; y < height; ++y) {
-      for (int x = 0; x < width; ++x) {
-	uchar r = uchar(*p++);
-	uchar g = uchar(*p++);
-	uchar b = uchar(*p++);
-	uint32_t pixel = 0xff000000 | (r << 16) | (g << 8) | b;
-	*q++ = pixel;
-      }
-    }
   } else {
-    for (int y = 0; y < height; ++y) {
-      for (int x = 0; x < width; ++x) {
-	uchar r = uchar(*p++);
-	uint32_t pixel = 0xff000000 | (r << 16) | (r << 8) | r;
-	*q++ = pixel;
+    const char *p = stream.data();
+    uint32_t *q = (uint32_t *) pixels.data();
+    if (components == 3) {
+      for (int y = 0; y < height; ++y) {
+	for (int x = 0; x < width; ++x) {
+	  uint8_t r = uint8_t(*p++);
+	  uint8_t g = uint8_t(*p++);
+	  uint8_t b = uint8_t(*p++);
+	  uint32_t pixel = 0xff000000 | (r << 16) | (g << 8) | b;
+	  *q++ = pixel;
+	}
+      }
+    } else {
+      for (int y = 0; y < height; ++y) {
+	for (int x = 0; x < width; ++x) {
+	  uint8_t r = uint8_t(*p++);
+	  uint32_t pixel = 0xff000000 | (r << 16) | (r << 8) | r;
+	  *q++ = pixel;
+	}
       }
     }
   }
   cairo_surface_t *image =
-    cairo_image_surface_create_for_data((uchar *) data.data(),
+    cairo_image_surface_create_for_data((uint8_t *) pixels.data(),
 					CAIRO_FORMAT_ARGB32,
 					width, height,
 					4 * width);
@@ -240,7 +239,7 @@ static void drawImage(cairo_t *cr, const PdfDict *d, const PdfResources *r)
   cairoTransform(cr, tf);
   cairo_set_source_surface(cr, image, 0, 0);
   cairo_pattern_set_filter(cairo_get_source(cr), CAIRO_FILTER_FAST);
-  cairo_paint(cr);
+  cairo_paint_with_alpha(cr, opacity);
   cairo_restore(cr);
 }
 
@@ -258,8 +257,7 @@ static void drawImage(cairo_t *cr, const PdfDict *d, const PdfResources *r)
   to avoid drawing text without Latex. */
 CairoPainter::CairoPainter(const Cascade *sheet, Fonts *fonts,
 			   cairo_t *cc, double zoom, bool pretty)
-  : Painter(sheet), iFonts(fonts),
-    iCairo(cc), iZoom(zoom), iPretty(pretty)
+  : Painter(sheet), iFonts(fonts), iCairo(cc), iZoom(zoom), iPretty(pretty)
 {
   iDimmed = false;
 }
@@ -362,11 +360,12 @@ void CairoPainter::doDrawPath(TPathMode mode)
       cairo_pattern_set_extend(p, g->iExtend ?
 			       CAIRO_EXTEND_PAD : CAIRO_EXTEND_NONE);
 
-      for (uint i = 0; i < g->iStops.size(); ++i)
-	cairo_pattern_add_color_stop_rgb(p, g->iStops[i].offset,
-					 g->iStops[i].color.iRed.toDouble(),
-					 g->iStops[i].color.iGreen.toDouble(),
-					 g->iStops[i].color.iBlue.toDouble());
+      for (const auto & stop : g->iStops) {
+	cairo_pattern_add_color_stop_rgb(p, stop.offset,
+					 stop.color.iRed.toDouble(),
+					 stop.color.iGreen.toDouble(),
+					 stop.color.iBlue.toDouble());
+      }
 
       const Matrix &m0 = (matrix()* g->iMatrix).inverse();
       cairo_matrix_t m;
@@ -387,7 +386,7 @@ void CairoPainter::doDrawPath(TPathMode mode)
 
       cairo_surface_t *s =
 	cairo_image_surface_create(CAIRO_FORMAT_ARGB32, 32, 32);
-      uchar *data = cairo_image_surface_get_data(s);
+      uint8_t *data = cairo_image_surface_get_data(s);
       memset(data, 0, 4 * 32 * 32);
 
       cairo_t *cc = cairo_create(s);
@@ -395,7 +394,7 @@ void CairoPainter::doDrawPath(TPathMode mode)
 			    fillColor.iRed.toDouble(),
 			    fillColor.iGreen.toDouble(),
 			    fillColor.iBlue.toDouble(),
-			    1.0);
+			    opacity().toDouble());
 
       cairo_rectangle(cc, 0, 0, 32, 32 * t->iWidth / t->iStep);
       cairo_fill(cc);
@@ -426,7 +425,7 @@ void CairoPainter::doDrawPath(TPathMode mode)
     cairo_set_source_rgba(iCairo, strokeColor.iRed.toDouble(),
 			  strokeColor.iGreen.toDouble(),
 			  strokeColor.iBlue.toDouble(),
-			  opacity().toDouble());
+			  strokeOpacity().toDouble());
 
     cairo_set_line_width(iCairo, pen().toDouble());
 
@@ -472,36 +471,14 @@ void CairoPainter::doAddClipPath()
 
 // --------------------------------------------------------------------
 
-class RenderData : public Bitmap::MRenderData {
-public:
-  virtual ~RenderData() { /* Nothing */ }
-  Buffer pixels;
-};
-
 void CairoPainter::doDrawBitmap(Bitmap bitmap)
 {
-  // make caching optional, or cache only most recent bitmaps?
-#if 1
-  // The original data in the bitmap may be deflated or dct encoded
-  // cache the decoded data for faster rendering
-  if (!bitmap.renderData()) {
-    RenderData *render = new RenderData;
-    render->pixels = bitmap.pixelData(); // empty if failed
-    // may need to scale down?
-    bitmap.setRenderData(render);
-  }
-  if (!bitmap.renderData())
-    return;
-  RenderData *render = static_cast<RenderData *>(bitmap.renderData());
-  Buffer data = render->pixels;
-#else
   Buffer data = bitmap.pixelData();
-#endif
   if (!data.size())
     return;
   // is this legal?  I don't want cairo to modify my bitmap temporarily.
   cairo_surface_t *image =
-    cairo_image_surface_create_for_data((uchar *) data.data(),
+    cairo_image_surface_create_for_data((uint8_t *) data.data(),
 					CAIRO_FORMAT_ARGB32,
 					bitmap.width(), bitmap.height(),
 					4 * bitmap.width());
@@ -512,7 +489,7 @@ void CairoPainter::doDrawBitmap(Bitmap bitmap)
   cairoTransform(iCairo, tf);
   cairo_set_source_surface(iCairo, image, 0, 0);
   cairo_pattern_set_filter(cairo_get_source(iCairo), CAIRO_FILTER_FAST);
-  cairo_paint(iCairo);
+  cairo_paint_with_alpha(iCairo, opacity().toDouble());
   cairo_restore(iCairo);
 }
 
@@ -572,26 +549,31 @@ void CairoPainter::doDrawText(const Text *text)
   } else {
     transform(Matrix(xf->iStretch, 0, 0, xf->iStretch, 0, 0));
     translate(xf->iTranslation);
-    cairo_save(iCairo);
-    cairoTransform(iCairo, matrix());
-    PdfState ps;
-    ps.iFont = nullptr;
-    ps.iFillRgb[0] = ps.iFillRgb[1] = ps.iFillRgb[2] = 0.0;
-    ps.iStrokeRgb[0] = ps.iStrokeRgb[1] = ps.iStrokeRgb[2] = 0.0;
-    ps.iStrokeOpacity = ps.iFillOpacity = opacity().toDouble();
-    ps.iCharacterSpacing = 0.0;
-    ps.iWordSpacing = 0.0;
-    ps.iHorizontalScaling = 1.0;
-    ps.iLeading = 0.0;
-    ps.iTextRise = 0.0;
-    iPdfState.push_back(ps);
     const PdfDict *form = findResource("XObject", xf->iName);
     if (form)
-      execute(form);
-    cairo_restore(iCairo);
+      executeStream(form, form);
   }
 }
 
+void CairoPainter::executeStream(const PdfDict *stream, const PdfDict *resources)
+{
+  cairo_save(iCairo);
+  cairoTransform(iCairo, matrix());
+  PdfState ps;
+  ps.iFont = nullptr;
+  ps.iFillRgb[0] = ps.iFillRgb[1] = ps.iFillRgb[2] = 0.0;
+  ps.iStrokeRgb[0] = ps.iStrokeRgb[1] = ps.iStrokeRgb[2] = 0.0;
+  ps.iStrokeOpacity = ps.iFillOpacity = opacity().toDouble();
+  ps.iCharacterSpacing = 0.0;
+  ps.iWordSpacing = 0.0;
+  ps.iHorizontalScaling = 1.0;
+  ps.iLeading = 0.0;
+  ps.iTextRise = 0.0;
+  iPdfState.push_back(ps);
+  execute(stream, resources);
+  cairo_restore(iCairo);
+}
+
 // --------------------------------------------------------------------
 
 //! Clear PDF argument stack
@@ -615,12 +597,12 @@ const PdfDict *CairoPainter::findResource(String kind, String name)
 }
 
 
-void CairoPainter::execute(const PdfDict *xform)
+void CairoPainter::execute(const PdfDict *xform, const PdfDict *resources, bool applyMatrix)
 {
   // ipeDebug("execute %s", xform->dictRepr().z());
-  iResourceStack.push_back(xform);
+  iResourceStack.push_back(resources);
   std::vector<double> m;
-  if (xform->getNumberArray("Matrix", nullptr, m) && m.size() == 6) {
+  if (applyMatrix && xform->getNumberArray("Matrix", nullptr, m) && m.size() == 6) {
     Matrix mx;
     for (int i = 0; i < 6; ++i)
       mx.a[i] = m[i];
@@ -659,6 +641,10 @@ void CairoPainter::execute(const PdfDict *xform)
 	opk(false);
       else if (op == "K")
 	opk(true);
+      else if (op == "scn")
+	opscn(false);
+      else if (op == "SCN")
+	opscn(true);
       else if (op == "w")
 	opw();
       else if (op == "d")
@@ -752,8 +738,8 @@ void CairoPainter::execute(const PdfDict *xform)
 	// set rendering intent, ignore
       } else {
 	String a;
-	for (uint i = 0; i < iArgs.size(); ++i)
-	  a += iArgs[i]->repr() + " ";
+	for (const auto & arg : iArgs)
+	  a += arg->repr() + " ";
 	ipeDebug("op %s (%s)", op.z(), a.z());
       }
       clearArgs();
@@ -768,7 +754,7 @@ void CairoPainter::opg(bool stroke)
   if (iArgs.size() != 1 || !iArgs[0]->number())
     return;
   double gr = iArgs[0]->number()->value();
-  PdfState &ps = iPdfState.back();
+  auto & ps = iPdfState.back();
   if (stroke)
     ps.iStrokeRgb[0] = ps.iStrokeRgb[1] = ps.iStrokeRgb[2] = gr;
   else
@@ -793,8 +779,7 @@ void CairoPainter::opk(bool stroke)
       || !iArgs[2]->number() || !iArgs[3]->number())
     return;
   ipeDebug("PDF setting CMYK color");
-  // should really use the colorspace of the monitor instead of this
-  // crude conversion
+  // should use the colorspace of the monitor instead of this crude conversion
   double v = 1.0  - iArgs[3]->number()->value();
   double *col = (stroke ? iPdfState.back().iStrokeRgb :
 		 iPdfState.back().iFillRgb);
@@ -802,6 +787,31 @@ void CairoPainter::opk(bool stroke)
     col[i] = v * (1.0 - iArgs[i]->number()->value());
 }
 
+void CairoPainter::opscn(bool stroke)
+{
+  // uncolored tiling pattern arguments actually depend on colorspace set with cs,
+  // we simply assume here that it's DeviceRGB
+  String pattern;
+  auto & ps = iPdfState.back();
+  if (iArgs.size() == 1 && iArgs[0]->name()) {
+    // colored tiling pattern
+    pattern = iArgs[0]->name()->value();
+  } else {
+    if (iArgs.size() != 4 || !iArgs[0]->number() || !iArgs[1]->number()
+	|| !iArgs[2]->number() || !iArgs[3]->name())
+      return;
+    // uncolored tiling pattern
+    pattern = iArgs[3]->name()->value();
+    double *col = (stroke ? ps.iStrokeRgb : ps.iFillRgb);
+    for (int i = 0; i < 3; ++i)
+      col[i] = iArgs[i]->number()->value();
+  }
+  if (stroke)
+    ipeDebug("op scn /%s: stroke pattern not implemented.", pattern.z());
+  else
+    ps.iFillPattern = pattern;
+}
+
 void CairoPainter::opcm()
 {
   if (iArgs.size() != 6)
@@ -828,7 +838,7 @@ void CairoPainter::opd()
     return;
   std::vector<double> dashes;
   for (int i = 0; i < iArgs[0]->array()->count(); ++i) {
-    const PdfObj *obj = iArgs[0]->array()->obj(i, 0);
+    const PdfObj *obj = iArgs[0]->array()->obj(i, nullptr);
     if (!obj->number())
       return;
     dashes.emplace_back(obj->number()->value());
@@ -928,10 +938,10 @@ void CairoPainter::opDo()
   String subtype = subtypeObj->name()->value();
   if (subtype == "Form") {
     cairo_save(iCairo);
-    execute(xf);
+    execute(xf, xf);
     cairo_restore(iCairo);
   } else if (subtype == "Image") {
-    drawImage(iCairo, xf, iFonts->resources());
+    drawImage(iCairo, xf, iFonts->resources(), iPdfState.back().iFillOpacity);
   } else
     ipeDebug("Do operator with unsupported XObject subtype %s", subtype.z());
 }
@@ -1025,10 +1035,99 @@ void CairoPainter::opre()
 
 void CairoPainter::opn()
 {
-  // TODO: this is a hack - isn't there a better way?
-  cairo_set_operator(iCairo, CAIRO_OPERATOR_DEST);
-  cairo_stroke(iCairo);
-  cairo_set_operator(iCairo, CAIRO_OPERATOR_OVER);
+  // the sequence "W n" updates the clipping path and then clears the current path
+  cairo_new_path(iCairo);
+}
+
+// TODO: cache patterns instead of recreating them for every object?
+// caching would need different handling of uncolored tiling patterns.
+// shading patterns are not implemented here, because Ipe and tikz create
+// them using the 'sh' operator.
+void CairoPainter::createPattern()
+{
+  auto & ps = iPdfState.back();
+  const PdfDict *pat = findResource("Pattern", ps.iFillPattern);
+  double patternType, paintType, xstep, ystep;
+  if (pat && pat->getNumber("PatternType", patternType, nullptr)
+      && pat->getNumber("PaintType", paintType, nullptr)
+      && pat->getNumber("XStep", xstep, nullptr)
+      && pat->getNumber("YStep", ystep, nullptr)) {
+    ipeDebug("Fill pattern /%s PatternType %g PaintType %g xstep %g ystep %g", ps.iFillPattern.z(),
+	     patternType, paintType, xstep, ystep);
+    // handle tiling patterns only
+    if (patternType != 1.0)
+      return;
+
+    // to get good quality patterns, the pattern surface cannot be too small
+    // except that for Ipe patterns this isn't necessary.
+    double xscale = 1.0;
+    double yscale = 1.0;
+    // Heuristic: if a matrix exists, let's assume small cell is okay
+    if (pat->get("Matrix", nullptr) == nullptr) {
+      while (xscale * xstep < 100)
+	xscale *= 2.0;
+      while (yscale * ystep < 100)
+	yscale *= 2.0;
+    }
+    int width = (int) std::ceil(xscale * xstep);
+    int height = (int) std::ceil(yscale * ystep);
+    ipeDebug("Using pattern surface of size %d x %d", width, height);
+    cairo_surface_t *sf = cairo_image_surface_create(CAIRO_FORMAT_ARGB32, width, height);
+    cairo_t *cc = cairo_create(sf);
+    cairo_paint_with_alpha(cc, 0.0); // clear surface to transparent
+    cairo_translate(cc, 0.0, height);
+    cairo_scale(cc, 1.0, -1.0);
+
+    auto & ps0 = iPdfState.back();
+    PdfState ps;
+    ps.iFont = nullptr;
+    for (int i = 0; i < 3; ++i) {
+      ps.iFillRgb[i] = ps0.iFillRgb[i];
+      ps.iStrokeRgb[i] = ps0.iStrokeRgb[i];
+    }
+    ps.iStrokeOpacity = ps0.iStrokeOpacity;
+    ps.iFillOpacity = ps0.iFillOpacity;
+    ps.iCharacterSpacing = 0.0;
+    ps.iWordSpacing = 0.0;
+    ps.iHorizontalScaling = 1.0;
+    ps.iLeading = 0.0;
+    ps.iTextRise = 0.0;
+
+    // drawing the pattern four times is also not necessary for Ipe patterns...
+    // we can avoid this if we consult the BBox
+    for (int dx = 0; dx < 2; ++dx) {
+      for (int dy = 0; dy < 2; ++dy) {
+	cairo_save(cc);
+	cairo_translate(cc, -dx * width, -dy * height);
+	cairo_scale(cc, xscale, yscale);
+	CairoPainter painter(iCascade, iFonts, cc, 1.0, false);
+	painter.iPdfState.push_back(ps);
+	painter.execute(pat, pat, false);
+	cairo_restore(cc);
+      }
+    }
+    cairo_surface_flush(sf);
+    cairo_destroy(cc);
+
+    cairo_pattern_t *cpat = cairo_pattern_create_for_surface(sf);
+    cairo_pattern_set_extend(cpat, CAIRO_EXTEND_REPEAT);
+
+    Matrix mx;
+    std::vector<double> m;
+    if (pat->getNumberArray("Matrix", nullptr, m) && m.size() == 6) {
+      for (int i = 0; i < 6; ++i)
+	mx.a[i] = m[i];
+      // PDF pattern matrix goes from user space to pattern space,
+      // Cairo pattern matrix is the opposite
+      mx = mx.inverse();
+    }
+    mx = Matrix(xscale, 0.0, 0.0, -yscale, 0.0, height) * mx;
+    cairo_matrix_t cm;
+    cairoMatrix(cm, mx);
+    cairo_pattern_set_matrix(cpat, &cm);
+    cairo_set_source(iCairo, cpat);
+    cairo_pattern_destroy(cpat);
+  }
 }
 
 void CairoPainter::opStrokeFill(bool close, bool fill, bool stroke, bool eofill)
@@ -1037,9 +1136,12 @@ void CairoPainter::opStrokeFill(bool close, bool fill, bool stroke, bool eofill)
     cairo_close_path(iCairo);
   PdfState &ps = iPdfState.back();
   if (fill) {
-    cairo_set_source_rgba(iCairo,
-			  ps.iFillRgb[0], ps.iFillRgb[1], ps.iFillRgb[2],
-			  ps.iFillOpacity);
+    if (!ps.iFillPattern.empty())
+      createPattern();
+    else
+      cairo_set_source_rgba(iCairo,
+			    ps.iFillRgb[0], ps.iFillRgb[1], ps.iFillRgb[2],
+			    ps.iFillOpacity);
     cairo_set_fill_rule(iCairo, eofill ? CAIRO_FILL_RULE_EVEN_ODD :
 			CAIRO_FILL_RULE_WINDING);
   }
@@ -1101,7 +1203,8 @@ void CairoPainter::opTf()
   String name = iArgs[0]->name()->value();
   iPdfState.back().iFontSize = iArgs[1]->number()->value();
   const PdfDict *fd = findResource("Font", name);
-  iPdfState.back().iFont = iFonts->getFace(fd);
+  if (fd)
+    iPdfState.back().iFont = iFonts->getFace(fd);
 }
 
 void CairoPainter::opTd(bool setLeading)
@@ -1161,7 +1264,7 @@ void CairoPainter::opTJ()
   std::vector<cairo_glyph_t> glyphs;
   Vector textPos(0, 0);
   for (int i = 0; i < iArgs[0]->array()->count(); ++i) {
-    const PdfObj *obj = iArgs[0]->array()->obj(i, 0);
+    const PdfObj *obj = iArgs[0]->array()->obj(i, nullptr);
     if (obj->number())
       textPos.x -=
 	0.001 * ps.iFontSize * obj->number()->value() * ps.iHorizontalScaling;
@@ -1180,9 +1283,9 @@ void CairoPainter::collectGlyphs(String s, std::vector<cairo_glyph_t> &glyphs,
 	      ps.iFont->type() == FontType::CIDType2);
   int j = 0;
   while (j < s.size()) {
-    int ch = uchar(s[j++]);
+    int ch = uint8_t(s[j++]);
     if (ucs && j < s.size())
-      ch = (ch << 8) | uchar(s[j++]);
+      ch = (ch << 8) | uint8_t(s[j++]);
     cairo_glyph_t g;
     g.index = ps.iFont->glyphIndex(ch);
     Vector p = iTextMatrix.linear() * textPos;
