@@ -53,6 +53,8 @@ namespace ipe {
 
     void setDimmed(bool dim) { iDimmed = dim; }
 
+    void executeStream(const PdfDict *stream, const PdfDict *resources);
+
   protected:
     virtual void doPush();
     virtual void doPop();
@@ -73,7 +75,7 @@ namespace ipe {
     void drawGlyphs(std::vector<cairo_glyph_t> &glyphs);
     void collectGlyphs(String s, std::vector<cairo_glyph_t> &glyphs,
 		       Vector &textPos);
-    void execute(const PdfDict *form);
+    void execute(const PdfDict *stream, const PdfDict *resources, bool applyMatrix = true);
     void clearArgs();
     void opcm();
     void opBT();
@@ -89,6 +91,7 @@ namespace ipe {
     void opk(bool stroke);
     void opg(bool stroke);
     void oprg(bool stroke);
+    void opscn(bool stroke);
     void opw();
     void opd();
     void opDo();
@@ -110,6 +113,7 @@ namespace ipe {
     void opn();
     void opStrokeFill(bool close, bool fill, bool stroke, bool eofill);
     void opsh();
+    void createPattern();
 
   private:
     Fonts *iFonts;
@@ -138,7 +142,9 @@ namespace ipe {
       double iWordSpacing;
       double iHorizontalScaling;
       double iLeading;
+      String iFillPattern;
     };
+
     std::vector<PdfState> iPdfState;
 
     // text matrix is only valid inside BT-ET pairs
