@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2016  Otfried Cheong
+    Copyright (c) 1993-2019 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -36,6 +36,7 @@
 #include "ipegeo.h"
 #include "iperesources.h"
 
+#include <list>
 #include <cairo.h>
 
 //------------------------------------------------------------------------
@@ -44,14 +45,14 @@ struct FT_FaceRec_;
 
 namespace ipe {
 
-  class PdfResources;
+  class PdfResourceBase;
   class PdfDict;
 
   enum class FontType { Type1, Truetype, CIDType0, CIDType2, Unsupported };
 
   class Face {
   public:
-    Face(const PdfDict *d, const PdfResources *resources) noexcept;
+    Face(const PdfDict *d, const PdfResourceBase *resources) noexcept;
     ~Face() noexcept;
     inline bool matches(const PdfDict *d) const noexcept {
       return d == iFontDict; }
@@ -70,7 +71,7 @@ namespace ipe {
 
   private:
     const PdfDict *iFontDict;
-    const PdfResources *iResources;
+    const PdfResourceBase *iResources;
     FontType iType;
     String iName;
     cairo_font_face_t *iCairoFont { nullptr };
@@ -82,21 +83,16 @@ namespace ipe {
 
   class Fonts {
   public:
-    static Fonts *New(const PdfResources *resources);
-    ~Fonts();
+    Fonts(const PdfResourceBase *resources);
+
     Face *getFace(const PdfDict *d);
     static cairo_font_face_t *screenFont();
     static String freetypeVersion();
-    const PdfResources *resources() const noexcept { return iResources; }
+    const PdfResourceBase *resources() const noexcept { return iResources; }
 
   private:
-    Fonts(const PdfResources *resources);
-
-  private:
-    const PdfResources *iResources;
-
-    typedef std::list<Face *> FaceSeq;
-    FaceSeq iFaces;
+    const PdfResourceBase *iResources;
+    std::list<std::unique_ptr<Face>> iFaces;
   };
 
 } // namespace
