@@ -1,10 +1,10 @@
 // --------------------------------------------------------------------
-// Making thumbnails of Ipe pages
+// Making thumbnails of Ipe pages and PDF pages
 // --------------------------------------------------------------------
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2016  Otfried Cheong
+    Copyright (c) 1993-2019 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -43,10 +43,7 @@
 #include <cairo-ps.h>
 #endif
 
-#ifdef WIN32
-#include <windows.h>
-#include <gdiplus.h>
-#endif
+#include <cstring>
 
 using namespace ipe;
 
@@ -61,12 +58,7 @@ Thumbnail::Thumbnail(const Document *doc, int width)
   Rect paper = iLayout->paper();
   iHeight = int(iWidth * paper.height() / paper.width());
   iZoom = iWidth / paper.width();
-  iFonts = Fonts::New(doc->resources());
-}
-
-Thumbnail::~Thumbnail()
-{
-  delete iFonts;
+  iFonts = std::make_unique<Fonts>(doc->resources());
 }
 
 Buffer Thumbnail::render(const Page *page, int view)
@@ -75,7 +67,7 @@ Buffer Thumbnail::render(const Page *page, int view)
   memset(buffer.data(), 0xff, iWidth * iHeight * 4);
 
   cairo_surface_t* surface =
-    cairo_image_surface_create_for_data((uchar *) buffer.data(),
+    cairo_image_surface_create_for_data((uint8_t *) buffer.data(),
 					CAIRO_FORMAT_ARGB32,
 					iWidth, iHeight, iWidth * 4);
   cairo_t *cc = cairo_create(surface);
@@ -83,7 +75,7 @@ Buffer Thumbnail::render(const Page *page, int view)
   Vector offset = iLayout->iOrigin - iLayout->paper().topLeft();
   cairo_translate(cc, offset.x, offset.y);
 
-  CairoPainter painter(iDoc->cascade(), iFonts, cc, iZoom, true);
+  CairoPainter painter(iDoc->cascade(), iFonts.get(), cc, iZoom, true);
   painter.pushMatrix();
   for (int i = 0; i < page->count(); ++i) {
     if (page->objectVisible(view, i))
@@ -98,59 +90,13 @@ Buffer Thumbnail::render(const Page *page, int view)
   return buffer;
 }
 
-#ifdef WIN32
-static bool getEncoderClsid(const WCHAR* format, CLSID &clsid)
+static cairo_status_t stream_writer(void *closure, const unsigned char *data, unsigned int length)
 {
-  uint num = 0;
-  uint size = 0;
-  Gdiplus::GetImageEncodersSize(&num, &size);
-  if (size == 0)
-    return false;
-
-  Gdiplus::ImageCodecInfo* pImageCodecInfo =
-    (Gdiplus::ImageCodecInfo *) malloc(size);
-  if (pImageCodecInfo == NULL)
-    return false;
-
-  Gdiplus::GetImageEncoders(num, size, pImageCodecInfo);
-  for (uint j = 0; j < num; ++j) {
-    if (!wcscmp(pImageCodecInfo[j].MimeType, format)) {
-      clsid = pImageCodecInfo[j].Clsid;
-      free(pImageCodecInfo);
-      return true;
-    }
-  }
-  free(pImageCodecInfo);
-  return false;
+  if (::fwrite(data, 1, length, (std::FILE *) closure) != length)
+    return CAIRO_STATUS_WRITE_ERROR;
+  return CAIRO_STATUS_SUCCESS;
 }
 
-void Thumbnail::savePNG(cairo_surface_t* surface, const char *dst)
-{
-  int w = cairo_image_surface_get_width(surface);
-  int h = cairo_image_surface_get_height(surface);
-  int stride = cairo_image_surface_get_stride(surface);
-  uchar *data = cairo_image_surface_get_data(surface);
-
-  Gdiplus::Bitmap *bitmap = new Gdiplus::Bitmap(w, h, stride,
-						PixelFormat32bppARGB,
-						data);
-
-  CLSID pngClsid;
-  getEncoderClsid(L"image/png", pngClsid);
-
-  std::vector<wchar_t> wname;
-  ipe::Platform::utf8ToWide(dst, wname);
-  bitmap->Save(&wname[0], &pngClsid, NULL);
-  delete bitmap;
-}
-
-#else
-void Thumbnail::savePNG(cairo_surface_t* surface, const char *dst)
-{
-  cairo_surface_write_to_png(surface, dst);
-}
-#endif
-
 bool Thumbnail::saveRender(TargetFormat fm, const char *dst,
 			   const Page *page, int view, double zoom,
 			   bool transparent, bool nocrop)
@@ -168,7 +114,10 @@ bool Thumbnail::saveRender(TargetFormat fm, const char *dst,
   }
 
   Buffer data;
-  cairo_surface_t* surface = 0;
+  cairo_surface_t* surface = nullptr;
+  std::FILE *file = Platform::fopen(dst, "wb");
+  if (!file)
+    return false;
 
   if (fm == EPNG) {
     if (wid * ht > 20000000)
@@ -178,21 +127,21 @@ bool Thumbnail::saveRender(TargetFormat fm, const char *dst,
       memset(data.data(), 0x00, wid * ht * 4);
     else
       memset(data.data(), 0xff, wid * ht * 4);
-    surface = cairo_image_surface_create_for_data((uchar *) data.data(),
+    surface = cairo_image_surface_create_for_data((uint8_t *) data.data(),
 						  CAIRO_FORMAT_ARGB32,
 						  wid, ht, wid * 4);
 #ifdef CAIRO_HAS_SVG_SURFACE
   } else if (fm == ESVG) {
-    surface = cairo_svg_surface_create(dst, wid, ht);
+    surface = cairo_svg_surface_create_for_stream(&stream_writer, (void *) file, wid, ht);
 #endif
 #ifdef CAIRO_HAS_PS_SURFACE
   } else if (fm == EPS) {
-    surface = cairo_ps_surface_create(dst, wid, ht);
+    surface = cairo_ps_surface_create_for_stream(&stream_writer, (void *) file, wid, ht);
     cairo_ps_surface_set_eps(surface, true);
 #endif
 #ifdef CAIRO_HAS_PDF_SURFACE
   } else if (fm == EPDF) {
-    surface = cairo_pdf_surface_create(dst, wid, ht);
+    surface = cairo_pdf_surface_create_for_stream(&stream_writer, (void *) file, wid, ht);
 #endif
   }
 
@@ -200,7 +149,7 @@ bool Thumbnail::saveRender(TargetFormat fm, const char *dst,
   cairo_scale(cc, zoom, -zoom);
   cairo_translate(cc, -bbox.topLeft().x, -bbox.topLeft().y);
 
-  CairoPainter painter(iDoc->cascade(), iFonts, cc, zoom, true);
+  CairoPainter painter(iDoc->cascade(), iFonts.get(), cc, zoom, true);
   // painter.Transform(IpeLinear(zoom, 0, 0, -zoom));
   // painter.Translate(-bbox.TopLeft());
   painter.pushMatrix();
@@ -222,11 +171,61 @@ bool Thumbnail::saveRender(TargetFormat fm, const char *dst,
   cairo_show_page(cc);
 
   if (fm == EPNG)
-    savePNG(surface, dst);
+    cairo_surface_write_to_png_stream(surface, &stream_writer, (void *) file);
 
   cairo_destroy(cc);
   cairo_surface_destroy(surface);
+
+  ::fclose(file);
   return true;
 }
 
 // --------------------------------------------------------------------
+
+PdfThumbnail::PdfThumbnail(const PdfFile *pdf, int width)
+{
+  iPdf = pdf;
+  iCascade = std::make_unique<Cascade>();
+  iCascade->insert(0, StyleSheet::standard());
+
+  iResources = std::make_unique<PdfFileResources>(iPdf);
+  iFonts = std::make_unique<Fonts>(iResources.get());
+
+  iWidth = width;
+  iHeight = 0;
+  for (int i = 0; i < iPdf->countPages(); ++i) {
+    Rect paper = iPdf->mediaBox(iPdf->page(i));
+    iHeight = std::max(iHeight, (int) (paper.height() * iWidth / paper.width()));
+  }
+}
+
+Buffer PdfThumbnail::render(const PdfDict *page)
+{
+  Rect paper = iPdf->mediaBox(page);
+  double zoom = iWidth / paper.width();
+
+  const PdfObj *stream0 = page->get("Contents", iPdf);
+  const PdfDict *stream = stream0 ? stream0->dict() : nullptr;
+
+  Buffer buffer(iWidth * iHeight * 4);
+  cairo_surface_t* surface =
+    cairo_image_surface_create_for_data((uint8_t *) buffer.data(), CAIRO_FORMAT_ARGB32,
+					iWidth, iHeight, iWidth * 4);
+  cairo_t *cc = cairo_create(surface);
+  cairo_set_source_rgb(cc, 1.0, 1.0, 1.0);
+  cairo_paint(cc);
+
+  cairo_translate(cc, 0.0, iHeight);
+  cairo_scale(cc, zoom, -zoom);
+
+  if (stream) {
+    CairoPainter painter(iCascade.get(), iFonts.get(), cc, 1.0, false);
+    painter.executeStream(stream, page);
+  }
+  cairo_surface_flush(surface);
+  cairo_destroy(cc);
+  cairo_surface_destroy(surface);
+  return buffer;
+}
+
+// --------------------------------------------------------------------
