@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2016  Otfried Cheong
+    Copyright (c) 1993-2019 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -44,7 +44,6 @@ namespace ipe {
     enum TargetFormat { ESVG, EPNG, EPS, EPDF };
 
     Thumbnail(const Document *doc, int width);
-    ~Thumbnail();
 
     int width() const { return iWidth; }
     int height() const { return iHeight; }
@@ -52,14 +51,28 @@ namespace ipe {
     bool saveRender(TargetFormat fm, const char *dst,
 		    const Page *page, int view, double zoom,
 		    bool transparent, bool nocrop);
-    static void savePNG(cairo_surface_t* surface, const char *dst);
   private:
     const Document *iDoc;
     int iWidth;
     int iHeight;
     double iZoom;
     const Layout *iLayout;
-    Fonts *iFonts;
+    std::unique_ptr<Fonts> iFonts;
+  };
+
+  class PdfThumbnail {
+  public:
+    PdfThumbnail(const PdfFile *pdf, int width);
+    int width() const { return iWidth; }
+    int height() const { return iHeight; }
+    Buffer render(const PdfDict *page);
+  private:
+    const PdfFile *iPdf;
+    int iWidth;
+    int iHeight;
+    std::unique_ptr<Cascade> iCascade; // dummy stylesheet
+    std::unique_ptr<PdfFileResources> iResources;
+    std::unique_ptr<Fonts> iFonts;
   };
 
 } // namespace
