@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2016  Otfried Cheong
+    Copyright (c) 1993-2019 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -50,8 +50,9 @@ using namespace ipe;
 // --------------------------------------------------------------------
 
 CanvasObserver::~CanvasObserver() { /* nothing */ }
-void CanvasObserver::canvasObserverWheelMoved(double degrees,
-					      bool precise) { /* nothing */ }
+void CanvasObserver::canvasObserverWheelMoved(double xDegrees,
+					      double yDegrees,
+					      int kind) { /* nothing */ }
 void CanvasObserver::canvasObserverMouseAction(int button) { /* nothing */ }
 void CanvasObserver::canvasObserverPositionChanged() { /* nothing */ }
 void CanvasObserver::canvasObserverToolChanged(bool hasTool) { /* nothing */ }
@@ -67,9 +68,9 @@ CanvasBase::CanvasBase()
 {
   iObserver = nullptr;
   iTool = nullptr;
-  iPage = 0;
-  iCascade = 0;
-  iSurface = 0;
+  iPage = nullptr;
+  iCascade = nullptr;
+  iSurface = nullptr;
   iPan = Vector::ZERO;
   iZoom = 1.0;
   iDimmed = false;
@@ -78,7 +79,6 @@ CanvasBase::CanvasBase()
   iBWidth = 0;
   iBHeight = 0;
   iRepaintObjects = false;
-  iFonts = nullptr;
   iResources = nullptr;
   iAutoSnap = false;
   iFifiVisible = false;
@@ -115,7 +115,6 @@ CanvasBase::~CanvasBase()
 {
   if (iSurface)
     cairo_surface_destroy(iSurface);
-  delete iFonts;
   delete iTool;
   ipeDebug("CanvasBase::~CanvasBase");
 }
@@ -125,9 +124,9 @@ CanvasBase::~CanvasBase()
 //! set information about Latex fonts (from ipe::Document)
 void CanvasBase::setResources(const PdfResources *resources)
 {
-  delete iFonts;
+  iFonts.reset();
   iResources = resources;
-  iFonts = Fonts::New(resources);
+  iFonts = std::make_unique<Fonts>(resources);
 }
 
 // --------------------------------------------------------------------
@@ -357,7 +356,7 @@ void CanvasBase::drawObjects(cairo_t *cc)
     cairo_clip(cc);
   }
 
-  CairoPainter painter(iCascade, iFonts, cc, iZoom, iStyle.pretty);
+  CairoPainter painter(iCascade, iFonts.get(), cc, iZoom, iStyle.pretty);
   painter.setDimmed(iDimmed);
   // painter.Transform(CanvasTfm());
   painter.pushMatrix();
@@ -521,7 +520,7 @@ void CanvasBase::setTool(Tool *tool)
 void CanvasBase::finishTool()
 {
   delete iTool;
-  iTool = 0;
+  iTool = nullptr;
   iDimmed = false;
   iAutoSnap = false;
   update();
@@ -636,7 +635,7 @@ void CanvasBase::computeFifi(double x, double y)
   if (iSnap.iSnap & ~mask) {
     iFifiMode = iSnap.snap(iMousePos, iPage, iView,
 			   iSnap.iSnapDistance / iZoom,
-			   iTool, (iAutoSnap ? &iAutoOrigin : 0));
+			   iTool, (iAutoSnap ? &iAutoOrigin : nullptr));
     if (iFifiMode == Snap::ESnapNone)
       snapToPaperAndFrame();
 
@@ -677,8 +676,7 @@ void CanvasBase::refreshSurface()
   if (iRepaintObjects) {
     iRepaintObjects = false;
     if (!iSurface)
-      iSurface = cairo_image_surface_create(CAIRO_FORMAT_RGB24,
-					    iBWidth, iBHeight);
+      iSurface = cairo_image_surface_create(CAIRO_FORMAT_RGB24, iBWidth, iBHeight);
     cairo_t *cc = cairo_create(iSurface);
     // background
     cairo_set_source_rgb(cc, 0.4, 0.4, 0.4);
