@@ -152,7 +152,7 @@ namespace ipe {
     int canvasHeight() const { return iHeight; }
 
     virtual void setCursor(TCursor cursor, double w = 1.0,
-			   Color *color = 0) = 0;
+			   Color *color = nullptr) = 0;
 
     static int selectPageOrView(Document *doc, int page = -1,
 				int startIndex = 0,
@@ -209,7 +209,7 @@ namespace ipe {
     bool iSelectionVisible;
 
     const PdfResources *iResources;
-    Fonts *iFonts;
+    std::unique_ptr<Fonts> iFonts;
   };
 
 } // namespace
