@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2016  Otfried Cheong
+    Copyright (c) 1993-2019 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -253,6 +253,7 @@ void Canvas::tabletEvent(QTabletEvent *ev)
 
   switch (ev->type()) {
   case QEvent::TabletPress:
+    ipeDebug("TabletPress: %d %d", ev->button(), ev->pointerType());
     iGlobalPos = globalPos;
     computeFifi(hiPos.x(), hiPos.y());
     if (ev->pointerType() == QTabletEvent::Eraser) {
@@ -260,9 +261,9 @@ void Canvas::tabletEvent(QTabletEvent *ev)
 	iObserver->canvasObserverMouseAction(Qt::XButton1 |
 					     iAdditionalModifiers);
     } else if (iTool)
-      iTool->mouseButton(Qt::LeftButton | iAdditionalModifiers, true);
+      iTool->mouseButton(ev->button() | iAdditionalModifiers, true);
     else if (iObserver)
-      iObserver->canvasObserverMouseAction(Qt::LeftButton |
+      iObserver->canvasObserverMouseAction(ev->button() |
 					   iAdditionalModifiers);
     break;
   case QEvent::TabletMove:
@@ -289,8 +290,10 @@ void Canvas::tabletEvent(QTabletEvent *ev)
 
 void Canvas::wheelEvent(QWheelEvent *ev)
 {
+  QPoint p = ev->angleDelta();
+  int kind = (ev->modifiers() & Qt::ControlModifier) ? 2 : 0;
   if (iObserver)
-    iObserver->canvasObserverWheelMoved(ev->delta() / 120.0, true);
+    iObserver->canvasObserverWheelMoved(p.x() / 8.0, p.y() / 8.0, kind);
   ev->accept();
 }
 
