@@ -76,11 +76,11 @@ namespace ipe {
 #if QT_VERSION < 0x050000
     Canvas(QWidget* parent, Qt::WFlags f=0);
 #else
-    Canvas(QWidget* parent, Qt::WindowFlags f=0);
+    Canvas(QWidget* parent, Qt::WindowFlags f=nullptr);
 #endif
 
     virtual void setCursor(TCursor cursor, double w = 1.0,
-			   Color *color = 0);
+			   Color *color = nullptr);
 
   protected:
     virtual void invalidate();
