@@ -43,11 +43,11 @@ namespace ipe {
   public:
     static void init(HINSTANCE hInstance);
     static UINT getDpiForWindow(HWND hwnd);
-    Canvas(HWND parent);
+    Canvas(HWND parent, HINSTANCE hInstance = nullptr);
 
     HWND windowId() const { return hwnd; }
 
-    static HBITMAP createBitmap(unsigned char *p, int w, int h);
+    static HBITMAP createBitmap(uint8_t *p, int w, int h);
 
   private:
     void button(int button, bool down, int mod, Vector v);
