@@ -58,13 +58,19 @@ NSSize thumbnail_size;
 
 - (NSString *) title:(int) index
 {
-  if (self.page >= 0)
-    return [NSString stringWithFormat:@"View %d", index + 1];
-  String t = self.doc->page(index)->title();
-  if (t.empty())
-    return [NSString stringWithFormat:@"Page %d", index + 1];
-  else
-    return [NSString stringWithFormat:@"%s: %d", t.z(), index + 1];
+  if (self.page >= 0) {
+    String t = self.doc->page(self.page)->viewName(index);
+    if (t.empty())
+      return [NSString stringWithFormat:@"View %d", index + 1];
+    else
+      return [NSString stringWithFormat:@"%d: %@", index + 1, C2N(t.z())];
+  } else {
+    String t = self.doc->page(index)->title();
+    if (t.empty())
+      return [NSString stringWithFormat:@"Page %d", index + 1];
+    else
+      return [NSString stringWithFormat:@"%d: %@", index + 1, C2N(t.z())];
+  }
 }
 
 - (NSImage *) image:(int) index
@@ -75,32 +81,31 @@ NSSize thumbnail_size;
       [self.images addObject:[NSNull null]];
   }
   if (self.images[index] == [NSNull null]) {
-    Buffer b;
-    if (self.page >= 0)
-      b = self.thumb->render(self.doc->page(self.page), index);
-    else {
-      Page *p = self.doc->page(index);
-      b = self.thumb->render(p, p->countViews() - 1);
-    }
+    Buffer b = [self renderImage:index];
     self.images[index] = [self createImage:b];
   }
   return (NSImage *) self.images[index];
 }
 
+- (Buffer) renderImage:(int) index
+{
+  if (self.page >= 0)
+    return self.thumb->render(self.doc->page(self.page), index);
+  else {
+    Page *p = self.doc->page(index);
+    return self.thumb->render(p, p->countViews() - 1);
+  }
+}
+
 - (NSImage *) createImage:(Buffer) b
 {
-  // thumbnail has twice the resolution in case we have a retina
-  // display
-  int w = self.thumb->width();
-  int h = self.thumb->height();
-  NSSize size;
-  size.width = w / 2.0;
-  size.height = h / 2.0;
-  return [NSImage imageWithSize:size
+  int w = 2 * self.tnSize.width;
+  int h = 2 * self.tnSize.height;
+  return [NSImage imageWithSize:self.tnSize
 			flipped:YES
 		 drawingHandler:^(NSRect rect) {
       cairo_surface_t *image =
-	cairo_image_surface_create_for_data((uchar *) b.data(),
+	cairo_image_surface_create_for_data((uint8_t *) b.data(),
 					    CAIRO_FORMAT_ARGB32,
 					    w, h, 4 * w);
       CGContextRef ctx = [[NSGraphicsContext currentContext] CGContext];
@@ -206,14 +211,10 @@ NSSize thumbnail_size;
 
 // --------------------------------------------------------------------
 
-int CanvasBase::selectPageOrView(Document *doc, int page, int startIndex,
-				 int pageWidth, int width, int height)
+int showPageSelectDialog(int width, int height, const char * title,
+			 IpeSelectorProvider *provider, int startIndex)
 {
-  // for retina displays, create bitmap with twice the resolution
-  Thumbnail thumbs(doc, 2 * pageWidth);
-
-  thumbnail_size.width = thumbs.width() / 2.0;
-  thumbnail_size.height = thumbs.height() / 2.0;
+  thumbnail_size = provider.tnSize;
 
   NSPanel *panel = [[NSPanel alloc]
 		     initWithContentRect:NSMakeRect(200., 100., width, height)
@@ -221,17 +222,11 @@ int CanvasBase::selectPageOrView(Document *doc, int page, int startIndex,
 		     NSResizableWindowMask|NSClosableWindowMask
 				 backing:NSBackingStoreBuffered
 				   defer:YES];
-
-  panel.title = C2N((page >= 0) ? "Ipe: Select view" : "Ipe: Select page");
+  panel.title = C2N(title);
 
   IpeSelectorDelegate *delegate = [IpeSelectorDelegate new];
   panel.delegate = delegate;
 
-  IpeSelectorProvider *provider = [IpeSelectorProvider new];
-  provider.doc = doc;
-  provider.thumb = &thumbs;
-  provider.page = page;
-
   NSMutableArray *elements =
     [NSMutableArray arrayWithCapacity:[provider count]];
 
@@ -260,3 +255,26 @@ int CanvasBase::selectPageOrView(Document *doc, int page, int startIndex,
 }
 
 // --------------------------------------------------------------------
+
+int CanvasBase::selectPageOrView(Document *doc, int page, int startIndex,
+				 int pageWidth, int width, int height)
+{
+  // for retina displays, create bitmap with twice the resolution
+  Thumbnail thumbs(doc, 2 * pageWidth);
+
+  NSSize tnSize;
+  tnSize.width = thumbs.width() / 2.0;
+  tnSize.height = thumbs.height() / 2.0;
+
+  const char *title = (page >= 0) ? "Ipe: Select view" : "Ipe: Select page";
+
+  IpeSelectorProvider *provider = [IpeSelectorProvider new];
+  provider.doc = doc;
+  provider.thumb = &thumbs;
+  provider.page = page;
+  provider.tnSize = tnSize;
+
+  return showPageSelectDialog(width, height, title, provider, startIndex);
+}
+
+// --------------------------------------------------------------------
