@@ -53,11 +53,9 @@ using namespace ipe;
   is computed automatically).
 */
 
-PageSelector::PageSelector(Document *doc, int page, int startIndex,
-			   int itemWidth, QWidget *parent)
+PageSelector::PageSelector(QWidget *parent)
   : QListWidget(parent)
 {
-  iDoc = doc;
   setViewMode(QListView::IconMode);
   setSelectionMode(QAbstractItemView::SingleSelection);
   setResizeMode(QListView::Adjust);
@@ -67,10 +65,44 @@ PageSelector::PageSelector(Document *doc, int page, int startIndex,
   setSpacing(10);
   setMovement(QListView::Static);
 
-  Thumbnail r(iDoc, itemWidth);
-  setGridSize(QSize(itemWidth, r.height() + 50));
-  setIconSize(QSize(itemWidth, r.height()));
+  connect(this, SIGNAL(itemActivated(QListWidgetItem *)),
+	  SLOT(pageSelected(QListWidgetItem *)));
+}
 
+void PageSelector::pageSelected(QListWidgetItem *item)
+{
+  emit selectionMade();
+}
+
+void PageSelector::fill(std::vector<QPixmap> &icons, std::vector<String> &labels)
+{
+  int maxWidth = 0;
+  int maxHeight = 0;
+  for (const auto & icon : icons) {
+    if (icon.width() > maxWidth)
+      maxWidth = icon.width();
+    if (icon.height() > maxHeight)
+      maxHeight = icon.height();
+  }
+  setGridSize(QSize(maxWidth + 10, maxHeight + 50));
+  setIconSize(QSize(maxWidth, maxHeight));
+
+  for (size_t i = 0; i < icons.size(); ++i) {
+    QString s = QString::fromUtf8(labels[i].z());
+    QListWidgetItem *item = new QListWidgetItem(QIcon(icons[i]), s);
+    item->setFlags(Qt::ItemIsSelectable|Qt::ItemIsEnabled);
+    item->setToolTip(s);
+    addItem(item);
+  }
+}
+
+// --------------------------------------------------------------------
+
+static void fillWithPages(PageSelector *sel, Document *doc, int page, int itemWidth)
+{
+  Thumbnail r(doc, itemWidth);
+  std::vector<QPixmap> icons;
+  std::vector<String> labels;
   if (page >= 0) {
     Page *p = doc->page(page);
     for (int i = 0; i < p->countViews(); ++i) {
@@ -78,13 +110,14 @@ PageSelector::PageSelector(Document *doc, int page, int startIndex,
       QImage bits((const uchar *) b.data(), itemWidth, r.height(),
 		  QImage::Format_RGB32);
       // need to copy bits since buffer b is temporary
-      QIcon icon(QPixmap::fromImage(bits.copy()));
-      QString s;
-      s.sprintf("View %d", i+1);
-      QListWidgetItem *item = new QListWidgetItem(icon, s);
-      item->setFlags(Qt::ItemIsSelectable|Qt::ItemIsEnabled);
-      item->setToolTip(s);
-      addItem(item);
+      icons.push_back(QPixmap::fromImage(bits.copy()));
+      String s;
+      StringStream ss(s);
+      if (!p->viewName(i).empty())
+	ss << i+1 << ": " << p->viewName(i);
+      else
+	ss << "View " << i+1;
+      labels.push_back(s);
     }
   } else {
     for (int i = 0; i < doc->countPages(); ++i) {
@@ -93,30 +126,18 @@ PageSelector::PageSelector(Document *doc, int page, int startIndex,
       QImage bits((const uchar *) b.data(), itemWidth, r.height(),
 		  QImage::Format_RGB32);
       // need to copy bits since buffer b is temporary
-      QIcon icon(QPixmap::fromImage(bits.copy()));
+      icons.push_back(QPixmap::fromImage(bits.copy()));
 
-      QString s;
-      QString t = QString::fromUtf8(p->title().z());
-      if (t != "") {
-	s.sprintf("%d: ", i+1);
-	s += t;
-      } else {
-	s.sprintf("Page %d", i+1);
-      }
-      QListWidgetItem *item = new QListWidgetItem(icon, s);
-      item->setFlags(Qt::ItemIsSelectable|Qt::ItemIsEnabled);
-      item->setToolTip(s);
-      addItem(item);
+      String s;
+      StringStream ss(s);
+      if (!p->title().empty())
+	ss << i+1 << ": " << p->title();
+      else
+	ss << "Page " << i+1;
+      labels.push_back(s);
     }
   }
-  setCurrentRow(startIndex);
-  connect(this, SIGNAL(itemActivated(QListWidgetItem *)),
-	  SLOT(pageSelected(QListWidgetItem *)));
-}
-
-void PageSelector::pageSelected(QListWidgetItem *item)
-{
-  emit selectionMade();
+  sel->fill(icons, labels);
 }
 
 // --------------------------------------------------------------------
@@ -137,13 +158,16 @@ int CanvasBase::selectPageOrView(Document *doc, int page, int startIndex,
 		    "Ipe: Select page");
 
   QLayout *lo = new QVBoxLayout;
-  PageSelector *p = new PageSelector(doc, page, startIndex, pageWidth);
+  PageSelector *p = new PageSelector(d);
+  fillWithPages(p, doc, page, pageWidth);
+
   lo->addWidget(p);
   d->setLayout(lo);
 
   QWidget::connect(p, SIGNAL(selectionMade()), d, SLOT(accept()));
 
   d->resize(width, height);
+  p->setCurrentRow(startIndex);
   int result = d->exec();
   int sel = p->selectedIndex();
   delete d;
