@@ -119,6 +119,22 @@ void StreamParser::writeAttributes(const XmlAttributes &attr)
   fprintf(iOut, ">\n");
 }
 
+static void writeBits(FILE *out, Buffer bits)
+{
+  const char *data = bits.data();
+  const char *fin = data + bits.size();
+  int col = 0;
+  while (data != fin) {
+    fprintf(out, "%02x", (*data++ & 0xff));
+    if (++col == 36) {
+      fputc('\n', out);
+      col = 0;
+    }
+  }
+  if (col > 0)
+    fputc('\n', out);
+}
+
 bool StreamParser::parseBitmap()
 {
   XmlAttributes attr;
@@ -126,22 +142,19 @@ bool StreamParser::parseBitmap()
     return false;
   String objNumStr;
   if (attr.slash() && attr.has("pdfObject", objNumStr)) {
-    int objNum = Lex(objNumStr).getInt();
-    Buffer bits = image(objNum);
-    writeAttributes(attr);
-    // write bits
-    const char *data = bits.data();
-    const char *fin = data + bits.size();
-    int col = 0;
-    while (data != fin) {
-      fprintf(iOut, "%02x", (*data++ & 0xff));
-      if (++col == 36) {
-	fputc('\n', iOut);
-	col = 0;
-      }
+    Lex lex(objNumStr);
+    Buffer bits = image(lex.getInt());
+    Buffer alpha;
+    lex.skipWhitespace();
+    if (!lex.eos()) {
+      alpha = image(lex.getInt());
+      fprintf(iOut, " alphaLength=\"%d\"", alpha.size());
     }
-    if (col > 0)
-      fputc('\n', iOut);
+    fprintf(iOut, " length=\"%d\"", bits.size());
+    writeAttributes(attr);
+    writeBits(iOut, bits);
+    if (alpha.size() > 0)
+      writeBits(iOut, alpha);
     fprintf(iOut, "</bitmap>\n");
   } else {
     // just write out attributes
@@ -316,20 +329,31 @@ static bool extractPdf(DataSource &source, std::FILE *out)
 
   // try ancient format version first (early previews of Ipe 6.0)
   const PdfObj *obj = loader.catalog()->get("Ipe", &loader);
-  if (!obj) {
-    obj = loader.object(1);
-    if (!obj || !obj->dict()) {
-      fprintf(stderr, "Input file does not contain an Ipe XML stream.\n");
-      return false;
-    }
 
-    const PdfObj *type = obj->dict()->get("Type", 0);
-    if (!type || !type->name() || type->name()->value() != "Ipe") {
-      fprintf(stderr, "Input file does not contain an Ipe XML stream.\n");
-      return false;
+  // otherwise try most recent format (>= 7.2.11)
+  if (!obj) {
+    obj = loader.catalog()->get("PieceInfo", &loader);
+    if (obj && obj->dict()) {
+      obj = obj->dict()->get("Ipe", &loader);
+      if (obj && obj->dict())
+	obj = obj->dict()->get("Private", &loader);
     }
   }
 
+  if (!obj)
+    obj = loader.object(1);
+
+  if (!obj || !obj->dict()) {
+    fprintf(stderr, "Input file does not contain an Ipe XML stream.\n");
+    return false;
+  }
+
+  const PdfObj *type = obj->dict()->get("Type", nullptr);
+  if (!type || !type->name() || type->name()->value() != "Ipe") {
+    fprintf(stderr, "Input file does not contain an Ipe XML stream.\n");
+    return false;
+  }
+
   Buffer buffer = obj->dict()->stream();
   BufferSource xml(buffer);
 
