@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2016  Otfried Cheong
+    Copyright (c) 1993-2019 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -59,7 +59,8 @@ bool KGonIpelet::run(int, IpeletData *data, IpeletHelper *helper)
     return false;
   }
 
-  String str;
+
+  String str = helper->getParameter("n");   // get default value from Lua wrapper
   if (!helper->getString("Enter k (number of corners)", str))
     return false;
   int k = Lex(str).getInt();
