@@ -29,8 +29,10 @@
 */
 
 #include "ipebase.h"
+
 #include <cmath>
 #include <cstdlib>
+#include <cstring>
 
 using namespace ipe;
 
@@ -94,10 +96,10 @@ String::String() noexcept
 //! Construct a string by making copy of \a str.
 String::String(const char *str) noexcept
 {
-  int len = strlen(str);
-  if (!len)
+  if (!str || !str[0]) {
     iImp = emptyString();
-  else {
+  } else {
+    int len = strlen(str);
     iImp = new Imp;
     iImp->iRefCount = 1;
     iImp->iSize = len;
@@ -125,7 +127,7 @@ String String::withData(char *data, int len) noexcept
 //! Construct string by making copy of \a str with given \a len.
 String::String(const char *str, int len) noexcept
 {
-  if (!len)
+  if (!str || !len)
     iImp = emptyString();
   else {
     iImp = new Imp;
@@ -360,7 +362,7 @@ String String::operator+(const String &rhs) const noexcept
   return s;
 }
 
-static const uchar bytesFromUTF8[256] = {
+static const uint8_t bytesFromUTF8[256] = {
   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
   0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0, 0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
@@ -371,7 +373,7 @@ static const uchar bytesFromUTF8[256] = {
   2,2,2,2,2,2,2,2,2,2,2,2,2,2,2,2, 3,3,3,3,3,3,3,3,4,4,4,4,5,5,5,5
 };
 
-static const uchar firstByteMark[7] = {
+static const uint8_t firstByteMark[7] = {
   0x00, 0xC0, 0xE0, 0xF0, 0xF8, 0xFC, 0
 };
 
@@ -380,7 +382,7 @@ static const uchar firstByteMark[7] = {
   This returns 0xfffd if there is any problem in parsing UTF-8. */
 int String::unicode(int &index) const noexcept
 {
-  int wch = uchar(iImp->iData[index++]);
+  int wch = uint8_t(iImp->iData[index++]);
   if ((wch & 0xc0) == 0x80) {
     // not on first byte of a UTF-8 sequence
     while (index < iImp->iSize && (iImp->iData[index] & 0xc0) == 0x80)
@@ -463,8 +465,8 @@ String Lex::token()
 String Lex::nextToken()
 {
   skipWhitespace();
-  uint mark = iPos;
-  while (!(eos() || uchar(iString[iPos]) <= ' '))
+  int mark = iPos;
+  while (!(eos() || uint8_t(iString[iPos]) <= ' '))
     ++iPos;
   return iString.substr(mark, iPos - mark);
 }
@@ -473,7 +475,7 @@ String Lex::nextToken()
 int Lex::getInt()
 {
   String str = nextToken();
-  return std::atoi(str.z());
+  return std::strtol(str.z(), nullptr, 10);
 }
 
 inline int hexDigit(int ch)
@@ -504,7 +506,7 @@ int Lex::getHexByte()
 unsigned long int Lex::getHexNumber()
 {
   String str = nextToken();
-  return std::strtoul(str.z(), 0, 16);
+  return std::strtoul(str.z(), nullptr, 16);
 }
 
 //! Extract Fixed token (skipping whitespace).
@@ -514,11 +516,11 @@ Fixed Lex::getFixed()
   int i = 0;
   while (i < str.size() && str[i] != '.')
     ++i;
-  int integral = std::strtol(str.substr(0, i).z(), 0, 10);
+  int integral = std::strtol(str.substr(0, i).z(), nullptr, 10);
   int fractional = 0;
   if (i < str.size()) {
     String s = (str.substr(i+1) + "000").substr(0, 3);
-    fractional = std::strtol(s.z(), 0, 10);
+    fractional = std::strtol(s.z(), nullptr, 10);
   }
   return Fixed::fromInternal(integral * 1000 + fractional);
 }
@@ -532,7 +534,7 @@ double Lex::getDouble()
 //! Skip over whitespace.
 void Lex::skipWhitespace()
 {
-  while (!eos() && uchar(iString[iPos]) <= ' ')
+  while (!eos() && uint8_t(iString[iPos]) <= ' ')
     ++iPos;
 }
 
@@ -542,92 +544,20 @@ void Lex::skipWhitespace()
   \ingroup base
   \brief A memory buffer.
 
-  Implements sharing semantics using reference counting.
+  Can be be copied in constant time, the actual data is shared.
 */
 
 //! Create buffer of specified size.
-Buffer::Buffer(int size) noexcept
+Buffer::Buffer(int size)
 {
-  iImp = new Imp;
-  iImp->iRefCount = 1;
-  iImp->iSize = size;
-  iImp->iData = new (std::nothrow) char[size];
-  if (iImp->iData == nullptr) {
-    fprintf(stderr, "Failure to allocate %d bytes of memory, exiting.\n", size);
-    exit(9);
-  }
-}
-
-//! Copy constructor. Constant time, buffer is shared.
-Buffer::Buffer(const Buffer &rhs) noexcept
-{
-  iImp = rhs.iImp;
-  if (iImp)
-    iImp->iRefCount++;
-}
-
-//! Move constructor. Constant time.
-Buffer::Buffer(Buffer &&rhs) noexcept
-{
-  iImp = rhs.iImp;
-  rhs.iImp = nullptr;
+  iData = std::shared_ptr<std::vector<char>>(new std::vector<char>(size));
 }
 
 //! Create buffer by copying the data.
-Buffer::Buffer(const char *data, int size) noexcept
+Buffer::Buffer(const char *data, int size)
 {
-  iImp = new Imp;
-  iImp->iRefCount = 1;
-  iImp->iSize = size;
-  iImp->iData = new char[size];
-  memcpy(iImp->iData, data, size);
-}
-
-//! Destructor.
-Buffer::~Buffer() noexcept
-{
-  if (iImp) {
-    iImp->iRefCount--;
-    if (iImp->iRefCount == 0) {
-      delete [] iImp->iData;
-      delete iImp;
-    }
-  }
-}
-
-//! Assignment operator (constant-time).
-Buffer &Buffer::operator=(const Buffer &rhs) noexcept
-{
-  if (this != &rhs) {
-    if (iImp) {
-      iImp->iRefCount--;
-      if (iImp->iRefCount == 0) {
-	delete [] iImp->iData;
-	delete iImp;
-      }
-    }
-    iImp = rhs.iImp;
-    if (iImp)
-      iImp->iRefCount++;
-  }
-  return *this;
-}
-
-//! Assignment move operator (constant-time).
-Buffer &Buffer::operator=(Buffer &&rhs) noexcept
-{
-  if (this != &rhs) {
-    if (iImp) {
-      iImp->iRefCount--;
-      if (iImp->iRefCount == 0) {
-	delete [] iImp->iData;
-	delete iImp;
-      }
-    }
-    iImp = rhs.iImp;
-    rhs.iImp = nullptr;
-  }
-  return *this;
+  iData = std::shared_ptr<std::vector<char>>(new std::vector<char>(size));
+  std::memcpy(&(*iData)[0], data, size);
 }
 
 // --------------------------------------------------------------------
@@ -892,7 +822,7 @@ int BufferSource::getChar()
 {
   if (iPos >= iBuffer.size())
     return EOF;
-  return uchar(iBuffer[iPos++]);
+  return uint8_t(iBuffer[iPos++]);
 }
 
 // --------------------------------------------------------------------
