@@ -106,45 +106,45 @@ String readLine(DataSource &source)
 }
 
 //! Determine format of file in \a source.
-Document::TFormat Document::fileFormat(DataSource &source)
+FileFormat Document::fileFormat(DataSource &source)
 {
   String s1 = readLine(source);
   String s2 = readLine(source);
   if (s1.substr(0, 5) == "<?xml" ||
       s1.substr(0, 9) == "<!DOCTYPE" ||
       s1.substr(0, 4) == "<ipe")
-    return EXml;
+    return FileFormat::Xml;
   if (s1.substr(0, 4) == "%PDF")
-    return EPdf;  // let's assume it contains an Ipe stream
+    return FileFormat::Pdf;  // let's assume it contains an Ipe stream
   if (s1.substr(0, 4) == "%!PS") {
     if (s2.substr(0, 11) != "%%Creator: ")
-      return EUnknown;
+      return FileFormat::Unknown;
     if (s2.substr(11, 6) == "Ipelib" || s2.substr(11, 4) == "xpdf")
-      return EEps;
+      return FileFormat::Eps;
     if (s2.substr(11, 3) == "Ipe")
-      return EIpe5;
-    return EUnknown;
+      return FileFormat::Ipe5;
+    return FileFormat::Unknown;
   }
   if (s1.substr(0, 5) == "%\\Ipe" || s1.substr(0, 6) == "%\\MIPE")
-    return EIpe5;
-  return EUnknown;
+    return FileFormat::Ipe5;
+  return FileFormat::Unknown;
 }
 
 //! Determine format of file from filename \a fn.
-Document::TFormat Document::formatFromFilename(String fn)
+FileFormat Document::formatFromFilename(String fn)
 {
   if (fn.size() < 5)
-    return Document::EUnknown;
+    return FileFormat::Unknown;
   // fn = fn.toLower();
   String s = fn.right(4);
   if (s == ".xml" || s == ".ipe")
-    return Document::EXml;
+    return FileFormat::Xml;
   else if (s == ".pdf")
-    return Document::EPdf;
+    return FileFormat::Pdf;
   else if (s == ".eps")
-    return Document::EEps;
+    return FileFormat::Eps;
   else
-    return Document::EUnknown;
+    return FileFormat::Unknown;
 }
 
 // --------------------------------------------------------------------
@@ -286,7 +286,7 @@ Document *doParse(Document *self, ImlParser &parser, int &reason)
   int res = parser.parseDocument(*self);
   if (res) {
     delete self;
-    self = 0;
+    self = nullptr;
     if (res == ImlParser::ESyntaxError)
       reason = parser.parsePosition();
     else
@@ -307,7 +307,7 @@ Document *doParsePs(DataSource &source, int &reason)
   PsSource psSource(source);
   reason = Document::EFileOpenError; // could not find Xml stream
   if (!psSource.skipToXml())
-    return 0;
+    return nullptr;
 
   Document *self = new Document;
   if (psSource.deflated()) {
@@ -326,14 +326,21 @@ Document *doParsePdf(DataSource &source, int &reason)
   PdfFile loader;
   reason = Document::ENotAnIpeFile;
   if (!loader.parse(source))  // could not parse PDF container
-    return 0;
-  const PdfObj *obj = loader.object(1);
+    return nullptr;
+  const PdfObj *obj = loader.catalog()->get("PieceInfo", &loader);
+  if (obj && obj->dict()) {
+    obj = obj->dict()->get("Ipe", &loader);
+    if (obj && obj->dict())
+      obj = obj->dict()->get("Private", &loader);
+  }
+  if (!obj)
+    obj = loader.object(1);
   // was the object really created by Ipe?
   if (!obj || !obj->dict())
-    return 0;
-  const PdfObj *type = obj->dict()->get("Type", 0);
+    return nullptr;
+  const PdfObj *type = obj->dict()->get("Type", nullptr);
   if (!type || !type->name() || type->name()->value() != "Ipe")
-    return 0;
+    return nullptr;
 
   Buffer buffer = obj->dict()->stream();
   BufferSource xml(buffer);
@@ -355,20 +362,20 @@ Document *doParsePdf(DataSource &source, int &reason)
   file (stream) offset where parsing failed.  If \a reason is
   negative, it is an error code, see Document::LoadErrors.
 */
-Document *Document::load(DataSource &source, TFormat format,
+Document *Document::load(DataSource &source, FileFormat format,
 			 int &reason)
 {
-  if (format == EXml)
+  if (format == FileFormat::Xml)
     return doParseXml(source, reason);
 
-  if (format == EPdf)
+  if (format == FileFormat::Pdf)
     return doParsePdf(source, reason);
 
-  if (format == EEps)
+  if (format == FileFormat::Eps)
     return doParsePs(source, reason);
 
-  reason = (format == EIpe5) ? EVersionTooOld : ENotAnIpeFile;
-  return 0;
+  reason = (format == FileFormat::Ipe5) ? EVersionTooOld : ENotAnIpeFile;
+  return nullptr;
 }
 
 Document *Document::load(const char *fname, int &reason)
@@ -376,9 +383,9 @@ Document *Document::load(const char *fname, int &reason)
   reason = EFileOpenError;
   std::FILE *fd = Platform::fopen(fname, "rb");
   if (!fd)
-    return 0;
+    return nullptr;
   FileSource source(fd);
-  TFormat format = fileFormat(source);
+  FileFormat format = fileFormat(source);
   std::rewind(fd);
   Document *self = load(source, format, reason);
   std::fclose(fd);
@@ -412,7 +419,7 @@ Document *Document::loadWithErrorReport(const char *fname)
     fprintf(stderr, "Error parsing the document at position %d\n.", reason);
     break;
   }
-  return 0;
+  return nullptr;
 }
 
 // --------------------------------------------------------------------
@@ -420,9 +427,9 @@ Document *Document::loadWithErrorReport(const char *fname)
 //! Save in a stream.
 /*! Returns true if sucessful.
 */
-bool Document::save(TellStream &stream, TFormat format, uint flags) const
+bool Document::save(TellStream &stream, FileFormat format, uint32_t flags) const
 {
-  if (format == EXml) {
+  if (format == FileFormat::Xml) {
     stream << "<?xml version=\"1.0\"?>\n";
     stream << "<!DOCTYPE ipe SYSTEM \"ipe.dtd\">\n";
     saveAsXml(stream);
@@ -430,15 +437,15 @@ bool Document::save(TellStream &stream, TFormat format, uint flags) const
   }
 
   int compresslevel = 9;
-  if (flags & ENoZip)
+  if (flags & SaveFlag::NoZip)
     compresslevel = 0;
 
-  if (format == EPdf) {
-    PdfWriter writer(stream, this, iResources, (flags & EMarkedView),
-		     0, -1, compresslevel);
+  if (format == FileFormat::Pdf) {
+    PdfWriter writer(stream, this, iResources, flags, 0, -1, compresslevel);
     writer.createPages();
     writer.createBookmarks();
-    if (!(flags & EExport)) {
+    writer.createNamedDests();
+    if (!(flags & SaveFlag::Export)) {
       String xmlData;
       StringStream stream(xmlData);
       if (compresslevel > 0) {
@@ -459,7 +466,7 @@ bool Document::save(TellStream &stream, TFormat format, uint flags) const
   return false;
 }
 
-bool Document::save(const char *fname, TFormat format, uint flags) const
+bool Document::save(const char *fname, FileFormat format, uint32_t flags) const
 {
   std::FILE *fd = Platform::fopen(fname, "wb");
   if (!fd)
@@ -471,14 +478,14 @@ bool Document::save(const char *fname, TFormat format, uint flags) const
 }
 
 //! Export a single view to PDF
-bool Document::exportView(const char *fname, TFormat format, uint flags,
+bool Document::exportView(const char *fname, FileFormat format, uint32_t flags,
 			  int pno, int vno) const
 {
-  if (format != EPdf)
+  if (format != FileFormat::Pdf)
     return false;
 
   int compresslevel = 9;
-  if (flags & ENoZip)
+  if (flags & SaveFlag::NoZip)
     compresslevel = 0;
 
   std::FILE *fd = Platform::fopen(fname, "wb");
@@ -486,8 +493,7 @@ bool Document::exportView(const char *fname, TFormat format, uint flags,
     return false;
   FileStream stream(fd);
 
-  PdfWriter writer(stream, this, iResources, (flags & EMarkedView),
-		   pno, pno, compresslevel);
+  PdfWriter writer(stream, this, iResources, flags, pno, pno, compresslevel);
   writer.createPageView(pno, vno);
   writer.createTrailer();
   std::fclose(fd);
@@ -495,18 +501,17 @@ bool Document::exportView(const char *fname, TFormat format, uint flags,
 }
 
 //! Export a range of pages to PDF.
-bool Document::exportPages(const char *fname, uint flags,
+bool Document::exportPages(const char *fname, uint32_t flags,
 			   int fromPage, int toPage) const
 {
   int compresslevel = 9;
-  if (flags & ENoZip)
+  if (flags & SaveFlag::NoZip)
     compresslevel = 0;
   std::FILE *fd = Platform::fopen(fname, "wb");
   if (!fd)
     return false;
   FileStream stream(fd);
-  PdfWriter writer(stream, this, iResources, (flags & EMarkedView),
-		   fromPage, toPage, compresslevel);
+  PdfWriter writer(stream, this, iResources, flags, fromPage, toPage, compresslevel);
   writer.createPages();
   writer.createTrailer();
   std::fclose(fd);
@@ -685,7 +690,7 @@ bool Document::addStyleSheet(DataSource &source)
 #endif
 
 //! Return total number of views in all pages.
-size_t Document::countTotalViews() const
+int Document::countTotalViews() const
 {
   int views = 0;
   for (int i = 0; i < countPages(); ++i) {
@@ -708,7 +713,7 @@ int Document::findPage(String s) const
       return -1;
     return no - 1;
   }
-  for (size_t i = 0; i < countPages(); ++i) {
+  for (int i = 0; i < countPages(); ++i) {
     if (s == page(i)->section(0))
       return i;
   }
@@ -749,7 +754,7 @@ Page *Document::remove(int no)
 // --------------------------------------------------------------------
 
 //! Run PdfLatex or Xelatex
-int Document::runLatex(String &texLog)
+int Document::runLatex(String docname, String &texLog)
 {
   texLog = "";
   Latex converter(cascade(), iProperties.iTexEngine);
@@ -798,7 +803,7 @@ int Document::runLatex(String &texLog)
   if (err < 0)
     return ErrWritingSource;
 
-  int result = Platform::runLatex(latexDir, iProperties.iTexEngine);
+  int result = Platform::runLatex(latexDir, iProperties.iTexEngine, docname);
 
   if (result != 0 && result != 1)
     return ErrRunLatex;
@@ -837,10 +842,10 @@ int Document::runLatex(String &texLog)
 
 //! Run Pdflatex (suitable for console applications)
 /*! Success/error is reported on stderr. */
-int Document::runLatex()
+int Document::runLatex(String docname)
 {
   String logFile;
-  switch (runLatex(logFile)) {
+  switch (runLatex(docname, logFile)) {
   case ErrNoText:
     fprintf(stderr, "No text objects in document, no need to run Pdflatex.\n");
     return 0;
