@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2016  Otfried Cheong
+    Copyright (c) 1993-2019 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -56,7 +56,7 @@ Object *ObjectFactory::createObject(String name, const XmlAttributes &attr,
   else if (name == "use")
     return new Reference(attr, data);
   else
-    return 0;
+    return nullptr;
 }
 
 //! Create an Image with a given bitmap.
