@@ -264,14 +264,19 @@ void Rect::clipTo(const Rect &cbox)
 {
   if (isEmpty() || cbox.isEmpty())
     return;
-  if (iMin.x < cbox.iMin.x)
-    iMin.x = cbox.iMin.x;
-  if (iMin.y < cbox.iMin.y)
-    iMin.y = cbox.iMin.y;
-  if (iMax.x > cbox.iMax.x)
-    iMax.x = cbox.iMax.x;
-  if (iMax.y > cbox.iMax.y)
-    iMax.y = cbox.iMax.y;
+  if (!intersects(cbox)) {  // make box empty
+    iMin = Vector(1, 0);
+    iMax = Vector(-1, 0);
+  } else {
+    if (iMin.x < cbox.iMin.x)
+      iMin.x = cbox.iMin.x;
+    if (iMin.y < cbox.iMin.y)
+      iMin.y = cbox.iMin.y;
+    if (iMax.x > cbox.iMax.x)
+      iMax.x = cbox.iMax.x;
+    if (iMax.y > cbox.iMax.y)
+      iMax.y = cbox.iMax.y;
+  }
 }
 
 /*! Returns false if the distance between the box and v is smaller
@@ -1105,7 +1110,7 @@ void Arc::intersect(const Segment &s, std::vector<Vector> &result) const
   std::vector<Vector> pt;
   intersect(s.line(), pt);
   Vector dir = s.iQ - s.iP;
-  for (uint i = 0; i < pt.size(); ++i) {
+  for (int i = 0; i < size(pt); ++i) {
     // check whether it's on the segment
     Vector v = pt[i];
     if (dot(v - s.iP, dir) >= 0 && dot(v - s.iQ, dir) <= 0)
