@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2016  Otfried Cheong
+    Copyright (c) 1993-2019 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -51,6 +51,7 @@ Group::Group() : Object()
   iImp = new Imp;
   iImp->iRefCount = 1;
   iImp->iPinned = ENoPin;
+  iDecoration = Attribute::NORMAL();
 }
 
 //! Create empty group with these attributes (objects can be added later).
@@ -67,6 +68,8 @@ Group::Group(const XmlAttributes &attr)
       iClip = clip;
   }
   iUrl = attr["url"];
+  iDecoration = attr.has("decoration", str) ?
+    Attribute(true, str) : Attribute::NORMAL();
 }
 
 //! Copy constructor. Constant time --- components are not copied!
@@ -77,6 +80,7 @@ Group::Group(const Group &rhs)
   iImp->iRefCount++;
   iClip = rhs.iClip;
   iUrl = rhs.iUrl;
+  iDecoration = rhs.iDecoration;
 }
 
 //! Destructor.
@@ -86,7 +90,7 @@ Group::~Group()
     for (List::iterator it = iImp->iObjects.begin();
 	 it != iImp->iObjects.end(); ++it) {
       delete *it;
-      *it = 0;
+      *it = nullptr;
     }
     delete iImp;
   } else
@@ -105,6 +109,7 @@ Group &Group::operator=(const Group &rhs)
     iImp->iRefCount++;
     iClip = rhs.iClip;
     iUrl = rhs.iUrl;
+    iDecoration = rhs.iDecoration;
     Object::operator=(rhs);
   }
   return *this;
@@ -171,13 +176,111 @@ void Group::saveAsXml(Stream &stream, String layer) const
     stream.putXmlString(iUrl);
     stream << "\"";
   }
+  if (!iDecoration.isNormal())
+    stream << " decoration=\"" << iDecoration.string() << "\"";
   stream << ">\n";
   saveComponentsAsXml(stream);
   stream << "</group>\n";
 }
 
+// --------------------------------------------------------------------
+
+class DecorationPainter : public Painter {
+public:
+  DecorationPainter(Painter &painter, const Vector &center,
+		    double dx, double dy);
+protected:
+  virtual void doPush();
+  virtual void doPop();
+  virtual void doNewPath();
+  virtual void doMoveTo(const Vector &v);
+  virtual void doLineTo(const Vector &v);
+  virtual void doCurveTo(const Vector &v1, const Vector &v2,
+			 const Vector &v3);
+  virtual void doClosePath();
+  virtual void doDrawPath(TPathMode mode);
+  Vector adapt(const Vector &v);
+private:
+  Painter &iPainter;
+  Vector iCenter;
+  double iX, iY;
+};
+
+DecorationPainter::DecorationPainter(Painter &painter, const Vector &center,
+				     double dx, double dy) :
+  Painter(painter.cascade()), iPainter(painter), iCenter(center),
+  iX(dx), iY(dy)
+{
+  // nothing
+}
+
+Vector DecorationPainter::adapt(const Vector &v)
+{
+  Vector r;
+  r.x = (v.x < iCenter.x) ? v.x - iX : v.x + iX;
+  r.y = (v.y < iCenter.y) ? v.y - iY : v.y + iY;
+  return r;
+}
+
+void DecorationPainter::doPush()
+{
+  iPainter.push();
+}
+
+void DecorationPainter::doPop()
+{
+  iPainter.pop();
+}
+
+void DecorationPainter::doNewPath()
+{
+  iPainter.setState(iState.back());
+  iPainter.newPath();
+}
+
+void DecorationPainter::doMoveTo(const Vector &v)
+{
+  iPainter.moveTo(adapt(v));
+}
+
+void DecorationPainter::doLineTo(const Vector &v)
+{
+  iPainter.lineTo(adapt(v));
+}
+
+void DecorationPainter::doCurveTo(const Vector &v1, const Vector &v2,
+				  const Vector &v3)
+{
+  iPainter.curveTo(adapt(v1), adapt(v2), adapt(v3));
+}
+
+void DecorationPainter::doClosePath()
+{
+  iPainter.closePath();
+}
+
+void DecorationPainter::doDrawPath(TPathMode mode)
+{
+  iPainter.drawPath(mode);
+}
+
+// --------------------------------------------------------------------
+
 void Group::draw(Painter &painter) const
 {
+  if (!iDecoration.isNormal()) {
+    painter.pushMatrix();
+    auto m = painter.matrix();
+    painter.untransform(ETransformationsTranslations);
+    Rect r;
+    addToBBox(r, m, false);
+    double dx = 0.5 * (r.width() - 200.0);
+    double dy = 0.5 * (r.height() - 100.0);
+    DecorationPainter dp(painter, r.center(), dx, dy);
+    dp.translate(r.center() - Vector(200.0, 150.0));
+    dp.drawSymbol(iDecoration);
+    painter.popMatrix();
+  }
   painter.pushMatrix();
   painter.transform(matrix());
   painter.untransform(transformations());
@@ -303,11 +406,24 @@ void Group::detach()
 
 }
 
+Attribute Group::getAttribute(Property prop) const noexcept
+{
+  if (prop == EPropDecoration)
+    return iDecoration;
+  else
+    return Object::getAttribute(prop);
+}
+
 //! Set attribute on all children.
 bool Group::setAttribute(Property prop, Attribute value)
 {
   if (prop == EPropPinned || prop == EPropTransformations)
     return Object::setAttribute(prop, value);
+  if (prop == EPropDecoration) {
+    auto old = iDecoration;
+    iDecoration = value;
+    return (old != iDecoration);
+  }
   // all others are handled by elements themselves
   detach();
   bool result = false;
