@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2016  Otfried Cheong
+    Copyright (c) 1993-2019 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -46,6 +46,7 @@ Image::Image(const Rect &rect, Bitmap bitmap)
 {
   iRect = rect;
   iBitmap = bitmap;
+  iOpacity = Attribute::OPAQUE();
   assert(!iBitmap.isNull());
 }
 
@@ -66,6 +67,12 @@ Image::Image(const XmlAttributes &attr, Bitmap bitmap)
 
 void Image::init(const XmlAttributes &attr)
 {
+  String str;
+  if (attr.has("opacity", str))
+    iOpacity = Attribute(true, str);
+  else
+    iOpacity = Attribute::OPAQUE();
+
   // parse rect
   Lex st(attr["rect"]);
   Vector v;
@@ -104,6 +111,8 @@ void Image::saveAsXml(Stream &stream, String layer) const
   stream << "<image";
   saveAttributesAsXml(stream, layer);
   stream << " rect=\"" << rect() << "\"";
+  if (iOpacity != Attribute::OPAQUE())
+    stream << " opacity=\"" << iOpacity.string() << "\"";
   stream << " bitmap=\"" << iBitmap.objNum() << "\"";
   stream << "/>\n";
 }
@@ -118,6 +127,7 @@ void Image::draw(Painter &painter) const
   painter.untransform(transformations());
   painter.transform(m);
   painter.push();
+  painter.setOpacity(iOpacity);
   painter.drawBitmap(iBitmap);
   painter.pop();
   painter.popMatrix();
@@ -176,4 +186,35 @@ void Image::snapCtl(const Vector &mouse, const Matrix &m,
   (m1 * iRect.topLeft()).snap(mouse, pos, bound);
 }
 
+//! Set opacity of the object.
+void Image::setOpacity(Attribute opaq)
+{
+  iOpacity = opaq;
+}
+
+bool Image::setAttribute(Property prop, Attribute value)
+{
+  switch (prop) {
+  case EPropOpacity:
+    if (value != opacity()) {
+      setOpacity(value);
+      return true;
+    }
+    break;
+  default:
+    return Object::setAttribute(prop, value);
+  }
+  return false;
+}
+
+Attribute Image::getAttribute(Property prop) const noexcept
+{
+  switch (prop) {
+  case EPropOpacity:
+    return opacity();
+  default:
+    return Object::getAttribute(prop);
+  }
+}
+
 // --------------------------------------------------------------------
