@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2016  Otfried Cheong
+    Copyright (c) 1993-2019 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -176,9 +176,13 @@ bool ImlParser::parseBitmap()
     return false;
   String objNumStr;
   if (att.slash() && att.has("pdfObject", objNumStr)) {
-    int objNum = Lex(objNumStr).getInt();
-    // ipeDebug("ParseBitmap: objNum = %d", objNum);
-    Bitmap bitmap(att, pdfStream(objNum));
+    Lex lex(objNumStr);
+    Buffer data = pdfStream(lex.getInt());
+    Buffer alpha;
+    lex.skipWhitespace();
+    if (!lex.eos())
+      alpha = pdfStream(lex.getInt());
+    Bitmap bitmap(att, data, alpha);
     iBitmaps.push_back(bitmap);
   } else {
     String bits;
@@ -220,9 +224,9 @@ bool ImlParser::parsePage(Page &page)
   if (tag == "notes") {
     XmlAttributes att;
     if (!parseAttributes(att))
-      return ESyntaxError;
+      return false;
     if (!parsePCDATA("notes", str))
-      return ESyntaxError;
+      return false;
     page.setNotes(str);
 
     tag = parseToTag();
@@ -270,6 +274,9 @@ bool ImlParser::parsePage(Page &page)
     if (att["marked"] == "yes")
       page.setMarkedView(page.countViews() - 1, true);
 
+    if (att.has("name", str))
+      page.setViewName(page.countViews() - 1, str);
+
     tag = parseToTag();
   }
 
@@ -292,6 +299,8 @@ bool ImlParser::parsePage(Page &page)
     if (tag == "/page") {
       return true;
     }
+    if (tag.empty())
+      return false;
     Object *obj = parseObject(tag, &page, &currentLayer);
     if (!obj)
       return false;
@@ -305,28 +314,28 @@ Page *ImlParser::parsePageSelection()
 {
   String tag = parseToTag();
   if (tag != "ipepage")
-    return 0;
+    return nullptr;
   XmlAttributes attr;
   if (!parseAttributes(attr))
-    return 0;
+    return nullptr;
   tag = parseToTag();
 
   while (tag == "bitmap") {
     if (!parseBitmap())
-      return 0;
+      return nullptr;
     tag = parseToTag();
   }
 
   if (tag != "page")
-    return 0;
+    return nullptr;
 
   std::unique_ptr<Page> page(new Page);
   if (!parsePage(*page))
-    return 0;
+    return nullptr;
 
   tag = parseToTag();
   if (tag != "/ipepage")
-    return 0;
+    return nullptr;
   return page.release();
 }
 
@@ -341,7 +350,7 @@ bool ImlParser::parseSelection(PageObjectSeq &seq)
     return false;
   XmlAttributes attr;
   if (!parseAttributes(attr))
-    return 0;
+    return nullptr;
   tag = parseToTag();
 
   while (tag == "bitmap") {
@@ -368,11 +377,11 @@ Object *ImlParser::parseObject(String tag, Page *page,
 			       int *currentLayer)
 {
   if (tag[0] == '/')
-    return 0;
+    return nullptr;
 
   XmlAttributes attr;
   if (!parseAttributes(attr))
-    return 0;
+    return nullptr;
 
   String layer;
   if (page && currentLayer && attr.has("layer", layer)) {
@@ -393,14 +402,14 @@ Object *ImlParser::parseObject(String tag, Page *page,
       }
       Object *obj = parseObject(tag);
       if (!obj)
-	return 0;
+	return nullptr;
       group.push_back(obj);
     }
   }
 
   String pcdata;
   if (!attr.slash() && !parsePCDATA(tag, pcdata))
-    return 0;
+    return nullptr;
   String bitmapId;
   if (tag == "image" && attr.has("bitmap", bitmapId)) {
     int objNum = Lex(bitmapId).getInt();
@@ -459,7 +468,7 @@ bool ImlParser::parseStyle(StyleSheet &sheet)
       else if (att["transformations"] == "translations")
 	symbol.iTransformations = ETransformationsTranslations;
       if (att["xform"] == "yes") {
-	uint flags = Reference::flagsFromName(name);
+	uint32_t flags = Reference::flagsFromName(name);
 	if ((flags & (Reference::EHasStroke|
 		      Reference::EHasFill|
 		      Reference::EHasPen|
@@ -468,6 +477,13 @@ bool ImlParser::parseStyle(StyleSheet &sheet)
 	  symbol.iTransformations = ETransformationsTranslations;
 	}
       }
+      Lex snaplex(att["snap"]);
+      while (!snaplex.eos()) {
+	Vector pos;
+	snaplex >> pos.x >> pos.y;
+	snaplex.skipWhitespace();
+	symbol.iSnap.push_back(pos);
+      }
       sheet.addSymbol(Attribute(true, name), symbol);
       if (parseToTag() != "/symbol")
 	return false;
@@ -616,7 +632,7 @@ bool ImlParser::parseStyle(StyleSheet &sheet)
       }
       if (s.iStops.front().offset < 0.0 || s.iStops.front().offset > 1.0)
 	return false;
-      for (uint i = 1; i < s.iStops.size(); ++i) {
+      for (int i = 1; i < size(s.iStops); ++i) {
 	if (s.iStops[i].offset < s.iStops[i-1].offset)
 	  return false;
       }
@@ -703,16 +719,16 @@ StyleSheet *ImlParser::parseStyleSheet()
   if (tag == "?xml") {
     XmlAttributes attr;
     if (!parseAttributes(attr, true))
-      return 0;
+      return nullptr;
     tag = parseToTag();
   }
   if (tag != "ipestyle")
-    return 0;
+    return nullptr;
   StyleSheet *sheet = new StyleSheet();
   if (parseStyle(*sheet))
     return sheet;
   delete sheet;
-  return 0;
+  return nullptr;
 }
 
 // --------------------------------------------------------------------
