@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2016  Otfried Cheong
+    Copyright (c) 1993-2019 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -181,7 +181,7 @@ void Latex::addPageNumber(int pno, int vno, int npages, int nviews)
 */
 int Latex::createLatexSource(Stream &stream, String preamble)
 {
-  bool ancient = (getenv("IPEANCIENTPDFTEX") != 0);
+  bool ancient = (getenv("IPEANCIENTPDFTEX") != nullptr);
   int count = 0;
   stream << "\\nonstopmode\n";
   if (!iXetex) {
@@ -197,12 +197,13 @@ int Latex::createLatexSource(Stream &stream, String preamble)
 	     << "\\RequirePackage{luatex85}\\fi\n";
   }
   stream << "\\documentclass{article}\n"
-	 << "\\usepackage{xcolor}\n"
 	 << "\\newdimen\\ipefs\n"
 	 << "\\newcounter{ipePage}\\newcounter{ipeView}\n"
 	 << "\\newcounter{ipePages}\\newcounter{ipeViews}\n"
 	 << "\\newcommand{\\PageTitle}[1]{#1}\n"
 	 << "\\newcommand{\\ipesymbol}[4]{$\\bullet$}\n";
+  stream << "\\def\\ipedefinecolors#1{\\ipecolorpreamble{#1}\\let\\ipecolorpreamble\\relax}\n"
+	 << "\\def\\ipecolorpreamble#1{\\usepackage[#1]{xcolor}\n";
   AttributeSeq colors;
   iCascade->allNames(EColor, colors);
   for (AttributeSeq::const_iterator it = colors.begin();
@@ -218,18 +219,23 @@ int Latex::createLatexSource(Stream &stream, String preamble)
 	     << value.iRed << "," << value.iGreen << ","
 	     << value.iBlue << "}\n";
   }
+  stream << "}\n";
   if (iXetex) {
     stream << "\\def\\ipesetcolor#1#2#3{\\special{pdf:bc [#1 #2 #3]}}\n"
 	   << "\\def\\iperesetcolor{\\special{pdf:ec}}\n";
   } else if (!ancient) {
-    stream << "\\def\\ipesetcolor#1#2#3{\\pdfcolorstack0 push{#1 #2 #3 rg #1 #2 #3 RG}}\n"
-	   << "\\def\\iperesetcolor{\\pdfcolorstack0 pop}\n";
+    stream << "\\makeatletter\n"
+	   << "\\def\\ipesetcolor#1#2#3{\\def\\current@color{#1 #2 #3 rg #1 #2 #3 RG}"
+	   << "\\pdfcolorstack\\@pdfcolorstack push{\\current@color}}\n"
+	   << "\\def\\iperesetcolor{\\pdfcolorstack\\@pdfcolorstack pop}\n"
+	   << "\\makeatother\n";
   } else {
     stream << "\\def\\ipesetcolor#1#2#3{\\color[rgb]{#1,#2,#3}}\n"
 	   << "\\def\\iperesetcolor{}\n";
   }
   stream << iCascade->findPreamble() << "\n"
 	 << preamble << "\n"
+	 << "\\ipedefinecolors{}\n"
 	 << "\\pagestyle{empty}\n"
 	 << "\\newcount\\bigpoint\\dimen0=0.01bp\\bigpoint=\\dimen0\n"
 	 << "\\begin{document}\n"
