@@ -148,6 +148,8 @@ void Page::saveAsXml(Stream &stream) const
       stream << " effect=\"" << effect(i).string() << "\"";
     if (markedView(i))
       stream << " marked=\"yes\"";
+    if (!viewName(i).empty())
+      stream << " name=\"" << viewName(i) << "\"";
     stream << "/>\n";
   }
   int currentLayer = -1;
@@ -295,10 +297,10 @@ void Page::renameLayer(String oldName, String newName)
 }
 
 //! Returns a precise bounding box for the artwork on the page.
-/*! This is meant to be used as the bounding box in PDF and EPS
-  output.  It is computed by rendering all objects on the page that
-  are visible in at least one view, plus all objects in a layer named
-  "BBOX" (even if that is not visible), using a BBoxPainter. */
+/*! This is meant to be used as the bounding box in PDF output.  It is
+  computed by rendering all objects on the page that are visible in at
+  least one view, plus all objects in a layer named "BBOX" (even if
+  that is not visible), using a BBoxPainter. */
 Rect Page::pageBBox(const Cascade *sheet) const
 {
   // make table with all layers to be rendered
@@ -399,7 +401,7 @@ void Page::setMarkedView(int index, bool marked)
   iViews[index].iMarked = marked;
 }
 
-size_t Page::countMarkedViews() const
+int Page::countMarkedViews() const
 {
   int count = 0;
   for (int i = 0; i < countViews(); ++i) {
@@ -412,7 +414,6 @@ size_t Page::countMarkedViews() const
 //! Return index of view with given number or name.
 /*! Input numbers are one-based.
   Returns -1 if no such view exists.
-  View names are not yet implemented!!!
 */
 int Page::findView(String s) const
 {
@@ -424,12 +425,10 @@ int Page::findView(String s) const
       return -1;
     return no - 1;
   }
-  /*
-  for (size_t i = 0; i < countViews(); ++i) {
-    if (s == <name of view i>)
+  for (int i = 0; i < countViews(); ++i) {
+    if (s == viewName(i))
       return i;
-  }
-  */
+    }
   return -1;
 }
 
@@ -437,7 +436,7 @@ int Page::findView(String s) const
 
 Page::SObject::SObject()
 {
-  iObject = 0;
+  iObject = nullptr;
   iLayer = 0;
   iSelect = ENotSelected;
 }
@@ -448,7 +447,7 @@ Page::SObject::SObject(const SObject &rhs)
   if (rhs.iObject)
     iObject = rhs.iObject->clone();
   else
-    iObject = 0;
+    iObject = nullptr;
 }
 
 Page::SObject &Page::SObject::operator=(const SObject &rhs)
@@ -460,7 +459,7 @@ Page::SObject &Page::SObject::operator=(const SObject &rhs)
     if (rhs.iObject)
       iObject = rhs.iObject->clone();
     else
-      iObject = 0;
+      iObject = nullptr;
     iBBox.clear(); // invalidate
   }
   return *this;
@@ -538,14 +537,15 @@ void Page::invalidateBBox(int i) const
 
 //! Return a bounding box for the object at index \a i.
 /*! This is a bounding box including the control points of the object.
-    If you need a tight bounding box, you'll need to use the Object
-    directly.
+  If you need a tight bounding box, you'll need to use the Object
+  directly.
 
-    The Page caches the box the first time it is computed.
+  The Page caches the box the first time it is computed.
 
-    Make sure you call Page::transform instead of Object::setMatrix,
-    as the latter would not invalidate the bounding box.
+  Make sure you call Page::transform instead of Object::setMatrix,
+  as the latter would not invalidate the bounding box.
 */
+
 Rect Page::bbox(int i) const
 {
   if (iObjects[i].iBBox.isEmpty())
@@ -588,7 +588,7 @@ void Page::snapBnd(int i, const Vector &mouse,
 
 //! Set attribute \a prop of object at index \a i to \a value.
 /*! This method automatically invalidates the bounding box if a
-    ETextSize property is actually changed. */
+  ETextSize property is actually changed. */
 bool Page::setAttribute(int i, Property prop, Attribute value)
 {
   bool changed = object(i)->setAttribute(prop, value);
@@ -654,7 +654,7 @@ void Page::setMarked(bool marked)
 const Text *Page::titleText() const
 {
   if (title().empty())
-    return 0;
+    return nullptr;
   return &iTitleObject;
 }
 
