@@ -91,6 +91,7 @@ Painter::Painter(const Cascade *style)
   state.iSymFill = Color(1000, 1000, 1000);
   state.iSymPen = Fixed(1);
   state.iOpacity = Fixed(1);
+  state.iStrokeOpacity = Fixed(1);
   state.iTiling = Attribute::NORMAL();
   state.iGradient = Attribute::NORMAL();
   iState.push_back(state);
@@ -370,6 +371,13 @@ void Painter::setOpacity(Attribute opaq)
   iState.back().iOpacity = iCascade->find(EOpacity, opaq).number();
 }
 
+//! Set stroke opacity.
+void Painter::setStrokeOpacity(Attribute opaq)
+{
+  assert(!iInPath);
+  iState.back().iStrokeOpacity = iCascade->find(EOpacity, opaq).number();
+}
+
 //! Set tiling pattern.
 /*! If \a tiling is not \c normal, resets the gradient pattern. */
 void Painter::setTiling(Attribute tiling)
