@@ -62,7 +62,7 @@ Path *Path::create(const XmlAttributes &attr, String data)
 {
   std::unique_ptr<Path> self(new Path(attr));
   if (!self->iShape.load(data))
-    return 0;
+    return nullptr;
   self->makeArrowData();
   return self.release();
 }
@@ -100,6 +100,11 @@ Path::Path(const XmlAttributes &attr)
   else
     iOpacity = Attribute::OPAQUE();
 
+  if (attr.has("stroke-opacity", str))
+    iStrokeOpacity = Attribute(true, str);
+  else
+    iStrokeOpacity = iOpacity;
+
   iGradient = Attribute::NORMAL();
   iTiling = Attribute::NORMAL();
 
@@ -159,6 +164,7 @@ void Path::init(const AllAttributes &attr, bool withArrows)
   iDashStyle = attr.iDashStyle;
   iPen = attr.iPen;
   iOpacity = attr.iOpacity;
+  iStrokeOpacity = attr.iStrokeOpacity;
   iTiling = attr.iTiling;
   if (iTiling.isNormal())
     iGradient = attr.iGradient;
@@ -290,6 +296,8 @@ void Path::saveAsXml(Stream &stream, String layer) const
   }
   if (iOpacity != Attribute::OPAQUE())
     stream << " opacity=\"" << iOpacity.string() << "\"";
+  if (iStrokeOpacity != iOpacity)
+    stream << " stroke-opacity=\"" << iStrokeOpacity.string() << "\"";
   if (filled && !iTiling.isNormal())
     stream << " tiling=\"" << iTiling.string() << "\"";
   if (filled && !iGradient.isNormal())
@@ -403,6 +411,7 @@ void Path::draw(Painter &painter) const
     painter.setGradient(iGradient);
   }
   painter.setOpacity(iOpacity);
+  painter.setStrokeOpacity(iStrokeOpacity);
   painter.pushMatrix();
   painter.transform(matrix());
   painter.untransform(transformations());
@@ -531,6 +540,12 @@ void Path::setOpacity(Attribute opaq)
   iOpacity = opaq;
 }
 
+//! Set stroke opacity of the object.
+void Path::setStrokeOpacity(Attribute opaq)
+{
+  iStrokeOpacity = opaq;
+}
+
 //! Set pen.
 void Path::setPen(Attribute pen)
 {
@@ -588,6 +603,7 @@ void Path::checkStyle(const Cascade *sheet, AttributeSeq &seq) const
   checkSymbol(ESymbol, iFArrowShape, sheet, seq);
   checkSymbol(ESymbol, iRArrowShape, sheet, seq);
   checkSymbol(EOpacity, iOpacity, sheet, seq);
+  checkSymbol(EOpacity, iStrokeOpacity, sheet, seq);
   if (!iTiling.isNormal())
     checkSymbol(ETiling, iTiling, sheet, seq);
   if (!iGradient.isNormal())
@@ -645,6 +661,12 @@ bool Path::setAttribute(Property prop, Attribute value)
       return true;
     }
     break;
+  case EPropStrokeOpacity:
+    if (value != strokeOpacity()) {
+      setStrokeOpacity(value);
+      return true;
+    }
+    break;
   case EPropFArrow:
     if (value.boolean() != iHasFArrow) {
       iHasFArrow = value.boolean();
@@ -723,6 +745,8 @@ Attribute Path::getAttribute(Property prop) const noexcept
     return dashStyle();
   case EPropOpacity:
     return opacity();
+  case EPropStrokeOpacity:
+    return strokeOpacity();
   case EPropTiling:
     return tiling();
   case EPropGradient:
