@@ -62,7 +62,7 @@ static char specialChars[256] = {
 
 inline int toInt(String &s)
 {
-  return std::strtol(s.z(), 0, 10);
+  return std::strtol(s.z(), nullptr, 10);
 }
 
 // --------------------------------------------------------------------
@@ -226,7 +226,7 @@ void PdfArray::write(Stream &stream, const PdfRenumber *renumber,
   for (int i = 0; i < count(); ++i) {
     stream << sep;
     sep = " ";
-    obj(i, 0)->write(stream, renumber);
+    obj(i, nullptr)->write(stream, renumber);
   }
   stream << "]";
 }
@@ -236,7 +236,7 @@ PdfArray::~PdfArray()
   for (std::vector<const PdfObj *>::iterator it = iObjects.begin();
        it != iObjects.end(); ++it) {
     delete *it;
-    *it = 0;
+    *it = nullptr;
   }
 }
 
@@ -319,7 +319,7 @@ PdfDict::~PdfDict()
   for (std::vector<Item>::iterator it = iItems.begin();
        it != iItems.end(); ++it) {
     delete it->iVal;
-    it->iVal = 0;
+    it->iVal = nullptr;
   }
 }
 
@@ -487,7 +487,7 @@ void PdfParser::getToken()
 	    getChar();
 	  }
 	  buf[i] = '\0';
-	  iTok.iString.append(char(std::strtol(buf, 0, 8)));
+	  iTok.iString.append(char(std::strtol(buf, nullptr, 8)));
 	} else {
 	  iTok.iString.append(char(iCh));
 	  getChar();
@@ -627,7 +627,7 @@ PdfDict *PdfParser::makeDict()
 	getChar();
       getChar(); // skip '\n'
       // now at beginning of stream
-      const PdfObj *len = dict->get("Length", 0);
+      const PdfObj *len = dict->get("Length", nullptr);
       if (len && len->ref()) {
 	ipeDebug("/Length entry of dictionary is a reference.");
 	return nullptr;
@@ -787,6 +787,7 @@ bool PdfFile::parseObjectStream(const PdfDict *d)
     PdfObj *obj = parser.getObject();
     if (!obj)
       return false;
+    // ipeDebug("Object: %s", obj->repr().z());
     iObjects[num] = std::unique_ptr<const PdfObj>(obj);
   }
   return true;
@@ -826,11 +827,11 @@ bool PdfFile::parse(DataSource &source)
 	  ipeDebug("Failed to get trailer");
 	  return false;
 	}
-	return true;
+	return readPageTree();
       } else if (t.iString == "xref") {
 	parser.skipXRef();
       } else if (t.iString == "startxref") {
-	return true; // this is the end
+	return readPageTree(); // this is the end
       } else {
 	ipeDebug("Weird token: %s", t.iString.z());
 	// don't know what's happening
@@ -874,20 +875,50 @@ const PdfDict *PdfFile::catalog() const noexcept
   return root->dict();
 }
 
-//! Return first page of the document.
-const PdfDict *PdfFile::page() const noexcept
+bool PdfFile::readPageTree(const PdfObj *ptn)
 {
-  const PdfObj *pages = catalog()->get("Pages", this);
-  assert(pages && pages->dict());
-  const PdfObj *kids = pages->dict()->get("Kids", this);
-  assert(kids);
-  if (!kids->array())
+  if (ptn == nullptr)
+    ptn = catalog()->get("Pages", this);
+  if (!ptn || !ptn->dict())
+    return false;
+  const PdfObj *kids = ptn->dict()->get("Kids", this);
+  if (!kids || !kids->array())
+    return false;
+  for (int i = 0; i < kids->array()->count(); ++i) {
+    const PdfObj *page = kids->array()->obj(i, this);
+    if (!page || !page->dict())
+      return false;
+    const PdfObj *type = page->dict()->get("Type", this);
+    if (!type || !type->name())
+      return false;
+    if (type->name()->value() == "Pages")
+      readPageTree(page);
+    else if (type->name()->value() == "Page")
+      iPages.push_back(page->dict());
+    else
+      return false;
+  }
+  return true;
+}
+
+//! Return a page of the document.
+const PdfDict *PdfFile::page(int pno) const noexcept
+{
+  if (pno < 0 || pno >= countPages())
     return nullptr;
-  const PdfObj *page = kids->array()->obj(0, this);
-  // this should be page 1
-  if (!page || !page->dict())
-    return nullptr;
-  return page->dict();
+  return iPages[pno];
+}
+
+//! Return mediabox of a page.
+Rect PdfFile::mediaBox(const PdfDict *pg) const
+{
+  Rect box;
+  std::vector<double> a;
+  if (pg && pg->getNumberArray("MediaBox", this, a) && a.size() == 4) {
+    box.addPoint(Vector(a[0], a[1]));
+    box.addPoint(Vector(a[2], a[3]));
+  }
+  return box;
 }
 
 // --------------------------------------------------------------------
