@@ -56,6 +56,7 @@ PdfPainter::PdfPainter(const Cascade *style, Stream &stream)
   state.iLineCap = style->lineCap();
   state.iLineJoin = style->lineJoin();
   state.iOpacity = Fixed(1);
+  state.iStrokeOpacity = Fixed(1);
   iStream << state.iLineCap - 1 << " J "
 	  << state.iLineJoin - 1 << " j\n";
   iActiveState.push_back(state);
@@ -147,7 +148,7 @@ void PdfPainter::drawAttributes()
     } else
       drawColor(iStream, s.iFill, "g", "rg");
   }
-  drawOpacity();
+  drawOpacity(true);
 }
 
 static const char *opacityName(Fixed alpha)
@@ -157,14 +158,18 @@ static const char *opacityName(Fixed alpha)
   return buf;
 }
 
-void PdfPainter::drawOpacity()
+void PdfPainter::drawOpacity(bool withStroke)
 {
   State &s = iState.back();
   State &sa = iActiveState.back();
   if (s.iOpacity != sa.iOpacity) {
     sa.iOpacity = s.iOpacity;
+    sa.iStrokeOpacity = s.iOpacity;
     iStream << opacityName(s.iOpacity) << " gs\n";
   }
+  if (withStroke && s.iStrokeOpacity != sa.iStrokeOpacity) {
+    iStream << opacityName(s.iStrokeOpacity) << "s gs\n";
+  }
 }
 
 // If gradient fill is set, then StrokeAndFill does NOT stroke!
@@ -175,7 +180,7 @@ void PdfPainter::doDrawPath(TPathMode mode)
   // if (!mode)
   // iStream << "n\n"; // no op path
 
-  const Gradient *g = 0;
+  const Gradient *g = nullptr;
   Attribute grad = iState.back().iGradient;
   if (!grad.isNormal())
     g = iCascade->findGradient(grad);
@@ -201,6 +206,7 @@ void PdfPainter::doDrawBitmap(Bitmap bitmap)
 {
   if (bitmap.objNum() < 0)
     return;
+  drawOpacity(false);
   iStream << matrix() << " cm /Image" << bitmap.objNum() << " Do\n";
 }
 
@@ -209,7 +215,7 @@ void PdfPainter::doDrawText(const Text *text)
   const Text::XForm *xf = text->getXForm();
   if (!xf)
     return;
-  drawOpacity();
+  drawOpacity(false);
   pushMatrix();
   transform(Matrix(xf->iStretch, 0, 0, xf->iStretch, 0, 0));
   translate(xf->iTranslation);
@@ -254,11 +260,10 @@ void PdfPainter::doDrawSymbol(Attribute symbol)
 */
 
 //! Create a PDF writer operating on this (open and empty) file.
-PdfWriter::PdfWriter(TellStream &stream, const Document *doc,
-		     const PdfResources *resources,
-		     bool markedView, int fromPage, int toPage,
+PdfWriter::PdfWriter(TellStream &stream, const Document *doc, const PdfResources *resources,
+		     uint32_t flags, int fromPage, int toPage,
 		     int compression)
-  : iStream(stream), iDoc(doc), iResources(resources), iMarkedView(markedView),
+  : iStream(stream), iDoc(doc), iResources(resources), iSaveFlags(flags),
     iFromPage(fromPage), iToPage(toPage)
 {
   iCompressLevel = compression;
@@ -267,6 +272,7 @@ PdfWriter::PdfWriter(TellStream &stream, const Document *doc,
   iExtGState = -1;
   iPatternNum = -1;
   iBookmarks = -1;
+  iDests = -1;
 
   if (iFromPage < 0 || iFromPage >= iDoc->countPages())
     iFromPage = 0;
@@ -300,11 +306,13 @@ PdfWriter::PdfWriter(TellStream &stream, const Document *doc,
   if (os.size() > 0) {
     iExtGState = startObject();
     iStream << "<<\n";
-    for (uint i = 0; i < os.size(); ++i) {
-      Attribute alpha = iDoc->cascade()->find(EOpacity, os[i]);
+    for (const auto & obj : os) {
+      Attribute alpha = iDoc->cascade()->find(EOpacity, obj);
       assert(alpha.isNumber());
       iStream << opacityName(alpha.number()) << " << /CA " << alpha.number()
 	      << " /ca " << alpha.number() << " >>\n";
+      iStream << opacityName(alpha.number()) << "s << /CA " << alpha.number()
+	      << " >>\n";
     }
     iStream << ">> endobj\n";
   }
@@ -312,8 +320,8 @@ PdfWriter::PdfWriter(TellStream &stream, const Document *doc,
   // embed all gradients
   AttributeSeq gs;
   iDoc->cascade()->allNames(EGradient, gs);
-  for (uint i = 0; i < gs.size(); ++i) {
-    const Gradient *g = iDoc->cascade()->findGradient(gs[i]);
+  for (const auto & grad : gs) {
+    const Gradient *g = iDoc->cascade()->findGradient(grad);
     int num = startObject();
     iStream << "<<\n"
 	    << " /ShadingType " << int(g->iType) << "\n"
@@ -339,7 +347,7 @@ PdfWriter::PdfWriter(TellStream &stream, const Document *doc,
 	      << "  /FunctionType 3 /Domain [ 0 1 ]\n"
 	      << "  /Bounds [";
       int count = 0;
-      for (uint i = 1; i < g->iStops.size() - 1; ++i) {
+      for (int i = 1; i < size(g->iStops) - 1; ++i) {
 	if (g->iStops[i].offset > g->iStops[i-1].offset) {
 	  iStream << g->iStops[i].offset << " ";
 	  ++count;
@@ -349,7 +357,7 @@ PdfWriter::PdfWriter(TellStream &stream, const Document *doc,
       for (int i = 0; i <= count; ++i)
 	iStream << "0.0 1.0 ";
       iStream << "]\n  /Functions [\n";
-      for (uint i = 1; i < g->iStops.size(); ++i) {
+      for (int i = 1; i < size(g->iStops); ++i) {
 	if (g->iStops[i].offset > g->iStops[i-1].offset) {
 	  iStream << "   << /FunctionType 2 /Domain [ 0 1 ] /N 1 /C0 [";
 	  g->iStops[i-1].color.saveRGB(iStream);
@@ -361,7 +369,7 @@ PdfWriter::PdfWriter(TellStream &stream, const Document *doc,
       iStream << "] >>\n";
     }
     iStream << ">> endobj\n";
-    iGradients[gs[i].index()] = num;
+    iGradients[grad.index()] = num;
   }
 
   // embed all tilings
@@ -369,8 +377,8 @@ PdfWriter::PdfWriter(TellStream &stream, const Document *doc,
   std::map<int, int> patterns;
   iDoc->cascade()->allNames(ETiling, ts);
   if (ts.size() > 0) {
-    for (uint i = 0; i < ts.size(); ++i) {
-      const Tiling *t = iDoc->cascade()->findTiling(ts[i]);
+    for (const auto & tiling : ts) {
+      const Tiling *t = iDoc->cascade()->findTiling(tiling);
       Linear m(t->iAngle);
       int num = startObject();
       iStream << "<<\n"
@@ -379,7 +387,7 @@ PdfWriter::PdfWriter(TellStream &stream, const Document *doc,
 	      << "/PaintType 2\n"    // uncolored pattern
 	      << "/TilingType 2\n"   // faster
 	      << "/BBox [ 0 0 100 " << t->iStep << " ]\n"
-	      << "/XStep 99\n"
+	      << "/XStep 100\n"
 	      << "/YStep " << t->iStep << "\n"
 	      << "/Resources << >>\n"
 	      << "/Matrix [" << m << " 0 0]\n";
@@ -387,15 +395,15 @@ PdfWriter::PdfWriter(TellStream &stream, const Document *doc,
       StringStream ss(s);
       ss << "0 0 100 " << t->iWidth << " re f\n";
       createStream(s.data(), s.size(), false);
-      patterns[ts[i].index()] = num;
+      patterns[tiling.index()] = num;
     }
 
     // create pattern dictionary
     iPatternNum = startObject();
     iStream << "<<\n";
-    for (uint i = 0; i < ts.size(); ++i) {
-      iStream << "/Pat" << ts[i].index() << " "
-	      << patterns[ts[i].index()] << " 0 R\n";
+    for (const auto & pattern : ts) {
+      iStream << "/Pat" << pattern.index() << " "
+	      << patterns[pattern.index()] << " 0 R\n";
     }
     iStream << ">> endobj\n";
   }
@@ -404,8 +412,8 @@ PdfWriter::PdfWriter(TellStream &stream, const Document *doc,
   AttributeSeq sys;
   iDoc->cascade()->allNames(ESymbol, sys);
   if (sys.size()) {
-    for (uint i = 0; i < sys.size(); ++i) {
-      const Symbol *sym = iDoc->cascade()->findSymbol(sys[i]);
+    for (const auto & sysi : sys) {
+      const Symbol *sym = iDoc->cascade()->findSymbol(sysi);
       if (sym->iXForm) {
 	// compute bbox for object
 	BBoxPainter bboxPainter(iDoc->cascade());
@@ -426,7 +434,7 @@ PdfWriter::PdfWriter(TellStream &stream, const Document *doc,
 	PdfPainter painter(iDoc->cascade(), ss);
 	sym->iObject->draw(painter);
 	createStream(s.data(), s.size(), false);
-	iSymbols[sys[i].index()] = num;
+	iSymbols[sysi.index()] = num;
       }
     }
   }
@@ -606,46 +614,50 @@ void PdfWriter::createStream(const char *data, int size, bool preCompressed)
 
 void PdfWriter::embedBitmap(Bitmap bitmap)
 {
+  int smaskNum = -1;
+  auto embed = bitmap.embed();
+  if (bitmap.hasAlpha() && embed.second.size() > 0) {
+    smaskNum = startObject();
+    iStream << "<<\n";
+    iStream << "/Type /XObject\n";
+    iStream << "/Subtype /Image\n";
+    iStream << "/Width " << bitmap.width() << "\n";
+    iStream << "/Height " << bitmap.height() << "\n";
+    iStream << "/ColorSpace /DeviceGray\n";
+    iStream << "/Filter /FlateDecode\n";
+    iStream << "/BitsPerComponent 8\n";
+    iStream << "/Length " << embed.second.size() << "\n>> stream\n";
+    iStream.putRaw(embed.second.data(), embed.second.size());
+    iStream << "\nendstream endobj\n";
+  }
   int objnum = startObject();
   iStream << "<<\n";
   iStream << "/Type /XObject\n";
   iStream << "/Subtype /Image\n";
   iStream << "/Width " << bitmap.width() << "\n";
   iStream << "/Height " << bitmap.height() << "\n";
-  switch (bitmap.colorSpace()) {
-  case Bitmap::EDeviceGray:
+  if (bitmap.isGray())
     iStream << "/ColorSpace /DeviceGray\n";
-    break;
-  case Bitmap::EDeviceRGB:
+  else
     iStream << "/ColorSpace /DeviceRGB\n";
-    break;
-  case Bitmap::EDeviceCMYK:
-    iStream << "/ColorSpace /DeviceCMYK\n";
-    // apparently JPEG CMYK images need this decode array??
-    // iStream << "/Decode [1 0 1 0 1 0 1 0]\n";
-    break;
-  }
-  switch (bitmap.filter()) {
-  case Bitmap::EFlateDecode:
-    iStream << "/Filter /FlateDecode\n";
-    break;
-  case Bitmap::EDCTDecode:
+  if (bitmap.isJpeg())
     iStream << "/Filter /DCTDecode\n";
-    break;
-  default:
-    // no filter
-    break;
-  }
-  iStream << "/BitsPerComponent " << bitmap.bitsPerComponent() << "\n";
-  if (bitmap.colorKey() >= 0) {
+  else
+    iStream << "/Filter /FlateDecode\n";
+  iStream << "/BitsPerComponent 8\n";
+  if (smaskNum >= 0) {
+    iStream << "/SMask " << smaskNum << " 0 R\n";
+  } else if (bitmap.colorKey() >= 0) {
     int r = (bitmap.colorKey() >> 16) & 0xff;
     int g = (bitmap.colorKey() >> 8) & 0xff;
     int b = bitmap.colorKey() & 0xff;
-    iStream << "/Mask [" << r << " " << r << " " << g << " " << g
-	    << " " << b << " " << b << "]\n";
+    iStream << "/Mask [" << r << " " << r;
+    if (!bitmap.isGray())
+      iStream << " " << g << " " << g << " " << b << " " << b;
+    iStream << "]\n";
   }
-  iStream << "/Length " << bitmap.size() << "\n>> stream\n";
-  iStream.putRaw(bitmap.data(), bitmap.size());
+  iStream << "/Length " << embed.first.size() << "\n>> stream\n";
+  iStream.putRaw(embed.first.data(), embed.first.size());
   iStream << "\nendstream endobj\n";
   bitmap.setObjNum(objnum);
 }
@@ -787,22 +799,54 @@ void PdfWriter::createPageView(int pno, int view)
       iStream << "<<\n"
 	      << "/Type /Annot\n"
 	      << "/Subtype /Link\n"
+	      << "/H /N\n"
 	      << "/Rect [" << page->bbox(i) << "]\n"
-	      << "/A <</Type/Action/S/URI/URI";
-      writeString(g->url());
+	      << "/A <</Type/Action/S";
+      String url = g->url();
+      if (url.left(6) == "named:") {
+	iStream << "/Named/N/" << url.substr(6);
+      } else {
+	if (url.left(7) == "launch:") {
+	  url = url.substr(7);
+	  iStream << "/Launch/F";
+	} else if (url.left(5) == "goto:") {
+	  url = url.substr(5);
+	  iStream << "/GoTo/D";
+	} else
+	  iStream << "/URI/URI";
+	writeString(url);
+      }
       iStream << ">>\n>> endobj\n";
     }
   }
+  int notesObj = -1;
+  if (!page->notes().empty() &&
+      (!(iSaveFlags & SaveFlag::Export) || (iSaveFlags & SaveFlag::KeepNotes))) {
+    notesObj = startObject();
+    iStream << "<<\n"
+	    << "/Type /Annot\n"
+	    << "/Subtype /Text\n"
+	    << "/Rect [20 40 30 40]\n"
+	    << "/F 4\n"
+	    << "/Contents ";
+    writeString(page->notes());
+    iStream << "\n>> endobj\n";
+  }
+
   int contentsobj = startObject();
   iStream << "<<\n";
   createStream(pagedata.data(), pagedata.size(), (iCompressLevel > 0));
   int pageobj = startObject();
   iStream << "<<\n";
   iStream << "/Type /Page\n";
-  if (firstLink >= 0) {
+  if (firstLink >= 0 || notesObj >= 0) {
     iStream << "/Annots [ ";
-    while (firstLink <= lastLink)
-      iStream << firstLink++ << " 0 R ";
+    if (firstLink >= 0) {
+      while (firstLink <= lastLink)
+	iStream << firstLink++ << " 0 R ";
+    }
+    if (notesObj >= 0)
+      iStream << notesObj << " 0 R";
     iStream << "]\n";
   }
   iStream << "/Contents " << contentsobj << " 0 R\n";
@@ -828,17 +872,17 @@ void PdfWriter::createPageView(int pno, int view)
     iStream << "/ArtBox [" << bbox << "]\n";
   iStream << "/Parent 2 0 R\n";
   iStream << ">> endobj\n";
-  iPageObjectNumbers.push_back(pageobj);
+  iPageObjectNumbers.push_back({ pno, view, pageobj });
 }
 
 //! Create all PDF pages.
 void PdfWriter::createPages()
 {
   for (int page = iFromPage; page <= iToPage; ++page) {
-    if (iMarkedView && !iDoc->page(page)->marked())
+    if ((iSaveFlags & SaveFlag::MarkedView) && !iDoc->page(page)->marked())
       continue;
     int nViews = iDoc->page(page)->countViews();
-    if (iMarkedView) {
+    if (iSaveFlags & SaveFlag::MarkedView) {
       bool shown = false;
       for (int view = 0; view < nViews; ++view) {
 	if (iDoc->page(page)->markedView(view)) {
@@ -901,12 +945,20 @@ void PdfWriter::writeString(String text)
 
 // --------------------------------------------------------------------
 
+int PdfWriter::pageObjectNumber(int page, int view)
+{
+  auto it = std::find_if(iPageObjectNumbers.begin(), iPageObjectNumbers.end(),
+			 [=](const PON &pon) {return pon.page == page && pon.view == view;});
+  if (it != iPageObjectNumbers.end())
+    return it->objNum;
+  ipeDebug("pageObjectNumber not found, this is a bug!");
+  return 0;
+}
+
 struct Section {
   int iPage;
-  int iSeqPage;
   int iObjNum;
   std::vector<int> iSubPages;
-  std::vector<int> iSubSeqPages;
 };
 
 //! Create the bookmarks (PDF outline).
@@ -914,33 +966,25 @@ void PdfWriter::createBookmarks()
 {
   // first collect all information
   std::vector<Section> sections;
-  int seqPg = 0;
   for (int pg = iFromPage; pg <= iToPage; ++pg) {
-    if (!iDoc->page(pg)->marked())
+    if ((iSaveFlags & SaveFlag::MarkedView) && !iDoc->page(pg)->marked())
       continue;
     String s = iDoc->page(pg)->section(0);
     String ss = iDoc->page(pg)->section(1);
     if (!s.empty()) {
       Section sec;
       sec.iPage = pg;
-      sec.iSeqPage = seqPg;
       sections.push_back(sec);
-      // ipeDebug("section on page %d, seq %d", pg, seqPg);
     }
-    if (!sections.empty() && !ss.empty()) {
+    if (!sections.empty() && !ss.empty())
       sections.back().iSubPages.push_back(pg);
-      sections.back().iSubSeqPages.push_back(seqPg);
-      // ipeDebug("subsection on page %d, seq %d", pg, seqPg);
-    }
-    seqPg += iMarkedView ? iDoc->page(pg)->countMarkedViews() :
-      iDoc->page(pg)->countViews();
   }
   if (sections.empty())
     return;
   // reserve outline object
   iBookmarks = iObjNum++;
   // assign object numbers
-  for (uint s = 0; s < sections.size(); ++s) {
+  for (int s = 0; s < size(sections); ++s) {
     sections[s].iObjNum = iObjNum++;
     iObjNum += sections[s].iSubPages.size(); // leave space for subsections
   }
@@ -949,7 +993,7 @@ void PdfWriter::createBookmarks()
   iStream << "<<\n/First " << sections[0].iObjNum << " 0 R\n"
 	  << "/Count " << int(sections.size()) << "\n"
 	  << "/Last " << sections.back().iObjNum << " 0 R\n>> endobj\n";
-  for (uint s = 0; s < sections.size(); ++s) {
+  for (int s = 0; s < size(sections); ++s) {
     int count = sections[s].iSubPages.size();
     int obj = sections[s].iObjNum;
     // embed section
@@ -957,11 +1001,11 @@ void PdfWriter::createBookmarks()
     iStream << "<<\n/Title ";
     writeString(iDoc->page(sections[s].iPage)->section(0));
     iStream << "\n/Parent " << iBookmarks << " 0 R\n"
-	    << "/Dest [ " << iPageObjectNumbers[sections[s].iSeqPage]
+	    << "/Dest [ " << pageObjectNumber(sections[s].iPage, 0)
 	    << " 0 R /XYZ null null null ]\n";
     if (s > 0)
       iStream << "/Prev " << sections[s-1].iObjNum << " 0 R\n";
-    if (s < sections.size() - 1)
+    if (s < size(sections) - 1)
       iStream << "/Next " << sections[s+1].iObjNum << " 0 R\n";
     if (count > 0)
       iStream << "/Count " << -count << "\n"
@@ -971,12 +1015,11 @@ void PdfWriter::createBookmarks()
     // using ids obj + 1 .. obj + count for the subsections
     for (int ss = 0; ss < count; ++ss) {
       int pageNo = sections[s].iSubPages[ss];
-      int seqPageNo = sections[s].iSubSeqPages[ss];
       startObject(obj + ss + 1);
       iStream << "<<\n/Title ";
       writeString(iDoc->page(pageNo)->section(1));
       iStream << "\n/Parent " << obj << " 0 R\n"
-	      << "/Dest [ " << iPageObjectNumbers[seqPageNo]
+	      << "/Dest [ " << pageObjectNumber(pageNo, 0)
 	      << " 0 R /XYZ null null null ]\n";
       if (ss > 0)
 	iStream << "/Prev " << (obj + ss) << " 0 R\n";
@@ -987,6 +1030,33 @@ void PdfWriter::createBookmarks()
   }
 }
 
+//! Create the named destinations.
+void PdfWriter::createNamedDests()
+{
+  std::vector<std::pair<String, int>> dests;
+  for (int pg = iFromPage; pg <= iToPage; ++pg) {
+    if ((iSaveFlags & SaveFlag::MarkedView) && !iDoc->page(pg)->marked())
+      continue;
+    String s = iDoc->page(pg)->section(0);
+    if (!s.empty())
+      dests.push_back(std::make_pair(s, pageObjectNumber(pg, 0)));
+  }
+  if (dests.empty())
+    return;
+  std::sort(dests.begin(), dests.end());
+  iDests = startObject();
+  iStream << "<<\n/Limits [";
+  writeString(dests.front().first);
+  iStream << " ";
+  writeString(dests.back().first);
+  iStream << "]\n/Names [\n";
+  for (const auto & dest : dests) {
+    writeString(dest.first);
+    iStream << " [" << dest.second << " 0 R /XYZ null null null]\n";
+  }
+  iStream << "]>> endobj\n";
+}
+
 // --------------------------------------------------------------------
 
 //! Create the root objects and trailer of the PDF file.
@@ -998,13 +1068,23 @@ void PdfWriter::createTrailer()
   iStream << "<<\n" << "/Type /Pages\n";
   iStream << "/Count " << int(iPageObjectNumbers.size()) << "\n";
   iStream << "/Kids [ ";
-  for (std::vector<int>::const_iterator it = iPageObjectNumbers.begin();
-       it != iPageObjectNumbers.end(); ++it)
-    iStream << (*it) << " 0 R ";
+  for (auto pon : iPageObjectNumbers)
+    iStream << pon.objNum << " 0 R ";
   iStream << "]\n>> endobj\n";
+  // create Name dictionary
+  int nameDict = -1;
+  if (iDests >= 0) {
+    nameDict = startObject();
+    iStream << "<</Dests " << iDests << " 0 R>> endobj\n";
+  }
+  // create PieceInfo
+  int pieceInfo = startObject();
+  iStream << "<</Ipe<</Private 1 0 R/LastModified("
+	  << props.iModified << ")>> >> endobj\n";
   // create /Catalog
   int catalogobj = startObject();
-  iStream << "<<\n/Type /Catalog\n/Pages 2 0 R\n";
+  iStream << "<<\n/Type /Catalog\n/Pages 2 0 R\n"
+	  << "/PieceInfo " << pieceInfo << " 0 R\n";
   if (props.iFullScreen)
     iStream << "/PageMode /FullScreen\n";
   if (iBookmarks >= 0) {
@@ -1012,12 +1092,14 @@ void PdfWriter::createTrailer()
       iStream << "/PageMode /UseOutlines\n";
     iStream << "/Outlines " << iBookmarks << " 0 R\n";
   }
+  if (nameDict >= 0)
+    iStream << "/Names " << nameDict << " 0 R\n";
   if (iDoc->countTotalViews() > 1) {
     iStream << "/PageLabels << /Nums [ ";
     int count = 0;
     for (int page = 0; page < iDoc->countPages(); ++page) {
-      if (!iMarkedView || iDoc->page(page)->marked()) {
-	int nviews = iMarkedView ? iDoc->page(page)->countMarkedViews() :
+      if (!(iSaveFlags & SaveFlag::MarkedView) || iDoc->page(page)->marked()) {
+	int nviews = (iSaveFlags & SaveFlag::MarkedView) ? iDoc->page(page)->countMarkedViews() :
 	  iDoc->page(page)->countViews();
 	if (nviews > 1) {
 	  iStream << count << " <</S /D /P (" << (page + 1) << "-)>>";
