@@ -43,6 +43,10 @@
 #ifdef __APPLE__
 #include <mach-o/dyld.h>
 #include <xlocale.h>
+// sys/param.h triggers legacy mode for realpath, so that the
+// last component of the path does not need to exist.
+#include <sys/param.h>
+#include <sys/errno.h>
 #endif
 
 #include <cstdlib>
@@ -51,6 +55,7 @@
 #include <cstdarg>
 #include <unistd.h>
 #include <clocale>
+#include <cstring>
 
 using namespace ipe;
 
@@ -94,6 +99,19 @@ static LPStrtodL p_strtod_l = nullptr;
 locale_t ipeLocale;
 #endif
 
+#ifndef WIN32
+static String dotIpe()
+{
+  const char *home = getenv("HOME");
+  if (!home)
+    return String();
+  String res = String(home) + "/.ipe";
+  if (!Platform::fileExists(res) && mkdir(res.z(), 0700) != 0)
+    return String();
+  return res + "/";
+}
+#endif
+
 #ifdef WIN32
 static void readIpeConf()
 {
@@ -106,9 +124,21 @@ static void readIpeConf()
   int i = 0;
   while (i < conf.size()) {
     String line = conf.getLine(i);
-    std::vector<wchar_t> wline;
-    Platform::utf8ToWide(line.z(), wline);
-    _wputenv(&wline[0]);
+    _wputenv(line.w().data());
+  }
+}
+#else
+static void readIpeConf()
+{
+  String fname = dotIpe() + "/ipe.conf";
+  String conf = Platform::readFile(fname);
+  if (conf.empty())
+    return;
+  ipeDebug("ipe.conf = %s", conf.z());
+  int i = 0;
+  while (i < conf.size()) {
+    String line = conf.getLine(i);
+    putenv(strdup(line.z()));  // a leak, but necessary
   }
 }
 #endif
@@ -151,8 +181,9 @@ void Platform::initLib(int version)
   if (initialized)
     return;
   initialized = true;
+  readIpeConf();
   showDebug = false;
-  if (getenv("IPEDEBUG") != 0) {
+  if (getenv("IPEDEBUG")) {
     showDebug = true;
     fprintf(stderr, "Debug messages enabled\n");
   }
@@ -165,7 +196,6 @@ void Platform::initLib(int version)
     appImageBase = ".";
 #endif
 #ifdef WIN32
-  readIpeConf();
   HMODULE hDll = LoadLibraryA("msvcrt.dll");
   if (hDll) {
     p_create_locale = (LPCreateLocale) GetProcAddress(hDll, "_create_locale");
@@ -175,12 +205,12 @@ void Platform::initLib(int version)
   if (p_create_locale != nullptr)
     ipeLocale = p_create_locale(LC_NUMERIC, "C");
   Gdiplus::GdiplusStartupInput gdiplusStartupInput;
-  Gdiplus::GdiplusStartup(&gdiplusToken, &gdiplusStartupInput, 0);
+  Gdiplus::GdiplusStartup(&gdiplusToken, &gdiplusStartupInput, nullptr);
 #else
-  ipeLocale = newlocale(LC_NUMERIC_MASK, "C", NULL);
+  ipeLocale = newlocale(LC_NUMERIC_MASK, "C", nullptr);
 #endif
   atexit(shutdownIpelib);
-#ifndef WIN32
+#ifndef IPE_NO_IPELIB_VERSION_CHECK
   if (version == IPELIB_VERSION)
     return;
   fprintf(stderr,
@@ -220,7 +250,7 @@ void ipeDebug(const char *msg, ...) noexcept
 String Platform::currentDirectory()
 {
 #ifdef WIN32
-  wchar_t *buffer = _wgetcwd(0, 0);
+  wchar_t *buffer = _wgetcwd(nullptr, 0);
   return String(buffer);
 #else
   char buffer[1024];
@@ -255,17 +285,15 @@ String Platform::ipeDir(const char *suffix, const char *fname)
 #else
 #ifdef WIN32
   wchar_t exename[OFS_MAXPATHNAME];
-  GetModuleFileNameW(0, exename, OFS_MAXPATHNAME);
+  GetModuleFileNameW(nullptr, exename, OFS_MAXPATHNAME);
   String exe(exename);
 #else
-  char path[PATH_MAX];
+  char path[PATH_MAX], rpath[PATH_MAX];
   uint32_t size = sizeof(path);
   String exe;
-  if (_NSGetExecutablePath(path, &size) == 0) {
-    char *rp = realpath(path, NULL);
-    exe = String(rp);
-    free(rp);
-  } else
+  if (_NSGetExecutablePath(path, &size) == 0 && realpath(path, rpath) != nullptr)
+    exe = String(rpath);
+  else
     ipeDebug("ipeDir: buffer too small; need size %u", size);
 #endif
   int i = exe.rfind(IPESEP);
@@ -295,19 +323,6 @@ String Platform::ipeDir(const char *suffix, const char *fname)
 }
 #endif
 
-#ifndef WIN32
-static String dotIpe()
-{
-  const char *home = getenv("HOME");
-  if (!home)
-    return String();
-  String res = String(home) + "/.ipe";
-  if (!Platform::fileExists(res) && mkdir(res.z(), 0700) != 0)
-    return String();
-  return res + "/";
-}
-#endif
-
 //! Return path for the directory containing pdflatex and xelatex.
 /*! If empty means look on PATH. */
 String Platform::latexPath()
@@ -337,14 +352,14 @@ String Platform::latexDirectory()
 #ifdef WIN32
   String latexDir;
   const wchar_t *p = _wgetenv(L"IPELATEXDIR");
-  if (p)
+  if (p) {
     latexDir = String(p);
-  if (latexDir.left(4) == "ipe:")
-    latexDir = ipeDrive() + latexDir.substr(4);
-  else {
+    if (latexDir.left(4) == "ipe:")
+      latexDir = ipeDrive() + latexDir.substr(4);
+  } else {
     wchar_t szPath[MAX_PATH];
-    if (SUCCEEDED(SHGetFolderPathW(NULL, CSIDL_LOCAL_APPDATA,
-				   NULL, 0, szPath))) {
+    if (SUCCEEDED(SHGetFolderPathW(nullptr, CSIDL_LOCAL_APPDATA,
+				   nullptr, 0, szPath))) {
       latexDir = String(szPath) + "\\ipe";
     } else {
       p = _wgetenv(L"LOCALAPPDATA");
@@ -383,24 +398,41 @@ String Platform::latexDirectory()
 bool Platform::fileExists(String fname)
 {
 #ifdef WIN32
-  std::vector<wchar_t> wname;
-  utf8ToWide(fname.z(), wname);
-  return (_waccess(&wname[0], F_OK) == 0);
+  return (_waccess(fname.w().data(), F_OK) == 0);
 #else
   return (access(fname.z(), F_OK) == 0);
 #endif
 }
 
+//! Convert relative filename to absolute.
+/*! This also works when the filename does not exist, or at least it tries. */
+String Platform::realPath(String fname)
+{
+#ifdef WIN32
+  wchar_t wresult[MAX_PATH];
+  // this function works also when fname does not exist
+  GetFullPathNameW(fname.w().data(), MAX_PATH, wresult, nullptr);
+  return String(wresult);
+#else
+  char rpath[PATH_MAX];
+  if (realpath(fname.z(), rpath))
+    return String(rpath);
+  if (errno != ENOENT || fname.left(1) == "/")
+    return fname;  // not much we can do
+  if (realpath(".", rpath) == nullptr)
+    return fname;  // nothing we can do
+  return String(rpath) + "/" + fname;
+#endif
+}
+
 //! List all files in directory
 /*! Return true if successful, false on error. */
 bool Platform::listDirectory(String path, std::vector<String> &files)
 {
 #ifdef WIN32
   String pattern = path + "\\*";
-  std::vector<wchar_t> wname;
-  utf8ToWide(pattern.z(), wname);
   struct _wfinddata_t info;
-  intptr_t h = _wfindfirst(&wname[0], &info);
+  intptr_t h = _wfindfirst(pattern.w().data(), &info);
   if (h == -1L)
     return false;
   files.push_back(String(info.name));
@@ -410,10 +442,10 @@ bool Platform::listDirectory(String path, std::vector<String> &files)
   return true;
 #else
   DIR *dir = opendir(path.z());
-  if (dir == 0)
+  if (dir == nullptr)
     return false;
   struct dirent *entry = readdir(dir);
-  while (entry != 0) {
+  while (entry != nullptr) {
     String s(entry->d_name);
     if (s != "." && s != "..")
       files.push_back(s);
@@ -440,14 +472,26 @@ String Platform::readFile(String fname)
   return s;
 }
 
-//! Runs latex on file text.tex in given directory.
-int Platform::runLatex(String dir, LatexType engine) noexcept
+//! Runs latex on file ipetemp.tex in given directory.
+/*! directory of docname is added to TEXINPUTS if its non-empty. */
+int Platform::runLatex(String dir, LatexType engine, String docname) noexcept
 {
   const char *latex = (engine == LatexType::Xetex) ?
     "xelatex" : (engine == LatexType::Luatex) ?
     "lualatex" : "pdflatex";
+  String url = Platform::readFile(dir + "url.txt");
+  bool online = (url.left(4) == "http");
+  String texinputs;
+  if (!online && !docname.empty()) {
+    docname = realPath(docname);
+    int i = docname.size();
+    while (i > 0 && docname[i-1] != IPESEP)
+      --i;
+    if (i > 0)
+      texinputs = docname.substr(0, i-1);
+  }
 #ifdef WIN32
-  if (getenv("IPETEXFORMAT")) {
+  if (!online && getenv("IPETEXFORMAT")) {
     latex = (engine == LatexType::Xetex) ?
       "xetex ^&latex" : (engine == LatexType::Luatex) ?
       "luatex ^&latex" : "pdftex ^&pdflatex";
@@ -461,22 +505,36 @@ int Platform::runLatex(String dir, LatexType engine) noexcept
   bat += "cd \"";
   bat += dir;
   bat += "\"\r\n";
-  String path = latexPath();
-  if (!path.empty()) {
-    bat += "PATH ";
-    bat += path;
-    bat += ";%PATH%\r\n";
+  if (!texinputs.empty()) {
+    bat += "setlocal\r\n";
+    bat += "set TEXINPUTS=.;";
+    bat += texinputs;
+    bat += ";%TEXINPUTS%\r\n";
   }
-  bat += latex;
-  bat += " ipetemp.tex\r\n";
+  if (online) {
+    bat += ipeDir("bin", "ipecurl.exe");
+    bat += " ";
+    bat += latex;
+    bat += "\r\n";
+  } else {
+    String path = latexPath();
+    if (!path.empty()) {
+      bat += "PATH ";
+      bat += path;
+      bat += ";%PATH%\r\n";
+    }
+    bat += latex;
+    bat += " ipetemp.tex\r\n";
+  }
+  if (!texinputs.empty())
+    bat += "endlocal\r\n";
   // bat += "pause\r\n";  // alternative for Wine
 
   // CMD.EXE input needs to be encoded in "OEM codepage",
   // which can be different from "Windows codepage"
-  std::vector<wchar_t> wbat;
-  utf8ToWide(bat.z(), wbat);
+  std::wstring wbat = bat.w();
   Buffer oemBat(2 * wbat.size() + 1);
-  CharToOemW(&wbat[0], oemBat.data());
+  CharToOemW(wbat.data(), oemBat.data());
 
   String s = dir + "runlatex.bat";
   std::FILE *f = Platform::fopen(s.z(), "wb");
@@ -494,12 +552,12 @@ int Platform::runLatex(String dir, LatexType engine) noexcept
   startupInfo.cb = sizeof(startupInfo);
 
   // Call the executable program
-  s = String("cmd /c call \"") + dir + String("runlatex.bat\"");
-  utf8ToWide(s.z(), wbat);
+  String cmd = String("cmd /c call \"") + dir + String("runlatex.bat\"");
+  std::wstring wcmd = cmd.w();
 
-  int result = CreateProcessW(NULL, &wbat[0], NULL, NULL, FALSE,
+  int result = CreateProcessW(nullptr, wcmd.data(), nullptr, nullptr, FALSE,
 			      NORMAL_PRIORITY_CLASS|CREATE_NO_WINDOW,
-			      NULL, NULL, &startupInfo, &processInformation);
+			      nullptr, nullptr, &startupInfo, &processInformation);
   if (result == 0)
     return -1;  // failure to create process
 
@@ -516,7 +574,7 @@ int Platform::runLatex(String dir, LatexType engine) noexcept
     Sleep(Lex(wine).getInt());
   return 0;
 #else
-  if (getenv("IPETEXFORMAT")) {
+  if (!online && getenv("IPETEXFORMAT")) {
     latex = (engine == LatexType::Xetex) ?
       "xetex \\&latex" : (engine == LatexType::Luatex) ?
       "luatex \\&latex" : "pdftex \\&pdflatex";
@@ -524,12 +582,27 @@ int Platform::runLatex(String dir, LatexType engine) noexcept
   String s("cd \"");
   s += dir;
   s += "\"; rm -f ipetemp.log; ";
-  String path = latexPath();
-  if (path.empty())
+  if (!texinputs.empty()) {
+    s += "export TEXINPUTS=\"";
+    s += texinputs;
+    s += ":$TEXINPUTS\"; ";
+  }
+  if (online) {
+#ifdef __APPLE__
+    s += ipeDir("../MacOS", "ipecurl ");
+#else
+    s += "ipecurl ";
+#endif
     s += latex;
-  else
-    s += String("\"") + path + "/" + latex + "\"";
-  s += " ipetemp.tex > /dev/null";
+  } else {
+    String path = latexPath();
+    if (path.empty())
+      s += latex;
+    else
+      s += String("\"") + path + "/" + latex + "\"";
+    s += " ipetemp.tex";
+  }
+  s += " > /dev/null";
   int result = std::system(s.z());
   if (result != -1)
     result = WEXITSTATUS(result);
@@ -540,18 +613,23 @@ int Platform::runLatex(String dir, LatexType engine) noexcept
 #ifdef WIN32
 FILE *Platform::fopen(const char *fname, const char *mode)
 {
-  std::vector<wchar_t> wname;
-  utf8ToWide(fname, wname);
-  std::vector<wchar_t> wmode;
-  utf8ToWide(mode, wmode);
-  return _wfopen(&wname[0], &wmode[0]);
+  return _wfopen(String(fname).w().data(), String(mode).w().data());
 }
 
 int Platform::mkdir(const char *dname)
 {
-  std::vector<wchar_t> wname;
-  utf8ToWide(dname, wname);
-  return _wmkdir(&wname[0]);
+  return _wmkdir(String(dname).w().data());
+}
+
+//! Return a wide string including a terminating zero character.
+std::wstring String::w() const noexcept
+{
+  if (empty())
+    return L"\0";
+  int rw = MultiByteToWideChar(CP_UTF8, 0, data(), size(), nullptr, 0);
+  std::wstring result(rw + 1, wchar_t(0));
+  MultiByteToWideChar(CP_UTF8, 0, data(), size(), &result[0], rw);
+  return result;
 }
 
 String::String(const wchar_t *wbuf)
@@ -559,22 +637,15 @@ String::String(const wchar_t *wbuf)
   if (!wbuf) {
     iImp = emptyString();
   } else {
-    int rm = WideCharToMultiByte(CP_UTF8, 0, wbuf, -1, NULL, 0, NULL, NULL);
+    int rm = WideCharToMultiByte(CP_UTF8, 0, wbuf, -1, nullptr, 0, nullptr, nullptr);
     iImp = new Imp;
     iImp->iRefCount = 1;
     iImp->iSize = rm - 1; // rm includes the trailing zero
     iImp->iCapacity = (rm + 32) & ~15;
     iImp->iData = new char[iImp->iCapacity];
-    WideCharToMultiByte(CP_UTF8, 0, wbuf, -1, iImp->iData, rm, NULL, NULL);
+    WideCharToMultiByte(CP_UTF8, 0, wbuf, -1, iImp->iData, rm, nullptr, nullptr);
   }
 }
-
-void Platform::utf8ToWide(const char *utf8, std::vector<wchar_t> &wbuf)
-{
-  int rw = MultiByteToWideChar(CP_UTF8, 0, utf8, -1, NULL, 0);
-  wbuf.resize(rw);
-  MultiByteToWideChar(CP_UTF8, 0, utf8, -1, &wbuf[0], rw);
-}
 #endif
 
 // --------------------------------------------------------------------
@@ -583,14 +654,38 @@ double Platform::toDouble(String s)
 {
 #ifdef WIN32
   if (p_create_locale != nullptr)
-    return _strtod_l(s.z(), 0, ipeLocale);
+    return _strtod_l(s.z(), nullptr, ipeLocale);
   else
-    return strtod(s.z(), 0);
+    return strtod(s.z(), nullptr);
 #else
-  return strtod_l(s.z(), 0, ipeLocale);
+  return strtod_l(s.z(), nullptr, ipeLocale);
 #endif
 }
 
+int Platform::toNumber(String s, int &iValue, double &dValue)
+{
+  char *fin = const_cast<char *>(s.z());
+  iValue = std::strtol(s.z(), &fin, 10);
+  while (*fin == ' ' || *fin == '\t')
+    ++fin;
+  if (*fin == '\0')
+    return 1; // integer
+#ifdef WIN32
+  if (p_create_locale != nullptr)
+    dValue = _strtod_l(s.z(), &fin, ipeLocale);
+  else
+    dValue = strtod(s.z(), &fin);
+#else
+  dValue = strtod_l(s.z(), &fin, ipeLocale);
+#endif
+  while (*fin == ' ' || *fin == '\t')
+    ++fin;
+  if (*fin == '\0')
+    return 2; // double
+  // error
+  return 0;
+}
+
 void ipeAssertionFailed(const char *file, int line, const char *assertion)
 {
   fprintf(stderr, "Assertion failed on line #%d (%s): '%s'\n",
