@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2016  Otfried Cheong
+    Copyright (c) 1993-2019 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -162,6 +162,7 @@ void Reference::draw(Painter &painter) const
 {
   const Symbol *symbol = painter.cascade()->findSymbol(iName);
   if (symbol) {
+    iSnap = symbol->iSnap;  // cache snap point information
     Attribute si = painter.cascade()->find(ESymbolSize, iSize);
     double s = si.number().toDouble();
     painter.pushMatrix();
@@ -188,11 +189,28 @@ void Reference::draw(Painter &painter) const
 
 void Reference::drawSimple(Painter &painter) const
 {
-  const int size = 10;
   painter.pushMatrix();
   painter.transform(matrix());
   painter.translate(iPos);
+  if (iSnap.size() > 0) {
+    const Symbol *symbol = painter.cascade()->findSymbol(iName);
+    if (symbol) {
+      painter.untransform(symbol->iTransformations);
+      if (iFlags & EHasSize) {
+	Attribute si = painter.cascade()->find(ESymbolSize, iSize);
+	double s = si.number().toDouble();
+	Matrix m(s, 0, 0, s, 0, 0);
+	painter.transform(m);
+      }
+      painter.push();
+      symbol->iObject->drawSimple(painter);
+      painter.pop();
+      painter.popMatrix();
+      return;
+    }
+  }
   painter.untransform(ETransformationsTranslations);
+  const int size = 10;
   painter.newPath();
   painter.moveTo(Vector(-size, 0));
   painter.lineTo(Vector(size, 0));
@@ -204,17 +222,24 @@ void Reference::drawSimple(Painter &painter) const
 
 /*! \copydoc Object::addToBBox
 
-  This only adds the position to the \a box. */
+  This only adds the position (or the snap positions) to the \a box. */
 void Reference::addToBBox(Rect &box, const Matrix &m, bool cp) const
 {
-  box.addPoint((m * matrix()) * iPos);
+  if (iSnap.size() > 0) {
+    for (const Vector & pos : iSnap)
+      box.addPoint((m * matrix()) * (iPos + pos));
+  } else
+    box.addPoint((m * matrix()) * iPos);
 }
 
 void Reference::checkStyle(const Cascade *sheet, AttributeSeq &seq) const
 {
-  if (!sheet->findSymbol(iName)) {
+  const Symbol *symbol = sheet->findSymbol(iName);
+  if (!symbol) {
     if (std::find(seq.begin(), seq.end(), iName) == seq.end())
       seq.push_back(iName);
+  } else {
+    iSnap = symbol->iSnap;  // cache snap positions
   }
   if (iFlags & EHasStroke)
     checkSymbol(EColor, iStroke, sheet, seq);
@@ -226,15 +251,28 @@ void Reference::checkStyle(const Cascade *sheet, AttributeSeq &seq) const
     checkSymbol(ESymbolSize, iSize, sheet, seq);
 }
 
-double Reference::distance(const Vector &v, const Matrix &m, double) const
+double Reference::distance(const Vector &v, const Matrix &m, double bound) const
 {
-  return (v - (m * (matrix() * iPos))).len();
+  if (iSnap.size() > 0) {
+    double d = bound;
+    for (const Vector & snapPos : iSnap) {
+      double d1 = (v - (m * (matrix() * (iPos + snapPos)))).len();
+      if (d1 < d)
+	d = d1;
+    }
+    return d;
+  } else
+    return (v - (m * (matrix() * iPos))).len();
 }
 
 void Reference::snapVtx(const Vector &mouse, const Matrix &m,
 			Vector &pos, double &bound) const
 {
-  (m * (matrix() * iPos)).snap(mouse, pos, bound);
+  if (iSnap.size() > 0) {
+    for (const Vector & snapPos : iSnap)
+      (m * (matrix() * (iPos + snapPos))).snap(mouse, pos, bound);
+  } else
+    (m * (matrix() * iPos)).snap(mouse, pos, bound);
 }
 
 void Reference::snapBnd(const Vector &, const Matrix &,
@@ -345,9 +383,9 @@ Attribute Reference::getAttribute(Property prop) const noexcept
 
 // --------------------------------------------------------------------
 
-uint Reference::flagsFromName(String name)
+uint32_t Reference::flagsFromName(String name)
 {
-  uint flags = 0;
+  uint32_t flags = 0;
   if (name.left(5) == "mark/")
     flags |= EIsMark;
   if (name.left(6) == "arrow/")
