@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2016  Otfried Cheong
+    Copyright (c) 1993-2019 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -126,9 +126,9 @@ void CurveSegment::draw(Painter &painter) const
   case ESpline: {
     std::vector<Bezier> bez;
     beziers(bez);
-    for (uint i = 0; i < bez.size(); ++i)
-      painter.curveTo(bez[i]); }
-    break;
+    for (const auto & b : bez)
+      painter.curveTo(b);
+    break; }
   case EArc:
     painter.drawArc(arc());
     break;
@@ -163,8 +163,8 @@ void CurveSegment::addToBBox(Rect &box, const Matrix &m, bool cpf) const
     } else {
       std::vector<Bezier> bez;
       beziers(bez);
-      for (uint i = 0; i < bez.size(); ++i)
-	box.addRect((m * bez[i]).bbox());
+      for (const auto & b : bez)
+	box.addRect((m * b).bbox());
     }
     break;
   }
@@ -185,8 +185,8 @@ double CurveSegment::distance(const Vector &v, const Matrix &m,
     beziers(bez);
     double d = bound;
     double d1;
-    for (uint i = 0; i < bez.size(); ++i) {
-      if ((d1 = (m * bez[i]).distance(v, d)) < d)
+    for (const auto & b : bez) {
+      if ((d1 = (m * b).distance(v, d)) < d)
 	d = d1;
     }
     return d; }
@@ -251,8 +251,8 @@ void CurveSegment::snapBnd(const Vector &mouse, const Matrix &m,
   case ESpline: {
     std::vector<Bezier> bez;
     beziers(bez);
-    for (uint i = 0; i < bez.size(); ++i)
-      snapBezier(mouse, m * bez[i], pos, bound);
+    for (const auto & b : bez)
+      snapBezier(mouse, m * b, pos, bound);
     break; }
   }
 }
@@ -307,7 +307,7 @@ void Curve::appendSpline(const std::vector<Vector> &v, CurveSegment::Type type)
   if (iSeg.empty())
     iCP.push_back(v[0]);
   assert(v[0] == iCP.back());
-  for (uint i = 1; i < v.size(); ++i)
+  for (int i = 1; i < size(v); ++i)
     iCP.push_back(v[i]);
   Seg seg;
   seg.iType = type;
@@ -449,7 +449,7 @@ CurveSegment Curve::closingSegment(Vector u[2]) const
   assert(iClosed);
   u[0] = iCP.back();
   u[1] = iCP.front();
-  return CurveSegment(CurveSegment::ESegment, 2, u, 0);
+  return CurveSegment(CurveSegment::ESegment, 2, u, nullptr);
 }
 
 // --------------------------------------------------------------------
@@ -475,38 +475,22 @@ bool SubPath::closed() const
   return true;
 }
 
-//! Return this object as an Ellipse, or 0 if it's not an ellipse.
+//! Return this object as an Ellipse, or nullptr if it's not an ellipse.
 const Ellipse *SubPath::asEllipse() const
 {
-  return 0;
+  return nullptr;
 }
 
-/*
-//! Return this object as an Ellipse, or 0 if it's not an ellipse.
-Ellipse *SubPath::asEllipse()
-{
-  return 0;
-}
-*/
-
-//! Return this object as an ClosedSpline, or 0 if it's not a closed spline.
+//! Return this object as an ClosedSpline, or nullptr if it's not a closed spline.
 const ClosedSpline *SubPath::asClosedSpline() const
 {
-  return 0;
+  return nullptr;
 }
 
-/*
-//! Return this object as an ClosedSpline, or 0 if it's not a closed spline.
-ClosedSpline *SubPath::asClosedSpline()
-{
-  return 0;
-}
-*/
-
-//! Return this object as an Curve, or else 0.
+//! Return this object as an Curve, or else nullptr.
 const Curve *SubPath::asCurve() const
 {
-  return 0;
+  return nullptr;
 }
 
 // --------------------------------------------------------------------
@@ -600,7 +584,7 @@ const ClosedSpline *ClosedSpline::asClosedSpline() const
 
 void ClosedSpline::save(Stream &stream) const
 {
-  for (uint i = 0; i < iCP.size() - 1; ++i)
+  for (int i = 0; i < size(iCP) - 1; ++i)
     stream << iCP[i] << "\n";
   stream << iCP.back() << " u\n";
 }
@@ -610,21 +594,21 @@ void ClosedSpline::draw(Painter &painter) const
   std::vector<Bezier> bez;
   beziers(bez);
   painter.moveTo(bez.front().iV[0]);
-  for (uint i = 0; i < bez.size(); ++i)
-    painter.curveTo(bez[i]);
+  for (const auto & b : bez)
+    painter.curveTo(b);
   painter.closePath();
 }
 
 void ClosedSpline::addToBBox(Rect &box, const Matrix &m, bool cpf) const
 {
   if (cpf) {
-    for (uint i = 0; i < iCP.size(); ++i)
-      box.addPoint(m * iCP[i]);
+    for (const auto & cp : iCP)
+      box.addPoint(m * cp);
   } else {
     std::vector<Bezier> bez;
     beziers(bez);
-    for (uint i = 0; i < bez.size(); ++i)
-      box.addRect((m * bez[i]).bbox());
+    for (const auto & b : bez)
+      box.addRect((m * b).bbox());
   }
 }
 
@@ -635,8 +619,8 @@ double ClosedSpline::distance(const Vector &v, const Matrix &m,
   beziers(bez);
   double d = bound;
   double d1;
-  for (uint i = 0; i < bez.size(); ++i) {
-    if ((d1 = (m * bez[i]).distance(v, d)) < d)
+  for (const auto & b : bez) {
+    if ((d1 = (m * b).distance(v, d)) < d)
       d = d1;
   }
   return d;
@@ -652,8 +636,8 @@ void ClosedSpline::snapVtx(const Vector &mouse, const Matrix &m,
 {
   if (ctl) {
     // snap to control points
-    for (uint i = 0; i < iCP.size(); ++i)
-      snapVertex(mouse, m * iCP[i], pos, bound);
+    for (const auto & cp : iCP)
+      snapVertex(mouse, m * cp, pos, bound);
   }
 }
 
@@ -662,8 +646,8 @@ void ClosedSpline::snapBnd(const Vector &mouse, const Matrix &m,
 {
   std::vector<Bezier> bez;
   beziers(bez);
-  for (uint i = 0; i < bez.size(); ++i)
-    snapBezier(mouse, m * bez[i], pos, bound);
+  for (const auto & b : bez)
+    snapBezier(mouse, m * b, pos, bound);
 }
 
 // --------------------------------------------------------------------
@@ -867,7 +851,7 @@ Shape::Imp::~Imp()
   for (SubPathSeq::iterator it = iSubPaths.begin();
        it != iSubPaths.end(); ++it) {
     delete *it;
-    *it = 0;
+    *it = nullptr;
   }
 }
 
@@ -909,7 +893,7 @@ bool Shape::load(String data)
   Lex stream(data);
   String word;
   String type;
-  Curve *sp = 0;
+  Curve *sp = nullptr;
   Vector org;
   std::vector<double> args;
   do {
@@ -918,7 +902,7 @@ bool Shape::load(String data)
 	return false;
       stream.nextToken(); // eat token
       sp->setClosed(true);
-      sp = 0;
+      sp = nullptr;
     } else if (stream.token() == "m") {
       if (args.size() != 2)
 	return false;
@@ -966,14 +950,14 @@ bool Shape::load(String data)
       if (args.size() != 6)
 	return false;
       stream.nextToken();
-      sp = 0;
+      sp = nullptr;
       Ellipse *e = new Ellipse(getMatrix(args));
       appendSubPath(e);
     } else if (stream.token() == "u") {
       if (args.size() < 6 || (args.size() % 2 != 0))
 	return false;
       stream.nextToken();
-      sp = 0;
+      sp = nullptr;
       std::vector<Vector> v;
       while (!args.empty())
 	v.push_back(getVector(args));
