@@ -112,8 +112,8 @@ void CollectSegs::visitPath(const Path *obj)
       std::vector<Bezier> bez;
       bool cont = false;
       sp->asClosedSpline()->beziers(bez);
-      for (uint i = 0; i < bez.size(); ++i) {
-	b = m * bez[i];
+      for (const Bezier & bz : bez) {
+	b = m * bz;
 	if (b.distance(iMouse, iDist) < iDist) {
 	  iBeziers.push_back(b);
 	  iBeziersCont.push_back(cont);
@@ -143,8 +143,8 @@ void CollectSegs::visitPath(const Path *obj)
 	  std::vector<Bezier> bez;
 	  seg.beziers(bez);
 	  bool cont = false;
-	  for (uint i = 0; i < bez.size(); ++i) {
-	    b = m * bez[i];
+	  for (const Bezier & bz : bez) {
+	    b = m * bz;
 	    if (b.distance(iMouse, iDist) < iDist) {
 	      iBeziers.push_back(b);
 	      iBeziersCont.push_back(cont);
@@ -188,40 +188,40 @@ void Snap::intersectionSnap(const Vector &pos,  Vector &fifi,
   std::vector<Vector> pts;
 
   // 1. seg-seg intersections
-  for (uint i = 0; i < segs.iSegs.size(); ++i) {
-    for (uint j = i + 1; j < segs.iSegs.size(); ++j) {
+  for (int i = 0; i < size(segs.iSegs); ++i) {
+    for (int j = i + 1; j < size(segs.iSegs); ++j) {
       if (segs.iSegs[i].intersects(segs.iSegs[j], v))
 	pts.push_back(v);
     }
   }
 
   // 2. bezier-bezier and bezier-seg intersections
-  for (uint i = 0; i < segs.iBeziers.size(); ++i) {
-    for (uint j = i + 1; j < segs.iBeziers.size(); ++j) {
+  for (int i = 0; i < size(segs.iBeziers); ++i) {
+    for (int j = i + 1; j < size(segs.iBeziers); ++j) {
       if (j > i+1 || !segs.iBeziersCont[j])
 	segs.iBeziers[i].intersect(segs.iBeziers[j], pts);
     }
-    for (uint j = 0; j < segs.iSegs.size(); ++j)
+    for (int j = 0; j < size(segs.iSegs); ++j)
       segs.iBeziers[i].intersect(segs.iSegs[j], pts);
   }
 
   // 3. arc-arc, arc-bezier, and arc-segment intersections
-  for (uint i = 0; i < segs.iArcs.size(); ++i) {
-    for (uint j = i+1; j < segs.iArcs.size(); ++j)
+  for (int i = 0; i < size(segs.iArcs); ++i) {
+    for (int j = i+1; j < size(segs.iArcs); ++j)
       segs.iArcs[i].intersect(segs.iArcs[j], pts);
-    for (uint j = 0; j < segs.iBeziers.size(); ++j)
+    for (int j = 0; j < size(segs.iBeziers); ++j)
       segs.iArcs[i].intersect(segs.iBeziers[j], pts);
-    for (uint j = 0; j < segs.iSegs.size(); ++j)
+    for (int j = 0; j < size(segs.iSegs); ++j)
       segs.iArcs[i].intersect(segs.iSegs[j], pts);
   }
 
   double d = snapDist;
   Vector pos1 = pos;
   double d1;
-  for (uint k = 0; k < pts.size(); ++k) {
-    if ((d1 = (pos - pts[k]).len()) < d) {
+  for (const auto & pt : pts) {
+    if ((d1 = (pos - pt).len()) < d) {
       d = d1;
-      pos1 = pts[k];
+      pos1 = pt;
     }
   }
 
@@ -259,10 +259,10 @@ bool Snap::snapAngularIntersection(Vector &pos, const Line &l,
   Vector pos1 = pos;
   double d1;
 
-  for (uint i = 0; i < pts.size(); ++i) {
-    if ((d1 = (pos - pts[i]).len()) < d) {
+  for (const auto & pt : pts) {
+    if ((d1 = (pos - pt).len()) < d) {
       d = d1;
-      pos1 = pts[i];
+      pos1 = pt;
     }
   }
 
