@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2016  Otfried Cheong
+    Copyright (c) 1993-2019 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -58,7 +58,7 @@ static const char *styleStandard[] = {
   "<opacity name=\"opaque\" value=\"1\"/>",
   "<textpad left=\"1\" right=\"1\" top=\"1\" bottom=\"1\"/>",
   "</ipestyle>",
-  0 };
+  nullptr };
 
 class StandardStyleSource : public DataSource {
 public:
