@@ -48,7 +48,7 @@ using namespace ipe;
 //! Default constructor
 Symbol::Symbol()
 {
-  iObject = 0;
+  iObject = nullptr;
   iXForm = false;
   iTransformations = ETransformationsAffine;
 }
@@ -64,9 +64,10 @@ Symbol::Symbol(Object *object)
 //! Copy constructor.
 Symbol::Symbol(const Symbol &rhs)
 {
-  iObject = rhs.iObject ? rhs.iObject->clone() : 0;
+  iObject = rhs.iObject ? rhs.iObject->clone() : nullptr;
   iXForm = rhs.iXForm;
   iTransformations = rhs.iTransformations;
+  iSnap = rhs.iSnap;
 }
 
 //! Assignment operator.
@@ -74,9 +75,10 @@ Symbol &Symbol::operator=(const Symbol &rhs)
 {
   if (this != &rhs) {
     delete iObject;
-    iObject = rhs.iObject ? rhs.iObject->clone() : 0;
+    iObject = rhs.iObject ? rhs.iObject->clone() : nullptr;
     iXForm = rhs.iXForm;
     iTransformations = rhs.iTransformations;
+    iSnap = rhs.iSnap;
   }
   return *this;
 }
@@ -134,7 +136,7 @@ void StyleSheet::setLayout(const Layout &layout)
 const Layout *StyleSheet::layout() const
 {
   if (iLayout.isNull())
-    return 0;
+    return nullptr;
   else
     return &iLayout;
 }
@@ -143,7 +145,7 @@ const Layout *StyleSheet::layout() const
 const TextPadding *StyleSheet::textPadding() const
 {
   if (iTextPadding.iLeft < 0)
-    return 0;
+    return nullptr;
   else
     return &iTextPadding;
 }
@@ -166,7 +168,7 @@ const StyleSheet::TitleStyle *StyleSheet::titleStyle() const
   if (iTitleStyle.iDefined)
     return &iTitleStyle;
   else
-    return 0;
+    return nullptr;
 }
 
 //! Set style of page numbering.
@@ -195,12 +197,12 @@ void StyleSheet::addGradient(Attribute name, const Gradient &s)
 const Gradient *StyleSheet::findGradient(Attribute sym) const
 {
   if (!sym.isSymbolic())
-    return 0;
+    return nullptr;
   GradientMap::const_iterator it = iGradients.find(sym.index());
   if (it != iGradients.end())
     return &it->second;
   else
-    return 0;
+    return nullptr;
 }
 
 //! Add tiling to this style sheet.
@@ -214,12 +216,12 @@ void StyleSheet::addTiling(Attribute name, const Tiling &s)
 const Tiling *StyleSheet::findTiling(Attribute sym) const
 {
   if (!sym.isSymbolic())
-    return 0;
+    return nullptr;
   TilingMap::const_iterator it = iTilings.find(sym.index());
   if (it != iTilings.end())
     return &it->second;
   else
-    return 0;
+    return nullptr;
 }
 
 void StyleSheet::addEffect(Attribute name, const Effect &e)
@@ -231,12 +233,12 @@ void StyleSheet::addEffect(Attribute name, const Effect &e)
 const Effect *StyleSheet::findEffect(Attribute sym) const
 {
   if (!sym.isSymbolic())
-    return 0;
+    return nullptr;
   EffectMap::const_iterator it = iEffects.find(sym.index());
   if (it != iEffects.end())
     return &it->second;
   else
-    return 0;
+    return nullptr;
 }
 
 // --------------------------------------------------------------------
@@ -273,12 +275,12 @@ void StyleSheet::addSymbol(Attribute name, const Symbol &symbol)
 const Symbol *StyleSheet::findSymbol(Attribute attr) const
 {
   if (!attr.isSymbolic())
-    return 0;
+    return nullptr;
   SymbolMap::const_iterator it = iSymbols.find(attr.index());
   if (it != iSymbols.end())
     return &it->second;
   else
-    return 0;
+    return nullptr;
 }
 
 // --------------------------------------------------------------------
@@ -397,9 +399,9 @@ void StyleSheet::allNames(Kind kind, AttributeSeq &seq) const
 	seq.push_back(attr);
     }
   } else {
-    uint kindMatch = (kind << SHIFT);
+    uint32_t kindMatch = (kind << SHIFT);
     for (Map::const_iterator it = iMap.begin(); it != iMap.end(); ++it) {
-      if (uint(it->first & KINDMASK) == kindMatch) {
+      if (uint32_t(it->first & KINDMASK) == kindMatch) {
 	Attribute attr(true, (it->first & MASK));
 	if (std::find(seq.begin(), seq.end(), attr) == seq.end())
 	  seq.push_back(attr);
@@ -450,6 +452,15 @@ void StyleSheet::saveAsXml(Stream &stream, bool saveBitmaps) const
       stream << " transformations=\"rigid\"";
     if (it->second.iXForm)
       stream << " xform=\"yes\"";
+    if (it->second.iSnap.size() > 0) {
+      stream << " snap=\"";
+      String sep = "";
+      for (const Vector & pos : it->second.iSnap) {
+	stream << sep << pos;
+	sep = " ";
+      }
+      stream << "\"";
+    }
     stream << ">\n";
     it->second.iObject->saveAsXml(stream, String());
     stream << "</symbol>\n";
@@ -541,7 +552,7 @@ void StyleSheet::saveAsXml(Stream &stream, bool saveBitmaps) const
     if (!g.iMatrix.isIdentity())
       stream << " matrix=\"" << g.iMatrix << "\"";
     stream << ">\n";
-    for (uint i = 0; i < g.iStops.size(); ++i)
+    for (int i = 0; i < size(g.iStops); ++i)
       stream << " <stop offset=\"" << g.iStops[i].offset
 	     << "\" color=\"" << g.iStops[i].color << "\"/>\n";
     stream << "</gradient>\n";
@@ -592,9 +603,9 @@ Cascade::Cascade()
 
 static void destruct_sheets(std::vector<StyleSheet *> &sheets)
 {
-  for (uint i = 0; i < sheets.size(); ++i) {
+  for (int i = 0; i < size(sheets); ++i) {
     delete sheets[i];
-    sheets[i] = 0;
+    sheets[i] = nullptr;
   }
   sheets.clear();
 }
@@ -678,7 +689,7 @@ const Symbol *Cascade::findSymbol(Attribute sym) const
     const Symbol *s = iSheets[i]->findSymbol(sym);
     if (s) return s;
   }
-  return 0;
+  return nullptr;
 }
 
 const Gradient *Cascade::findGradient(Attribute sym) const
@@ -687,7 +698,7 @@ const Gradient *Cascade::findGradient(Attribute sym) const
     const Gradient *s = iSheets[i]->findGradient(sym);
     if (s) return s;
   }
-  return 0;
+  return nullptr;
 }
 
 const Tiling *Cascade::findTiling(Attribute sym) const
@@ -696,7 +707,7 @@ const Tiling *Cascade::findTiling(Attribute sym) const
     const Tiling *s = iSheets[i]->findTiling(sym);
     if (s) return s;
   }
-  return 0;
+  return nullptr;
 }
 
 const Effect *Cascade::findEffect(Attribute sym) const
@@ -705,7 +716,7 @@ const Effect *Cascade::findEffect(Attribute sym) const
     const Effect *s = iSheets[i]->findEffect(sym);
     if (s) return s;
   }
-  return 0;
+  return nullptr;
 }
 
 //! Find page layout (such as text margins).
@@ -717,7 +728,7 @@ const Layout *Cascade::findLayout() const
   }
   // must never happen
   assert(false);
-  return 0;
+  return nullptr;
 }
 
 //! Find text padding (for text bbox computation).
@@ -729,7 +740,7 @@ const TextPadding *Cascade::findTextPadding() const
   }
   // must never happen
   assert(false);
-  return 0;
+  return nullptr;
 }
 
 //! Get style of page titles (or 0 if none defined).
@@ -739,7 +750,7 @@ const StyleSheet::TitleStyle *Cascade::findTitleStyle() const
     const StyleSheet::TitleStyle *ts = iSheets[i]->titleStyle();
     if (ts) return ts;
   }
-  return 0;
+  return nullptr;
 }
 
 //! Return style of page numbering (or 0 if none defined).
@@ -749,7 +760,7 @@ const StyleSheet::PageNumberStyle *Cascade::findPageNumberStyle() const
     const StyleSheet::PageNumberStyle *pns = iSheets[i]->pageNumberStyle();
     if (pns) return pns;
   }
-  return 0;
+  return nullptr;
 }
 
 //! Return total LaTeX preamble (of the whole cascade).
