@@ -59,7 +59,7 @@ using namespace ipe;
 //! Construct an empty internal text object.
 Text::Text() : Object()
 {
-  iXForm = 0;
+  iXForm = nullptr;
   iPos = Vector::ZERO;
   iType = TextType(0);
   iStroke = Attribute::BLACK();
@@ -77,7 +77,7 @@ Text::Text(const AllAttributes &attr, String data,
 	   const Vector &pos, TextType type, double width)
   : Object(attr)
 {
-  iXForm = 0;
+  iXForm = nullptr;
   iText = data;
   iStroke = attr.iStroke;
   iOpacity = attr.iOpacity;
@@ -133,7 +133,7 @@ Text::~Text()
 Text::Text(const XmlAttributes &attr, String data)
   : Object(attr)
 {
-  iXForm = 0;
+  iXForm = nullptr;
   iText = data;
 
   iStroke = Attribute::makeColor(attr["stroke"], Attribute::BLACK());
@@ -179,6 +179,9 @@ Text::Text(const XmlAttributes &attr, String data)
 
   if (iType == ELabel && iStyle == Attribute::NORMAL() &&
       iText.size() >= 3 && iText[0] == '$' && iText[iText.size()-1] == '$') {
+    for (int i = 1; i < iText.size() - 1; ++i)  // check if no other $ in text
+      if (iText[i] == '$')
+	return;
     iStyle = Attribute(true, "math");
     iText = iText.substr(1, iText.size() - 2);
   }
@@ -400,7 +403,7 @@ void Text::setWidth(double width)
 {
   assert(textType() == EMinipage);
   iWidth = width;
-  setXForm(0);
+  setXForm(nullptr);
 }
 
 //! Set font size of text.
@@ -408,7 +411,7 @@ void Text::setWidth(double width)
 void Text::setSize(Attribute size)
 {
   iSize = size;
-  setXForm(0);
+  setXForm(nullptr);
 }
 
 //! Set Latex style of text.
