@@ -30,6 +30,8 @@
 
 #include "ipelua.h"
 
+#include <cstring>
+
 using namespace ipe;
 using namespace ipelua;
 
@@ -62,7 +64,7 @@ int ipelua::direction_constructor(lua_State *L)
 static int vector_get(lua_State *L)
 {
   Vector *v = check_vector(L, 1);
-  const char *key = lua_tostring(L, 2);
+  const char *key = lua_tolstring(L, 2, nullptr);
   if (!strcmp(key, "x"))
     lua_pushnumber(L, v->x);
   else if (!strcmp(key, "y"))
@@ -183,14 +185,15 @@ static const struct luaL_Reg vector_methods[] = {
   { "__sub", vector_sub },
   { "__eq", vector_eq },
   { "__mul", vector_mul },
-  { "__concat", vector_dot },
+  { "__concat", vector_dot },  // for historical reasons
+  { "__pow", vector_dot },
   { "len", vector_len },
   { "sqLen", vector_sqLen },
   { "normalized", vector_normalized },
   { "orthogonal", vector_orthogonal },
   { "factorize", vector_factorize },
   { "angle", vector_angle },
-  { NULL, NULL }
+  { nullptr, nullptr },
 };
 
 // --------------------------------------------------------------------
@@ -351,7 +354,7 @@ static const struct luaL_Reg matrix_methods[] = {
   { "isSingular", matrix_isSingular },
   { "inverse", matrix_inverse },
   { "elements", matrix_elements },
-  { NULL, NULL }
+  { nullptr, nullptr }
 };
 
 // --------------------------------------------------------------------
@@ -509,7 +512,7 @@ static const struct luaL_Reg rect_methods[] = {
   { "clipTo", rect_clipTo },
   { "contains", rect_contains },
   { "intersects", rect_intersects },
-  { NULL, NULL }
+  { nullptr, nullptr }
 };
 
 // --------------------------------------------------------------------
@@ -629,7 +632,7 @@ static const struct luaL_Reg line_methods[] = {
   { "distance", line_distance },
   { "intersects", line_intersects },
   { "project", line_project },
-  { NULL, NULL }
+  { nullptr, nullptr }
 };
 
 // --------------------------------------------------------------------
@@ -720,7 +723,7 @@ static const struct luaL_Reg segment_methods[] = {
   { "project", segment_project },
   { "distance", segment_distance },
   { "intersects", segment_intersects },
-  { NULL, NULL }
+  { nullptr, nullptr }
 };
 
 // --------------------------------------------------------------------
@@ -825,7 +828,7 @@ static const struct luaL_Reg bezier_methods[] = {
   { "bbox", bezier_bbox },
   { "intersect", bezier_intersect },
   { "snap", bezier_snap },
-  { NULL, NULL }
+  { nullptr, nullptr }
 };
 
 // --------------------------------------------------------------------
@@ -950,7 +953,7 @@ static const struct luaL_Reg arc_methods[] = {
   { "isEllipse", arc_isEllipse },
   { "intersect", arc_intersect },
   { "snap", arc_snap },
-  { NULL, NULL }
+  { nullptr, nullptr }
 };
 
 // --------------------------------------------------------------------
