@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2016  Otfried Cheong
+    Copyright (c) 1993-2019 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -33,6 +33,7 @@
 #ifdef WIN32
 #include <windows.h>
 #else
+#include <cstdlib>
 #include <dlfcn.h>
 #endif
 
@@ -43,12 +44,62 @@ typedef Ipelet *(*PNewIpeletFn)();
 
 // --------------------------------------------------------------------
 
+static void snapFlag(lua_State *L, int &flags, const char *mode, uint32_t bits)
+{
+  lua_getfield(L, -1, mode);
+  if (!lua_isnil(L, -1))
+    flags = lua_toboolean(L, -1) ? (flags | bits) : (flags & ~bits);
+  lua_pop(L, 1);
+}
+
+void ipelua::get_snap(lua_State *L, int i, ipe::Snap &snap)
+{
+  luaL_checktype(L, i, LUA_TTABLE);
+  snapFlag(L, snap.iSnap, "snapvtx", Snap::ESnapVtx);
+  snapFlag(L, snap.iSnap, "snapctl", Snap::ESnapCtl);
+  snapFlag(L, snap.iSnap, "snapbd", Snap::ESnapBd);
+  snapFlag(L, snap.iSnap, "snapint", Snap::ESnapInt);
+  snapFlag(L, snap.iSnap, "snapgrid", Snap::ESnapGrid);
+  snapFlag(L, snap.iSnap, "snapangle", Snap::ESnapAngle);
+  snapFlag(L, snap.iSnap, "snapauto", Snap::ESnapAuto);
+  lua_getfield(L, i, "grid_visible");
+  if (!lua_isnil(L, -1))
+    snap.iGridVisible = lua_toboolean(L, -1);
+  lua_pop(L, 1);
+  lua_getfield(L, i, "gridsize");
+  if (!lua_isnil(L, -1))
+    snap.iGridSize = (int) luaL_checkinteger(L, -1);
+  lua_pop(L, 1);
+  lua_getfield(L, i, "anglesize");
+  if (!lua_isnil(L, -1))
+    snap.iAngleSize = IpePi * luaL_checknumber(L, -1) / 180.0;
+  lua_pop(L, 1);
+  lua_getfield(L, i, "snap_distance");
+  if (!lua_isnil(L, -1))
+    snap.iSnapDistance = (int) luaL_checkinteger(L, -1);
+  lua_pop(L, 1);
+  lua_getfield(L, i, "with_axes");
+  if (!lua_isnil(L, -1))
+    snap.iWithAxes = lua_toboolean(L, -1);
+  lua_pop(L, 1);
+  lua_getfield(L, i, "origin");
+  if (is_type(L, -1, "Ipe.vector"))
+    snap.iOrigin = *check_vector(L, -1);
+  lua_pop(L, 1);
+  lua_getfield(L, i, "orientation");
+  if (!lua_isnil(L, -1))
+    snap.iDir = luaL_checknumber(L, -1);
+  lua_pop(L, 1);
+}
+
+// --------------------------------------------------------------------
+
 static int ipelet_destructor(lua_State *L)
 {
   ipeDebug("Ipelet destructor");
   Ipelet **p = check_ipelet(L, 1);
   delete *p;
-  *p = 0;
+  *p = nullptr;
   return 0;
 }
 
@@ -63,18 +114,16 @@ static int ipelet_tostring(lua_State *L)
 
 int ipelua::ipelet_constructor(lua_State *L)
 {
-  String fname(luaL_checkstring(L, 1));
+  String fname(luaL_checklstring(L, 1, nullptr));
 #if defined(WIN32)
   String dllname = fname + ".dll";
 #else
   String dllname = fname + ".so";
 #endif
   ipeDebug("Loading dll '%s'", dllname.z());
-  PNewIpeletFn pIpelet = 0;
+  PNewIpeletFn pIpelet = nullptr;
 #ifdef WIN32
-  std::vector<wchar_t> wname;
-  Platform::utf8ToWide(dllname.z(), wname);
-  HMODULE hMod = LoadLibraryW(&wname[0]);
+  HMODULE hMod = LoadLibraryW(dllname.w().data());
   if (hMod) {
     pIpelet = (PNewIpeletFn) GetProcAddress(hMod, "newIpelet");
     if (!pIpelet)
@@ -82,12 +131,6 @@ int ipelua::ipelet_constructor(lua_State *L)
   }
 #else
   void *handle = dlopen(dllname.z(), RTLD_NOW);
-#if defined(__APPLE__)
-  if (!handle) {
-    dllname = fname + ".dylib";
-    handle = dlopen(dllname.z(), RTLD_NOW);
-  }
-#endif
   if (handle) {
     pIpelet = (PNewIpeletFn) dlsym(handle, "newIpelet");
     if (!pIpelet)
@@ -96,12 +139,12 @@ int ipelua::ipelet_constructor(lua_State *L)
 #endif
   if (pIpelet) {
     Ipelet **p = (Ipelet **) lua_newuserdata(L, sizeof(Ipelet *));
-    *p = 0;
+    *p = nullptr;
     luaL_getmetatable(L, "Ipe.ipelet");
     lua_setmetatable(L, -2);
 
     Ipelet *ipelet = pIpelet();
-    if (ipelet == 0) {
+    if (ipelet == nullptr) {
       lua_pushnil(L);
       lua_pushstring(L, "ipelet returns no object");
       return 2;
@@ -127,6 +170,10 @@ int ipelua::ipelet_constructor(lua_State *L)
 		      dllname.z());
 #else
     lua_pushfstring(L, "Error loading Ipelet '%s': %s", dllname.z(), dlerror());
+#if defined(__APPLE__)
+    ipeDebug("DYLD_LIBRARY_PATH=%s", getenv("DYLD_LIBRARY_PATH"));
+    ipeDebug("DYLD_FALLBACK_LIBRARY_PATH=%s", getenv("DYLD_FALLBACK_LIBRARY_PATH"));
+#endif
 #endif
     return 2;
   }
@@ -142,6 +189,7 @@ public:
   virtual int messageBox(const char *text, const char *details,
 			 int buttons);
   virtual bool getString(const char *prompt, String &str);
+  virtual String getParameter(const char *key);
 private:
   lua_State *L;
   int iHelper;
@@ -165,7 +213,7 @@ void Helper::message(const char *msg)
   lua_pushvalue(L, -2); // luahelper
   lua_remove(L, -3);
   lua_pushstring(L, msg);
-  lua_call(L, 2, 0);
+  luacall(L, 2, 0);
 }
 
 int Helper::messageBox(const char *text, const char *details, int buttons)
@@ -180,9 +228,9 @@ int Helper::messageBox(const char *text, const char *details, int buttons)
   else
     lua_pushnil(L);
   lua_pushnumber(L, buttons);
-  lua_call(L, 4, 1);
+  luacall(L, 4, 1);
   if (lua_isnumber(L, -1))
-    return int(lua_tonumber(L, -1));
+    return int(lua_tonumberx(L, -1, nullptr));
   else
     return 0;
 }
@@ -194,14 +242,31 @@ bool Helper::getString(const char *prompt, String &str)
   lua_pushvalue(L, -2); // luahelper
   lua_remove(L, -3);
   lua_pushstring(L, prompt);
-  lua_call(L, 2, 1);
+  push_string(L, str);
+  luacall(L, 3, 1);
   if (lua_isstring(L, -1)) {
-    str = lua_tostring(L, -1);
+    str = lua_tolstring(L, -1, nullptr);
     return true;
   } else
     return false;
 }
 
+String Helper::getParameter(const char *key)
+{
+  lua_rawgeti(L, LUA_REGISTRYINDEX, iHelper);
+  lua_getfield(L, -1, "parameters");
+  String value;
+  if (lua_istable(L, -1)) {
+    lua_getfield(L, -1, key);
+    const char *t = lua_tolstring(L, -1, nullptr);
+    if (t)
+      value = t;
+    lua_pop(L, 1); // parameters[key]
+  }
+  lua_pop(L, 2); // helper, parameters
+  return value;
+}
+
 // --------------------------------------------------------------------
 
 static int ipelet_run(lua_State *L)
@@ -215,8 +280,8 @@ static int ipelet_run(lua_State *L)
   data.iView = (int)luaL_checkinteger(L, 6);
   data.iLayer = check_layer(L, 7, data.iPage);
   check_allattributes(L, 8, data.iAttributes);
-  // data.iSnap;
-  lua_pushvalue(L, 9);
+  get_snap(L, 9, data.iSnap);
+  lua_pushvalue(L, 10);
   int luahelper = luaL_ref(L, LUA_REGISTRYINDEX);
   Helper helper(L, luahelper);
   bool result = (*p)->run(num, &data, &helper);
@@ -230,7 +295,7 @@ static const struct luaL_Reg ipelet_methods[] = {
   { "__tostring", ipelet_tostring },
   { "__gc", ipelet_destructor },
   { "run", ipelet_run },
-  { NULL, NULL }
+  { nullptr, nullptr }
 };
 
 // --------------------------------------------------------------------
