@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2016  Otfried Cheong
+    Copyright (c) 1993-2019 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -43,31 +43,31 @@ using namespace ipelua;
 // --------------------------------------------------------------------
 
 static const char *const type_names[] =
-  { "group", "path", "text", "image", "reference", NULL };
+  { "group", "path", "text", "image", "reference", nullptr };
 
 static const char *const pinned_names[] =
-  { "none", "horizontal", "vertical", "fixed", 0 };
+  { "none", "horizontal", "vertical", "fixed", nullptr };
 
 static const char *const pathmode_names[] =
-  { "stroked", "strokedfilled", "filled", 0 };
+  { "stroked", "strokedfilled", "filled", nullptr };
 
 static const char *const transformation_names[] =
-  { "translations", "rigid", "affine", 0 };
+  { "translations", "rigid", "affine", nullptr };
 
 static const char *const horizontal_alignment_names[] =
-  { "left", "right", "hcenter", 0 };
+  { "left", "right", "hcenter", nullptr };
 
 static const char *const vertical_alignment_names[] =
-  { "bottom", "baseline", "top", "vcenter", 0 };
+  { "bottom", "baseline", "top", "vcenter", nullptr };
 
 const char *const ipelua::linejoin_names[] =
-  { "normal", "miter", "round", "bevel", 0 };
+  { "normal", "miter", "round", "bevel", nullptr };
 
 const char *const ipelua::linecap_names[] =
-  { "normal", "butt", "round", "square", 0 };
+  { "normal", "butt", "round", "square", nullptr };
 
 const char *const ipelua::fillrule_names[] =
-  { "normal", "wind", "evenodd", 0 };
+  { "normal", "wind", "evenodd", nullptr };
 
 // --------------------------------------------------------------------
 
@@ -134,7 +134,7 @@ Attribute ipelua::check_attribute(lua_State *L, int i)
     double v = luaL_checknumber(L, i);
     return Attribute(Fixed::fromInternal(int(v * 1000 + 0.5)));
   } else if (lua_type(L, i) == LUA_TSTRING) {
-    const char *s = luaL_checkstring(L, i);
+    const char *s = luaL_checklstring(L, i, nullptr);
     if (!strcmp(s, "true"))
       return Attribute::Boolean(true);
     if (!strcmp(s, "false"))
@@ -159,7 +159,7 @@ Attribute ipelua::check_attribute(lua_State *L, int i)
 Attribute ipelua::check_color_attribute(lua_State *L, int i)
 {
   if (lua_type(L, i) == LUA_TSTRING) {
-    const char *s = luaL_checkstring(L, i);
+    const char *s = luaL_checklstring(L, i, nullptr);
     return Attribute(true, s);
   } else {
     Color color = check_color(L, i);
@@ -173,7 +173,7 @@ Attribute ipelua::check_bool_attribute(lua_State *L, int i)
   static const char * const bool_names[] = { "false", "true" };
   if (lua_type(L, i) == LUA_TBOOLEAN)
     return Attribute::Boolean(lua_toboolean(L, i));
-  int val = luaL_checkoption(L, i, 0, bool_names);
+  int val = luaL_checkoption(L, i, nullptr, bool_names);
   return Attribute::Boolean(val);
 }
 
@@ -184,7 +184,7 @@ Attribute ipelua::check_number_attribute(lua_State *L, int i)
     double v = luaL_checknumber(L, i);
     return Attribute(Fixed::fromInternal(int(v * 1000 + 0.5)));
   }
-  const char *s = luaL_checkstring(L, i);
+  const char *s = luaL_checklstring(L, i, nullptr);
   return Attribute(true, s);
 }
 
@@ -193,28 +193,28 @@ Attribute ipelua::check_property(Property prop, lua_State *L, int i)
   int val;
   switch (prop) {
   case EPropHorizontalAlignment:
-    val = luaL_checkoption(L, i, 0, horizontal_alignment_names);
+    val = luaL_checkoption(L, i, nullptr, horizontal_alignment_names);
     return Attribute(THorizontalAlignment(val));
   case EPropVerticalAlignment:
-    val = luaL_checkoption(L, i, 0, vertical_alignment_names);
+    val = luaL_checkoption(L, i, nullptr, vertical_alignment_names);
     return Attribute(TVerticalAlignment(val));
   case EPropLineJoin:
-    val = luaL_checkoption(L, i, 0, linejoin_names);
+    val = luaL_checkoption(L, i, nullptr, linejoin_names);
     return Attribute(TLineJoin(val));
   case EPropLineCap:
-    val = luaL_checkoption(L, i, 0, linecap_names);
+    val = luaL_checkoption(L, i, nullptr, linecap_names);
     return Attribute(TLineCap(val));
   case EPropFillRule:
-    val = luaL_checkoption(L, i, 0, fillrule_names);
+    val = luaL_checkoption(L, i, nullptr, fillrule_names);
     return Attribute(TFillRule(val));
   case EPropPinned:
-    val = luaL_checkoption(L, i, 0, pinned_names);
+    val = luaL_checkoption(L, i, nullptr, pinned_names);
     return Attribute(TPinned(val));
   case EPropTransformations:
-    val = luaL_checkoption(L, i, 0, transformation_names);
+    val = luaL_checkoption(L, i, nullptr, transformation_names);
     return Attribute(TTransformations(val));
   case EPropPathMode:
-    val = luaL_checkoption(L, i, 0, pathmode_names);
+    val = luaL_checkoption(L, i, nullptr, pathmode_names);
     return Attribute(TPathMode(val));
   case EPropPen:
   case EPropSymbolSize:
@@ -231,15 +231,16 @@ Attribute ipelua::check_property(Property prop, lua_State *L, int i)
   case EPropTextStyle:
   case EPropLabelStyle:
   case EPropOpacity:
+  case EPropStrokeOpacity:
   case EPropGradient:
   case EPropDecoration:
   case EPropTiling:  // symbolic string only
-    return Attribute(true, luaL_checkstring(L, i));
+    return Attribute(true, luaL_checklstring(L, i, nullptr));
   case EPropStrokeColor:
   case EPropFillColor:
     return check_color_attribute(L, i);
   case EPropDashStyle:
-    return Attribute::makeDashStyle(luaL_checkstring(L, i));
+    return Attribute::makeDashStyle(luaL_checklstring(L, i, nullptr));
   case EPropFArrow:
   case EPropRArrow:
   case EPropMinipage:
@@ -271,7 +272,7 @@ static int get_option(lua_State *L, int i, const char *key,
   lua_getfield(L, i, key);
   int val;
   if (!lua_isnil(L, -1))
-    val = luaL_checkoption(L, -1, 0, names);
+    val = luaL_checkoption(L, -1, nullptr, names);
   else
     val = -1;
   lua_pop(L, 1);
@@ -299,6 +300,7 @@ void ipelua::check_allattributes(lua_State *L, int i, AllAttributes &all)
   get_attribute(L, i, EPropTextStyle, "textstyle", all.iTextStyle);
   get_attribute(L, i, EPropTextStyle, "labelstyle", all.iLabelStyle);
   get_attribute(L, i, EPropOpacity, "opacity", all.iOpacity);
+  get_attribute(L, i, EPropStrokeOpacity, "strokeopacity", all.iStrokeOpacity);
   get_attribute(L, i, EPropTiling, "tiling", all.iTiling);
   get_attribute(L, i, EPropGradient, "gradient", all.iGradient);
 
@@ -334,7 +336,7 @@ int ipelua::reference_constructor(lua_State *L)
 {
   AllAttributes all;
   check_allattributes(L, 1, all);
-  Attribute name(true, luaL_checkstring(L, 2));
+  Attribute name(true, luaL_checklstring(L, 2, nullptr));
   Vector *v = check_vector(L, 3);
   Reference *r = new Reference(all, name, *v);
   push_object(L, r);
@@ -345,7 +347,7 @@ int ipelua::text_constructor(lua_State *L)
 {
   AllAttributes all;
   check_allattributes(L, 1, all);
-  const char *s = luaL_checkstring(L, 2);
+  const char *s = luaL_checklstring(L, 2, nullptr);
   Vector *v = check_vector(L, 3);
   double width = 10.0;
   Text::TextType type = Text::ELabel;
@@ -389,7 +391,7 @@ int ipelua::group_constructor(lua_State *L)
 
 int ipelua::xml_constructor(lua_State *L)
 {
-  String s = luaL_checkstring(L, 1);
+  String s = luaL_checklstring(L, 1, nullptr);
   Buffer buffer(s.data(), s.size());
   BufferSource source(buffer);
   ImlParser parser(source);
@@ -437,7 +439,7 @@ static int object_destructor(lua_State *L)
   SObject *r = check_object(L, 1);
   if (r->owned && r->obj)
     delete r->obj;
-  r->obj = 0;
+  r->obj = nullptr;
   return 0;
 }
 
@@ -460,7 +462,7 @@ static int object_type(lua_State *L)
 static int object_set(lua_State *L)
 {
   SObject *s = check_object(L, 1);
-  Property prop = Property(luaL_checkoption(L, 2, NULL, property_names));
+  Property prop = Property(luaL_checkoption(L, 2, nullptr, property_names));
   Attribute value = check_property(prop, L, 3);
   s->obj->setAttribute(prop, value);
   return 0;
@@ -469,7 +471,7 @@ static int object_set(lua_State *L)
 static int object_get(lua_State *L)
 {
   SObject *s = check_object(L, 1);
-  Property prop = Property(luaL_checkoption(L, 2, NULL, property_names));
+  Property prop = Property(luaL_checkoption(L, 2, nullptr, property_names));
   Attribute value = s->obj->getAttribute(prop);
   push_attribute(L, value);
   return 1;
@@ -506,7 +508,7 @@ static int object_text(lua_State *L)
 static int object_setText(lua_State *L)
 {
   Object *obj = check_object(L, 1)->obj;
-  String s = luaL_checkstring(L, 2);
+  String s = luaL_checklstring(L, 2, nullptr);
   if (obj->type() == Object::EGroup) {
     obj->asGroup()->setUrl(s);
   } else {
@@ -598,12 +600,12 @@ static int object_addToBBox(lua_State *L)
 // --------------------------------------------------------------------
 
 static const char * const segtype_names[] =
-  { "arc", "segment", "spline", "oldspline", 0 };
+  { "arc", "segment", "spline", "oldspline", nullptr };
 
 static int segtype_cp[] = { 2, 2, 0, 0 };
 
 static const char * const subpath_names[] =
-  { "curve", "ellipse", "closedspline", 0 };
+  { "curve", "ellipse", "closedspline", nullptr };
 
 static bool collect_cp(lua_State *L, std::vector<Vector> &cp)
 {
@@ -763,7 +765,7 @@ static void push_subpath(lua_State *L, const SubPath *sp)
     lua_createtable(L, cs->iCP.size(), 1);
     lua_pushstring(L, "closedspline");
     lua_setfield(L, -2, "type");
-    for (uint j = 0; j < cs->iCP.size(); ++j) {
+    for (int j = 0; j < size(cs->iCP); ++j) {
       push_vector(L, cs->iCP[j]);
       lua_rawseti(L, -2, j+1);
     }
@@ -854,32 +856,42 @@ static int object_symbol(lua_State *L)
 
 static int object_info(lua_State *L)
 {
-  static const char * const filters[] =
-    { "direct", "flatedecode", "dctdecode" };
-  static const char * const colorspaces[] =
-    { "devicergb", "devicegray", "devicecmyk" };
-
   Object *s = check_object(L, 1)->obj;
   luaL_argcheck(L, s->type() == Object::EImage, 1, "not an image object");
   Bitmap bm = s->asImage()->bitmap();
   lua_createtable(L, 0, 7);
-  lua_pushstring(L, colorspaces[bm.colorSpace()]);
-  lua_setfield(L, -2, "colorspace");
-  lua_pushstring(L, filters[bm.filter()]);
-  lua_setfield(L, -2, "filter");
-  lua_pushnumber(L, bm.components());
-  lua_setfield(L, -2, "components");
-  lua_pushnumber(L, bm.bitsPerComponent());
-  lua_setfield(L, -2, "bitsPerComponent");
   lua_pushnumber(L, bm.width());
   lua_setfield(L, -2, "width");
   lua_pushnumber(L, bm.height());
   lua_setfield(L, -2, "height");
-  lua_pushnumber(L, bm.size());
-  lua_setfield(L, -2, "size");
+  String format;
+  if (bm.isJpeg())
+    format = "jpg";
+  else {
+    if (bm.isGray())
+      format = "gray";
+    else
+      format = "rgb";
+    if (bm.hasAlpha())
+      format += " alpha";
+    else if (bm.colorKey() >= 0)
+      format += " colorkeyed";
+  }
+  push_string(L, format);
+  lua_setfield(L, -2, "format");
   return 1;
 }
 
+static int object_savePixels(lua_State *L)
+{
+  Object *s = check_object(L, 1)->obj;
+  luaL_argcheck(L, s->type() == Object::EImage, 1, "not an image object");
+  String fname = luaL_checklstring(L, 2, nullptr);
+  Bitmap bm = s->asImage()->bitmap();
+  bm.savePixels(fname.z());
+  return 0;
+}
+
 // --------------------------------------------------------------------
 
 static const struct luaL_Reg object_methods[] = {
@@ -901,13 +913,14 @@ static const struct luaL_Reg object_methods[] = {
   { "setClip", object_setclip },
   { "symbol", object_symbol },
   { "info", object_info },
+  { "savePixels", object_savePixels },
   { "position", object_position },
   { "text", object_text },
   { "setText", object_setText },
   { "elements", object_elements },
   { "element", object_element },
   { "elementType", object_elementType },
-  { NULL, NULL }
+  { nullptr, nullptr }
 };
 
 // --------------------------------------------------------------------
