@@ -57,7 +57,7 @@ static int check_objno(lua_State *L, int i, Page *p, int extra = 0)
 
 int ipelua::check_layer(lua_State *L, int i, Page *p)
 {
-  const char *name = luaL_checkstring(L, i);
+  const char *name = luaL_checklstring(L, i, nullptr);
   int l = p->findLayer(name);
   luaL_argcheck(L, l >= 0, i, "layer does not exist");
   return l;
@@ -96,7 +96,7 @@ static int page_destructor(lua_State *L)
   SPage *p = check_page(L, 1);
   if (p->owned)
     delete p->page;
-  p->page = 0;
+  p->page = nullptr;
   return 0;
 }
 
@@ -107,7 +107,7 @@ static int page_index(lua_State *L)
     int n = check_objno(L, 2, p);
     push_object(L, p->object(n), false);
   } else {
-    const char *key = luaL_checkstring(L, 2);
+    const char *key = luaL_checklstring(L, 2, nullptr);
     if (!luaL_getmetafield(L, 1, key))
       lua_pushnil(L);
   }
@@ -166,7 +166,7 @@ static int page_object_iterator(lua_State *L)
   int i = (int)luaL_checkinteger(L, 2);
   i = i + 1;
   if (i <= p->count()) {
-    lua_pushnumber(L, i);                            // new counter
+    lua_pushinteger(L, i);                           // new counter
     push_object(L, p->object(i-1), false);           // object
     push_select(L, p->select(i-1));
     push_string(L, p->layer(p->layerOf(i-1)));       // layer
@@ -182,7 +182,7 @@ static int page_objects(lua_State *L)
   (void) check_page(L, 1);
   lua_pushcfunction(L, page_object_iterator); // iterator function
   lua_pushvalue(L, 1);          // page
-  lua_pushnumber(L, 0);         // counter
+  lua_pushinteger(L, 0);        // counter
   return 3;
 }
 
@@ -191,9 +191,9 @@ static int page_objects(lua_State *L)
 static int page_xml(lua_State *L)
 {
   static const char * const option_names[] =
-    { "ipepage", "ipeselection", 0 };
+    { "ipepage", "ipeselection", nullptr };
   Page *p = check_page(L, 1)->page;
-  int t = luaL_checkoption(L, 2, 0, option_names);
+  int t = luaL_checkoption(L, 2, nullptr, option_names);
   String data;
   StringStream stream(data);
   if (t == 0)
@@ -257,8 +257,8 @@ static int page_setSnapping(lua_State *L)
 static int page_renameLayer(lua_State *L)
 {
   Page *p = check_page(L, 1)->page;
-  const char *s1 = luaL_checkstring(L, 2);
-  const char *s2 = luaL_checkstring(L, 3);
+  const char *s1 = luaL_checklstring(L, 2, nullptr);
+  const char *s2 = luaL_checklstring(L, 3, nullptr);
   p->renameLayer(s1, s2);
   return 0;
 }
@@ -269,7 +269,7 @@ static int page_addLayer(lua_State *L)
   if (lua_isnoneornil(L, 2)) {
     p->addLayer();
   } else {
-    const char *s = luaL_checkstring(L, 2);
+    const char *s = luaL_checklstring(L, 2, nullptr);
     p->addLayer(s);
   }
   push_string(L, p->layer(p->countLayers() - 1));
@@ -326,7 +326,7 @@ static int page_setLayerOf(lua_State *L)
 {
   Page *p = check_page(L, 1)->page;
   int n = check_objno(L, 2, p);
-  const char *s = luaL_checkstring(L, 3);
+  const char *s = luaL_checklstring(L, 3, nullptr);
   int l = p->findLayer(s);
   luaL_argcheck(L, l >= 0, 3, "layer does not exist");
   p->setLayerOf(n, l);
@@ -405,7 +405,7 @@ static int page_setAttribute(lua_State *L)
 {
   Page *p = check_page(L, 1)->page;
   int n = check_objno(L, 2, p);
-  Property prop = Property(luaL_checkoption(L, 3, NULL, property_names));
+  Property prop = Property(luaL_checkoption(L, 3, nullptr, property_names));
   Attribute value = check_property(prop, L, 4);
   lua_pushboolean(L, p->setAttribute(n, prop, value));
   return 1;
@@ -466,15 +466,15 @@ static int page_setTitles(lua_State *L)
   luaL_checktype(L, 2, LUA_TTABLE);
   lua_getfield(L, 2, "title");
   if (lua_isstring(L, -1))
-    p->setTitle(lua_tostring(L, -1));
+    p->setTitle(lua_tolstring(L, -1, nullptr));
   lua_getfield(L, 2, "section");
   if (lua_isstring(L, -1))
-    p->setSection(0, false, lua_tostring(L, -1));
+    p->setSection(0, false, lua_tolstring(L, -1, nullptr));
   else
     p->setSection(0, true, "");
   lua_getfield(L, 2, "subsection");
   if (lua_isstring(L, -1))
-    p->setSection(1, false, lua_tostring(L, -1));
+    p->setSection(1, false, lua_tolstring(L, -1, nullptr));
   else
     p->setSection(1, true, "");
   lua_pop(L, 3); // title, section, subsection
@@ -491,7 +491,7 @@ static int page_notes(lua_State *L)
 static int page_setNotes(lua_State *L)
 {
   Page *p = check_page(L, 1)->page;
-  String n = luaL_checkstring(L, 2);
+  String n = luaL_checklstring(L, 2, nullptr);
   p->setNotes(n);
   return 0;
 }
@@ -531,7 +531,7 @@ static int page_setEffect(lua_State *L)
 {
   Page *p = check_page(L, 1)->page;
   int n = check_viewno(L, 2, p);
-  const char *eff = luaL_checkstring(L, 3);
+  const char *eff = luaL_checklstring(L, 3, nullptr);
   p->setEffect(n, Attribute(true, eff));
   return 0;
 }
@@ -548,7 +548,7 @@ static int page_setActive(lua_State *L)
 {
   Page *p = check_page(L, 1)->page;
   int n = check_viewno(L, 2, p);
-  const char *name = luaL_checkstring(L, 3);
+  const char *name = luaL_checklstring(L, 3, nullptr);
   p->setActive(n, name);
   return 0;
 }
@@ -557,7 +557,7 @@ static int page_insertView(lua_State *L)
 {
   Page *p = check_page(L, 1)->page;
   int n = check_viewno(L, 2, p, 1);
-  const char *name = luaL_checkstring(L, 3);
+  const char *name = luaL_checklstring(L, 3, nullptr);
   p->insertView(n, name);
   return 0;
 }
@@ -593,6 +593,23 @@ static int page_setMarkedView(lua_State *L)
   return 0;
 }
 
+static int page_viewName(lua_State *L)
+{
+  Page *p = check_page(L, 1)->page;
+  int n = check_viewno(L, 2, p);
+  push_string(L, p->viewName(n));
+  return 1;
+}
+
+static int page_setViewName(lua_State *L)
+{
+  Page *p = check_page(L, 1)->page;
+  int n = check_viewno(L, 2, p);
+  const char *s = luaL_checklstring(L, 3, nullptr);
+  p->setViewName(n, s);
+  return 0;
+}
+
 // Either: view & layername
 // Or: view & object index
 static int page_visible(lua_State *L)
@@ -668,6 +685,8 @@ static const struct luaL_Reg page_methods[] = {
   { "clearViews", page_clearViews },
   { "markedView", page_markedView },
   { "setMarkedView", page_setMarkedView },
+  { "viewName", page_viewName },
+  { "setViewName", page_setViewName },
   { "visible", page_visible },
   { "setVisible", page_setVisible },
   { "bbox", page_bbox },
@@ -689,7 +708,7 @@ static const struct luaL_Reg page_methods[] = {
   { "setNotes", page_setNotes },
   { "marked", page_marked },
   { "setMarked", page_setMarked },
-  { 0, 0 }
+  { nullptr, nullptr }
 };
 
 // --------------------------------------------------------------------
