@@ -34,6 +34,7 @@
 #include "ipeiml.h"
 
 #include <cerrno>
+#include <cstring>
 
 using namespace ipe;
 using namespace ipelua;
@@ -42,11 +43,11 @@ using namespace ipelua;
 
 static const char * const set_names[] =
   { "preamble", "linecap", "linejoin", "fillrule", "symbol", "layout",
-    "gradient", 0 };
+    "gradient", nullptr };
 
 int ipelua::test_option(lua_State *L, int i, const char * const *names)
 {
-  const char *s = lua_tostring(L, i);
+  const char *s = lua_tolstring(L, i, nullptr);
   const char * const *p = names;
   int what = 0;
   while (*p) {
@@ -119,7 +120,7 @@ static int sheet_destructor(lua_State *L)
   SSheet *s = check_sheet(L, 1);
   if (s->owned)
     delete s->sheet;
-  s->sheet = 0;
+  s->sheet = nullptr;
   return 0;
 }
 
@@ -150,7 +151,7 @@ static Attribute check_absolute_attribute(Kind kind, lua_State *L, int i)
     Color color = check_color(L, i);
     return Attribute(color); }
   case EDashStyle: {
-    const char *s = luaL_checkstring(L, i);
+    const char *s = luaL_checklstring(L, i, nullptr);
     Attribute ds = Attribute::makeDashStyle(s);
     luaL_argcheck(L, !ds.isSymbolic(), i, "dashstyle is not absolute");
     return ds; }
@@ -169,17 +170,17 @@ static Attribute check_absolute_attribute(Kind kind, lua_State *L, int i)
 static int sheet_add(lua_State *L)
 {
   StyleSheet *s = check_sheet(L, 1)->sheet;
-  const char *what = luaL_checkstring(L, 2);
+  const char *what = luaL_checklstring(L, 2, nullptr);
   if (!strcmp(what, "symbol")) {
-    const char *name = luaL_checkstring(L, 3);
+    const char *name = luaL_checklstring(L, 3, nullptr);
     SObject *obj = check_object(L, 4);
     Symbol symbol;
     symbol.iObject = obj->obj->clone();
     symbol.iTransformations = ETransformationsAffine;
     s->addSymbol(Attribute(true, name), symbol);
   } else {
-    Kind kind = Kind(luaL_checkoption(L, 2, NULL, kind_names));
-    const char *name = luaL_checkstring(L, 3);
+    Kind kind = Kind(luaL_checkoption(L, 2, nullptr, kind_names));
+    const char *name = luaL_checklstring(L, 3, nullptr);
     Attribute sym(true, String(name));
     Attribute value = check_absolute_attribute(kind, L, 4);
     s->add(kind, sym, value);
@@ -191,8 +192,8 @@ static int sheet_addfrom(lua_State *L)
 {
   StyleSheet *s = check_sheet(L, 1)->sheet;
   StyleSheet *t = check_sheet(L, 2)->sheet;
-  Kind kind = Kind(luaL_checkoption(L, 3, NULL, kind_names));
-  const char *name = luaL_checkstring(L, 4);
+  Kind kind = Kind(luaL_checkoption(L, 3, nullptr, kind_names));
+  const char *name = luaL_checklstring(L, 4, nullptr);
   Attribute sym(true, name);
   switch (kind) {
   case EGradient: {
@@ -223,8 +224,8 @@ static int sheet_addfrom(lua_State *L)
 static int sheet_remove(lua_State *L)
 {
   StyleSheet *s = check_sheet(L, 1)->sheet;
-  Kind kind = Kind(luaL_checkoption(L, 2, NULL, kind_names));
-  const char *name = luaL_checkstring(L, 3);
+  Kind kind = Kind(luaL_checkoption(L, 2, nullptr, kind_names));
+  const char *name = luaL_checklstring(L, 3, nullptr);
   Attribute sym(true, name);
   s->remove(kind, sym);
   return 0;
@@ -262,7 +263,7 @@ static int sheet_xml(lua_State *L)
 static int sheet_setName(lua_State *L)
 {
   SSheet *p = check_sheet(L, 1);
-  const char *name = luaL_checkstring(L, 2);
+  const char *name = luaL_checklstring(L, 2, nullptr);
   p->sheet->setName(name);
   return 0;
 }
@@ -270,10 +271,10 @@ static int sheet_setName(lua_State *L)
 static int sheet_set(lua_State *L)
 {
   StyleSheet *s = check_sheet(L, 1)->sheet;
-  int what = luaL_checkoption(L, 2, 0, set_names);
+  int what = luaL_checkoption(L, 2, nullptr, set_names);
   switch (what) {
   case 0: // preamble
-    s->setPreamble(luaL_checkstring(L, 3));
+    s->setPreamble(luaL_checklstring(L, 3, nullptr));
     break;
   case 1: // linecap
     s->setLineCap(check_property(EPropLineCap, L, 3).lineCap());
@@ -305,7 +306,7 @@ static const struct luaL_Reg sheet_methods[] = {
   { "isStandard", sheet_isStandard },
   { "name", sheet_name },
   { "setName", sheet_setName },
-  { NULL, NULL }
+  { nullptr, nullptr }
 };
 
 // --------------------------------------------------------------------
@@ -337,7 +338,7 @@ static int cascade_destructor(lua_State *L)
   SCascade *s = check_cascade(L, 1);
   if (s->owned)
     delete s->cascade;
-  s->cascade = 0;
+  s->cascade = nullptr;
   return 0;
 }
 
@@ -354,11 +355,11 @@ static int cascade_tostring(lua_State *L)
 static int cascade_allNames(lua_State *L)
 {
   SCascade *p = check_cascade(L, 1);
-  Kind kind = Kind(luaL_checkoption (L, 2, NULL, kind_names));
+  Kind kind = Kind(luaL_checkoption (L, 2, nullptr, kind_names));
   AttributeSeq seq;
   p->cascade->allNames(kind, seq);
   lua_createtable(L, seq.size(), 0);
-  for (uint i = 0; i < seq.size(); ++i) {
+  for (int i = 0; i < size(seq); ++i) {
     push_string(L, seq[i].string());
     lua_rawseti(L, -2, i + 1);
   }
@@ -437,7 +438,7 @@ static int cascade_find(lua_State *L)
     push_attribute(L, Attribute(s->fillRule()));
     break;
   case 4: { // symbol
-    const char *name = luaL_checkstring(L, 3);
+    const char *name = luaL_checklstring(L, 3, nullptr);
     const Symbol *symbol = s->findSymbol(Attribute(true, name));
     if (symbol)
       push_object(L, symbol->iObject->clone());
@@ -448,12 +449,12 @@ static int cascade_find(lua_State *L)
     push_layout(L, *s->findLayout());
     break;
   case 6: { // gradient
-    const char *name = luaL_checkstring(L, 3);
+    const char *name = luaL_checklstring(L, 3, nullptr);
     push_gradient(L, *s->findGradient(Attribute(true, name)));
     break; }
   default: {
-    Kind kind = Kind(luaL_checkoption(L, 2, NULL, kind_names));
-    const char *name = luaL_checkstring(L, 3);
+    Kind kind = Kind(luaL_checkoption(L, 2, nullptr, kind_names));
+    const char *name = luaL_checklstring(L, 3, nullptr);
     push_attribute(L, s->find(kind, Attribute(true, String(name))));
     break; }
   }
@@ -463,8 +464,8 @@ static int cascade_find(lua_State *L)
 static int cascade_has(lua_State *L)
 {
   Cascade *p = check_cascade(L, 1)->cascade;
-  Kind kind = Kind(luaL_checkoption(L, 2, NULL, kind_names));
-  const char *name = luaL_checkstring(L, 3);
+  Kind kind = Kind(luaL_checkoption(L, 2, nullptr, kind_names));
+  const char *name = luaL_checklstring(L, 3, nullptr);
   Attribute sym(true, String(name));
   lua_pushboolean(L, p->has(kind, sym));
   return 1;
@@ -524,7 +525,7 @@ static const struct luaL_Reg cascade_methods[] = {
   { "sheet", cascade_sheet },
   { "insert", cascade_insert },
   { "remove", cascade_remove },
-  { NULL, NULL }
+  { nullptr, nullptr }
 };
 
 // --------------------------------------------------------------------
