@@ -33,6 +33,7 @@
 
 #include <cstdio>
 #include <cstdlib>
+#include <cstring>
 
 using ipe::Document;
 using ipe::Page;
@@ -66,7 +67,7 @@ static int renderPage(Thumbnail::TargetFormat fm,
     return 1;
   }
 
-  if (doc->runLatex()) {
+  if (doc->runLatex(src)) {
     delete doc;
     return 1;
   }
