@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2016  Otfried Cheong
+    Copyright (c) 1993-2019 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -30,31 +30,33 @@
 
 #include "ipedoc.h"
 #include <cstdlib>
+#include <cstring>
 
 using ipe::Document;
 using ipe::String;
+using ipe::FileFormat;
+using ipe::SaveFlag;
 
-static int topdf(Document *doc, String src, String dst,
-		 Document::TFormat fm, uint flags,
+static int topdf(Document *doc, String src, String dst, uint32_t flags,
 		 int fromPage = -1, int toPage = -1, int viewNo = -1)
 {
-  int res = doc->runLatex();
+  int res = doc->runLatex(src);
   if (res) return res;
 
   bool result = false;
   if (viewNo >= 0) {
-    result = doc->exportView(dst.z(), Document::EPdf, flags, fromPage, viewNo);
+    result = doc->exportView(dst.z(), FileFormat::Pdf, flags, fromPage, viewNo);
   } else if (toPage >= 0) {
     result = doc->exportPages(dst.z(), flags, fromPage, toPage);
   } else {
-    result = doc->save(dst.z(), Document::EPdf, flags);
+    result = doc->save(dst.z(), FileFormat::Pdf, flags);
   }
   if (!result) {
     fprintf(stderr, "Failed to save or export document!\n");
     return 1;
   }
 
-  if (flags & Document::EExport)
+  if (flags & SaveFlag::Export)
     fprintf(stderr, "Warning: the exported file contains no Ipe markup.\n"
 	    "It cannot be read by Ipe - make sure you keep the original!\n");
   return 0;
@@ -69,10 +71,11 @@ static void usage()
 	  " -export      : output contains no Ipe markup.\n"
 	  " -pages <n-m> : export only these pages (implies -export).\n"
 	  " -view <p-v>  : export only this view (implies -export).\n"
-	  " -markedview  "
-	  ": export only marked views on marked pages (implies -export).\n"
+	  " -markedview  : export only marked views on marked pages (implies -export).\n"
 	  " -runlatex    : run Latex even for XML output.\n"
-	  " -nozip:      : do not compress PDF streams.\n"
+	  " -nozip       : do not compress PDF streams.\n"
+	  " -keepnotes   : save page notes as PDF annotations even when exporting.\n"
+	  "Pages can be specified by page number or by section title.\n"
 	  );
   exit(1);
 }
@@ -85,20 +88,19 @@ int main(int argc, char *argv[])
   if (argc < 3)
     usage();
 
-  Document::TFormat frm = Document::EUnknown;
+  FileFormat frm = FileFormat::Unknown;
   if (!strcmp(argv[1], "-xml"))
-    frm = Document::EXml;
+    frm = FileFormat::Xml;
   else if (!strcmp(argv[1], "-pdf"))
-    frm = Document::EPdf;
+    frm = FileFormat::Pdf;
 
-  if (frm == Document::EUnknown)
+  if (frm == FileFormat::Unknown)
     usage();
 
-  uint flags = Document::ESaveNormal;
+  uint32_t flags = SaveFlag::SaveNormal;
   bool runLatex = false;
-  int fromPage = -1;
-  int toPage = -1;
-  int viewNo = -1;
+  const char *pages = nullptr;
+  const char *view = nullptr;
   int i = 2;
 
   String infile;
@@ -107,27 +109,32 @@ int main(int argc, char *argv[])
   while (i < argc) {
 
     if (!strcmp(argv[i], "-export")) {
-      flags |= Document::EExport;
+      flags |= SaveFlag::Export;
       ++i;
     } else if (!strcmp(argv[i], "-view")) {
-      if (sscanf(argv[i+1], "%d-%d", &fromPage, &viewNo) != 2)
+      flags |= SaveFlag::Export;
+      if (i + 1 == argc)
 	usage();
-      flags |= Document::EExport;
+      view = argv[i+1];
       i += 2;
     } else if (!strcmp(argv[i], "-pages")) {
-      if (sscanf(argv[i+1], "%d-%d", &fromPage, &toPage) != 2)
+      flags |= SaveFlag::Export;
+      if (i + 1 == argc)
 	usage();
-      flags |= Document::EExport;
+      pages = argv[i+1];
       i += 2;
     } else if (!strcmp(argv[i], "-markedview")) {
-      flags |= Document::EMarkedView;
-      flags |= Document::EExport;
+      flags |= SaveFlag::MarkedView;
+      flags |= SaveFlag::Export;
       ++i;
     } else if (!strcmp(argv[i], "-runlatex")) {
       runLatex = true;
       ++i;
     } else if (!strcmp(argv[i], "-nozip")) {
-      flags |= Document::ENoZip;
+      flags |= SaveFlag::NoZip;
+      ++i;
+    } else if (!strcmp(argv[i], "-keepnotes")) {
+      flags |= SaveFlag::KeepNotes;
       ++i;
     } else {
       // last one or two arguments must be filenames
@@ -145,17 +152,17 @@ int main(int argc, char *argv[])
   if (infile.empty())
     usage();
 
-  if ((flags & Document::EExport) && frm == Document::EXml) {
+  if ((flags & SaveFlag::Export) && frm == FileFormat::Xml) {
     fprintf(stderr, "-export only available with -pdf.\n");
     exit(1);
   }
 
-  if (toPage >= 0 && frm != Document::EPdf) {
+  if (pages && frm != FileFormat::Pdf) {
     fprintf(stderr, "-pages only available with -pdf.\n");
     exit(1);
   }
 
-  if (toPage >= 0 && viewNo >= 0) {
+  if (pages && view) {
     fprintf(stderr, "cannot specify both -pages and -view.\n");
     exit(1);
   }
@@ -166,10 +173,10 @@ int main(int argc, char *argv[])
     if (ext == ".ipe" || ext == ".pdf" || ext == ".xml")
       outfile = infile.left(infile.size() - 4);
     switch (frm) {
-    case Document::EXml:
+    case FileFormat::Xml:
       outfile += ".ipe";
       break;
-    case Document::EPdf:
+    case FileFormat::Pdf:
       outfile += ".pdf";
     default:
       break;
@@ -188,22 +195,34 @@ int main(int argc, char *argv[])
   fprintf(stderr, "Document %s has %d pages (%d views)\n",
 	  infile.z(), doc->countPages(), doc->countTotalViews());
 
-  // check frompage, topage, viewno
-  if (toPage >= 0) {
-    if (fromPage <= 0 || fromPage > toPage || toPage > doc->countPages()) {
+  // parse pages and view
+  int fromPage = -1;
+  int toPage = -1;
+  int viewNo = -1;
+
+  if (pages) {
+    String p(pages);
+    int j = p.find('-');
+    if (j >= 0) {
+      fromPage = (j > 0) ? doc->findPage(p.left(j)) : 0;
+      toPage = (j < p.size() - 1) ? doc->findPage(p.substr(j+1)) : doc->countPages() - 1;
+    }
+    if (fromPage < 0 || fromPage > toPage) {
       fprintf(stderr, "incorrect -pages specification.\n");
       exit(1);
     }
-    --fromPage;
-    --toPage;
-  } else if (fromPage >= 0) {
-    if (fromPage <= 0 || fromPage > doc->countPages() ||
-	viewNo <= 0 || viewNo > doc->page(fromPage - 1)->countViews()) {
+  } else if (view) {
+    String v(view);
+    int j = v.find('-');
+    if (j > 0) {
+      fromPage = doc->findPage(v.left(j));
+      if (fromPage >= 0)
+	viewNo = doc->page(fromPage)->findView(v.substr(j+1));
+    }
+    if (fromPage < 0 || viewNo < 0) {
       fprintf(stderr, "incorrect -view specification.\n");
       exit(1);
     }
-    --fromPage;
-    --viewNo;
   }
 
   char buf[64];
@@ -216,17 +235,16 @@ int main(int argc, char *argv[])
   doc->setProperties(props);
 
   switch (frm) {
-  case Document::EXml:
+  case FileFormat::Xml:
     if (runLatex)
-      return topdf(doc.get(), infile, outfile, Document::EXml, flags);
+      return topdf(doc.get(), infile, outfile, flags);
     else
-      doc->save(outfile.z(), Document::EXml, Document::ESaveNormal);
+      doc->save(outfile.z(), FileFormat::Xml, SaveFlag::SaveNormal);
   default:
     return 0;
 
-  case Document::EPdf:
-    return topdf(doc.get(), infile, outfile, Document::EPdf,
-		 flags, fromPage, toPage, viewNo);
+  case FileFormat::Pdf:
+    return topdf(doc.get(), infile, outfile, flags, fromPage, toPage, viewNo);
   }
 }
 
