@@ -194,7 +194,7 @@ NSString *ipeui_set_mnemonic(NSString *title, NSButton *button)
 {
   unichar mnemonic = 0;
   NSMutableString *result = [NSMutableString stringWithCapacity:[title length]];
-  int i = 0;
+  size_t i = 0;
   while (i < [title length]) {
     unichar ch = [title characterAtIndex:i];
     if (ch != '&') {
@@ -422,6 +422,7 @@ void PDialog::setTextView(NSTextView *tv, const std::string &s)
 {
   NSAttributedString *n = [[NSAttributedString alloc] initWithString:S2N(s)];
   [[tv textStorage] setAttributedString:n];
+  tv.textColor = [NSColor textColor];
 }
 
 std::string PDialog::getTextView(NSTextView *tv)
@@ -468,7 +469,7 @@ void PDialog::layoutControls()
   // layout buttons
   NSView *lastButton = nil;
   int bcount = 0;
-  for (int i = 0; i < iElements.size(); ++i) {
+  for (size_t i = 0; i < iElements.size(); ++i) {
     SElement &m = iElements[i];
     if (m.row >= 0)
       continue;
@@ -500,7 +501,7 @@ void PDialog::layoutControls()
     iRowStretch.push_back(0);
 
   // layout rows and columns
-  for (int i = 0; i < iElements.size(); ++i) {
+  for (size_t i = 0; i < iElements.size(); ++i) {
     SElement &m = iElements[i];
     if (m.row < 0)
       continue;
@@ -639,7 +640,6 @@ bool PDialog::buildAndRun(int w, int h)
 	  tv.automaticDashSubstitutionEnabled = NO;
 	  setTextView(tv, m.text);
 	  view = tv;
-	  tv.textColor = [NSColor textColor];
 	}
 	break;
       case ECombo:
@@ -699,8 +699,16 @@ bool PDialog::buildAndRun(int w, int h)
   // set keyboard focus
   if (focusCtrl)
     [iPanel makeFirstResponder:focusCtrl];
+  // this is such a hack, but it seems to work for Ipe
+  NSMenu *editMenu = [[[NSApp mainMenu] itemAtIndex:2] submenu];
+  NSMenuItem *undoItem = [editMenu itemAtIndex:0];
+  NSMenuItem *redoItem = [editMenu itemAtIndex:1];
+  undoItem.action = @selector(undo:);
+  redoItem.action = @selector(redo:);
   int result = [NSApp runModalForWindow:iPanel];
   retrieveValues(); // for future reference
+  undoItem.action = @selector(ipeMenuAction:);
+  redoItem.action = @selector(ipeMenuAction:);
   iPanel = nil;
   return result;
 }
@@ -713,7 +721,7 @@ static int dialog_constructor(lua_State *L)
   const char *s = luaL_checkstring(L, 2);
 
   Dialog **dlg = (Dialog **) lua_newuserdata(L, sizeof(Dialog *));
-  *dlg = 0;
+  *dlg = nullptr;
   luaL_getmetatable(L, "Ipe.dialog");
   lua_setmetatable(L, -2);
   *dlg = new PDialog(L, parent, s);
@@ -748,6 +756,7 @@ static int ipeui_wait(lua_State *L)
   panel.contentView = l;
   std::thread t(editor_thread, cmd);
   [NSApp runModalForWindow:panel];
+  [panel close];
   t.join();
   return 0;
 }
@@ -857,7 +866,7 @@ int PMenu::add(lua_State *L)
     if (hastable)
       luaL_argcheck(L, lua_istable(L, 5), 5,
 		    "argument is not a function or table");
-    const char *current = 0;
+    const char *current = nullptr;
     if (hascheck) {
       luaL_argcheck(L, lua_isstring(L, 6), 6,
 		    "argument is not a function or string");
@@ -916,7 +925,7 @@ int PMenu::add(lua_State *L)
     }
     NSMenuItem *mitem = [[NSMenuItem alloc]
 			  initWithTitle:ipeui_set_mnemonic(C2N(title),nil)
-				 action:NULL
+				 action:nullptr
 			  keyEquivalent:@""];
     [mitem setSubmenu:sm];
     [iMenu addItem:mitem];
@@ -930,7 +939,7 @@ static int menu_constructor(lua_State *L)
 {
   // check_winid(L, 1);
   Menu **m = (Menu **) lua_newuserdata(L, sizeof(Menu *));
-  *m = 0;
+  *m = nullptr;
   luaL_getmetatable(L, "Ipe.menu");
   lua_setmetatable(L, -2);
   *m = new PMenu();
@@ -1024,7 +1033,7 @@ static int timer_constructor(lua_State *L)
   const char *method = luaL_checkstring(L, 2);
 
   Timer **t = (Timer **) lua_newuserdata(L, sizeof(Timer *));
-  *t = 0;
+  *t = nullptr;
   luaL_getmetatable(L, "Ipe.timer");
   lua_setmetatable(L, -2);
 
@@ -1148,9 +1157,9 @@ static NSArray<NSString *> *make_filters(const char *s)
 
 static int ipeui_fileDialog(lua_State *L)
 {
-  static const char * const typenames[] = { "open", "save", 0 };
+  static const char * const typenames[] = { "open", "save", nullptr };
   // not needed: NSWindow *win = check_winid(L, 1);
-  int type = luaL_checkoption(L, 2, 0, typenames);
+  int type = luaL_checkoption(L, 2, nullptr, typenames);
   if (type == 0) { // open file
     NSOpenPanel *panel = [NSOpenPanel openPanel];
     IpeFileDialogHelper *helper = [[IpeFileDialogHelper alloc] initFor:panel];
@@ -1204,22 +1213,22 @@ static int ipeui_getColor(lua_State *L)
 static int ipeui_messageBox(lua_State *L)
 {
   static const char * const options[] =  {
-    "none", "warning", "information", "question", "critical", 0 };
+    "none", "warning", "information", "question", "critical", nullptr };
   static const char * const buttontype[] = {
     "ok", "okcancel", "yesnocancel", "discardcancel",
-    "savediscardcancel", 0 };
+    "savediscardcancel", nullptr };
 
   NSWindow *win = check_winid(L, 1);
   int type = luaL_checkoption(L, 2, "none", options);
   const char *text = luaL_checkstring(L, 3);
-  const char *details = 0;
+  const char *details = nullptr;
   if (!lua_isnoneornil(L, 4))
     details = luaL_checkstring(L, 4);
   int buttons = 0;
   if (lua_isnumber(L, 5))
     buttons = (int)luaL_checkinteger(L, 5);
   else if (!lua_isnoneornil(L, 5))
-    buttons = luaL_checkoption(L, 5, 0, buttontype);
+    buttons = luaL_checkoption(L, 5, nullptr, buttontype);
 
   (void) win;
   NSAlert *alert = [[NSAlert alloc] init];
@@ -1328,7 +1337,7 @@ static const struct luaL_Reg ipeui_functions[] = {
   { "messageBox", ipeui_messageBox },
   { "currentDateTime", ipeui_currentDateTime },
   { "startBrowser", ipeui_startBrowser },
-  { 0, 0},
+  { nullptr, nullptr }
 };
 
 // --------------------------------------------------------------------
