@@ -36,7 +36,7 @@
 WINID check_winid(lua_State *L, int i)
 {
   if (lua_isnil(L, i))
-    return 0;
+    return nullptr;
   WINID *w = (WINID *) luaL_checkudata(L, i, "Ipe.winid");
   return *w;
 }
@@ -58,7 +58,7 @@ static int winid_tostring(lua_State *L)
 
 static const struct luaL_Reg winid_methods[] = {
   { "__tostring", winid_tostring },
-  { 0, 0 }
+  { nullptr, nullptr }
 };
 
 // --------------------------------------------------------------------
@@ -69,12 +69,17 @@ Dialog::Dialog(lua_State *L0, WINID parent, const char *caption)
   L = L0;
   iParent = parent;
   iLuaDialog = LUA_NOREF;
-  hDialog = NULL;
+  hDialog = nullptr;
   iIgnoreEscape = false;
   iNoRows = 1;
   iNoCols = 1;
 }
 
+// Careful: On Qt, the dialog is owned by the parent window.
+// It's important that Lua deletes the dialog before Qt tries to, otherwise
+// we have a double deletion.
+// Make sure not to create a cyclic reference to the dialog by capturing it inside
+// the Lua method for an action, as this would stop it from being garbage collected.
 Dialog::~Dialog()
 {
   // dereference lua methods
@@ -91,14 +96,14 @@ void Dialog::callLua(int luaMethod)
 
   lua_rawgeti(L, LUA_REGISTRYINDEX, luaMethod);
   lua_rawgeti(L, LUA_REGISTRYINDEX, iLuaDialog);
-  lua_call(L, 1, 0);
+  luacall(L, 1, 0);
 }
 
 // name, label, action
 int Dialog::addButton(lua_State *L)
 {
   SElement m;
-  m.name = std::string(luaL_checkstring(L, 2));
+  m.name = std::string(luaL_checklstring(L, 2, nullptr));
   m.type = EButton;
   m.lua_method = LUA_NOREF;
   m.flags = 0;
@@ -106,9 +111,9 @@ int Dialog::addButton(lua_State *L)
   m.col = -1;
   m.rowspan = 1;
   m.colspan = 1;
-  m.text = std::string(luaL_checkstring(L, 3));
+  m.text = std::string(luaL_checklstring(L, 3, nullptr));
   if (lua_isstring(L, 4)) {
-    const char *s = lua_tostring(L, 4);
+    const char *s = lua_tolstring(L, 4, nullptr);
     if (!strcmp(s, "accept"))
       m.flags |= EAccept;
     else if (!strcmp(s, "reject"))
@@ -131,24 +136,24 @@ int Dialog::addButton(lua_State *L)
 int Dialog::add(lua_State *L)
 {
   static const char * const typenames[] =
-    { "button", "text", "list", "label", "combo", "checkbox", "input", 0 };
+    { "button", "text", "list", "label", "combo", "checkbox", "input", nullptr };
 
   SElement m;
-  m.name = std::string(luaL_checkstring(L, 2));
-  m.type = TType(luaL_checkoption(L, 3, 0, typenames));
+  m.name = checkstring(L, 2);
+  m.type = TType(luaL_checkoption(L, 3, nullptr, typenames));
   luaL_checktype(L, 4, LUA_TTABLE);
   m.lua_method = LUA_NOREF;
   m.flags = 0;
-  m.row = (int)luaL_checkinteger(L, 5) - 1;
+  m.row = luaL_checkinteger(L, 5) - 1;
   if (m.row < 0)
     m.row = iNoRows + 1 + m.row;
-  m.col = (int)luaL_checkinteger(L, 6) - 1;
+  m.col = luaL_checkinteger(L, 6) - 1;
   m.rowspan = 1;
   m.colspan = 1;
   if (!lua_isnoneornil(L, 7))
-    m.rowspan = (int)luaL_checkinteger(L, 7);
+    m.rowspan = luaL_checkinteger(L, 7);
   if (!lua_isnoneornil(L, 8))
-    m.colspan = (int)luaL_checkinteger(L, 8);
+    m.colspan = luaL_checkinteger(L, 8);
   if (m.row + m.rowspan > iNoRows)
     iNoRows = m.row + m.rowspan;
   if (m.col + m.colspan > iNoCols)
@@ -187,7 +192,7 @@ void Dialog::addLabel(lua_State *L, SElement &m)
 {
   lua_getfield(L, 4, "label");
   luaL_argcheck(L, lua_isstring(L, -1), 4, "no label");
-  m.text = std::string(lua_tostring(L, -1));
+  m.text = std::string(lua_tolstring(L, -1, nullptr));
   lua_pop(L, 1); // label
   m.minHeight = 16;
   const char *p = m.text.c_str();
@@ -211,14 +216,13 @@ void Dialog::addButtonItem(lua_State *L, SElement &m)
 {
   lua_getfield(L, 4, "label");
   luaL_argcheck(L, lua_isstring(L, -1), 4, "no button label");
-  const char *label = lua_tostring(L, -1);
-  m.text = std::string(label);
+  m.text = tostring(L, -1);
   lua_getfield(L, 4, "action");
   if (lua_isstring(L, -1)) {
-    const char *s = lua_tostring(L, -1);
-    if (!strcmp(s, "accept"))
+    auto s = tostring(L, -1);
+    if (s == "accept")
       m.flags |= EAccept;
-    else if (!strcmp(s, "reject"))
+    else if (s == "reject")
       m.flags |= EReject;
     else
       luaL_argerror(L, 4, "unknown action");
@@ -238,7 +242,7 @@ void Dialog::addCheckbox(lua_State *L, SElement &m)
 {
   lua_getfield(L, 4, "label");
   luaL_argcheck(L, lua_isstring(L, -1), 4, "no label");
-  m.text = std::string(lua_tostring(L, -1));
+  m.text = tostring(L, -1);
   lua_getfield(L, 4, "action");
   if (!lua_isnil(L, -1)) {
     luaL_argcheck(L, lua_isfunction(L, -1), 4, "unknown action type");
@@ -277,12 +281,12 @@ void Dialog::addTextEdit(lua_State *L, SElement &m)
     m.flags |= EFocused;
   lua_getfield(L, 4, "syntax");
   if (!lua_isnil(L, -1)) {
-    const char *s = lua_tostring(L, -1);
-    if (!strcmp(s, "logfile")) {
+    auto s = tostring(L, -1);
+    if (s == "logfile") {
       m.flags |= ELogFile;
-    } else if (!strcmp(s, "xml")) {
+    } else if (s == "xml") {
       m.flags |= EXml;
-    } else if (!strcmp(s, "latex")) {
+    } else if (s == "latex") {
       m.flags |= ELatex;
     } else
       luaL_argerror(L, 4, "unknown syntax");
@@ -302,13 +306,20 @@ void Dialog::setListItems(lua_State *L, int index, SElement &m)
   for (int i = 1; i <= no; ++i) {
     lua_rawgeti(L, index, i);
     luaL_argcheck(L, lua_isstring(L, -1), index, "items must be strings");
-    const char *item = lua_tostring(L, -1);
-    m.items.emplace_back(item);
-    int w = 4 * strlen(item) + 16;
+    auto item = tostring(L, -1);
+    int w = 4 * item.size() + 16;
     if (w > m.minWidth)
       m.minWidth = w;
+    m.items.emplace_back(item);
     lua_pop(L, 1); // item
   }
+  lua_getfield(L, index, "action");
+  if (!lua_isnil(L, -1)) {
+    luaL_argcheck(L, lua_isfunction(L, -1), index, "unknown action type");
+    lua_pushvalue(L, -1);
+    m.lua_method = luaL_ref(L, LUA_REGISTRYINDEX);
+  }
+  lua_pop(L, 1); // action
 }
 
 void Dialog::addList(lua_State *L, SElement &m)
@@ -327,9 +338,9 @@ void Dialog::addCombo(lua_State *L, SElement &m)
 
 int Dialog::findElement(lua_State *L, int index)
 {
-  const char *name = luaL_checkstring(L, index);
+  auto name = checkstring(L, index);
   for (int i = 0; i < int(iElements.size()); ++i) {
-    if (!strcmp(iElements[i].name.c_str(), name))
+    if (name == iElements[i].name)
       return i;
   }
   return luaL_argerror(L, index, "no such element in dialog");
@@ -337,8 +348,8 @@ int Dialog::findElement(lua_State *L, int index)
 
 int Dialog::set(lua_State *L)
 {
-  const char *s = luaL_checkstring(L, 2);
-  if (!strcmp(s, "ignore-escape")) {
+  auto s = checkstring(L, 2);
+  if (s == "ignore-escape") {
     iIgnoreEscape = lua_toboolean(L, 3);
     return 0;
   }
@@ -361,20 +372,19 @@ void Dialog::setUnmapped(lua_State *L, int idx)
   case ELabel:
   case ETextEdit:
   case EInput:
-    m.text = std::string(luaL_checkstring(L, 3));
+    m.text = checkstring(L, 3);
     break;
   case EList:
   case ECombo:
     if (lua_isnumber(L, 3)) {
-      int n = (int)luaL_checkinteger(L, 3);
+      int n = luaL_checkinteger(L, 3);
       luaL_argcheck(L, 1 <= n && n <= int(m.items.size()), 3,
 		    "list index out of bounds");
       m.value = n - 1;
     } else if (lua_isstring(L, 3)) {
-      const char *s = lua_tostring(L, 3);
+      auto s = tostring(L, 3);
       auto it = std::find_if(m.items.cbegin(), m.items.cend(),
-			     [s](const std::string &el)
-			     { return !strcmp(el.c_str(), s); });
+			     [&s](const std::string &el) { return s == el; });
       luaL_argcheck(L, it != m.items.end(), 3, "item not in list");
       m.value = (it - m.items.begin());
     } else {
@@ -445,14 +455,14 @@ int Dialog::setEnabled(lua_State *L)
 
 int Dialog::setStretch(lua_State *L)
 {
-  static const char * const typenames[] = { "row", "column", 0 };
+  static const char * const typenames[] = { "row", "column", nullptr };
 
   while (int(iColStretch.size()) < iNoCols)
     iColStretch.push_back(0);
   while (int(iRowStretch.size()) < iNoRows)
     iRowStretch.push_back(0);
 
-  int type = luaL_checkoption(L, 2, 0, typenames);
+  int type = luaL_checkoption(L, 2, nullptr, typenames);
   int rowcol = (int)luaL_checkinteger(L, 3) - 1;
   int stretch = (int)luaL_checkinteger(L, 4);
 
@@ -481,8 +491,9 @@ static int dialog_destructor(lua_State *L)
 {
   //fprintf(stderr, "Dialog::~Dialog()\n");
   Dialog **dlg = check_dialog(L, 1);
+  // on Qt, hope we destruct before the parent window does
   delete (*dlg);
-  *dlg = 0;
+  *dlg = nullptr;
   return 0;
 }
 
@@ -497,8 +508,8 @@ static int dialog_execute(lua_State *L)
     luaL_argcheck(L, lua_isnumber(L, -1), 2, "width is not a number");
     lua_rawgeti(L, 2, 2);
     luaL_argcheck(L, lua_isnumber(L, -1), 2, "height is not a number");
-    w = lua_tointeger(L, -2);
-    h = lua_tointeger(L, -1);
+    w = lua_tointegerx(L, -2, nullptr);
+    h = lua_tointegerx(L, -1, nullptr);
     lua_pop(L, 2); // w & h
   }
   lua_pushboolean(L, (*dlg)->execute(L, w, h));
@@ -561,7 +572,7 @@ static const struct luaL_Reg dialog_methods[] = {
   { "get", dialog_get },
   { "setEnabled", dialog_setEnabled },
   { "accept", dialog_accept },
-  { 0, 0 }
+  { nullptr, nullptr }
 };
 
 // --------------------------------------------------------------------
@@ -585,7 +596,7 @@ static int menu_destructor(lua_State *L)
   //fprintf(stderr, "Menu::~Menu()\n");
   Menu **m = check_menu(L, 1);
   delete *m;
-  *m = 0;
+  *m = nullptr;
   return 0;
 }
 
@@ -608,7 +619,7 @@ static const struct luaL_Reg menu_methods[] = {
   { "__gc", menu_destructor },
   { "execute", menu_execute },
   { "add", menu_add },
-  { 0, 0 }
+  { nullptr, nullptr }
 };
 
 // --------------------------------------------------------------------
@@ -641,7 +652,7 @@ void Timer::callLua()
   }
   lua_remove(L, -3); // remove weak table
   lua_insert(L, -2); // stack is now: method, table
-  lua_call(L, 1, 0); // call method
+  luacall(L, 1, 0); // call method
 }
 
 int Timer::setSingleShot(lua_State *L)
@@ -664,7 +675,7 @@ static int timer_destructor(lua_State *L)
   Timer **t = check_timer(L, 1);
   //fprintf(stderr, "Timer::~Timer()\n");
   delete *t;
-  *t = 0;
+  *t = nullptr;
   return 0;
 }
 
@@ -710,7 +721,7 @@ static const struct luaL_Reg timer_methods[] = {
   { "active", timer_active },
   { "setInterval", timer_setinterval },
   { "setSingleShot", timer_setsingleshot },
-  { 0, 0 }
+  { nullptr, nullptr }
 };
 
 // --------------------------------------------------------------------
