@@ -39,7 +39,6 @@ extern "C" {
 
 #include <cstdio>
 #include <cstdlib>
-#include <cstring>
 #include <vector>
 #include <string>
 
@@ -70,6 +69,20 @@ extern void push_winid(lua_State *L, WINID win);
 
 // --------------------------------------------------------------------
 
+inline std::string checkstring(lua_State *L, int i) {
+  return std::string(luaL_checklstring(L, i, nullptr));
+}
+
+inline std::string tostring(lua_State *L, int i) {
+  return std::string(lua_tolstring(L, i, nullptr));
+}
+
+inline void luacall(lua_State *L, int nargs, int nresults) {
+  lua_callk(L, nargs, nresults, 0, nullptr);
+}
+
+// --------------------------------------------------------------------
+
 class Dialog {
 public:
   Dialog(lua_State *L0, WINID parent, const char *caption);
