@@ -5,7 +5,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2016  Otfried Cheong
+    Copyright (c) 1993-2019 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -82,8 +82,6 @@ public:
   virtual int active(lua_State *L);
   virtual int start(lua_State *L);
   virtual int stop(lua_State *L);
-private slots:
-  void elapsed();
 private:
   QTimer *iTimer;
 };
@@ -99,9 +97,6 @@ public:
 
   QGridLayout *gridlayout() { return iGrid; }
 
-private slots:
-  void notifyLua();
-
 protected:
   virtual void setMapped(lua_State *L, int idx);
   virtual bool buildAndRun(int w, int h);
