@@ -4,7 +4,7 @@
 /*
 
     This file is part of the extensible drawing editor Ipe.
-    Copyright (C) 1993-2016  Otfried Cheong
+    Copyright (c) 1993-2019 Otfried Cheong
 
     Ipe is free software; you can redistribute it and/or modify it
     under the terms of the GNU General Public License as published by
@@ -42,17 +42,11 @@
 
 // --------------------------------------------------------------------
 
-void WString::init(const char *s)
+void WString::init(const char *s, int len)
 {
-  int rw = MultiByteToWideChar(CP_UTF8, 0, s, -1, NULL, 0);
-  iData.resize(rw);
-  MultiByteToWideChar(CP_UTF8, 0, s, -1, &iData[0], rw);
-}
-
-void WString::appendTo(std::vector<wchar_t> &out)
-{
-  for (int i = 0; i < int(iData.size()); ++i)
-    out.push_back(iData[i]);
+  int rw = MultiByteToWideChar(CP_UTF8, 0, s, len, nullptr, 0);
+  resize(rw + 1, wchar_t(0));
+  MultiByteToWideChar(CP_UTF8, 0, s, len, data(), rw);
 }
 
 BOOL setWindowText(HWND h, const char *s)
@@ -81,9 +75,9 @@ void sendMessage(HWND h, UINT code, const char *t, WPARAM wParam)
 
 static std::string wideToUtf8(const wchar_t *wbuf)
 {
-  int rm = WideCharToMultiByte(CP_UTF8, 0, wbuf, -1, NULL, 0, NULL, NULL);
+  int rm = WideCharToMultiByte(CP_UTF8, 0, wbuf, -1, nullptr, 0, nullptr, nullptr);
   std::vector<char> multi(rm);
-  WideCharToMultiByte(CP_UTF8, 0, wbuf, -1, &multi[0], rm, NULL, NULL);
+  WideCharToMultiByte(CP_UTF8, 0, wbuf, -1, &multi[0], rm, nullptr, nullptr);
   return std::string(&multi[0]);
 }
 
@@ -103,13 +97,13 @@ static void buildFlags(std::vector<short> &t, DWORD flags)
 static void buildString(std::vector<short> &t, const char *s)
 {
   WString w(s);
-  const wchar_t *p = w;
+  const wchar_t *p = w.data();
   while (*p)
     t.push_back(*p++);
   t.push_back(0);
 }
 
-static void buildControl(std::vector<short> &t, short what, const char *s=0)
+static void buildControl(std::vector<short> &t, short what, const char *s = nullptr)
 {
   t.push_back(0xffff);
   t.push_back(what);
@@ -409,9 +403,9 @@ BOOL CALLBACK PDialog::dialogProc(HWND hwnd, UINT message,
     // TODO: Use modern SetSubclass method
     for (int i = 0; i < int(d->iElements.size()); ++i) {
       if (d->iElements[i].type == ETextEdit)
-	wpOrigProc =
-	  (WNDPROC) SetWindowLong(GetDlgItem(hwnd, i + IDBASE),
-				  GWL_WNDPROC, (LONG) subclassProc);
+	wpOrigProc = (WNDPROC)
+	  SetWindowLongPtr(GetDlgItem(hwnd, i + IDBASE),
+			   GWLP_WNDPROC, (LONG_PTR) &subclassProc);
     }
     return d->initDialog();
 
@@ -424,8 +418,8 @@ BOOL CALLBACK PDialog::dialogProc(HWND hwnd, UINT message,
     // Remove the subclasses from text edits
     for (int i = 0; i < int(d->iElements.size()); ++i) {
       if (d->iElements[i].type == ETextEdit)
-	SetWindowLong(GetDlgItem(hwnd, i + IDBASE),
-		      GWL_WNDPROC, (LONG) wpOrigProc);
+	SetWindowLongPtr(GetDlgItem(hwnd, i + IDBASE),
+			 GWLP_WNDPROC, (LONG_PTR) wpOrigProc);
     }
     return FALSE;
   default:
@@ -522,11 +516,12 @@ bool PDialog::buildAndRun(int w, int h)
   buildString(t,"MS Shell Dlg");
   buildElements(t);
   int res =
-    DialogBoxIndirectParamW((HINSTANCE) GetWindowLong(iParent, GWL_HINSTANCE),
+    DialogBoxIndirectParamW((HINSTANCE) GetWindowLongPtr(iParent,
+							 GWLP_HINSTANCE),
 			    (LPCDLGTEMPLATE) &t[0],
-			    iParent, dialogProc, (LPARAM) this);
+			    iParent, (DLGPROC) dialogProc, (LPARAM) this);
   // retrieveValues() has been called before EndDialog!
-  hDialog = NULL; // already destroyed by Windows
+  hDialog = nullptr; // already destroyed by Windows
   return (res > 0);
 }
 
@@ -637,10 +632,10 @@ void PDialog::enableItem(int idx, bool value)
 static int dialog_constructor(lua_State *L)
 {
   HWND parent = check_winid(L, 1);
-  const char *s = luaL_checkstring(L, 2);
+  const char *s = luaL_checklstring(L, 2, nullptr);
 
   Dialog **dlg = (Dialog **) lua_newuserdata(L, sizeof(Dialog *));
-  *dlg = 0;
+  *dlg = nullptr;
   luaL_getmetatable(L, "Ipe.dialog");
   lua_setmetatable(L, -2);
   *dlg = new PDialog(L, parent, s);
@@ -678,7 +673,7 @@ PMenu::~PMenu()
 {
   if (hMenu)
     DestroyMenu(hMenu);
-  hMenu = 0;
+  hMenu = nullptr;
   for (int i = 0; i < int(bitmaps.size()); ++i)
     DeleteObject(bitmaps[i]);
 }
@@ -689,7 +684,7 @@ int PMenu::execute(lua_State *L)
   int vy = (int)luaL_checkinteger(L, 3);
   int result = TrackPopupMenu(hMenu,
 			      TPM_NONOTIFY | TPM_RETURNCMD | TPM_RIGHTBUTTON,
-			      vx, vy, 0, hwnd, NULL);
+			      vx, vy, 0, hwnd, nullptr);
   if (1 <= result && result <= int(items.size())) {
     result -= 1;
     lua_pushstring(L, items[result].name.c_str());
@@ -711,7 +706,7 @@ static HBITMAP colorIcon(double red, double green, double blue)
   COLORREF rgb = RGB(r, g, b);
   int cx = GetSystemMetrics(SM_CXMENUCHECK);
   int cy = GetSystemMetrics(SM_CYMENUCHECK);
-  HDC hdc = GetDC(NULL);
+  HDC hdc = GetDC(nullptr);
   HDC memDC = CreateCompatibleDC(hdc);
   HBITMAP bm = CreateCompatibleBitmap(hdc, cx, cy);
   SelectObject(memDC, bm);
@@ -721,16 +716,16 @@ static HBITMAP colorIcon(double red, double green, double blue)
     }
   }
   DeleteDC(memDC);
-  ReleaseDC(NULL, hdc);
+  ReleaseDC(nullptr, hdc);
   return bm;
 }
 
 int PMenu::add(lua_State *L)
 {
-  const char *name = luaL_checkstring(L, 2);
-  WString title(luaL_checkstring(L, 3));
+  const char *name = luaL_checklstring(L, 2, nullptr);
+  WString title(luaL_checklstring(L, 3, nullptr));
   if (lua_gettop(L) == 3) {
-    AppendMenuW(hMenu, MF_STRING, items.size() + 1, title);
+    AppendMenuW(hMenu, MF_STRING, items.size() + 1, title.data());
     Item item;
     item.name = name;
     item.itemIndex = 0;
@@ -744,11 +739,11 @@ int PMenu::add(lua_State *L)
     if (hastable)
       luaL_argcheck(L, lua_istable(L, 5), 5,
 		    "argument is not a function or table");
-    const char *current = 0;
+    const char *current = nullptr;
     if (hascheck) {
       luaL_argcheck(L, lua_isstring(L, 6), 6,
 		    "argument is not a function or string");
-      current = luaL_checkstring(L, 6);
+      current = luaL_checklstring(L, 6, nullptr);
     }
     int no = lua_rawlen(L, 4);
     HMENU sm = CreatePopupMenu();
@@ -756,7 +751,7 @@ int PMenu::add(lua_State *L)
       lua_rawgeti(L, 4, i);
       luaL_argcheck(L, lua_isstring(L, -1), 4, "items must be strings");
       int id = items.size() + 1;
-      const char *item = lua_tostring(L, -1);
+      const char *item = lua_tolstring(L, -1, nullptr);
       if (hastable) {
 	lua_rawgeti(L, 5, i);
 	luaL_argcheck(L, lua_isstring(L, -1), 5, "labels must be strings");
@@ -764,14 +759,14 @@ int PMenu::add(lua_State *L)
 	lua_pushvalue(L, 5);   // function
 	lua_pushnumber(L, i);  // index
  	lua_pushvalue(L, -3);  // name
-	lua_call(L, 2, 1);     // function returns label
+	luacall(L, 2, 1);      // function returns label
 	luaL_argcheck(L, lua_isstring(L, -1), 5,
 		      "function does not return string");
       } else
 	lua_pushvalue(L, -1);
 
-      WString text(lua_tostring(L, -1));
-      AppendMenuW(sm, MF_STRING, id, text);
+      WString text(tostring(L, -1));
+      AppendMenuW(sm, MF_STRING, id, text.data());
       Item mitem;
       mitem.name = name;
       mitem.itemName = item;
@@ -784,7 +779,7 @@ int PMenu::add(lua_State *L)
 	lua_pushvalue(L, 6);   // function
 	lua_pushnumber(L, i);  // index
  	lua_pushvalue(L, -4);  // name
-	lua_call(L, 2, 3);     // function returns red, green, blue
+	luacall(L, 2, 3);      // function returns red, green, blue
 	double red = luaL_checknumber(L, -3);
 	double green = luaL_checknumber(L, -2);
 	double blue = luaL_checknumber(L, -1);
@@ -795,7 +790,7 @@ int PMenu::add(lua_State *L)
       }
       lua_pop(L, 2); // item, text
     }
-    AppendMenuW(hMenu, MF_STRING | MF_POPUP, UINT(sm), title);
+    AppendMenuW(hMenu, MF_STRING | MF_POPUP, (UINT_PTR) sm, title.data());
   }
   return 0;
 }
@@ -806,7 +801,7 @@ static int menu_constructor(lua_State *L)
 {
   HWND hwnd = check_winid(L, 1);
   Menu **m = (Menu **) lua_newuserdata(L, sizeof(Menu *));
-  *m = 0;
+  *m = nullptr;
   luaL_getmetatable(L, "Ipe.menu");
   lua_setmetatable(L, -2);
   *m = new PMenu(hwnd);
@@ -859,7 +854,7 @@ PTimer::PTimer(lua_State *L0, int lua_object, const char *method)
 PTimer::~PTimer()
 {
   if (iTimer)
-    KillTimer(NULL, iTimer);
+    KillTimer(nullptr, iTimer);
   // remove it from all_timers
   for (int i = 0; i < int(all_timers.size()); ++i) {
     if (all_timers[i] == this) {
@@ -873,7 +868,7 @@ void PTimer::elapsed()
 {
   callLua();
   if (iSingleShot) {
-    KillTimer(NULL, iTimer);
+    KillTimer(nullptr, iTimer);
     iTimer = 0;
   }
 }
@@ -883,7 +878,7 @@ int PTimer::setInterval(lua_State *L)
   int t = (int)luaL_checkinteger(L, 2);
   iInterval = t;
   if (iTimer)
-    SetTimer(NULL, iTimer, iInterval, TimerProc);
+    SetTimer(nullptr, iTimer, iInterval, TimerProc);
   return 0;
 }
 
@@ -896,14 +891,14 @@ int PTimer::active(lua_State *L)
 int PTimer::start(lua_State *L)
 {
   if (iTimer == 0)
-    iTimer = SetTimer(NULL, 0, iInterval, TimerProc);
+    iTimer = SetTimer(nullptr, 0, iInterval, TimerProc);
   return 0;
 }
 
 int PTimer::stop(lua_State *L)
 {
   if (iTimer) {
-    KillTimer(NULL, iTimer);
+    KillTimer(nullptr, iTimer);
     iTimer = 0;
   }
   return 0;
@@ -914,10 +909,10 @@ int PTimer::stop(lua_State *L)
 static int timer_constructor(lua_State *L)
 {
   luaL_argcheck(L, lua_istable(L, 1), 1, "argument is not a table");
-  const char *method = luaL_checkstring(L, 2);
+  const char *method = luaL_checklstring(L, 2, nullptr);
 
   Timer **t = (Timer **) lua_newuserdata(L, sizeof(Timer *));
-  *t = 0;
+  *t = nullptr;
   luaL_getmetatable(L, "Ipe.timer");
   lua_setmetatable(L, -2);
 
@@ -971,29 +966,29 @@ static int ipeui_getColor(lua_State *L)
 
 static int ipeui_fileDialog(lua_State *L)
 {
-  static const char * const typenames[] = { "open", "save", 0 };
+  static const char * const typenames[] = { "open", "save", nullptr };
   HWND hwnd = check_winid(L, 1);
-  int type = luaL_checkoption(L, 2, 0, typenames);
-  WString caption(luaL_checkstring(L, 3));
+  int type = luaL_checkoption(L, 2, nullptr, typenames);
+  WString caption(luaL_checklstring(L, 3, nullptr));
   if (!lua_istable(L, 4))
     luaL_argerror(L, 4, "table expected for filters");
-  std::vector<wchar_t> filters;
+  std::wstring filters;
   int nFilters = lua_rawlen(L, 4);
   for (int i = 1; i <= nFilters; ++i) {
     lua_rawgeti(L, 4, i);
     luaL_argcheck(L, lua_isstring(L, -1), 4, "filter entry is not a string");
-    WString el(lua_tostring(L, -1));
-    el.appendTo(filters);
+    WString el(tostring(L, -1));
+    filters += el;
     lua_pop(L, 1); // element i
   }
   filters.push_back(0); // terminate list
 
-  const char *dir = 0;
+  const char *dir = nullptr;
   if (!lua_isnoneornil(L, 5))
-    dir = luaL_checkstring(L, 5);
-  const char *name = 0;
+    dir = luaL_checklstring(L, 5, nullptr);
+  const char *name = nullptr;
   if (!lua_isnoneornil(L, 6))
-    name = luaL_checkstring(L, 6);
+    name = luaL_checklstring(L, 6, nullptr);
   int selected = 0;
   if (!lua_isnoneornil(L, 7))
     selected = luaL_checkinteger(L, 7);
@@ -1017,9 +1012,9 @@ static int ipeui_fileDialog(lua_State *L)
   }
   if (dir) {
     WString wdir(dir);
-    ofn.lpstrInitialDir = wdir;
+    ofn.lpstrInitialDir = wdir.data();
   }
-  ofn.lpstrTitle = caption;
+  ofn.lpstrTitle = caption.data();
 
   BOOL result;
   if (type == 0) {
@@ -1043,22 +1038,22 @@ static int ipeui_fileDialog(lua_State *L)
 static int ipeui_messageBox(lua_State *L)
 {
   static const char * const options[] =  {
-    "none", "warning", "information", "question", "critical", 0 };
+    "none", "warning", "information", "question", "critical", nullptr };
   static const char * const buttontype[] = {
     "ok", "okcancel", "yesnocancel", "discardcancel",
-    "savediscardcancel", 0 };
+    "savediscardcancel", nullptr };
 
   HWND hwnd = check_winid(L, 1);
   int type = luaL_checkoption(L, 2, "none", options);
-  const char *text = luaL_checkstring(L, 3);
-  const char *details = 0;
+  const char *text = luaL_checklstring(L, 3, nullptr);
+  const char *details = nullptr;
   if (!lua_isnoneornil(L, 4))
-    details = luaL_checkstring(L, 4);
+    details = luaL_checklstring(L, 4, nullptr);
   int buttons = 0;
   if (lua_isnumber(L, 5))
     buttons = (int)luaL_checkinteger(L, 5);
   else if (!lua_isnoneornil(L, 5))
-    buttons = luaL_checkoption(L, 5, 0, buttontype);
+    buttons = luaL_checkoption(L, 5, nullptr, buttontype);
 
   UINT uType = MB_APPLMODAL;
   switch (type) {
@@ -1105,11 +1100,9 @@ static int ipeui_messageBox(lua_State *L)
     char buf[strlen(text) + strlen(details) + 8];
     sprintf(buf, "%s\n\n%s", text, details);
     WString wbuf(buf);
-    ret = MessageBoxW(hwnd, wbuf, L"Ipe", uType);
-  } else {
-    WString wbuf(text);
-    ret = MessageBoxW(hwnd, wbuf, L"Ipe", uType);
-  }
+    ret = MessageBoxW(hwnd, wbuf.data(), L"Ipe", uType);
+  } else
+    ret = MessageBoxW(hwnd, WString(text).data(), L"Ipe", uType);
 
   switch (ret) {
   case IDOK:
@@ -1193,16 +1186,16 @@ static int ipeui_wait(lua_State *L)
   startupInfo.cb = sizeof(startupInfo);
 
   // Call the executable program
-  const char *cmd = luaL_checkstring(L, 2);
+  const char *cmd = luaL_checklstring(L, 2, nullptr);
   WString wcmd(cmd);
 
-  int result = CreateProcessW(NULL, wcmd, NULL, NULL, FALSE,
+  int result = CreateProcessW(nullptr, wcmd.data(), nullptr, nullptr, FALSE,
 			      NORMAL_PRIORITY_CLASS|CREATE_NO_WINDOW,
-			      NULL, NULL, &startupInfo, &processInformation);
+			      nullptr, nullptr, &startupInfo, &processInformation);
   if (result == 0)
     return 0;
 
-  HWND dialogHandle = NULL;
+  HWND dialogHandle = nullptr;
   HANDLE waitHandle;
 
   RegisterWaitForSingleObject(&waitHandle,
@@ -1212,9 +1205,9 @@ static int ipeui_wait(lua_State *L)
 			      INFINITE,
 			      WT_EXECUTEINWAITTHREAD|WT_EXECUTEONLYONCE);
 
-  DialogBoxIndirectParamW((HINSTANCE) GetWindowLong(parent, GWL_HINSTANCE),
+  DialogBoxIndirectParamW((HINSTANCE) GetWindowLongPtr(parent, GWLP_HINSTANCE),
 			  (LPCDLGTEMPLATE) &t[0],
-			  parent, waitDialogProc,
+			  parent, (DLGPROC) waitDialogProc,
 			  (LPARAM) &dialogHandle);
 
   UnregisterWait(waitHandle);
@@ -1229,7 +1222,7 @@ static int ipeui_currentDateTime(lua_State *L)
 {
   SYSTEMTIME st;
   GetLocalTime(&st);
-  char buf[16];
+  char buf[32];
   sprintf(buf, "%04d%02d%02d%02d%02d%02d",
 	  st.wYear, st.wMonth, st.wDay,
 	  st.wHour, st.wMinute, st.wSecond);
@@ -1239,8 +1232,8 @@ static int ipeui_currentDateTime(lua_State *L)
 
 static int ipeui_startBrowser(lua_State *L)
 {
-  const char *url = luaL_checkstring(L, 1);
-  int res = int(ShellExecuteA(0, "open", url, 0, 0, SW_SHOWNORMAL));
+  const char *url = luaL_checklstring(L, 1, nullptr);
+  long long int res = (long long int) ShellExecuteA(nullptr, "open", url, nullptr, nullptr, SW_SHOWNORMAL);
   lua_pushboolean(L, (res >= 32));
   return 1;
 }
@@ -1257,7 +1250,7 @@ static const struct luaL_Reg ipeui_functions[] = {
   { "waitDialog", ipeui_wait },
   { "currentDateTime", ipeui_currentDateTime },
   { "startBrowser", ipeui_startBrowser },
-  { 0, 0},
+  { nullptr, nullptr }
 };
 
 // --------------------------------------------------------------------
